@@ -341,6 +341,16 @@ GSIMapMoreNodes(GSIMapTable map)
   GSIMapNode	*newArray;
   size_t	arraySize = (map->chunkCount+1)*sizeof(GSIMapNode);
 
+#if	GS_WITH_GC == 1
+  /*
+   * Our nodes may be allocated from the atomic zone - but we don't want
+   * them freed - so we must keep the array of pointers to memory chunks in
+   * the default zone
+   */
+  if (map->zone == GSAtomicMallocZone())
+    newArray = (GSIMapNode*)NSZoneMalloc(NSDefaultMallocZone(), arraySize);
+  else
+#endif
   newArray = (GSIMapNode*)NSZoneMalloc(map->zone, arraySize);
   if (newArray)
     {
@@ -364,17 +374,7 @@ GSIMapMoreNodes(GSIMapTable map)
 	  chunkCount = ((map->nodeCount>>2)+1)<<1;
 	}
       chunkSize = chunkCount * sizeof(GSIMapNode_t);
-#if	GS_WITH_GC
-      /*
-       *	If we use a nil zone, objects we point to are subject to GC
-       */
-      if (map->zone == 0)
-	newNodes = (GSIMapNode*)GC_MALLOC_ATOMIC(chunkSize);
-      else
-	newNodes = (GSIMapNode*)GC_MALLOC(chunkSize);
-#else
       newNodes = (GSIMapNode)NSZoneMalloc(map->zone, chunkSize);
-#endif
       if (newNodes)
 	{
 	  map->nodeChunks[map->chunkCount++] = newNodes;
@@ -517,20 +517,18 @@ GSIMapResize(GSIMapTable map, size_t new_capacity)
    *	around powers of two - we don't want lots of keys falling into a single
    *	bucket.
    */
-  if (size == 8) size++;
+  if (size == 8)
+    size++;
 
   /*
    *	Make a new set of buckets for this map
    */
   new_buckets = (GSIMapBucket)NSZoneCalloc(map->zone, size,
-		sizeof(GSIMapBucket_t));
+    sizeof(GSIMapBucket_t));
   if (new_buckets != 0)
     {
-      GSIMapRemangleBuckets(map,
-				  map->buckets,
-				  map->bucketCount,
-				  new_buckets,
-				  size);
+      GSIMapRemangleBuckets(map, map->buckets, map->bucketCount, new_buckets,
+	size);
 
       if (map->buckets != 0)
 	{
@@ -727,11 +725,7 @@ GSIMapEmptyMap(GSIMapTable map)
     {
       for (i = 0; i < map->chunkCount; i++)
 	{
-#if	GS_WITH_GC
-	  GC_FREE(map->nodeChunks[i]);
-#else
 	  NSZoneFree(map->zone, map->nodeChunks[i]);
-#endif
 	}
       map->chunkCount = 0;
       NSZoneFree(map->zone, map->nodeChunks);
