@@ -37,6 +37,7 @@
 #include <Foundation/NSUtilities.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSDebug.h>
 
 #include <base/fast.x>
 
@@ -50,9 +51,19 @@
 @interface NSMutableArrayNonCore : NSMutableArray
 @end
 
+static Class NSArray_abstract_class;
 static Class NSArray_concrete_class;
+static Class NSMutableArray_abstract_class;
 static Class NSMutableArray_concrete_class;
 
+static SEL	addSel = @selector(addObject:);
+static SEL	appSel = @selector(appendString:);
+static SEL	countSel = @selector(count);
+static SEL	eqSel = @selector(isEqual:);
+static SEL	oaiSel = @selector(objectAtIndex:);
+static SEL	remSel = @selector(removeObjectAtIndex:);
+static SEL	rlSel = @selector(removeLastObject);
+
 
 @implementation NSArray
 
@@ -60,6 +71,8 @@ static Class NSMutableArray_concrete_class;
 {
   if (self == [NSArray class])
     {
+      NSArray_abstract_class = [NSArray class];
+      NSMutableArray_abstract_class = [NSMutableArray class];
       NSArray_concrete_class = [NSGArray class];
       NSMutableArray_concrete_class = [NSGMutableArray class];
       behavior_class_add_class (self, [NSArrayNonCore class]);
@@ -86,31 +99,31 @@ static Class NSMutableArray_concrete_class;
   return NSMutableArray_concrete_class;
 }
 
-+ allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  if ([self class] == [NSArray class])
-    return NSAllocateObject ([self _concreteClass], 0, z);
+  if (self == NSArray_abstract_class)
+    return NSAllocateObject(NSArray_concrete_class, 0, z);
   return [super allocWithZone: z];
 }
 
-+ array
++ (id) array
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()] init]);
 }
 
-+ arrayWithArray: (NSArray*)array
++ (id) arrayWithArray: (NSArray*)array
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithArray: array]);
 }
 
-+ arrayWithContentsOfFile: (NSString*)file
++ (id) arrayWithContentsOfFile: (NSString*)file
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithContentsOfFile: file]);
 }
 
-+ arrayWithObject: anObject
++ (id) arrayWithObject: anObject
 {
   if (anObject == nil)
     [NSException raise: NSInvalidArgumentException
@@ -120,7 +133,7 @@ static Class NSMutableArray_concrete_class;
 }
 
 /* This is the designated initializer for NSArray. */
-- initWithObjects: (id*)objects count: (unsigned)count
+- (id) initWithObjects: (id*)objects count: (unsigned)count
 {
   [self subclassResponsibility: _cmd];
   return nil;
@@ -132,7 +145,7 @@ static Class NSMutableArray_concrete_class;
   return 0;
 }
 
-- objectAtIndex: (unsigned)index
+- (id) objectAtIndex: (unsigned)index
 {
   [self subclassResponsibility: _cmd];
   return nil;
@@ -142,55 +155,54 @@ static Class NSMutableArray_concrete_class;
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-    unsigned	count = [self count];
+  unsigned	count = [self count];
 
-    [aCoder encodeValueOfObjCType: @encode(unsigned)
-			       at: &count];
+  [aCoder encodeValueOfObjCType: @encode(unsigned)
+			     at: &count];
 
-    if (count > 0) {
-        unsigned	i;
+  if (count > 0)
+    {
+      id	a[count];
 
-	for (i = 0; i < count; i++) {
-	    [aCoder encodeObject: [self objectAtIndex: i]];
-	}
+      [self getObjects: a];
+      [aCoder encodeArrayOfObjCType: @encode(id)
+                              count: count
+                                 at: a];
     }
 }
 
 - (id) initWithCoder: (NSCoder*)aCoder
 {
-    unsigned    count;
+  unsigned    count;
 
-    if ([aCoder systemVersion] == 0) {
-	unsigned dummy;
-	[aCoder decodeValueOfObjCType: @encode(unsigned) at: &dummy];
-	[aCoder decodeValueOfObjCType: @encode(unsigned) at: &dummy];
-    }
-    [aCoder decodeValueOfObjCType: @encode(unsigned) at: &count];
-    if (count > 0) {
-	id	contents[count];
-	int	i;
+  [aCoder decodeValueOfObjCType: @encode(unsigned)
+			     at: &count];
+  if (count > 0)
+    {
+      id	contents[count];
 
-	for (i = 0; i < count; i++) {
-	    contents[i] = [aCoder decodeObject];
-	}
-	return [self initWithObjects: contents count: count];
+      [aCoder decodeArrayOfObjCType: @encode(id)
+                              count: count
+                                 at: contents];
+      return [self initWithObjects: contents count: count];
     }
+  else
     return [self initWithObjects: 0 count: 0];
 }
 
 /* The NSCopying Protocol */
 
-- copyWithZone: (NSZone*)zone
+- (id) copyWithZone: (NSZone*)zone
 {
   return RETAIN(self);
 }
 
 /* The NSMutableCopying Protocol */
 
-- mutableCopyWithZone: (NSZone*)zone
+- (id) mutableCopyWithZone: (NSZone*)zone
 {
-  return [[[[self class] _mutableConcreteClass] allocWithZone: zone] 
-	  initWithArray: self];
+  return [[NSMutableArray_concrete_class allocWithZone: zone] 
+    initWithArray: self];
 }
 
 @end
@@ -198,20 +210,26 @@ static Class NSMutableArray_concrete_class;
 
 @implementation NSArrayNonCore
 
-- (NSArray*) arrayByAddingObject: anObject
+- (NSArray*) arrayByAddingObject: (id)anObject
 {
   id na;
-  unsigned c;
+  unsigned	c = [self count];
  
-  c = [self count];
-  {
-    id	objects[c+1];
+  if (anObject == nil)
+    [NSException raise: NSInvalidArgumentException
+		format: @"Attempt to add nil to an array"];
+  if (c == 0)
+    na = [[NSArray_concrete_class allocWithZone: NSDefaultMallocZone()]
+      initWithObjects: &anObject count: 1];
+  else
+    {
+      id	objects[c+1];
 
-    [self getObjects: objects];
-    objects[c] = anObject;
-    na = [[NSArray allocWithZone: NSDefaultMallocZone()]
-      initWithObjects: objects count: c+1];
-  }
+      [self getObjects: objects];
+      objects[c] = anObject;
+      na = [[NSArray_concrete_class allocWithZone: NSDefaultMallocZone()]
+	initWithObjects: objects count: c+1];
+    }
   return AUTORELEASE(na);
 }
 
@@ -227,12 +245,12 @@ static Class NSMutableArray_concrete_class;
 
     [self getObjects: objects];
     [anotherArray getObjects: &objects[c]];
-    na = [NSArray arrayWithObjects: objects count: c+l];
+    na = [NSArray_abstract_class arrayWithObjects: objects count: c+l];
   }
   return na;
 }
 
-- initWithObjects: firstObject rest: (va_list) ap
+- (id) initWithObjects: firstObject rest: (va_list) ap
 {
   register	unsigned		i;
   register	unsigned		curSize;
@@ -244,7 +262,7 @@ static Class NSMutableArray_concrete_class;
   /*	Do initial allocation.	*/
   prevSize = 3;
   curSize  = 5;
-  OBJC_MALLOC(objsArray, id, curSize);
+  objsArray = (id*)NSZoneMalloc(NSDefaultMallocZone(), sizeof(id) * curSize);
   tmpId = firstObject;
 
   /*	Loop through adding objects to array until a nil is
@@ -266,7 +284,8 @@ static Class NSMutableArray_concrete_class;
 	  curSize  = newSize;
 
 	  /*	Reallocate object array.	*/
-	  OBJC_REALLOC(objsArray, id, curSize);
+	  objsArray = (id*)NSZoneRealloc(NSDefaultMallocZone(), objsArray,
+	    sizeof(id) * curSize);
 	}
       tmpId = va_arg(ap, id);
     }
@@ -274,11 +293,11 @@ static Class NSMutableArray_concrete_class;
 
   /*	Put object ids into NSArray.	*/
   self = [self initWithObjects: objsArray count: i];
-  OBJC_FREE( objsArray );
+  NSZoneFree(NSDefaultMallocZone(), objsArray);
   return( self );
 }
 
-- initWithObjects: firstObject, ...
+- (id) initWithObjects: firstObject, ...
 {
   va_list ap;
   va_start(ap, firstObject);
@@ -287,7 +306,7 @@ static Class NSMutableArray_concrete_class;
   return self;
 }
 
-- initWithContentsOfFile: (NSString*)file
+- (id) initWithContentsOfFile: (NSString*)file
 {
   NSString 	*myString;
 
@@ -295,7 +314,7 @@ static Class NSMutableArray_concrete_class;
     initWithContentsOfFile: file];
   if (myString)
     {
-      id result = [myString propertyList];
+      id result;
 
       NS_DURING
 	{
@@ -307,18 +326,18 @@ static Class NSMutableArray_concrete_class;
 	}
       NS_ENDHANDLER
       RELEASE(myString);
-      if ([result isKindOfClass: [NSArray class]])
+      if ([result isKindOfClass: NSArray_abstract_class])
 	{
 	  [self initWithArray: result];
 	  return self;
 	}
     }
-  NSLog(@"Contents of file does not contain an array");
+  NSWarnMLog(@"Contents of file does not contain an array", 0);
   RELEASE(self);
   return nil;
 }
 
-+ arrayWithObjects: firstObject, ...
++ (id) arrayWithObjects: firstObject, ...
 {
   va_list ap;
   va_start(ap, firstObject);
@@ -328,13 +347,13 @@ static Class NSMutableArray_concrete_class;
   return AUTORELEASE(self);
 }
 
-+ arrayWithObjects: (id*)objects count: (unsigned)count
++ (id) arrayWithObjects: (id*)objects count: (unsigned)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithObjects: objects count: count]);
 }
 
-- initWithArray: (NSArray*)array
+- (id) initWithArray: (NSArray*)array
 {
   unsigned c;
 
@@ -351,19 +370,21 @@ static Class NSMutableArray_concrete_class;
 - (void) getObjects: (id*)aBuffer
 {
   unsigned i, c = [self count];
+  IMP	get = [self methodForSelector: oaiSel];
 
   for (i = 0; i < c; i++)
-    aBuffer[i] = [self objectAtIndex: i];
+    aBuffer[i] = (*get)(self, oaiSel, i);
 }
 
 - (void) getObjects: (id*)aBuffer range: (NSRange)aRange
 {
   unsigned i, j = 0, c = [self count], e = aRange.location + aRange.length;
+  IMP	get = [self methodForSelector: oaiSel];
 
   GS_RANGE_CHECK(aRange, c);
 
   for (i = aRange.location; i < e; i++)
-    aBuffer[j++] = [self objectAtIndex: i];
+    aBuffer[j++] = (*get)(self, oaiSel, i);
 }
 
 - (unsigned) hash
@@ -373,46 +394,65 @@ static Class NSMutableArray_concrete_class;
 
 - (unsigned) indexOfObjectIdenticalTo: anObject
 {
-  unsigned i, c = [self count];
-  for (i = 0; i < c; i++)
-    if (anObject == [self objectAtIndex: i])
-      return i;
+  unsigned c = [self count];
+
+  if (c > 0)
+    {
+      IMP	get = [self methodForSelector: oaiSel];
+      unsigned	i;
+
+      for (i = 0; i < c; i++)
+	if (anObject == (*get)(self, oaiSel, i))
+	  return i;
+    }
   return NSNotFound;
 }
 
 - (unsigned) indexOfObjectIdenticalTo: anObject inRange: (NSRange)aRange
 {
   unsigned i, e = aRange.location + aRange.length, c = [self count];
+  IMP	get = [self methodForSelector: oaiSel];
 
   GS_RANGE_CHECK(aRange, c);
 
   for (i = aRange.location; i < e; i++)
-    if (anObject == [self objectAtIndex: i])
+    if (anObject == (*get)(self, oaiSel, i))
       return i;
   return NSNotFound;
 }
 
 /* Inefficient, should be overridden. */
-- (unsigned) indexOfObject: anObject
+- (unsigned) indexOfObject: (id)anObject
 {
-  unsigned i, c = [self count];
-  for (i = 0; i < c; i++)
-    if ([[self objectAtIndex: i] isEqual: anObject])
-      return i;
+  unsigned	c = [self count];
+
+  if (c > 0 && anObject != nil)
+    {
+      unsigned	i;
+      IMP	get = [self methodForSelector: oaiSel];
+      BOOL	(*eq)(id, SEL, id)
+	= (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
+
+      for (i = 0; i < c; i++)
+	if ((*eq)(anObject, eqSel, (*get)(self, oaiSel, i)) == YES)
+	  return i;
+    }
   return NSNotFound;
 }
 
 /* Inefficient, should be overridden. */
-- (unsigned) indexOfObject: anObject inRange: (NSRange)aRange
+- (unsigned) indexOfObject: (id)anObject inRange: (NSRange)aRange
 {
   unsigned i, e = aRange.location + aRange.length, c = [self count];
+  IMP	get = [self methodForSelector: oaiSel];
+  BOOL	(*eq)(id, SEL, id)
+    = (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
 
   GS_RANGE_CHECK(aRange, c);
 
   for (i = aRange.location; i < e; i++)
     {
-      id o = [self objectAtIndex: i];
-      if (anObject == o || [o isEqual: anObject])
+      if ((*eq)(anObject, eqSel, (*get)(self, oaiSel, i)) == YES)
         return i;
     }
   return NSNotFound;
@@ -423,26 +463,37 @@ static Class NSMutableArray_concrete_class;
   return ([self indexOfObject: anObject] != NSNotFound);
 }
 
-- (BOOL) isEqual: anObject
+- (BOOL) isEqual: (id)anObject
 {
-  if ([anObject isKindOf: [NSArray class]])
+  if (self == anObject)
+    return YES;
+  if ([anObject isKindOfClass: NSArray_abstract_class])
     return [self isEqualToArray: anObject];
   return NO;
 }
 
 - (BOOL) isEqualToArray: (NSArray*)otherArray
 {
-  unsigned i, c = [self count];
+  unsigned i, c;
  
+  if (self == (id)otherArray)
+    return YES;
+  c = [self count];
   if (c != [otherArray count])
     return NO;
-  for (i = 0; i < c; i++)
-    if (![[self objectAtIndex: i] isEqual: [otherArray objectAtIndex: i]])
-      return NO;
+  if (c > 0)
+    {
+      IMP	get0 = [self methodForSelector: oaiSel];
+      IMP	get1 = [otherArray methodForSelector: oaiSel];
+
+      for (i = 0; i < c; i++)
+	if (![(*get0)(self, oaiSel, i) isEqual: (*get1)(otherArray, oaiSel, i)])
+	  return NO;
+    }
   return YES;
 }
 
-- lastObject
+- (id) lastObject
 {
   unsigned count = [self count];
   if (count == 0)
@@ -453,8 +504,14 @@ static Class NSMutableArray_concrete_class;
 - (void) makeObjectsPerformSelector: (SEL)aSelector
 {
   unsigned i = [self count];
-  while (i-- > 0)
-    [[self objectAtIndex: i] performSelector: aSelector];
+
+  if (i > 0)
+    {
+      IMP	get = [self methodForSelector: oaiSel];
+
+      while (i-- > 0)
+	[(*get)(self, oaiSel, i) performSelector: aSelector];
+    }
 }
 
 - (void) makeObjectsPerform: (SEL)aSelector
@@ -462,14 +519,20 @@ static Class NSMutableArray_concrete_class;
    [self makeObjectsPerformSelector: aSelector];
 }
 
-- (void) makeObjectsPerformSelector: (SEL)aSelector withObject: argument
+- (void) makeObjectsPerformSelector: (SEL)aSelector withObject: (id) arg
 {
   unsigned i = [self count];
-  while (i-- > 0)
-    [[self objectAtIndex: i] performSelector: aSelector withObject: argument];
+
+  if (i > 0)
+    {
+      IMP	get = [self methodForSelector: oaiSel];
+
+      while (i-- > 0)
+	[(*get)(self, oaiSel, i) performSelector: aSelector withObject: arg];
+    }
 }
 
-- (void) makeObjectsPerform: (SEL)aSelector withObject: argument
+- (void) makeObjectsPerform: (SEL)aSelector withObject: (id)argument
 {
    [self makeObjectsPerformSelector: aSelector withObject: argument];
 }
@@ -481,7 +544,7 @@ static Class NSMutableArray_concrete_class;
       return (int)[elem1 performSelector: comparator withObject: elem2];
     }
 
-    return [self sortedArrayUsingFunction: compare context: NULL];
+  return [self sortedArrayUsingFunction: compare context: NULL];
 }
 
 - (NSArray*) sortedArrayUsingFunction: (int(*)(id,id,void*))comparator 
@@ -492,7 +555,7 @@ static Class NSMutableArray_concrete_class;
 
 - (NSData*) sortedArrayHint
 {
-    return nil;
+  return nil;
 }
 
 - (NSArray*) sortedArrayUsingFunction: (int(*)(id,id,void*))comparator 
@@ -502,10 +565,10 @@ static Class NSMutableArray_concrete_class;
   NSMutableArray	*sortedArray;
   NSArray		*result;
 
-  sortedArray = [[NSMutableArray allocWithZone: NSDefaultMallocZone()]
-    initWithArray: self];
+  sortedArray = [[NSMutableArray_abstract_class allocWithZone:
+    NSDefaultMallocZone()] initWithArray: self];
   [sortedArray sortUsingFunction: comparator context: context];
-  result = [NSArray arrayWithArray: sortedArray];
+  result = [NSArray_abstract_class arrayWithArray: sortedArray];
   RELEASE(sortedArray);
   return result;
 }
@@ -530,17 +593,22 @@ static Class NSMutableArray_concrete_class;
 {
   unsigned i, c = [self count];
   NSMutableArray *a = [NSMutableArray arrayWithCapacity: 1];
+  Class	cls = [NSString class];
+  IMP	get = [self methodForSelector: oaiSel];
+  IMP	add = [a methodForSelector: addSel];
+
   for (i = 0; i < c; i++)
     {
-      id o = [self objectAtIndex: i];
-      if ([o isKindOfClass: [NSString class]])
+      id o = (*get)(self, oaiSel, i);
+
+      if ([o isKindOfClass: cls])
 	if ([extensions containsObject: [o pathExtension]])
-	  [a addObject: o];
+	  (*add)(a, addSel, o);
     }
   return a;
 }
 
-- firstObjectCommonWithArray: (NSArray*)otherArray
+- (id) firstObjectCommonWithArray: (NSArray*)otherArray
 {
   unsigned i, c = [self count];
   id o;
@@ -627,10 +695,8 @@ static NSString	*indentStrings[] = {
   unsigned		count = [self count];
   NSString		*plists[count];
   unsigned		i;
-  SEL			appSel;
   IMP			appImp;
 
-  appSel = @selector(appendString:);
   appImp = [(NSObject*)result methodForSelector: appSel];
 
   if (level < sizeof(indentStrings)/sizeof(NSString*))
@@ -679,6 +745,11 @@ static NSString	*indentStrings[] = {
   (*appImp)(result, appSel, @")");
 }
 
+- (BOOL) writeToFile: (NSString *)path atomically: (BOOL)useAuxiliaryFile
+{
+  return [[self description] writeToFile: path atomically: useAuxiliaryFile];
+}
+
 @end
 
 
@@ -693,16 +764,16 @@ static NSString	*indentStrings[] = {
     }
 }
 
-+ allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  if ([self class] == [NSMutableArray class])
-    return NSAllocateObject ([self _mutableConcreteClass], 0, z);
+  if (self == NSMutableArray_abstract_class)
+    return NSAllocateObject(NSMutableArray_concrete_class, 0, z);
   return [super allocWithZone: z];
 }
 
 /* The NSCopying Protocol */
 
-- copyWithZone: (NSZone*)zone
+- (id) copyWithZone: (NSZone*)zone
 {
   /* a deep copy */
   unsigned	count = [self count];
@@ -713,15 +784,15 @@ static NSString	*indentStrings[] = {
   [self getObjects: objects];
   for (i = 0; i < count; i++)
     objects[i] = [objects[i] copyWithZone: zone];
-  newArray = [[[[self class] _concreteClass] allocWithZone: zone]
-	      initWithObjects: objects count: count];
+  newArray = [[NSArray_concrete_class allocWithZone: zone]
+    initWithObjects: objects count: count];
   while (i > 0)
     RELEASE(objects[--i]);
   return newArray;
 }
 
 /* This is the desgnated initializer for NSMutableArray */
-- initWithCapacity: (unsigned)numItems
+- (id) initWithCapacity: (unsigned)numItems
 {
   [self subclassResponsibility: _cmd];
   return nil;
@@ -774,127 +845,233 @@ static NSString	*indentStrings[] = {
 
 @implementation NSMutableArrayNonCore
 
-+ arrayWithCapacity: (unsigned)numItems
++ (id) arrayWithCapacity: (unsigned)numItems
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithCapacity: numItems]);
 }
 
-- (BOOL)writeToFile: (NSString *)path atomically: (BOOL)useAuxiliaryFile
-{
-  return [[self description] writeToFile: path atomically: useAuxiliaryFile];
-}
-
 /* Override our superclass's designated initializer to go our's */
-- initWithObjects: (id*)objects count: (unsigned)count
+- (id) initWithObjects: (id*)objects count: (unsigned)count
 {
-  unsigned i;
   self = [self initWithCapacity: count];
-  for (i = 0; i < count; i++)
-    [self addObject: objects[i]];
+  if (count > 0)
+    {
+      unsigned	i;
+      IMP	add = [self methodForSelector: addSel];
+
+      for (i = 0; i < count; i++)
+	(*add)(self, addSel, objects[i]);
+    }
   return self;
 }
 
 - (void) removeLastObject
 {
-  unsigned count = [self count];
+  unsigned	count = [self count];
+
   if (count == 0)
     [NSException raise: NSRangeException
 		 format: @"Trying to remove from an empty array."];
   [self removeObjectAtIndex: count-1];
 }
 
-- (void) removeObjectIdenticalTo: anObject
+- (void) removeObjectIdenticalTo: (id)anObject
 {
-  unsigned pos = NSNotFound;
-  unsigned i = [self count];
+  unsigned	i;
 
-  while (i-- > 0)
+  if (anObject == nil)
     {
-      id o = [self objectAtIndex: i];
-      if (o == anObject)
+      NSWarnMLog(@"attempt to remove nil object", 0);
+      return;
+    }
+  i = [self count];
+  if (i > 0)
+    {
+      IMP	rem = 0;
+      IMP	get = [self methodForSelector: oaiSel];
+
+      while (i-- > 0)
 	{
-	  if (pos != NSNotFound)
-	    [self removeObjectAtIndex: pos];
-	  pos = i;
+	  id	o = (*get)(self, oaiSel, i);
+
+	  if (o == anObject)
+	    {
+	      if (rem == 0)
+		{
+		  rem = [self methodForSelector: remSel];
+		}
+	      (*rem)(self, remSel, i);
+	    }
 	}
     }
-  if (pos != NSNotFound)
-    [self removeObjectAtIndex: pos];
 }
 
-- (void) removeObject: anObject inRange: (NSRange)aRange
+- (void) removeObject: (id)anObject inRange: (NSRange)aRange
 {
-  unsigned c = [self count], s = aRange.location;
-  unsigned i = aRange.location + aRange.length;
-  unsigned pos = NSNotFound;
+  unsigned	c;
+  unsigned	s;
+  unsigned	i;
+
+  if (anObject == nil)
+    {
+      NSWarnMLog(@"attempt to remove nil object", 0);
+      return;
+    }
+  c = [self count];
+  s = aRange.location;
+  i = aRange.location + aRange.length;
   if (i > c)
-    i = c;
-  while (i-- > s)
     {
-      id o = [self objectAtIndex: i];
-      if (o == anObject || [o isEqual: anObject])
+      i = c;
+    }
+  if (i > s)
+    {
+      IMP	rem = 0;
+      IMP	get = [self methodForSelector: oaiSel];
+      BOOL	(*eq)(id, SEL, id)
+	= (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
+
+      while (i-- > s)
 	{
-	  if (pos != NSNotFound)
-	    [self removeObjectAtIndex: pos];
-	  pos = i;
+	  id	o = (*get)(self, oaiSel, i);
+
+	  if (o == anObject || (*eq)(anObject, eqSel, o) == YES)
+	    {
+	      if (rem == 0)
+		{
+		  rem = [self methodForSelector: remSel];
+		  /*
+		   * We need to retain the object so that when we remove the
+		   * first equal object we don't get left with a bad object
+		   * pointer for later comparisons.
+		   */
+		  RETAIN(anObject);
+		}
+	      (*rem)(self, remSel, i);
+	    }
 	}
+#ifndef GS_WITH_GC
+      if (rem != 0)
+	{
+	  RELEASE(anObject);
+	}
+#endif
     }
-  if (pos != NSNotFound)
-    [self removeObjectAtIndex: pos];
 }
 
-- (void) removeObjectIdenticalTo: anObject inRange: (NSRange)aRange
+- (void) removeObjectIdenticalTo: (id)anObject inRange: (NSRange)aRange
 {
-  unsigned c = [self count], s = aRange.location;
-  unsigned i = aRange.location + aRange.length;
-  unsigned pos = NSNotFound;
+  unsigned	c;
+  unsigned	s;
+  unsigned	i;
+
+  if (anObject == nil)
+    {
+      NSWarnMLog(@"attempt to remove nil object", 0);
+      return;
+    }
+  c = [self count];
+  s = aRange.location;
+  i = aRange.location + aRange.length;
   if (i > c)
-    i = c;
-  while (i-- > s)
     {
-      id o = [self objectAtIndex: i];
-      if (o == anObject)
+      i = c;
+    }
+  if (i > s)
+    {
+      IMP	rem = 0;
+      IMP	get = [self methodForSelector: oaiSel];
+
+      while (i-- > s)
 	{
-	  if (pos != NSNotFound)
-	    [self removeObjectAtIndex: pos];
-	  pos = i;
+	  id	o = (*get)(self, oaiSel, i);
+
+	  if (o == anObject)
+	    {
+	      if (rem == 0)
+		{
+		  rem = [self methodForSelector: remSel];
+		}
+	      (*rem)(self, remSel, i);
+	    }
 	}
     }
-  if (pos != NSNotFound)
-    [self removeObjectAtIndex: pos];
 }
 
-- (void) removeObject: anObject
+- (void) removeObject: (id)anObject
 {
-  unsigned pos = NSNotFound;
-  unsigned i = [self count];
+  unsigned	i;
 
-  while (i-- > 0)
+  if (anObject == nil)
     {
-      id o = [self objectAtIndex: i];
-      if (o == anObject || [o isEqual: anObject])
+      NSWarnMLog(@"attempt to remove nil object", 0);
+      return;
+    }
+  i = [self count];
+  if (i > 0)
+    {
+      IMP	rem = 0;
+      IMP	get = [self methodForSelector: oaiSel];
+      BOOL	(*eq)(id, SEL, id)
+	= (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
+
+      while (i-- > 0)
 	{
-	  if (pos != NSNotFound)
-	    [self removeObjectAtIndex: pos];
-	  pos = i;
+	  id	o = (*get)(self, oaiSel, i);
+
+	  if (o == anObject || (*eq)(anObject, eqSel, o) == YES)
+	    {
+	      if (rem == 0)
+		{
+		  rem = [self methodForSelector: remSel];
+		  /*
+		   * We need to retain the object so that when we remove the
+		   * first equal object we don't get left with a bad object
+		   * pointer for later comparisons.
+		   */
+		  RETAIN(anObject);
+		}
+	      (*rem)(self, remSel, i);
+	    }
 	}
+#ifndef GS_WITH_GC
+      if (rem != 0)
+	{
+	  RELEASE(anObject);
+	}
+#endif
     }
-  if (pos != NSNotFound)
-    [self removeObjectAtIndex: pos];
 }
 
 - (void) removeAllObjects
 {
-  while ([self count])
-    [self removeLastObject];
+  unsigned	c = [self count];
+
+  if (c > 0)
+    {
+      IMP	remLast = [self methodForSelector: rlSel];
+
+      while (c--)
+	{
+	  (*remLast)(self, rlSel);
+	}
+    }
 }
 
 - (void) addObjectsFromArray: (NSArray*)otherArray
 {
-  unsigned i, c = [otherArray count];
-  for (i = 0; i < c; i++)
-    [self addObject: [otherArray objectAtIndex: i]];
+  unsigned c = [otherArray count];
+
+  if (c > 0)
+    {
+      unsigned	i;
+      IMP	get = [otherArray methodForSelector: oaiSel];
+      IMP	add = [self methodForSelector: addSel];
+
+      for (i = 0; i < c; i++)
+	(*add)(self, addSel,  (*get)(otherArray, oaiSel, i));
+    }
 }
 
 - (void) setArray: (NSArray *)otherArray
@@ -943,28 +1120,53 @@ static NSString	*indentStrings[] = {
 	    }
 	}
 
-      while (to--)
+      if (to > 0)
 	{
-	  [self removeObjectAtIndex: sorted[to]];
+	  IMP	rem = [self methodForSelector: remSel];
+
+	  while (to--)
+	    {
+	      (*rem)(self, remSel, sorted[to]);
+	    }
 	}
     }
 }
 
 - (void) removeObjectsInArray: (NSArray*)otherArray
 {
-  unsigned i, c = [otherArray count];
-  for (i = 0; i < c; i++)
-    [self removeObject: [otherArray objectAtIndex: i]];
+  unsigned	c = [otherArray count];
+
+  if (c > 0)
+    {
+      unsigned	i;
+      IMP	get = [otherArray methodForSelector: oaiSel];
+      IMP	rem = [self methodForSelector: @selector(removeObject:)];
+
+      for (i = 0; i < c; i++)
+	(*rem)(self, @selector(removeObject:), (*get)(otherArray, oaiSel, i));
+    }
 }
 
 - (void) removeObjectsInRange: (NSRange)aRange
 {
-  unsigned i, s = aRange.location, c = [self count];
+  unsigned	i;
+  unsigned	s = aRange.location;
+  unsigned	c = [self count];
+
   i = aRange.location + aRange.length;
+
   if (c < i)
     i = c;
-  while (i-- > s)
-    [self removeObjectAtIndex: i];
+
+  if (i > s)
+    {
+      IMP	rem = [self methodForSelector: remSel];
+
+      while (i-- > s)
+	{
+	  (*rem)(self, remSel, i);
+	}
+    }
 }
 
 - (void) sortUsingSelector: (SEL)comparator
@@ -974,75 +1176,116 @@ static NSString	*indentStrings[] = {
       return (int)[elem1 performSelector: comparator withObject: elem2];
     }
 
-    [self sortUsingFunction: compare context: NULL];
+  [self sortUsingFunction: compare context: NULL];
 }
 
 - (void) sortUsingFunction: (int(*)(id,id,void*))compare 
-   context: (void*)context
+		   context: (void*)context
 {
   /* Shell sort algorithm taken from SortingInAction - a NeXT example */
 #define STRIDE_FACTOR 3	// good value for stride factor is not well-understood
                         // 3 is a fairly good choice (Sedgewick)
-  unsigned c,d, stride;
-  BOOL found;
-  int count = [self count];
+  unsigned	c,d, stride;
+  BOOL		found;
+  int		count = [self count];
+#ifdef	GSWARN
+  BOOL		badComparison = NO;
+#endif
 
   stride = 1;
   while (stride <= count)
-    stride = stride * STRIDE_FACTOR + 1;
-    
-  while(stride > (STRIDE_FACTOR - 1)) {
-    // loop to sort for each value of stride
-    stride = stride / STRIDE_FACTOR;
-    for (c = stride; c < count; c++) {
-      found = NO;
-      if (stride > c)
-	break;
-      d = c - stride;
-      while (!found) {
-	// move to left until correct place
-	id a = [self objectAtIndex: d + stride];
-	id b = [self objectAtIndex: d];
-	if ((*compare)(a, b, context) == NSOrderedAscending) {
-	  RETAIN(a);
-	  [self replaceObjectAtIndex: d + stride withObject: b];
-	  [self replaceObjectAtIndex: d withObject: a];
-	  RELEASE(a);
-	  if (stride > d)
-	    break;
-	  d -= stride;		// jump by stride factor
-	}
-	else found = YES;
-      }
+    {
+      stride = stride * STRIDE_FACTOR + 1;
     }
-  }
+    
+  while(stride > (STRIDE_FACTOR - 1))
+    {
+      // loop to sort for each value of stride
+      stride = stride / STRIDE_FACTOR;
+      for (c = stride; c < count; c++)
+	{
+	  found = NO;
+	  if (stride > c)
+	    {
+	      break;
+	    }
+	  d = c - stride;
+	  while (!found)	/* move to left until correct place */
+	    {
+	      id			a = [self objectAtIndex: d + stride];
+	      id			b = [self objectAtIndex: d];
+	      NSComparisonResult	r;
+
+	      r = (*compare)(a, b, context);
+	      if (r < 0)
+		{
+#ifdef	GSWARN
+		  if (r != NSOrderedAscending)
+		    {
+		      badComparison = YES;
+		    }
+#endif
+		  IF_NO_GC(RETAIN(a));
+		  [self replaceObjectAtIndex: d + stride withObject: b];
+		  [self replaceObjectAtIndex: d withObject: a];
+		  RELEASE(a);
+		  if (stride > d)
+		    {
+		      break;
+		    }
+		  d -= stride;		// jump by stride factor
+		}
+	      else
+		{
+#ifdef	GSWARN
+		  if (r != NSOrderedDescending && r != NSOrderedSame)
+		    {
+		      badComparison = YES;
+		    }
+#endif
+		  found = YES;
+		}
+	    }
+	}
+    }
+#ifdef	GSWARN
+  if (badComparison == YES)
+    {
+      NSWarnMLog(@"Detected bad return value from comparison", 0);
+    }
+#endif
 }
 
 @end
 
 @interface NSArrayEnumerator : NSEnumerator
 {
-  id array;
-  int next_index;
+  NSArray	*array;
+  unsigned	pos;
+  IMP		get;
+  unsigned	(*cnt)(NSArray*, SEL);
 }
+- (id) initWithArray: (NSArray*)anArray;
 @end
 
 @implementation NSArrayEnumerator
 
-- initWithArray: (NSArray*)anArray
+- (id) initWithArray: (NSArray*)anArray
 {
   [super init];
   array = anArray;
-  RETAIN(array);
-  next_index = 0;
+  IF_NO_GC(RETAIN(array));
+  pos = 0;
+  get = [array methodForSelector: oaiSel];
+  cnt = (unsigned (*)(NSArray*, SEL))[array methodForSelector: countSel];
   return self;
 }
 
 - (id) nextObject
 {
-  if (next_index >= [array count])
+  if (pos >= (*cnt)(array, countSel))
     return nil;
-  return [array objectAtIndex: next_index++];
+  return (*get)(array, oaiSel, pos++);
 }
 
 - (void) dealloc
@@ -1058,19 +1301,17 @@ static NSString	*indentStrings[] = {
 
 @implementation NSArrayEnumeratorReverse
 
-- initWithArray: (NSArray*)anArray
+- (id) initWithArray: (NSArray*)anArray
 {
-  [super init];
-  array = anArray;
-  RETAIN(array);
-  next_index = [array count]-1;
+  [super initWithArray: anArray];
+  pos = (*cnt)(array, countSel);
   return self;
 }
 
 - (id) nextObject
 {
-  if (next_index < 0)
+  if (pos == 0)
     return nil;
-  return [array objectAtIndex: next_index--];
+  return (*get)(array, oaiSel, --pos);
 }
 @end
