@@ -117,6 +117,16 @@
   return AUTORELEASE(d);
 }
 
+- (id) addTimeInterval: (NSTimeInterval)seconds
+{
+  id newObj = [[self class] dateWithTimeIntervalSinceReferenceDate:
+     [self timeIntervalSinceReferenceDate] + seconds];
+	
+  [newObj setTimeZone: [self timeZoneDetail]];
+
+  return newObj;
+}
+
 - (Class) classForPortCoder
 {
   return [self class];
@@ -129,23 +139,25 @@
 
 - (void) encodeWithCoder: (NSCoder*)coder
 {
-  [coder encodeValueOfObjCType: @encode(NSTimeInterval) at: &seconds_since_ref];
-  [coder encodeObject: calendar_format];
-  [coder encodeObject: time_zone];
+  [coder encodeValueOfObjCType: @encode(NSTimeInterval)
+			    at: &_seconds_since_ref];
+  [coder encodeObject: _calendar_format];
+  [coder encodeObject: _time_zone];
 }
 
 - (id) initWithCoder: (NSCoder*)coder
 {
-  [coder decodeValueOfObjCType: @encode(NSTimeInterval) at: &seconds_since_ref];
-  [coder decodeValueOfObjCType: @encode(id) at: &calendar_format];
-  [coder decodeValueOfObjCType: @encode(id) at: &time_zone];
+  [coder decodeValueOfObjCType: @encode(NSTimeInterval)
+			    at: &_seconds_since_ref];
+  [coder decodeValueOfObjCType: @encode(id) at: &_calendar_format];
+  [coder decodeValueOfObjCType: @encode(id) at: &_time_zone];
   return self;
 }
 
 - (void) dealloc
 {
-  RELEASE(calendar_format);
-  RELEASE(time_zone);
+  RELEASE(_calendar_format);
+  RELEASE(_time_zone);
   [super dealloc];
 }
 
@@ -169,48 +181,54 @@
 		       locale: nil];
 }
 
-/*
- * This function could possibly be written better
- * but it works ok; currently ignores locale
- * information and some specifiers.
- */
-- (id) initWithString: (NSString *)description
-       calendarFormat: (NSString *)format
-	       locale: (NSDictionary *)locale
-{
-  const char *d = [description cString];
-  const char *f = [format cString];
-  char *newf;
-  int lf = strlen(f);
-  BOOL mtag = NO, dtag = NO, ycent = NO;
-  BOOL fullm = NO;
-  char ms[80] = "", ds[80] = "", timez[80] = "", ampm[80] = "";
-  int tznum = 0;
-  int yd = 0, md = 0, dd = 0, hd = 0, mnd = 0, sd = 0;
-  void *pntr[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-  int order;
-  int yord = 0, mord = 0, dord = 0, hord = 0, mnord = 0, sord = 0, tzord = 0;
-  int ampmord = 0;
-  int i;
-  NSTimeZone *tz;
-  BOOL zoneByAbbreviation = YES;
 
-  // If either the string or format is nil then raise exception
-  if (!description)
-    [NSException raise: NSInvalidArgumentException
-		format: @"NSCalendar date description is nil"];
+- (id) initWithString: (NSString *)description 
+       calendarFormat: (NSString *)fmt
+               locale: (NSDictionary *)locale
+{
+  int		year = 0, month = 1, day = 1, hour = 0, min = 0, sec = 0;
+  NSTimeZone	*tz = [NSTimeZone localTimeZone];
+  BOOL		ampm = NO;
+  BOOL		twelveHrClock = NO; 
+  int		julianWeeks = -1, weekStartsMonday = 0, dayOfWeek = -1;
+  const char	*source = [description cString];
+  int		sourceLen = strlen(source);
+  const char	*format = [fmt cString];
+  int		formatLen = strlen(format);
+  int		formatIdx = 0;
+  int		sourceIdx = 0;
+  char		tmpStr[20];
+  int		tmpIdx;
 
   if (locale == nil)
-    locale = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
-  if (format == nil)
-    format = [locale objectForKey: NSTimeDateFormatString];
+    {
+      locale = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
+    }
+  if (fmt == nil)
+    {
+      format = [[locale objectForKey: NSTimeDateFormatString] cString];
+      formatLen = strlen(format);
+    }
 
-  // The strftime specifiers
+  //
+  // WARNING:
+  //   -%c, %F, %x, %X do NOT work. (NSTimeDateFormatString isn't defined and
+  //    the underlying call has granularity to the second.
+  //   -Most locale stuff is dubious at best.
+  //   -Long day and month names depend on a non-alpha character after the last
+  //    digit to work.
+  //
+  // The strftime specifiers as used by OpenStep + %U.
+  //
+  // %%   literal % character
   // %a   abbreviated weekday name according to locale
   // %A   full weekday name according to locale
   // %b   abbreviated month name according to locale
   // %B   full month name according to locale
+  // %c   same as '%X %x'
   // %d   day of month as decimal number
+  // %e   same as %d without leading zero (you get a leading space instead)
+  // %F   milliseconds as a decimal number
   // %H   hour as a decimal number using 24-hour clock
   // %I   hour as a decimal number using 12-hour clock
   // %j   day of year as a decimal number
@@ -221,253 +239,366 @@
   // %U   week of the current year as decimal number (Sunday first day)
   // %W   week of the current year as decimal number (Monday first day)
   // %w   day of the week as decimal number (Sunday = 0)
-  // %y   year as a decimal number without century
+  // %x   date with date representation for locale
+  // %X   time with time representation for locale
+  // %y   year as a decimal number without century 
   // %Y   year as a decimal number with century
-  // %Z   time zone
-  // %%   literal % character
+  // %z   time zone offset in hours and minutes from GMT (HHMM)
+  // %Z   time zone abbreviation
 
-  // Find the order of date elements
-  // and translate format string into scanf ready string
-  order = 1;
-  newf = objc_malloc(lf+1);
-  for (i = 0;i < lf; ++i)
+  while (formatIdx < formatLen)
     {
-      newf[i] = f[i];
-
-      // Only care about a format specifier
-      if (f[i] == '%')
+      if (format[formatIdx] != '%')
 	{
-	  // check the character that comes after
-	  switch (f[i+1])
-	    {
-	      // skip literal %
-	    case '%':
-	      ++i;
-	      newf[i] = f[i];
-	      break;
-
-	      // is it the year
-	    case 'Y':
-	      ycent = YES;
-	    case 'y':
-	      yord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 'd';
-	      pntr[yord] = (void *)&yd;
-	      break;
-
-	      // is it the month
-	    case 'B':
-	      fullm = YES;   // Full month name
-	    case 'b':
-	      mtag = YES;    // Month is character string
-	    case 'm':
-	      mord = order;
-	      ++order;
-	      ++i;
-	      if (mtag)
-		{
-		  newf[i] = 's';
-		  pntr[mord] = (void *)ms;
-		}
-	      else
-		{
-		  newf[i] = 'd';
-		  pntr[mord] = (void *)&md;
-		}
-	      break;
-
-	      // is it the day
-	    case 'a':
-	    case 'A':
-	      dtag = YES;   // Day is character string
-	    case 'd':
-	    case 'j':
-	    case 'w':
-	      dord = order;
-	      ++order;
-	      ++i;
-	      if (dtag)
-		{
-		  newf[i] = 's';
-		  pntr[dord] = (void *)ds;
-		}
-	      else
-		{
-		  newf[i] = 'd';
-		  pntr[dord] = (void *)&dd;
-		}
-	      break;
-
-	      // is it the hour
-	    case 'H':
-	    case 'I':
-	      hord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 'd';
-	      pntr[hord] = (void *)&hd;
-	      break;
-
-	      // is it the minute
-	    case 'M':
-	      mnord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 'd';
-	      pntr[mnord] = (void *)&mnd;
-	      break;
-
-	      // is it the second
-	    case 'S':
-	      sord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 'd';
-	      pntr[sord] = (void *)&sd;
-	      break;
-
-	      // the time zone abbreviation
-	    case 'Z':
-	      tzord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 's';
-	      pntr[tzord] = (void *)timez;
-	      break;
-
-	      // the time zone in numeric format
-	    case 'z':
-	      tzord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 'd';
-	      pntr[tzord] = (void *)&tznum;
-	      zoneByAbbreviation = NO;
-	      break;
-
-	      // AM PM indicator
-	    case 'p':
-	      ampmord = order;
-	      ++order;
-	      ++i;
-	      newf[i] = 's';
-	      pntr[ampmord] = (void *)ampm;
-	      break;
-
-	      // Anything else is an invalid format
-	    default:
-	      free(newf);
-	      [NSException raise: NSInvalidArgumentException
-			   format: @"Invalid NSCalendar date, specifier %c not recognized in format %s", f[i+1], f];
-	    }
-	}
-    }
-  newf[lf] = '\0';
-
-  // Have sscanf parse and retrieve the values for us
-  if (order != 1)
-    sscanf(d, newf, pntr[1], pntr[2], pntr[3], pntr[4], pntr[5], pntr[6],
-	   pntr[7], pntr[8], pntr[9]);
-  else
-    // nothing in the string?
-    ;
-
-  // Put century on year if need be
-  // +++ How do we be year 2000 compliant?
-  if (!ycent)
-    yd += 1900;
-
-  // Possibly convert month from string to decimal number
-  if (mtag)
-    {
-      int i;
-      NSString *m = [NSString stringWithCString: ms];
-
-      if (fullm)
-	{
-	  NSArray	*names = [locale objectForKey: NSMonthNameArray];
-
-	  for (i = 0;i < 12; ++i)
-	    if ([[names objectAtIndex: i] isEqual: m] == YES)
-	      break;
+	  // If it's not a format specifier, ignore it.
+	  sourceIdx++;
 	}
       else
 	{
-	  NSArray	*names = [locale objectForKey: NSShortMonthNameArray];
+	  // Skip '%'
+	  formatIdx++;
 
-	  for (i = 0;i < 12; ++i)
-	    if ([[names objectAtIndex: i] isEqual: m] == YES)
-	      break;
-	}
-      md = i + 1;
-    }
-
-  // Possibly convert day from string to decimal number
-  // +++ how do we take locale into account?
-  if (dtag)
-    {
-    }
-
-  // +++ We need to take 'am' and 'pm' into account
-  if (ampmord)
-    {
-      // If its PM then we shift
-      if ((ampm[0] == 'p') || (ampm[0] == 'P'))
-	{
-	  // 12pm is 12pm not 24pm
-	  if (hd != 12)
-	    hd += 12;
-	}
-    }
-
-  // +++ then there is the time zone
-  if (tzord)
-    {
-      if (zoneByAbbreviation)
-	{
-	  tz = [NSTimeZone timeZoneWithAbbreviation:
-			     [NSString stringWithCString: timez]];
-	  if (!tz)
+	  switch (format[formatIdx])
 	    {
-	      tz = [NSTimeZone localTimeZone];
+	      case '%':
+		// skip literal %
+		sourceIdx++;
+		break;
+
+	      case 'a':
+		// Are Short names three chars in all locales?????
+		tmpStr[0] = toupper(source[sourceIdx++]);
+		tmpStr[1] = tolower(source[sourceIdx++]);
+		tmpStr[2] = tolower(source[sourceIdx++]);
+		tmpStr[3] = '\0';
+		{
+		  NSString	*currDay;
+		  NSArray	*dayNames;
+
+		  currDay = [NSString stringWithCString: tmpStr];
+		  dayNames = [locale objectForKey: NSShortWeekDayNameArray];
+		  for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
+		    {
+		      if ([[dayNames objectAtIndex: tmpIdx] isEqual: currDay]
+			== YES)
+			{
+			  break;
+			}
+		    }
+		  dayOfWeek = tmpIdx; 
+		}
+		break;
+
+	      case 'A':
+		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		  {
+		    if (isalpha(source[tmpIdx]))
+		      {
+			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+		      }
+		    else
+		      {
+			break;
+		      }
+		  }
+		tmpStr[tmpIdx - sourceIdx] = '\0';
+		sourceIdx += tmpIdx - sourceIdx;
+	        {
+		  NSString	*currDay;
+		  NSArray	*dayNames;
+
+		  currDay = [NSString stringWithCString: tmpStr];
+		  dayNames = [locale objectForKey: NSWeekDayNameArray];
+		  for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
+		    {
+		      if ([[dayNames objectAtIndex: tmpIdx] isEqual: currDay]
+			== YES)
+			{
+			  break;
+			}
+		    }
+		  dayOfWeek = tmpIdx;
+	        }
+		break;
+
+	      case 'b':
+		// Are Short names three chars in all locales?????
+		tmpStr[0] = toupper(source[sourceIdx++]);
+		tmpStr[1] = tolower(source[sourceIdx++]);
+		tmpStr[2] = tolower(source[sourceIdx++]);
+		tmpStr[3] = '\0';
+	        {
+		  NSString	*currMonth;
+		  NSArray	*monthNames;
+
+		  currMonth = [NSString stringWithCString: tmpStr];
+		  monthNames = [locale objectForKey: NSShortMonthNameArray];
+
+		  for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
+		    {
+		      if ([[monthNames objectAtIndex: tmpIdx]
+			isEqual: currMonth] == YES)
+			{
+			  break;
+			}
+		    }
+		  month = tmpIdx+1;
+	        }
+		break;
+
+	      case 'B':
+		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		  {
+		    if (isalpha(source[tmpIdx]))
+		      {
+			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+		      }
+		    else
+		      {
+			break;
+		      }
+		  }
+		tmpStr[tmpIdx - sourceIdx] = '\0';
+		sourceIdx += tmpIdx - sourceIdx;
+	        {
+		  NSString	*currMonth;
+		  NSArray	*monthNames;
+
+		  currMonth = [NSString stringWithCString: tmpStr];
+		  monthNames = [locale objectForKey: NSMonthNameArray];
+
+		  for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
+		    {
+		      if ([[monthNames objectAtIndex: tmpIdx]
+			isEqual: currMonth] == YES)
+			{
+			  break;
+			}
+		    }
+		  month = tmpIdx+1;
+	        }
+	        break;
+
+//	case 'c':
+//	break;
+
+	      case 'd': // fall through
+	      case 'e':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		day = atoi(tmpStr);
+		break;
+
+//	case 'F':
+//	break;
+
+	      case 'I': // fall through
+		twelveHrClock = YES;
+	      case 'H':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		hour = atoi(tmpStr);
+		break;
+
+	      case 'j':
+		memcpy(tmpStr, &source[sourceIdx], 3);
+		tmpStr[3] = '\0';
+		sourceIdx += 3;
+		day = atoi(tmpStr);
+		break;
+
+	      case 'm':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		month = atoi(tmpStr);
+		break;
+
+	      case 'M':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		min = atoi(tmpStr);
+		break;
+
+	      case 'p':
+		// Questionable assumption that all am/pm indicators are 2
+		// characters and in upper case....
+		tmpStr[0] = toupper(source[sourceIdx++]);
+		tmpStr[1] = toupper(source[sourceIdx++]);
+		tmpStr[2] = '\0';
+		{
+		  NSString	*currAMPM;
+		  NSArray	*amPMNames;
+
+		  currAMPM = [NSString stringWithCString: tmpStr];
+		  amPMNames = [locale objectForKey: NSAMPMDesignation];
+
+		  /*
+		   * The time addition is handled below because this
+		   * indicator only modifies the time on a 12hour clock.
+		   */
+		  if ([[amPMNames objectAtIndex: 1] isEqual: currAMPM] == YES)
+		    {
+		      ampm = YES;
+		    }
+		}
+		break;
+
+	      case 'S':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		sec = atoi(tmpStr);
+		break;
+
+	      case 'w':
+		tmpStr[0] = source[sourceIdx++];
+		tmpStr[1] = '\0';
+		dayOfWeek = atoi(tmpStr);
+		break;
+
+	      case 'W': // Fall through
+		weekStartsMonday = 1;
+	      case 'U':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		julianWeeks = atoi(tmpStr);
+		break;
+
+//	case 'x':
+//	break;
+
+//	case 'X':
+//	break;
+
+	      case 'y':
+		memcpy(tmpStr, &source[sourceIdx], 2);
+		tmpStr[2] = '\0';
+		sourceIdx += 2;
+		year = atoi(tmpStr);
+		if (year >= 70)
+		  {
+		    year += 1900;
+		  }
+		else
+		  {
+		    year += 2000;
+		  }
+		break;
+
+	      case 'Y':
+		memcpy(tmpStr, &source[sourceIdx], 4);
+		tmpStr[4] = '\0';
+		sourceIdx += 4;
+		year = atoi(tmpStr);
+		break;
+
+	      case 'z':
+		tmpStr[0] = toupper(source[sourceIdx++]);
+		tmpStr[1] = tolower(source[sourceIdx++]);
+		tmpStr[2] = tolower(source[sourceIdx++]);
+		tmpStr[3] = tolower(source[sourceIdx++]);
+		tmpStr[4] = '\0';
+	        {
+		  int zone = atoi(tmpStr);
+
+		  if ((tz = [NSTimeZone timeZoneForSecondsFromGMT: 
+		    (zone / 100 * 60 + (zone % 100)) * 60]) == nil)
+		    {
+		      tz = [NSTimeZone localTimeZone];
+		    }
+		}
+		break;
+
+	      case 'Z':
+		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		  {
+		    if (isalpha(source[tmpIdx]) || source[tmpIdx] == '-'
+		      || source[tmpIdx] == '+')
+		      {
+			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+		      }
+		    else
+		      {
+			break;
+		      }
+		  }
+		tmpStr[tmpIdx - sourceIdx] = '\0';
+		sourceIdx += tmpIdx - sourceIdx;
+	        {
+		  if ((tz = [NSTimeZone timeZoneWithAbbreviation: 
+		    [NSString stringWithCString: tmpStr]]) == nil)
+		    {
+		      tz = [NSTimeZone localTimeZone];
+		    }
+	        }
+		break;
+
+	      default:
+		[NSException raise: NSInvalidArgumentException format:
+		  @"Invalid NSCalendar date, specifier %c not recognized in format %s", format[formatIdx], format];
 	    }
-	}
-      else
-	{
-	  int tzm, tzh, sign;
+	} 
+      formatIdx++;
+    }
 
-	  if (tznum < 0)
+  if (twelveHrClock == YES)
+    {
+      if (ampm == YES && hour != 12)
+	{
+	   hour += 12;
+        }
+    }
+
+  if (julianWeeks != -1)
+    {
+      NSTimeZone	*gmtZone;
+      NSCalendarDate	*d;
+      int		currDay;
+
+      gmtZone = [NSTimeZone timeZoneForSecondsFromGMT: 0];
+      d  = [NSCalendarDate dateWithYear: year
+				  month: 1
+				    day: 1
+				   hour: 0
+				 minute: 0
+				 second: 0
+			       timeZone: gmtZone];
+      currDay = [d dayOfWeek];
+
+      /*
+       * The julian weeks are either sunday relative or monday relative but all
+       * of the day of week specifiers are sunday relative.  This means that if
+       * no day of week specifier was used the week starts on monday.
+       */
+      if (dayOfWeek == -1)
+	{
+	  if (weekStartsMonday)
 	    {
-	      sign = -1;
-	      tznum = -tznum;
+	      dayOfWeek = 1;
 	    }
 	  else
 	    {
-	      sign = 1;
-	    }
-	  tzm = tznum % 100;
-	  tzh = tznum / 100;
-	  tz = [NSTimeZone timeZoneForSecondsFromGMT: (tzh*60 + tzm)*60*sign];
-	  if (!tz)
-	    {
-	      tz = [NSTimeZone localTimeZone];
+	      dayOfWeek = 0;
 	    }
 	}
-    }
-  else
-    {
-      tz = [NSTimeZone localTimeZone];
+      day = dayOfWeek + (julianWeeks * 7 - (currDay - 1));
     }
 
-  free(newf);
-
-  return [self initWithYear: yd month: md day: dd hour: hd
-	       minute: mnd second: sd
-	       timeZone: tz];
+  return [[NSCalendarDate alloc] initWithYear: year
+					month: month
+					  day: day
+					 hour: hour
+				       minute: min
+				       second: sec
+				     timeZone: tz];
 }
 
+
 - (id) initWithYear: (int)year
 	      month: (unsigned int)month
 	        day: (unsigned int)day
@@ -490,12 +621,12 @@
   s += second;
 
   // Assign time zone detail
-  time_zone = RETAIN([aTimeZone
+  _time_zone = RETAIN([aTimeZone
 		timeZoneDetailForDate:
 		  [NSDate dateWithTimeIntervalSinceReferenceDate: s]]);
 
   // Adjust date so it is correct for time zone.
-  s -= [time_zone timeZoneSecondsFromGMT];
+  s -= [_time_zone timeZoneSecondsFromGMT];
   self = [self initWithTimeIntervalSinceReferenceDate: s];
 
   /* Now permit up to five cycles of adjustment to allow for daylight savings.
@@ -516,14 +647,14 @@
       z = [aTimeZone
 		timeZoneDetailForDate:
 		  [NSDate dateWithTimeIntervalSinceReferenceDate: s]];
-      if (z != time_zone)
+      if (z != _time_zone)
 	{
 	  NSTimeInterval	oldOffset;
 	  NSTimeInterval	newOffset;
 
-	  oldOffset = [time_zone timeZoneSecondsFromGMT];
-	  ASSIGN(time_zone, z);
-	  newOffset = [time_zone timeZoneSecondsFromGMT];
+	  oldOffset = [_time_zone timeZoneSecondsFromGMT];
+	  ASSIGN(_time_zone, z);
+	  newOffset = [_time_zone timeZoneSecondsFromGMT];
 	  s += newOffset - oldOffset;
 	}
       else
@@ -562,11 +693,11 @@
 // Default initializer
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)seconds
 {
-  seconds_since_ref = seconds;
-  if (!calendar_format)
-    calendar_format = @"%Y-%m-%d %H:%M:%S %z";
-  if (!time_zone)
-    time_zone = RETAIN([[NSTimeZone localTimeZone]
+  _seconds_since_ref = seconds;
+  if (!_calendar_format)
+    _calendar_format = @"%Y-%m-%d %H:%M:%S %z";
+  if (!_time_zone)
+    _time_zone = RETAIN([[NSTimeZone localTimeZone]
       timeZoneDetailForDate: self]);
   return self;
 }
@@ -588,7 +719,7 @@
   // Calculate hour, minute, and seconds
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]));
+  a = abs(d - (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]));
   b = a / 3600;
   *hour = (int)b;
   h = *hour;
@@ -608,7 +739,7 @@
   int r;
 
   // Get reference date in terms of days
-  a = (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]) / 86400.0;
+  a = (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]) / 86400.0;
   // Offset by Gregorian reference
   a += GREGORIAN_REFERENCE;
   r = (int)a;
@@ -659,7 +790,7 @@
   double a, d = [self dayOfCommonEra];
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]));
+  a = abs(d - (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]));
   a = a / 3600;
   h = (int)a;
 
@@ -677,7 +808,7 @@
   double a, b, d = [self dayOfCommonEra];
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]));
+  a = abs(d - (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]));
   b = a / 3600;
   h = (int)b;
   h = h * 3600;
@@ -704,7 +835,7 @@
   double a, b, c, d = [self dayOfCommonEra];
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]));
+  a = abs(d - (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]));
   b = a / 3600;
   h = (int)b;
   h = h * 3600;
@@ -721,13 +852,9 @@
 - (int) yearOfCommonEra
 {
   int m, d, y;
-  int a;
 
-  // Get reference date in terms of days
-  a = (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]) / 86400;
-  // Offset by Gregorian reference
-  a += GREGORIAN_REFERENCE;
-  [self gregorianDateFromAbsolute: a day: &d month: &m year: &y];
+  [self gregorianDateFromAbsolute: [self dayOfCommonEra]
+	day: &d month: &m year: &y];
 
   return y;
 }
@@ -751,7 +878,7 @@
 // Getting String Descriptions of Dates
 - (NSString*) description
 {
-  return [self descriptionWithCalendarFormat: calendar_format locale: nil];
+  return [self descriptionWithCalendarFormat: _calendar_format locale: nil];
 }
 
 - (NSString*) descriptionWithCalendarFormat: (NSString *)format
@@ -884,7 +1011,7 @@
 
 	    case 'F': 	// milliseconds
 	      s = ([self dayOfCommonEra] - GREGORIAN_REFERENCE) * 86400.0;
-	      s -= (seconds_since_ref+[time_zone timeZoneSecondsFromGMT]);
+	      s -= (_seconds_since_ref+[_time_zone timeZoneSecondsFromGMT]);
 	      s = abs(s);
 	      s -= floor(s);
 	      ++i;
@@ -985,13 +1112,13 @@
 	    case 'Z':
 	      ++i;
 	      k = VSPRINTF_LENGTH(sprintf(&(buf[j]), "%s",
-			  [[time_zone timeZoneAbbreviation] cString]));
+			  [[_time_zone timeZoneAbbreviation] cString]));
 	      j += k;
 	      break;
 
 	    case 'z':
 	      ++i;
-	      z = [time_zone timeZoneSecondsFromGMT];
+	      z = [_time_zone timeZoneSecondsFromGMT];
 	      if (z < 0) {
 		z = -z;
 		z /= 60;
@@ -1040,8 +1167,8 @@
 
       if (newDate)
 	{
-	  newDate->calendar_format = [calendar_format copyWithZone: zone];
-	  newDate->time_zone = RETAIN(time_zone);
+	  newDate->_calendar_format = [_calendar_format copyWithZone: zone];
+	  newDate->_time_zone = RETAIN(_time_zone);
 	}
     }
   return newDate;
@@ -1049,19 +1176,19 @@
 
 - (NSString*) descriptionWithLocale: (NSDictionary *)locale
 {
-  return [self descriptionWithCalendarFormat: calendar_format locale: locale];
+  return [self descriptionWithCalendarFormat: _calendar_format locale: locale];
 }
 
 // Getting and Setting Calendar Formats
 - (NSString*) calendarFormat
 {
-  return calendar_format;
+  return _calendar_format;
 }
 
 - (void) setCalendarFormat: (NSString *)format
 {
-  RELEASE(calendar_format);
-  calendar_format = [format copyWithZone: [self zone]];
+  RELEASE(_calendar_format);
+  _calendar_format = [format copyWithZone: [self zone]];
 }
 
 // Getting and Setting Time Zones
@@ -1069,12 +1196,12 @@
 {
   NSTimeZoneDetail	*detail = [aTimeZone timeZoneDetailForDate: self];
 
-  ASSIGN(time_zone, detail);
+  ASSIGN(_time_zone, detail);
 }
 
 - (NSTimeZoneDetail*) timeZoneDetail
 {
-  return time_zone;
+  return _time_zone;
 }
 
 @end
@@ -1252,7 +1379,7 @@
 			         hour: hour
 			       minute: minute
 			       second: second
-			     timeZone: nil];
+			     timeZone: [self timeZoneDetail]];
 }
 
 - (void) years: (int*)years
