@@ -20,10 +20,10 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 
-/* To do:
+/* To do: 
    Make it thread-safe.
    */
 
@@ -47,6 +47,8 @@
 #include <Foundation/NSException.h>
 #include <Foundation/NSLock.h>
 #include <Foundation/NSThread.h>
+#include <Foundation/NSPort.h>
+#include <Foundation/NSPortMessage.h>
 #include <Foundation/NSNotification.h>
 
 NSString* NSConnectionReplyMode = @"NSConnectionReplyMode";
@@ -111,7 +113,7 @@ static unsigned local_object_counter = 0;
 
   counter = (GSLocalCounter*)NSAllocateObject(self, 0, NSDefaultMallocZone());
   counter->ref = 1;
-  counter->object = [obj retain];
+  counter->object = RETAIN(obj);
   counter->target = ++local_object_counter;
   return counter;
 }
@@ -147,7 +149,7 @@ static unsigned local_object_counter = 0;
 {
   CachedLocalObject	*item = [[self alloc] init];
 
-  item->obj = [o retain];
+  item->obj = RETAIN(o);
   item->time = t;
   return [item autorelease];
 }
@@ -184,7 +186,8 @@ static unsigned local_object_counter = 0;
 
 @interface NSConnection (Private)
 - _superInit;
-+ setDebug: (int)val;
+- (void) handlePortMessage: (NSPortMessage*)msg;
++ (void) setDebug: (int)val;
 @end
 
 #define proxiesHashGate refGate
@@ -216,18 +219,6 @@ compare_ints (const void *k1, const void *k2)
   return !(k1 - k2);
 }
 
-static int
-type_get_number_of_arguments (const char *type)
-{
-  int i = 0;
-  while (*type)
-    {
-      type = objc_skip_argspec (type);
-      i += 1;
-    }
-  return i - 1;
-}
-
 /* class defaults */
 static id default_receive_port_class;
 static id default_send_port_class;
@@ -275,7 +266,7 @@ static int messages_received_count;
 {
     NSDistantObject	*proxy;
 
-    proxy = [self rootProxyForConnectionWithRegisteredName:n host:h];
+    proxy = [self rootProxyForConnectionWithRegisteredName: n host: h];
     if (proxy) {
 	return [proxy connectionForProxy];
     }
@@ -290,24 +281,24 @@ static int messages_received_count;
  */
 + (NSConnection*) defaultConnection
 {
-  static NSString*	tkey = @"NSConnectionThreadKey";
-  NSConnection*	c;
-  NSThread*	t;
+  static NSString	*tkey = @"NSConnectionThreadKey";
+  NSConnection		*c;
+  NSMutableDictionary	*d;
 
-  t = [NSThread currentThread];
-  c = (NSConnection*)[[t threadDictionary] objectForKey:tkey];
+  d = GSCurrentThreadDictionary();
+  c = (NSConnection*)[d objectForKey: tkey];
   if (c != nil && [c isValid] == NO) {
     /*
      *	If the default connection for this thread has been invalidated -
      *	release it and create a new one.
      */
-    [[t threadDictionary] removeObjectForKey:tkey];
+    [d removeObjectForKey: tkey];
     c = nil;
   }
   if (c == nil) {
     c = [NSConnection new];
-    [[t threadDictionary] setObject:c forKey:tkey];
-    [c release];	/* retained in dictionary.	*/
+    [d setObject: c forKey: tkey];
+    RELEASE(c);
   }
   return c;
 }
@@ -327,11 +318,11 @@ static int messages_received_count;
   all_connections_local_cached =
     NSCreateMapTable (NSIntMapKeyCallBacks,
 		      NSObjectMapValueCallBacks, 0);
-  received_request_rmc_queue = [[NSMutableArray alloc] initWithCapacity:32];
+  received_request_rmc_queue = [[NSMutableArray alloc] initWithCapacity: 32];
   received_request_rmc_queue_gate = [NSLock new];
-  received_reply_rmc_queue = [[NSMutableArray alloc] initWithCapacity:32];
+  received_reply_rmc_queue = [[NSMutableArray alloc] initWithCapacity: 32];
   received_reply_rmc_queue_gate = [NSLock new];
-  root_object_dictionary = [[NSMutableDictionary alloc] initWithCapacity:8];
+  root_object_dictionary = [[NSMutableDictionary alloc] initWithCapacity: 8];
   root_object_dictionary_gate = [NSLock new];
   receive_port_2_ancestor =
     NSCreateMapTable (NSNonOwnedPointerMapKeyCallBacks,
@@ -349,9 +340,9 @@ static int messages_received_count;
 + new
 {
   id newPort = [[default_receive_port_class newForReceiving] autorelease];
-  id newConn = [NSConnection newForInPort:newPort
-				  outPort:nil
-		       ancestorConnection:nil];
+  id newConn = [NSConnection newForInPort: newPort
+				  outPort: nil
+		       ancestorConnection: nil];
   return newConn;
 }
 
@@ -393,12 +384,17 @@ static int messages_received_count;
 
 - (void) addRequestMode: (NSString*)mode
 {
-    if (![request_modes containsObject:mode]) {
-	[request_modes addObject:mode];
+    if (![request_modes containsObject: mode]) {
+	[request_modes addObject: mode];
         [[NSRunLoop currentRunLoop] addPort: receive_port forMode: mode];
     }
 }
 
+- (void) addRunLoop: (NSRunLoop*)loop
+{
+  [self notImplemented: _cmd];
+}
+
 - (void) dealloc
 {
   if (debug_connection)
@@ -411,9 +407,14 @@ static int messages_received_count;
   return delegate;
 }
 
-- (BOOL) independantConversationQueueing
+- (void) handlePortMessage: (NSPortMessage*)msg
 {
-    return independant_queueing;
+  [self notImplemented: _cmd];
+}
+
+- (BOOL) independentConversationQueueing
+{
+  return independent_queueing;
 }
 
 - (void) enableMultipleThreads
@@ -431,9 +432,9 @@ static int messages_received_count;
 - (id) init
 {
   id newPort = [[default_receive_port_class newForReceiving] autorelease];
-  id newConn = [NSConnection newForInPort:newPort
-				  outPort:nil
-		       ancestorConnection:nil];
+  id newConn = [NSConnection newForInPort: newPort
+				  outPort: nil
+		       ancestorConnection: nil];
   [self release];
   return newConn;
 }
@@ -458,7 +459,7 @@ static int messages_received_count;
   /*
    *	Don't need notifications any more - so remove self as observer.
    */
-  [NSNotificationCenter removeObserver: self];
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
 
   /*
    *	We can't be the ancestor of anything if we are invalid.
@@ -477,10 +478,11 @@ static int messages_received_count;
     unsigned 	i;
 
     [proxiesHashGate lock];
-    targets = [NSAllMapTableValues(local_targets) retain];
+    targets = NSAllMapTableValues(local_targets);
+    IF_NO_GC(RETAIN(targets));
     for (i = 0; i < [targets count]; i++)
       {
-	id	t = [[targets objectAtIndex:i] localForProxy];
+	id	t = [[targets objectAtIndex: i] localForProxy];
 
 	[self removeLocalObject: t];
       }
@@ -502,8 +504,9 @@ static int messages_received_count;
   {
     NSAutoreleasePool	*arp = [NSAutoreleasePool new];
 
-    [NSNotificationCenter postNotificationName: NSConnectionDidDieNotification
-					object: self];
+    [[NSNotificationCenter defaultCenter]
+      postNotificationName: NSConnectionDidDieNotification
+		    object: self];
     [arp release];
   }
 }
@@ -538,9 +541,9 @@ static int messages_received_count;
 
 - (void) removeRequestMode: (NSString*)mode
 {
-  if ([request_modes containsObject:mode])
+  if ([request_modes containsObject: mode])
     {
-      [request_modes removeObject:mode];
+      [request_modes removeObject: mode];
       [[NSRunLoop currentRunLoop] removePort: receive_port forMode: mode];
     }
 }
@@ -595,9 +598,9 @@ static int messages_received_count;
   delegate = anObj;
 }
 
-- (void) setIndependantConversationQueueing: (BOOL)flag
+- (void) setIndependentConversationQueueing: (BOOL)flag
 {
-  independant_queueing = flag;
+  independent_queueing = flag;
 }
 
 - (void) setReplyTimeout: (NSTimeInterval)to
@@ -687,8 +690,8 @@ static int messages_received_count;
 
   /* Remove rootObject from root_object_dictionary
      if this is last connection */
-  if (![NSConnection connectionsCountWithInPort:receive_port])
-    [NSConnection setRootObject:nil forInPort:receive_port];
+  if (![NSConnection connectionsCountWithInPort: receive_port])
+    [NSConnection setRootObject: nil forInPort: receive_port];
 
   /* Remove receive port from run loop. */
   [self setRequestMode: nil];
@@ -704,8 +707,6 @@ static int messages_received_count;
   NSFreeMapTable (remote_proxies);
   NSFreeMapTable (local_objects);
   NSFreeMapTable (local_targets);
-  NSFreeMapTable (incoming_xref_2_const_ptr);
-  NSFreeMapTable (outgoing_const_ptr_2_xref);
   [proxiesHashGate unlock];
 
   [arp release];
@@ -816,14 +817,14 @@ static int messages_received_count;
   newPort = [[default_receive_port_class newForReceiving] autorelease];
   newConn = [self newForInPort: newPort outPort: nil
 		  ancestorConnection: nil];
-  [self setRootObject:anObj forInPort:newPort];
+  [self setRootObject: anObj forInPort: newPort];
   return newConn;
 }
 
 /* I want this method name to clearly indicate that we're not connecting
    to a pre-existing registration name, we're registering a new name,
    and this method will fail if that name has already been registered.
-   This is why I don't like "newWithRegisteredName:" --- it's unclear
+   This is why I don't like "newWithRegisteredName: " --- it's unclear
    if we're connecting to another NSConnection that already registered
    with that name. */
 
@@ -857,7 +858,7 @@ static int messages_received_count;
 
 + (NSDistantObject*) rootProxyAtName: (NSString*)n onHost: (NSString*)h
 {
-  return [self rootProxyForConnectionWithRegisteredName:n host:h];
+  return [self rootProxyForConnectionWithRegisteredName: n host: h];
 }
 
 + (NSDistantObject*) rootProxyAtPort: (NSPort*)anOutPort
@@ -894,7 +895,8 @@ static int messages_received_count;
 		       outPort: (NSPort*)op
 	    ancestorConnection: (NSConnection*)ancestor
 {
-  NSConnection *newConn;
+  NSNotificationCenter	*nCenter;
+  NSConnection		*newConn;
 
   NSParameterAssert (ip);
 
@@ -941,13 +943,6 @@ static int messages_received_count;
     NSCreateMapTable (NSIntMapKeyCallBacks,
 		      NSNonOwnedPointerMapValueCallBacks, 0);
 
-  newConn->incoming_xref_2_const_ptr =
-    NSCreateMapTable (NSIntMapKeyCallBacks,
-		      NSNonOwnedPointerMapValueCallBacks, 0);
-  newConn->outgoing_const_ptr_2_xref =
-    NSCreateMapTable (NSIntMapKeyCallBacks,
-		      NSNonOwnedPointerMapValueCallBacks, 0);
-
   newConn->reply_timeout = [self defaultInTimeout];
   newConn->request_timeout = [self defaultOutTimeout];
   newConn->encoding_class = default_encoding_class;
@@ -978,14 +973,14 @@ static int messages_received_count;
       newConn->receive_port_class = default_receive_port_class;
       newConn->send_port_class = default_send_port_class;
     }
-  newConn->independant_queueing = NO;
+  newConn->independent_queueing = NO;
   newConn->reply_depth = 0;
   newConn->delegate = nil;
   /*
    *	Set up request modes array and make sure the receiving port is
    *	added to the run loop to get data.
    */
-  newConn->request_modes = [[NSMutableArray arrayWithObject:
+  newConn->request_modes = [[NSMutableArray arrayWithObject: 
 		NSDefaultRunLoopMode] retain];
   [[NSRunLoop currentRunLoop] addPort: (NSPort*)ip
 			      forMode: NSDefaultRunLoopMode];
@@ -995,7 +990,7 @@ static int messages_received_count;
   /* Ssk the delegate for permission, (OpenStep-style and GNUstep-style). */
 
   /* Preferred OpenStep version, which just allows the returning of BOOL */
-  if ([[ancestor delegate] respondsTo:@selector(connection:shouldMakeNewConnection:)])
+  if ([[ancestor delegate] respondsTo: @selector(connection: shouldMakeNewConnection: )])
     {
       if (![[ancestor delegate] connection: ancestor
 		   shouldMakeNewConnection: (NSConnection*)newConn])
@@ -1006,7 +1001,7 @@ static int messages_received_count;
 	}
     }
   /* Deprecated OpenStep version, which just allows the returning of BOOL */
-  if ([[ancestor delegate] respondsTo:@selector(makeNewConnection:sender:)])
+  if ([[ancestor delegate] respondsTo: @selector(makeNewConnection: sender: )])
     {
       if (![[ancestor delegate] makeNewConnection: (NSConnection*)newConn
 				sender: ancestor])
@@ -1019,21 +1014,22 @@ static int messages_received_count;
   /* Here is the GNUstep version, which allows the delegate to specify
      a substitute.  Note: The delegate is responsible for freeing
      newConn if it returns something different. */
-  if ([[ancestor delegate] respondsTo:@selector(connection:didConnect:)])
-    newConn = [[ancestor delegate] connection:ancestor
-	       didConnect:newConn];
+  if ([[ancestor delegate] respondsTo: @selector(connection: didConnect: )])
+    newConn = [[ancestor delegate] connection: ancestor
+	       didConnect: newConn];
 
   /* Register ourselves for invalidation notification when the
      ports become invalid. */
-  [NSNotificationCenter addObserver: newConn
-			   selector: @selector(portIsInvalid:)
-			       name: NSPortDidBecomeInvalidNotification
-			     object: ip];
+  nCenter = [NSNotificationCenter defaultCenter];
+  [nCenter addObserver: newConn
+	      selector: @selector(portIsInvalid: )
+		  name: NSPortDidBecomeInvalidNotification
+		object: ip];
   if (op)
-    [NSNotificationCenter addObserver: newConn
-			     selector: @selector(portIsInvalid:)
-				 name: NSPortDidBecomeInvalidNotification
-			       object: op];
+    [nCenter addObserver: newConn
+		selector: @selector(portIsInvalid: )
+		    name: NSPortDidBecomeInvalidNotification
+		  object: op];
   /* if OP is nil, making this notification request would have
      registered us to receive all NSPortDidBecomeInvalidNotification
      requests, independent of which port posted them.  This isn't
@@ -1045,7 +1041,7 @@ static int messages_received_count;
   NSHashInsert(connection_table, (void*)newConn);
   [connection_table_gate unlock];
 
-  [NSNotificationCenter
+  [[NSNotificationCenter defaultCenter]
     postNotificationName: NSConnectionDidInitializeNotification
 		  object: newConn];
 
@@ -1112,7 +1108,7 @@ static int messages_received_count;
   return self;
 }
 
-+ setDebug: (int)val
++ (void) setDebug: (int)val
 {
   debug_connection = val;
 }
@@ -1145,7 +1141,7 @@ static int messages_received_count;
 
 /* Methods for handling client and server, requests and replies */
 
-/* NSDistantObject's -forward:: method calls this to the the message over the wire. */
+/* NSDistantObject's -forward: : method calls this to the the message over the wire. */
 - (retval_t) forwardForProxy: (NSDistantObject*)object
 		    selector: (SEL)sel
                     argFrame: (arglist_t)argframe
@@ -1158,7 +1154,7 @@ static int messages_received_count;
 #define ENCODED_ARGNAME @"argument value"
       switch (*type)
 	{
-	case _C_ID:
+	case _C_ID: 
 	  if (flags & _F_BYCOPY)
 	    [op encodeBycopyObject: *(id*)datum withName: ENCODED_ARGNAME];
 #ifdef	_F_BYREF
@@ -1168,7 +1164,7 @@ static int messages_received_count;
 	  else
 	    [op encodeObject: *(id*)datum withName: ENCODED_ARGNAME];
 	  break;
-	default:
+	default: 
 	  [op encodeValueOfObjCType: type at: datum withName: ENCODED_ARGNAME];
 	}
     }
@@ -1187,7 +1183,7 @@ static int messages_received_count;
     [NSException
       raise: NSGenericException
       format: @"Sorry, distributed objects does not work with NeXT runtime"];
-    /* type = [object selectorTypeForProxy:sel]; */
+    /* type = [object selectorTypeForProxy: sel]; */
 #else
     type = sel_get_type(sel);
 #endif
@@ -1250,7 +1246,7 @@ static int messages_received_count;
 		}
 	      /* xxx Why do we get the reply packet in here, and not
 		 just before calling dissect_method_return() below? */
-	      ip = [self _getReceivedReplyRmcWithSequenceNumber:seq_num];
+	      ip = [self _getReceivedReplyRmcWithSequenceNumber: seq_num];
 	      /* Find out if the server is returning an exception instead
 		 of the return values. */
 	      [ip decodeValueOfCType: @encode(BOOL)
@@ -1270,7 +1266,7 @@ static int messages_received_count;
 		}
 	    }
 	  [ip decodeValueOfObjCType: type at: datum withName: NULL];
-	  /* -decodeValueOfCType:at:withName: malloc's new memory
+	  /* -decodeValueOfCType: at: withName: malloc's new memory
 	     for char*'s.  We need to make sure it gets freed eventually
 	     so we don't have a memory leak.  Request here that it be
 	     autorelease'ed. Also autorelease created objects. */
@@ -1311,10 +1307,10 @@ static int messages_received_count;
 	  return;
 	}
 
-      [aRmc decodeValueOfObjCType:type
-	    at:datum
-	    withName:NULL];
-      /* -decodeValueOfCType:at:withName: malloc's new memory
+      [aRmc decodeValueOfObjCType: type
+	    at: datum
+	    withName: NULL];
+      /* -decodeValueOfCType: at: withName: malloc's new memory
 	 for char*'s.  We need to make sure it gets freed eventually
 	 so we don't have a memory leak.  Request here that it be
 	 autorelease'ed. Also autorelease created objects. */
@@ -1335,7 +1331,7 @@ static int messages_received_count;
 	     back to the remote application!	*/
 	  if (!is_valid)
 	    return;
-	  op = [self newSendingReplyRmcWithSequenceNumber:
+	  op = [self newSendingReplyRmcWithSequenceNumber: 
 		       reply_sequence_number];
 	  [op encodeValueOfCType: @encode(BOOL)
 	      at: &is_exception
@@ -1343,18 +1339,18 @@ static int messages_received_count;
 	}
       switch (*type)
 	{
-	case _C_ID:
+	case _C_ID: 
 	  if (flags & _F_BYCOPY)
-	    [op encodeBycopyObject:*(id*)datum withName:ENCODED_RETNAME];
+	    [op encodeBycopyObject: *(id*)datum withName: ENCODED_RETNAME];
 #ifdef	_F_BYREF
 	  else if (flags & _F_BYREF)
 	    [op encodeByrefObject: *(id*)datum withName: ENCODED_ARGNAME];
 #endif
 	  else
-	    [op encodeObject:*(id*)datum withName:ENCODED_RETNAME];
+	    [op encodeObject: *(id*)datum withName: ENCODED_RETNAME];
 	  break;
-	default:
-	  [op encodeValueOfObjCType:type at:datum withName:ENCODED_RETNAME];
+	default: 
+	  [op encodeValueOfObjCType: type at: datum withName: ENCODED_RETNAME];
 	}
     }
 
@@ -1372,9 +1368,9 @@ static int messages_received_count;
 	 If all selectors included qualifiers and I could make
 	 sel_types_match() work the way I wanted, we wouldn't need
 	 to do this. */
-      [aRmc decodeValueOfCType:@encode(char*)
-	    at:&forward_type
-	    withName:NULL];
+      [aRmc decodeValueOfCType: @encode(char*)
+	    at: &forward_type
+	    withName: NULL];
 
       if (debug_connection > 1)
         NSLog(@"Handling message from 0x%x\n", (gsaddr)self);
@@ -1412,7 +1408,7 @@ static int messages_received_count;
 
 - (void) _service_rootObject: rmc
 {
-  id rootObject = [NSConnection rootObjectForInPort:receive_port];
+  id rootObject = [NSConnection rootObjectForInPort: receive_port];
   NSPortCoder* op = [[self encodingClass]
 			newForWritingWithConnection: [rmc connection]
 			sequenceNumber: [rmc sequenceNumber]
@@ -1588,17 +1584,17 @@ static int messages_received_count;
 	newForWritingWithConnection: self
 	sequenceNumber: seq_num
 	identifier: METHODTYPE_REQUEST];
-  [op encodeValueOfObjCType:":"
-      at:&sel
-      withName:NULL];
-  [op encodeValueOfCType:@encode(unsigned)
-      at:&target
-      withName:NULL];
+  [op encodeValueOfObjCType: ": "
+      at: &sel
+      withName: NULL];
+  [op encodeValueOfCType: @encode(unsigned)
+      at: &target
+      withName: NULL];
   [op dismiss];
-  ip = [self _getReceivedReplyRmcWithSequenceNumber:seq_num];
-  [ip decodeValueOfCType:@encode(char*)
-      at:&type
-      withName:NULL];
+  ip = [self _getReceivedReplyRmcWithSequenceNumber: seq_num];
+  [ip decodeValueOfCType: @encode(char*)
+      at: &type
+      withName: NULL];
   [ip dismiss];
   return type;
 }
@@ -1621,12 +1617,12 @@ static int messages_received_count;
 	sequenceNumber: [rmc sequenceNumber]
 	identifier: METHODTYPE_REPLY];
 
-  [rmc decodeValueOfObjCType:":"
-       at:&sel
-       withName:NULL];
-  [rmc decodeValueOfCType:@encode(unsigned)
-       at:&target
-       withName:NULL];
+  [rmc decodeValueOfObjCType: ": "
+       at: &sel
+       withName: NULL];
+  [rmc decodeValueOfCType: @encode(unsigned)
+       at: &target
+       withName: NULL];
   p = [self includesLocalTarget: target];
   o = [p localForProxy];
 
@@ -1640,9 +1636,9 @@ static int messages_received_count;
     type = m->method_types;
   else
     type = "";
-  [op encodeValueOfCType:@encode(char*)
-      at:&type
-      withName:@"Requested Method Type for Target"];
+  [op encodeValueOfCType: @encode(char*)
+      at: &type
+      withName: @"Requested Method Type for Target"];
   [op dismiss];
   [rmc dismiss];
 }
@@ -1670,24 +1666,24 @@ static int messages_received_count;
 
   switch (ident)
     {
-    case ROOTPROXY_REQUEST:
+    case ROOTPROXY_REQUEST: 
       /* It won't take much time to handle this, so go ahead and service
 	 it, even if we are waiting for a reply. */
       [conn _service_rootObject: rmc];
       break;
-    case METHODTYPE_REQUEST:
+    case METHODTYPE_REQUEST: 
       /* It won't take much time to handle this, so go ahead and service
 	 it, even if we are waiting for a reply. */
       [conn _service_typeForSelector: rmc];
       break;
-    case METHOD_REQUEST:
+    case METHOD_REQUEST: 
       /* We just got a new request; we need to decide whether to queue
 	 it or service it now.
 	 If the REPLY_DEPTH is 0, then we aren't in the middle of waiting
 	 for a reply, we are waiting for requests---so service it now.
 	 If REPLY_DEPTH is non-zero, we may still want to service it now
-	 if independant_queuing is NO. */
-      if (reply_depth == 0 || independant_queueing == NO)
+	 if independent_queuing is NO. */
+      if (reply_depth == 0 || independent_queueing == NO)
 	{
 	  [conn _service_forwardForProxy: rmc];
 	  /* Service any requests that were queued while we
@@ -1703,31 +1699,31 @@ static int messages_received_count;
 	  [received_request_rmc_queue_gate unlock];
 	}
       break;
-    case ROOTPROXY_REPLY:
-    case METHOD_REPLY:
-    case METHODTYPE_REPLY:
-    case RETAIN_REPLY:
+    case ROOTPROXY_REPLY: 
+    case METHOD_REPLY: 
+    case METHODTYPE_REPLY: 
+    case RETAIN_REPLY: 
       /* Remember multi-threaded callbacks will have to be handled specially */
       [received_reply_rmc_queue_gate lock];
       [received_reply_rmc_queue addObject: rmc];
       [received_reply_rmc_queue_gate unlock];
       break;
-    case CONNECTION_SHUTDOWN:
+    case CONNECTION_SHUTDOWN: 
       {
 	[conn _service_shutdown: rmc forConnection: self];
 	break;
       }
-    case PROXY_RELEASE:
+    case PROXY_RELEASE: 
       {
 	[conn _service_release: rmc forConnection: self];
 	break;
       }
-    case PROXY_RETAIN:
+    case PROXY_RETAIN: 
       {
 	[conn _service_retain: rmc forConnection: self];
 	break;
       }
-    default:
+    default: 
       [rmc dismiss];
       [NSException raise: NSGenericException
 		   format: @"unrecognized NSPortCoder identifier"];
@@ -1947,7 +1943,7 @@ static int messages_received_count;
 		{
 		  timer = [NSTimer scheduledTimerWithTimeInterval: 1.0
 					 target: [NSConnection class]
-					 selector: @selector(_timeout:)
+					 selector: @selector(_timeout: )
 					 userInfo: nil
 					  repeats: YES];
 		}
@@ -2028,7 +2024,6 @@ static int messages_received_count;
 	{
 	  id		op;
 	  id		ip;
-	  unsigned int 	i;
 	  id		result;
 	  int seq_num = [self _newMsgNumber];
 
@@ -2206,7 +2201,7 @@ static int messages_received_count;
   id ro;
 
   [root_object_dictionary_gate lock];
-  ro = [root_object_dictionary objectForKey:aPort];
+  ro = [root_object_dictionary objectForKey: aPort];
   [root_object_dictionary_gate unlock];
   return ro;
 }
@@ -2251,56 +2246,17 @@ static int messages_received_count;
   return default_decoding_class;
 }
 
-
-/* Support for cross-connection const-ptr cache. */
-
-- (unsigned) _encoderCreateReferenceForConstPtr: (const void*)ptr
-{
-  unsigned xref;
-
-  NSParameterAssert (is_valid);
-  /* This must match the assignment of xref in _decoderCreateRef... */
-  xref = NSCountMapTable (outgoing_const_ptr_2_xref) + 1;
-  NSParameterAssert (! NSMapGet (outgoing_const_ptr_2_xref, (void*)xref));
-  NSMapInsert (outgoing_const_ptr_2_xref, ptr, (void*)xref);
-  return xref;
-}
-
-- (unsigned) _encoderReferenceForConstPtr: (const void*)ptr
-{
-  NSParameterAssert (is_valid);
-  return (unsigned) NSMapGet (outgoing_const_ptr_2_xref, ptr);
-}
-
-- (unsigned) _decoderCreateReferenceForConstPtr: (const void*)ptr
-{
-  unsigned xref;
-
-  NSParameterAssert (is_valid);
-  /* This must match the assignment of xref in _encoderCreateRef... */
-  xref = NSCountMapTable (incoming_xref_2_const_ptr) + 1;
-  NSMapInsert (incoming_xref_2_const_ptr, (void*)xref, ptr);
-  return xref;
-}
-
-- (const void*) _decoderConstPtrAtReference: (unsigned)xref
-{
-  NSParameterAssert (is_valid);
-  return NSMapGet (incoming_xref_2_const_ptr, (void*)xref);
-}
-
-
 
 /* Prevent trying to encode the connection itself */
 
 - (void) encodeWithCoder: anEncoder
 {
-  [self shouldNotImplement:_cmd];
+  [self shouldNotImplement: _cmd];
 }
 
 + newWithCoder: aDecoder;
 {
-  [self shouldNotImplement:_cmd];
+  [self shouldNotImplement: _cmd];
   return self;
 }
 
@@ -2325,7 +2281,7 @@ static int messages_received_count;
 	/* We shouldn't be getting any port invalidation notifications,
 	    except from our own ports; this is how we registered ourselves
 	    with the NSNotificationCenter in
-	    +newForInPort:outPort:ancestorConnection. */
+	    +newForInPort: outPort: ancestorConnection. */
 	NSParameterAssert (port == receive_port || port == send_port);
 
 	[self invalidate];
@@ -2343,7 +2299,7 @@ static int messages_received_count;
 {
     NSConnection	*c;
 
-    c = [self newForInPort:r outPort:s ancestorConnection:nil];
+    c = [self newForInPort: r outPort: s ancestorConnection: nil];
     return [c autorelease];
 }
 
@@ -2351,7 +2307,7 @@ static int messages_received_count;
 	     sendPort: (NSPort*)s
 {
     [self dealloc];
-    return [NSConnection newForInPort:r outPort:s ancestorConnection:nil];
+    return [NSConnection newForInPort: r outPort: s ancestorConnection: nil];
 }
 
 - (NSPort*) receivePort
