@@ -122,7 +122,7 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
   if ([path getFileSystemRepresentation: thePath
 			      maxLength: sizeof(thePath)-1] == NO)
     {
-      NSLog(@"Open (%s) attempt failed - bad path", thePath);
+      NSDebugLog(@"Open (%s) attempt failed - bad path", thePath);
       return NO;
     }
 #if	defined(__WIN32__)
@@ -133,7 +133,7 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
 
   if (theFile == NULL)		/* We failed to open the file. */
     {
-      NSLog(@"Open (%s) attempt failed - %s", thePath, strerror(errno));
+      NSDebugLog(@"Open (%s) attempt failed - %s", thePath, strerror(errno));
       goto failure;
     }
 
@@ -158,7 +158,11 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
       goto failure;
     }
 
+#if	GS_WITH_GC == 1
+  tmp = NSZoneMalloc(GSAtomicMallocZone(), fileLength);
+#else
   tmp = NSZoneMalloc(zone, fileLength);
+#endif
   if (tmp == 0)
     {
       NSLog(@"Malloc failed for file of length %d- %s",
@@ -397,7 +401,11 @@ failure:
 #define num2char(num) ((num) < 0xa ? ((num)+'0') : ((num)+0x57))
 
   /* we can just build a cString and convert it to an NSString */
+#if	GS_WITH_GC
+  dest = (char*) NSZoneMalloc(GSAtomicMallocZone(), 2*length+length/4+3);
+#else
   dest = (char*) NSZoneMalloc(z, 2*length+length/4+3);
+#endif
   if (dest == 0)
     [NSException raise: NSMallocException
 		format: @"No memory for description of NSData object"];
@@ -412,9 +420,14 @@ failure:
     }
   dest[j++] = '>';
   dest[j] = '\0';
+#if	GS_WITH_GC
+  str = [[NSString allocWithZone: z]
+    initWithCStringNoCopy: dest length: j fromZone: GSAtomicMallocZone()];
+#else
   str = [[NSString allocWithZone: z] initWithCStringNoCopy: dest
 						    length: j
 						  fromZone: z];
+#endif
   return AUTORELEASE(str);
 }
 
@@ -450,7 +463,11 @@ failure:
 
   GS_RANGE_CHECK(aRange, l);
 
+#if	GS_WITH_GC
+  buffer = NSZoneMalloc(GSAtomicMallocZone(), aRange.length);
+#else
   buffer = NSZoneMalloc([self zone], aRange.length);
+#endif
   if (buffer == 0)
     [NSException raise: NSMallocException
 		format: @"No memory for subdata of NSData object"];
@@ -501,7 +518,7 @@ failure:
   if ([path getFileSystemRepresentation: theRealPath
 			      maxLength: sizeof(theRealPath)-1] == NO)
     {
-      NSLog(@"Open (%s) attempt failed - bad path", theRealPath);
+      NSDebugLog(@"Open (%s) attempt failed - bad path", theRealPath);
       return NO;
     }
 
@@ -598,14 +615,16 @@ failure:
 
       if ([mgr fileExistsAtPath: path])
 	{
-	  att = [[mgr fileAttributesAtPath:path traverseLink:YES] mutableCopy];
-	  TEST_AUTORELEASE(att);
+	  att = [[mgr fileAttributesAtPath: path
+			      traverseLink: YES] mutableCopy];
+	  IF_NO_GC(TEST_AUTORELEASE(att));
 	}
 
       c = rename(thePath, theRealPath);
       if (c != 0)               /* Many things could go wrong, I guess. */
         {
-          NSLog(@"Rename (%s) failed - %s", thePath, strerror(errno));
+          NSLog(@"Rename ('%s' to '%s') failed - %s",
+	    thePath, theRealPath, strerror(errno));
           goto failure;
         }
 
@@ -639,6 +658,13 @@ failure:
 
   /* Just in case the failure action needs to be changed. */
  failure:
+  /*
+   * Attempt to tidy up by removing temporary file on failure.
+   */
+  if (useAuxiliaryFile)
+    {
+      unlink(thePath);
+    }
   return NO;
 }
 
@@ -702,7 +728,7 @@ failure:
 	      d = [d initWithBytesNoCopy: *(void**)data
 				  length: len
 			        fromZone: z];
-	      AUTORELEASE(d);
+	      IF_NO_GC(AUTORELEASE(d));
 	    }
 
 	  [self deserializeBytes: *(char**)data
@@ -772,7 +798,7 @@ failure:
 	  d = [d initWithBytesNoCopy: *(void**)data
 			      length: len
 			    fromZone: z];
-	  AUTORELEASE(d);
+	  IF_NO_GC(AUTORELEASE(d));
 	  [self deserializeDataAt: *(char**)data
 		       ofObjCType: type
 			 atCursor: cursor
@@ -820,7 +846,6 @@ failure:
 	  *(unsigned long*)data = NSSwapBigLongToHost(nl);
 	  return;
 	}
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:
       case _C_ULNG_LNG:
 	{
@@ -832,7 +857,6 @@ failure:
 	  *(unsigned long long*)data = NSSwapBigLongLongToHost(nl);
 	  return;
 	}
-#endif
       case _C_FLT:
 	{
 	  NSSwappedFloat nf;
@@ -1403,7 +1427,6 @@ failure:
 	  [self appendBytes: &nl length: sizeof(unsigned long)];
 	  return;
 	}
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:
       case _C_ULNG_LNG:
 	{
@@ -1413,7 +1436,6 @@ failure:
 	  [self appendBytes: &nl length: sizeof(unsigned long long)];
 	  return;
 	}
-#endif
       case _C_FLT:
 	{
 	  NSSwappedFloat nf = NSSwapHostFloatToBig(*(float*)data);
@@ -1445,9 +1467,9 @@ failure:
       case _C_SEL:
 	{
 	  const char  *name = *(SEL*)data?fastSelectorName(*(SEL*)data):"";
-	  gsu16	ln = (gsu16)strlen(name);
+	  gsu16	ln = (name == 0) ? 0 : (gsu16)strlen(name);
 	  const char  *types = *(SEL*)data?fastSelectorTypes(*(SEL*)data):"";
-	  gsu16	lt = (gsu16)strlen(types);
+	  gsu16	lt = (types == 0) ? 0 : (gsu16)strlen(types);
 	  gsu16	ni;
 
 	  ni = GSSwapHostI16ToBig(ln);
@@ -1886,7 +1908,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  *(unsigned long*)data = NSSwapBigLongToHost(nl);
 	  return;
 	}
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:
       case _C_ULNG_LNG:
 	{
@@ -1896,7 +1917,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  *(unsigned long long*)data = NSSwapBigLongLongToHost(nl);
 	  return;
 	}
-#endif
       case _C_FLT:
 	{
 	  NSSwappedFloat nf;
@@ -2090,7 +2110,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
   if (aBuffer != 0 && bufferSize > 0)
     {
+#if	GS_WITH_GC
+      zone = GSAtomicMallocZone();
+#else
       zone = [self zone];
+#endif
       tmp = NSZoneMalloc(zone, bufferSize);
       if (tmp == 0)
 	{
@@ -2133,7 +2157,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       return data;
     }
 
+#if	GS_WITH_GC
+  zone = GSAtomicMallocZone();
+#else
   zone = aZone;
+#endif
   bytes = aBuffer;
   if (bytes)
     {
@@ -2147,7 +2175,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   unsigned	l;
   void*		b;
 
+#if	GS_WITH_GC
+  zone = GSAtomicMallocZone();
+#else
   zone = [self zone];
+#endif
 
   [aCoder decodeValueOfObjCType: @encode(unsigned long) at: &l];
   if (l)
@@ -2170,7 +2202,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (id) initWithContentsOfFile: (NSString *)path
 {
+#if	GS_WITH_GC
+  zone = GSAtomicMallocZone();
+#else
   zone = [self zone];
+#endif
   if (readContentsOfFile(path, &bytes, &length, zone) == NO)
     {
       RELEASE(self);
@@ -2247,7 +2283,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   if ([path getFileSystemRepresentation: thePath
 			      maxLength: sizeof(thePath)-1] == NO)
     {
-      NSLog(@"Open (%s) attempt failed - bad path", thePath);
+      NSDebugLog(@"Open (%s) attempt failed - bad path", thePath);
       return NO;
     }
   fd = open(thePath, O_RDONLY);
@@ -2401,8 +2437,9 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 @implementation	NSMutableDataMalloc
 + (void) initialize
 {
-    if ([self class] == [NSMutableDataMalloc class]) {
-        behavior_class_add_class(self, [NSDataMalloc class]);
+  if (self == [NSMutableDataMalloc class])
+    {
+      behavior_class_add_class(self, [NSDataMalloc class]);
     }
 }
 
@@ -2413,17 +2450,17 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (Class) classForArchiver
 {
-    return mutableDataMalloc;
+  return mutableDataMalloc;
 }
 
 - (Class) classForCoder
 {
-    return mutableDataMalloc;
+  return mutableDataMalloc;
 }
 
 - (Class) classForPortCoder
 {
-    return mutableDataMalloc;
+  return mutableDataMalloc;
 }
 
 - (id) copy
@@ -2435,7 +2472,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 - (id) copyWithZone: (NSZone*)z
 {
   return [[dataMalloc allocWithZone: z]
-	    initWithBytes: bytes length: length];
+    initWithBytes: bytes length: length];
 }
 
 - (id) initWithBytes: (const void*)aBuffer length: (unsigned)bufferSize
@@ -2463,30 +2500,39 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 		    length: (unsigned)bufferSize
 		  fromZone: (NSZone*)aZone
 {
-    if (aZone == 0) {
-	self = [self initWithBytes: aBuffer length: bufferSize];
-	return self;
+  if (aZone == 0)
+    {
+      self = [self initWithBytes: aBuffer length: bufferSize];
+      return self;
     }
 
-    if (aBuffer == 0) {
-	self = [self initWithCapacity: bufferSize];
-	if (self) {
-	    [self setLength: bufferSize];
+  if (aBuffer == 0)
+    {
+      self = [self initWithCapacity: bufferSize];
+      if (self)
+	{
+	  [self setLength: bufferSize];
 	}
-	return self;
+      return self;
     }
-    self = [self initWithCapacity: 0];
-    if (self) {
-	zone = aZone;
-	bytes = aBuffer;
-	length = bufferSize;
-	capacity = bufferSize;
-	growth = capacity/2;
-	if (growth == 0) {
-	    growth = 1;
+  self = [self initWithCapacity: 0];
+  if (self)
+    {
+#if	GS_WITH_GC
+      zone = GSAtomicMallocZone();
+#else
+      zone = aZone;
+#endif
+      bytes = aBuffer;
+      length = bufferSize;
+      capacity = bufferSize;
+      growth = capacity/2;
+      if (growth == 0)
+	{
+	  growth = 1;
 	}
     }
-    return self;
+  return self;
 }
 
 /*
@@ -2494,23 +2540,30 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
  */
 - (id) initWithCapacity: (unsigned)size
 {
-    zone = [self zone];
-    if (size) {
-	bytes = NSZoneMalloc(zone, size);
-	if (bytes == 0) {
-	    NSLog(@"[NSMutableDataMalloc -initWithCapacity:] out of memory for %u bytes - %s", size, strerror(errno));
-	    RELEASE(self);
-	    return nil;
+#if	GS_WITH_GC
+  zone = GSAtomicMallocZone();
+#else
+  zone = [self zone];
+#endif
+  if (size)
+    {
+      bytes = NSZoneMalloc(zone, size);
+      if (bytes == 0)
+	{
+	  NSLog(@"[NSMutableDataMalloc -initWithCapacity:] out of memory for %u bytes - %s", size, strerror(errno));
+	  RELEASE(self);
+	  return nil;
 	}
     }
-    capacity = size;
-    growth = capacity/2;
-    if (growth == 0) {
-	growth = 1;
+  capacity = size;
+  growth = capacity/2;
+  if (growth == 0)
+    {
+      growth = 1;
     }
-    length = 0;
+  length = 0;
 
-    return self;
+  return self;
 }
 
 - (id) initWithCoder: (NSCoder*)aCoder
@@ -2537,56 +2590,62 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (id) initWithLength: (unsigned)size
 {
-    self = [self initWithCapacity: size];
-    if (self) {
-	memset(bytes, '\0', size);
-	length = size;
+  self = [self initWithCapacity: size];
+  if (self)
+    {
+      memset(bytes, '\0', size);
+      length = size;
     }
-    return self;
+  return self;
 }
 
 - (id) initWithContentsOfFile: (NSString *)path
 {
-    self = [self initWithCapacity: 0];
-    if (readContentsOfFile(path, &bytes, &length, zone) == NO) {
-	RELEASE(self);
-	self = nil;
+  self = [self initWithCapacity: 0];
+  if (readContentsOfFile(path, &bytes, &length, zone) == NO)
+    {
+      RELEASE(self);
+      self = nil;
     }
-    else {
-	capacity = length;
+  else
+    {
+      capacity = length;
     }
-    return self;
+  return self;
 }
 
 - (id) initWithContentsOfMappedFile: (NSString *)path
 {
-    return [self initWithContentsOfFile: path];
+  return [self initWithContentsOfFile: path];
 }
 
 - (id) initWithData: (NSData*)anObject
 {
-    if (anObject == nil) {
-	return [self initWithCapacity: 0];
+  if (anObject == nil)
+    {
+      return [self initWithCapacity: 0];
     }
-    if ([anObject isKindOfClass: [NSData class]] == NO) {
-        NSLog(@"-initWithData: passed a non-data object");
-	RELEASE(self);
-	return nil;
+  if ([anObject isKindOfClass: [NSData class]] == NO)
+    {
+      NSLog(@"-initWithData: passed a non-data object");
+      RELEASE(self);
+      return nil;
     }
-    return [self initWithBytes: [anObject bytes] length: [anObject length]];
+  return [self initWithBytes: [anObject bytes] length: [anObject length]];
 }
 
 - (void) appendBytes: (const void*)aBuffer
 	      length: (unsigned)bufferSize
 {
-    unsigned	oldLength = length;
-    unsigned	minimum = length + bufferSize;
+  unsigned	oldLength = length;
+  unsigned	minimum = length + bufferSize;
 
-    if (minimum > capacity) {
-	[self _grow: minimum];
+  if (minimum > capacity)
+    {
+      [self _grow: minimum];
     }
-    memcpy(bytes + oldLength, aBuffer, bufferSize);
-    length = minimum;
+  memcpy(bytes + oldLength, aBuffer, bufferSize);
+  length = minimum;
 }
 
 - (unsigned) capacity
@@ -2596,34 +2655,38 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (void) _grow: (unsigned)minimum
 {
-    if (minimum > capacity) {
-	unsigned	nextCapacity = capacity + growth;
-	unsigned	nextGrowth = capacity ? capacity : 1;
+  if (minimum > capacity)
+    {
+      unsigned	nextCapacity = capacity + growth;
+      unsigned	nextGrowth = capacity ? capacity : 1;
 
-	while (nextCapacity < minimum) {
-	    unsigned	tmp = nextCapacity + nextGrowth;
-	    nextGrowth = nextCapacity;
-	    nextCapacity = tmp;
+      while (nextCapacity < minimum)
+	{
+	  unsigned	tmp = nextCapacity + nextGrowth;
+
+	  nextGrowth = nextCapacity;
+	  nextCapacity = tmp;
 	}
-	[self setCapacity: nextCapacity];
-	growth = nextGrowth;
+      [self setCapacity: nextCapacity];
+      growth = nextGrowth;
     }
 }
 
 - (void*) mutableBytes
 {
-    return bytes;
+  return bytes;
 }
 
 - (void*) relinquishAllocatedBytesFromZone: (NSZone*)aZone
 {
-    void	*ptr = [super relinquishAllocatedBytesFromZone: aZone];
+  void	*ptr = [super relinquishAllocatedBytesFromZone: aZone];
 
-    if (ptr != 0) {
-	capacity = 0;
-	growth = 1;
+  if (ptr != 0)
+    {
+      capacity = 0;
+      growth = 1;
     }
-    return ptr;
+  return ptr;
 }
 
 - (void) replaceBytesInRange: (NSRange)aRange
@@ -2776,7 +2839,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  (*appendImp)(self, appendSel, &nl, sizeof(unsigned long));
 	  return;
 	}
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:
       case _C_ULNG_LNG:
 	{
@@ -2786,7 +2848,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  (*appendImp)(self, appendSel, &nl, sizeof(unsigned long long));
 	  return;
 	}
-#endif
       case _C_FLT:
 	{
 	  NSSwappedFloat nf = NSSwapHostFloatToBig(*(float*)data);
@@ -2823,9 +2884,9 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       case _C_SEL:
 	{
 	  const char  *name = *(SEL*)data?fastSelectorName(*(SEL*)data):"";
-	  gsu16	ln = (gsu16)strlen(name);
+	  gsu16	ln = (name == 0) ? 0 : (gsu16)strlen(name);
 	  const char  *types = *(SEL*)data?fastSelectorTypes(*(SEL*)data):"";
-	  gsu16	lt = (gsu16)strlen(types);
+	  gsu16	lt = (types == 0) ? 0 : (gsu16)strlen(types);
 	  gsu16	minimum = length + ln + lt + 2*sizeof(gsu16);
 	  gsu16	ni;
 
@@ -2909,30 +2970,36 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (id) setCapacity: (unsigned)size
 {
-    if (size != capacity) {
-	void*	tmp;
+  if (size != capacity)
+    {
+      void*	tmp;
 
-	if (bytes) {
-	    tmp = NSZoneRealloc(zone, bytes, size);
+      if (bytes)
+	{
+	  tmp = NSZoneRealloc(zone, bytes, size);
 	}
-	else {
-	    tmp = NSZoneMalloc(zone, size);
+      else
+	{
+	  tmp = NSZoneMalloc(zone, size);
 	}
-	if (tmp == 0) {
-	    [NSException raise: NSMallocException
-			format: @"Unable to set data capacity to '%d'", size];
+      if (tmp == 0)
+	{
+	  [NSException raise: NSMallocException
+		      format: @"Unable to set data capacity to '%d'", size];
 	}
-	bytes = tmp;
-	capacity = size;
-	growth = capacity/2;
-	if (growth == 0) {
-	    growth = 1;
+      bytes = tmp;
+      capacity = size;
+      growth = capacity/2;
+      if (growth == 0)
+	{
+	  growth = 1;
 	}
     }
-    if (size < length) {
-	length = size;
+  if (size < length)
+    {
+      length = size;
     }
-    return self;
+  return self;
 }
 
 - (void) setLength: (unsigned)size
