@@ -20,15 +20,9 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 
-/*
-  1995-03-31 02:41:00 -0600	Jeremy Bettis <jeremy@hksys.com>
-  Release the first draft of NSDate.
-  Three methods not implemented, and NSCalendarDate/NSTimeZone don't exist.
-*/
-
 #include <config.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDictionary.h>
@@ -39,6 +33,9 @@
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSCharacterSet.h>
 #include <Foundation/NSScanner.h>
+#include <base/preface.h>
+#include <base/behavior.h>
+#include <base/fast.x>
 #ifndef __WIN32__
 #include <time.h>
 #endif /* !__WIN32__ */
@@ -49,7 +46,7 @@
 #endif /* !__WIN32__ */
 
 /* The number of seconds between 1/1/2001 and 1/1/1970 = -978307200. */
-/* This number comes from: 
+/* This number comes from:
 -(((31 years * 365 days) + 8 days for leap years) =total number of days
   * 24 hours
   * 60 minutes
@@ -62,6 +59,26 @@
 #define DISTANT_FUTURE	(DISTANT_YEARS * 365.0 * 24 * 60 * 60)
 #define DISTANT_PAST	(-DISTANT_FUTURE)
 
+
+
+static BOOL	debug = NO;
+static Class	abstractClass = nil;
+static Class	concreteClass = nil;
+static Class	calendarClass = nil;
+
+@interface	GSDateSingle : NSGDate
+@end
+
+@interface	GSDatePast : GSDateSingle
+@end
+
+@interface	GSDateFuture : GSDateSingle
+@end
+
+static NSDate	*_distantPast = nil;
+static NSDate	*_distantFuture = nil;
+
+
 static NSString*
 findInArray(NSArray *array, unsigned pos, NSString *str)
 {
@@ -75,22 +92,25 @@ findInArray(NSArray *array, unsigned pos, NSString *str)
       item = [array objectAtIndex: index];
       if ([str caseInsensitiveCompare: item] == NSOrderedSame)
 	return item;
-    } 
+    }
   return nil;
 }
 
-
-/* The implementation of NSDate. */
-
-@implementation NSDate
-
-static BOOL	debug = NO;
-
-// Getting current time
-
-+ (NSTimeInterval) timeIntervalSinceReferenceDate
+static inline NSTimeInterval
+otherTime(NSDate* other)
 {
-#if !defined(__WIN32__) && !defined(_WIN32)
+  Class	c = fastClass(other);
+
+  if (c == concreteClass || c == calendarClass)
+    return ((NSGDate*)other)->_seconds_since_ref;
+  else
+    return [other timeIntervalSinceReferenceDate];
+}
+
+NSTimeInterval
+GSTimeNow()
+{
+#if !defined(__WIN32__)
   volatile NSTimeInterval interval;
   struct timeval tp;
 
@@ -98,10 +118,6 @@ static BOOL	debug = NO;
   gettimeofday (&tp, NULL);
   interval += tp.tv_sec;
   interval += (double)tp.tv_usec / 1000000.0;
-
-  /* There seems to be a problem with bad double arithmetic... */
-  NSAssert(interval < 0, NSInternalInconsistencyException);
-
   return interval;
 #else
   SYSTEMTIME sys_time;
@@ -120,23 +136,60 @@ static BOOL	debug = NO;
      minute: sys_time.wMinute
      second: sys_time.wSecond
      timeZone: [NSTimeZone localTimeZone]];
-  t = [d timeIntervalSinceReferenceDate];
-  [d release];
+  t = otherTime(d);
+  RELEASE(d);
   return t + sys_time.wMilliseconds / 1000.0;
 #endif /* __WIN32__ */
 }
 
+/* The implementation of NSDate. */
+
+@implementation NSDate
+
++ (void) initialize
+{
+  if (self == [NSDate class])
+    {
+      [self setVersion: 1];
+      abstractClass = self;
+      concreteClass = [NSGDate class];
+      calendarClass = [NSCalendarDate class];
+    }
+}
+
++ (id) alloc
+{
+  if (self == abstractClass)
+    return NSAllocateObject(concreteClass, 0, NSDefaultMallocZone());
+  else
+    return NSAllocateObject(self, 0, NSDefaultMallocZone());
+}
+
++ (id) allocWithZone: (NSZone*)z
+{
+  if (self == abstractClass)
+    return NSAllocateObject(concreteClass, 0, z);
+  else
+    return NSAllocateObject(self, 0, z);
+}
+
++ (NSTimeInterval) timeIntervalSinceReferenceDate
+{
+  return GSTimeNow();
+}
+
 // Allocation and initializing
 
 + (id) date
 {
-  return [[[self alloc] init] autorelease];
+  return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
+	initWithTimeIntervalSinceReferenceDate: GSTimeNow()]);
 }
 
 + (id) dateWithNaturalLanguageString: (NSString*)string
 {
-  [self dateWithNaturalLanguageString: string
-			       locale: nil];
+  return [self dateWithNaturalLanguageString: string
+				      locale: nil];
 }
 
 + (id) dateWithNaturalLanguageString: (NSString*)string
@@ -165,7 +218,6 @@ static BOOL	debug = NO;
   int			modYear = 0;
   int			modDay = 0;
   int			D, M, Y;
-  int			modWeek;
   int			h = 12;
   int			m = 0;
   int			s = 0;
@@ -179,7 +231,7 @@ static BOOL	debug = NO;
   scanner = [NSScanner scannerWithString: string];
   words = [NSMutableArray arrayWithCapacity: 10];
 
-  theDate = (NSCalendarDate*)[NSCalendarDate date];
+  theDate = (NSCalendarDate*)[calendarClass date];
   Y = [theDate yearOfCommonEra];
   M = [theDate monthOfYear];
   D = [theDate dayOfMonth];
@@ -309,7 +361,6 @@ static BOOL	debug = NO;
       unsigned	c = [ymw count];
       NSString	*yname = [ymw objectAtIndex: 0];
       NSString	*mname = c > 1 ? [ymw objectAtIndex: 1] : nil;
-      NSString	*wname = c > 2 ? [ymw objectAtIndex: 2] : nil;
       NSArray	*early = [locale objectForKey: NSEarlierTimeDesignations];
       NSArray	*later = [locale objectForKey: NSLaterTimeDesignations];
 
@@ -563,26 +614,26 @@ static BOOL	debug = NO;
 		    else
 		      mustSkip = NO;
 		    break;
-			
+
 		  case 'Y':
 		    if (hadYear)
 		      dtoIndex++;
 		    else
 		      mustSkip = NO;
 		    break;
-			
+
 		  case 'H':
 		    if (hadHour)
 		      dtoIndex++;
 		    else
 		      mustSkip = NO;
 		    break;
-			
+
 		  default:
-		    dtoIndex++;
 		    if (debug)
 		      NSLog(@"odd char (unicode %d) in NSDateTimeOrdering.\n",
-			    [dto characterAtIndex: index]);
+			    [dto characterAtIndex: dtoIndex]);
+		    dtoIndex++;
 		    break;
 		}
 	    }
@@ -708,17 +759,17 @@ static BOOL	debug = NO;
 	      default:
 		if (debug)
 		  NSLog(@"unexpected char (unicode%d) in NSDateTimeOrdering.\n",
-		    [dto characterAtIndex: index]);
+		    [dto characterAtIndex: dtoIndex]);
 		break;
 	    }
 	}
     }
-  
+
   /*
    *	If we had no date or time information - we give up, otherwise
    *	we can use reasonable defaults for any missing info.
    *	Missing date => today
-   *	Missing time => 12:00
+   *	Missing time => 12: 00
    *	If we had a week/month/year modifier without a day, we assume today.
    *	If we had a day name without any more day detail - adjust to that
    *	day this week.
@@ -739,7 +790,7 @@ static BOOL	debug = NO;
   /*
    *	Build a calendar date we can adjust easily.
    */
-  theDate = [NSCalendarDate dateWithYear: Y
+  theDate = [calendarClass dateWithYear: Y
 				   month: M
 				     day: D
 				    hour: h
@@ -766,51 +817,48 @@ static BOOL	debug = NO;
       return nil;
     }
   return [self dateWithTimeIntervalSinceReferenceDate:
-		[theDate timeIntervalSinceReferenceDate]];
+		otherTime(theDate)];
 }
 
 + (id) dateWithString: (NSString*)description
 {
-  return [[[self alloc] initWithString: description]  autorelease];
+  return AUTORELEASE([[self alloc] initWithString: description]);
 }
 
 + (id) dateWithTimeIntervalSinceNow: (NSTimeInterval)seconds
 {
-  return [[[self alloc] initWithTimeIntervalSinceNow: seconds]  autorelease];
+  return AUTORELEASE([[self alloc] initWithTimeIntervalSinceNow: seconds]);
 }
 
-+ (id)dateWithTimeIntervalSince1970:(NSTimeInterval)seconds
++ (id)dateWithTimeIntervalSince1970: (NSTimeInterval)seconds
 {
-  return [[[self alloc] initWithTimeIntervalSinceReferenceDate: 
-		       UNIX_REFERENCE_INTERVAL + seconds] autorelease];
+  return AUTORELEASE([[self alloc] initWithTimeIntervalSinceReferenceDate:
+		       UNIX_REFERENCE_INTERVAL + seconds]);
 }
 
 + (id) dateWithTimeIntervalSinceReferenceDate: (NSTimeInterval)seconds
 {
-  return [[[self alloc] initWithTimeIntervalSinceReferenceDate: seconds]
-	   autorelease];
+  return AUTORELEASE([[self alloc] initWithTimeIntervalSinceReferenceDate: seconds]);
 }
 
 + (id) distantFuture
 {
-  static id df = nil;
-  if (!df)
-    df = [[self alloc] initWithTimeIntervalSinceReferenceDate: DISTANT_FUTURE];
-  return df;
+  if (_distantFuture == nil)
+    return [GSDateFuture allocWithZone: 0];
+  return _distantFuture;
 }
 
 + (id) distantPast
 {
-  static id dp = nil;
-  if (!dp)
-    dp = [[self alloc] initWithTimeIntervalSinceReferenceDate: DISTANT_PAST];
-  return dp;
+  if (_distantPast == nil)
+    return [GSDatePast allocWithZone: 0];
+  return _distantPast;
 }
 
-- (id) copyWithZone:(NSZone*)zone
+- (id) copyWithZone: (NSZone*)zone
 {
   if (NSShouldRetainWithZone(self, zone))
-    return [self retain];
+    return RETAIN(self);
   else
     return NSCopyObject(self, 0, zone);
 }
@@ -819,73 +867,78 @@ static BOOL	debug = NO;
 {
   /* Make sure that Connection's always send us bycopy,
      i.e. as our own class, not a Proxy class. */
-  return [self class];
+  return abstractClass;
 }
 
 - (Class) classForPortCoder
 {
-  return [self class];
+  return abstractClass;
 }
+
 - replacementObjectForPortCoder: aRmc
 {
   return self;
 }
 
-- (void) encodeWithCoder:(NSCoder*)coder
+- (void) encodeWithCoder: (NSCoder*)coder
 {
-  [super encodeWithCoder:coder];
-  [coder encodeValueOfObjCType:@encode(NSTimeInterval) at:&seconds_since_ref];
+  NSTimeInterval	interval = [self timeIntervalSinceReferenceDate];
+
+  [coder encodeValueOfObjCType: @encode(NSTimeInterval) at: &interval];
 }
 
-- (id) initWithCoder:(NSCoder*)coder
+- (id) initWithCoder: (NSCoder*)coder
 {
-  self = [super initWithCoder:coder];
-  [coder decodeValueOfObjCType:@encode(NSTimeInterval) at:&seconds_since_ref];
-  return self;
+  NSTimeInterval	interval;
+  id			o;
+
+  [coder decodeValueOfObjCType: @encode(NSTimeInterval) at: &interval];
+  o = [[concreteClass alloc] initWithTimeIntervalSinceReferenceDate: interval];
+  [self release];
+  return o;
 }
 
 - (id) init
 {
-  return [self initWithTimeIntervalSinceReferenceDate:
-		 [[self class] timeIntervalSinceReferenceDate]];
+  return [self initWithTimeIntervalSinceReferenceDate: GSTimeNow()];
 }
 
 - (id) initWithString: (NSString*)description
 {
   // Easiest to just have NSCalendarDate do the work for us
-  NSCalendarDate *d = [NSCalendarDate alloc];
+  NSCalendarDate	*d = [calendarClass alloc];
+
   [d initWithString: description];
   [self initWithTimeIntervalSinceReferenceDate:
-	[d timeIntervalSinceReferenceDate]];
-  [d release];
+	otherTime(d)];
+  RELEASE(d);
   return self;
 }
 
 - (id) initWithTimeInterval: (NSTimeInterval)secsToBeAdded
-		       sinceDate: (NSDate*)anotherDate;
+		  sinceDate: (NSDate*)anotherDate;
 {
   // Get the other date's time, add the secs and init thyself
   return [self initWithTimeIntervalSinceReferenceDate:
-	       [anotherDate timeIntervalSinceReferenceDate] + secsToBeAdded];
+	       otherTime(anotherDate) + secsToBeAdded];
 }
 
 - (id) initWithTimeIntervalSinceNow: (NSTimeInterval)secsToBeAdded;
 {
   // Get the current time, add the secs and init thyself
   return [self initWithTimeIntervalSinceReferenceDate:
-	       [[self class] timeIntervalSinceReferenceDate] + secsToBeAdded];
+	       GSTimeNow() + secsToBeAdded];
 }
 
-- (id)initWithTimeIntervalSince1970:(NSTimeInterval)seconds
+- (id)initWithTimeIntervalSince1970: (NSTimeInterval)seconds
 {
-  return [self initWithTimeIntervalSinceReferenceDate: 
+  return [self initWithTimeIntervalSinceReferenceDate:
 		       UNIX_REFERENCE_INTERVAL + seconds];
 }
 
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
 {
-  [super init];
-  seconds_since_ref = secs;
+  [self subclassResponsibility: _cmd];
   return self;
 }
 
@@ -894,11 +947,11 @@ static BOOL	debug = NO;
 - (NSCalendarDate *) dateWithCalendarFormat: (NSString*)formatString
 				   timeZone: (NSTimeZone*)timeZone
 {
-  NSCalendarDate *d = [NSCalendarDate alloc];
-  [d initWithTimeIntervalSinceReferenceDate: seconds_since_ref];
+  NSCalendarDate *d = [calendarClass alloc];
+  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   [d setCalendarFormat: formatString];
   [d setTimeZone: timeZone];
-  return [d autorelease];
+  return AUTORELEASE(d);
 }
 
 // Representing dates
@@ -907,10 +960,10 @@ static BOOL	debug = NO;
 {
   // Easiest to just have NSCalendarDate do the work for us
   NSString *s;
-  NSCalendarDate *d = [NSCalendarDate alloc];
-  [d initWithTimeIntervalSinceReferenceDate: seconds_since_ref];
+  NSCalendarDate *d = [calendarClass alloc];
+  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   s = [d description];
-  [d release];
+  RELEASE(d);
   return s;
 }
 
@@ -920,10 +973,10 @@ static BOOL	debug = NO;
 {
   // Easiest to just have NSCalendarDate do the work for us
   NSString *s;
-  NSCalendarDate *d = [NSCalendarDate alloc];
+  NSCalendarDate *d = [calendarClass alloc];
   id f;
 
-  [d initWithTimeIntervalSinceReferenceDate: seconds_since_ref];
+  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   if (!format)
     f = [d calendarFormat];
   else
@@ -932,7 +985,7 @@ static BOOL	debug = NO;
     [d setTimeZone: aTimeZone];
 
   s = [d descriptionWithCalendarFormat: f locale: l];
-  [d release];
+  RELEASE(d);
   return s;
 }
 
@@ -940,10 +993,10 @@ static BOOL	debug = NO;
 {
   // Easiest to just have NSCalendarDate do the work for us
   NSString *s;
-  NSCalendarDate *d = [NSCalendarDate alloc];
-  [d initWithTimeIntervalSinceReferenceDate: seconds_since_ref];
+  NSCalendarDate *d = [calendarClass alloc];
+  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   s = [d descriptionWithLocale: locale];
-  [d release];
+  RELEASE(d);
   return s;
 }
 
@@ -953,70 +1006,297 @@ static BOOL	debug = NO;
 {
   /* xxx We need to check for overflow? */
   return [[self class] dateWithTimeIntervalSinceReferenceDate:
-		       seconds_since_ref + seconds];
+		       otherTime(self) + seconds];
 }
 
 - (NSTimeInterval) timeIntervalSince1970
 {
-  return seconds_since_ref - UNIX_REFERENCE_INTERVAL;
+  return otherTime(self) - UNIX_REFERENCE_INTERVAL;
 }
 
 - (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate
 {
-  return seconds_since_ref - [otherDate timeIntervalSinceReferenceDate];
+  return otherTime(self) - otherTime(otherDate);
 }
 
 - (NSTimeInterval) timeIntervalSinceNow
 {
-  NSTimeInterval now = [[self class] timeIntervalSinceReferenceDate];
-  return seconds_since_ref - now;
+  return otherTime(self) - GSTimeNow();
 }
 
 - (NSTimeInterval) timeIntervalSinceReferenceDate
 {
-  return seconds_since_ref;
+  [self subclassResponsibility: _cmd];
+  return 0;
 }
 
 // Comparing dates
 
 - (NSComparisonResult) compare: (NSDate*)otherDate
 {
-  if (seconds_since_ref > [otherDate timeIntervalSinceReferenceDate])
+  if (otherTime(self) > otherTime(otherDate))
     return NSOrderedDescending;
-		
-  if (seconds_since_ref < [otherDate timeIntervalSinceReferenceDate])
+
+  if (otherTime(self) < otherTime(otherDate))
     return NSOrderedAscending;
-		
+
   return NSOrderedSame;
 }
 
 - (NSDate*) earlierDate: (NSDate*)otherDate
 {
-  if (seconds_since_ref > [otherDate timeIntervalSinceReferenceDate])
+  if (otherTime(self) > otherTime(otherDate))
     return otherDate;
   return self;
 }
 
 - (BOOL) isEqual: (id)other
 {
-  if ([other isKindOf: [NSDate class]] 
-      && 1.0 > ABS(seconds_since_ref - [other timeIntervalSinceReferenceDate]))
+  if ([other isKindOf: abstractClass]
+      && 1.0 > ABS(otherTime(self) - otherTime(other)))
     return YES;
   return NO;
-}		
+}
 
 - (BOOL) isEqualToDate: (NSDate*)other
 {
-  if (1.0 > ABS(seconds_since_ref - [other timeIntervalSinceReferenceDate]))
+  if (1.0 > ABS(otherTime(self) - otherTime(other)))
     return YES;
   return NO;
-}		
+}
 
 - (NSDate*) laterDate: (NSDate*)otherDate
 {
-  if (seconds_since_ref < [otherDate timeIntervalSinceReferenceDate])
+  if (otherTime(self)
+    < otherTime(otherDate))
     return otherDate;
   return self;
 }
 
 @end
+
+@implementation NSGDate
+
++ (void) initialize
+{
+  if (self == [NSDate class])
+    {
+      [self setVersion: 1];
+    }
+}
+
+- (Class) classForPortCoder
+{
+  return [self class];
+}
+
+- replacementObjectForPortCoder: aRmc
+{
+  return self;
+}
+
+- (void) encodeWithCoder: (NSCoder*)coder
+{
+  [coder encodeValueOfObjCType: @encode(NSTimeInterval) at: &_seconds_since_ref];
+}
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  [coder decodeValueOfObjCType: @encode(NSTimeInterval) at: &_seconds_since_ref];
+  return self;
+}
+
+- (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
+{
+  _seconds_since_ref = secs;
+  return self;
+}
+
+
+// Adding and getting intervals
+
+- (NSTimeInterval) timeIntervalSince1970
+{
+  return _seconds_since_ref - UNIX_REFERENCE_INTERVAL;
+}
+
+- (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate
+{
+  return _seconds_since_ref - otherTime(otherDate);
+}
+
+- (NSTimeInterval) timeIntervalSinceNow
+{
+  return _seconds_since_ref - GSTimeNow();
+}
+
+- (NSTimeInterval) timeIntervalSinceReferenceDate
+{
+  return _seconds_since_ref;
+}
+
+// Comparing dates
+
+- (NSComparisonResult) compare: (NSDate*)otherDate
+{
+  if (_seconds_since_ref > otherTime(otherDate))
+    return NSOrderedDescending;
+
+  if (_seconds_since_ref < otherTime(otherDate))
+    return NSOrderedAscending;
+
+  return NSOrderedSame;
+}
+
+- (NSDate*) earlierDate: (NSDate*)otherDate
+{
+  if (_seconds_since_ref > otherTime(otherDate))
+    return otherDate;
+  return self;
+}
+
+- (BOOL) isEqual: (id)other
+{
+  if ([other isKindOfClass: abstractClass]
+      && 1.0 > ABS(_seconds_since_ref - otherTime(other)))
+    return YES;
+  return NO;
+}
+
+- (BOOL) isEqualToDate: (NSDate*)other
+{
+  if (1.0 > ABS(_seconds_since_ref - otherTime(other)))
+    return YES;
+  return NO;
+}
+
+- (NSDate*) laterDate: (NSDate*)otherDate
+{
+  if (_seconds_since_ref < otherTime(otherDate))
+    return otherDate;
+  return self;
+}
+
+@end
+
+
+
+/*
+ *	This abstract class represents a date of which there can be only
+ *	one instance.
+ */
+@implementation GSDateSingle
+
++ (void) initialize
+{
+  if (self == [GSDateSingle class])
+    {
+      [self setVersion: 1];
+      behavior_class_add_class(self, [NSGDate class]);
+    }
+}
+
+- (Class) classForPortCoder
+{
+  return [self class];
+}
+
+- replacementObjectForPortCoder: aRmc
+{
+  return self;
+}
+
+- (void) encodeWithCoder: (NSCoder*)coder
+{
+}
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  return self;
+}
+
+- (id) autorelease
+{
+  return self;
+}
+
+- (void) release
+{
+}
+
+- (id) retain
+{
+  return self;
+}
+
++ (id) allocWithZone: (NSZone*)z
+{
+  [NSException raise: NSInternalInconsistencyException
+	      format: @"Attempt to allocate fixed date"];
+  return nil;
+}
+
+- (id) copyWithZone: (NSZone*)z
+{
+  return self;
+}
+
+- (void) dealloc
+{
+  [NSException raise: NSInternalInconsistencyException
+	      format: @"Attempt to deallocate fixed date"];
+}
+
+- (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
+{
+  return self;
+}
+
+@end
+
+
+
+@implementation GSDatePast
+
++ (id) allocWithZone: (NSZone*)z
+{
+  if (_distantPast == nil)
+    {
+      id	obj = NSAllocateObject(self, 0, NSDefaultMallocZone());
+
+      _distantPast = [obj init];
+    }
+  return _distantPast;
+}
+
+- (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
+{
+  _seconds_since_ref = DISTANT_PAST;
+  return self;
+}
+
+@end
+
+
+@implementation GSDateFuture
+
++ (id) allocWithZone: (NSZone*)z
+{
+  if (_distantFuture == nil)
+    {
+      id	obj = NSAllocateObject(self, 0, NSDefaultMallocZone());
+
+      _distantFuture = [obj init];
+    }
+  return _distantFuture;
+}
+
+- (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
+{
+  _seconds_since_ref = DISTANT_FUTURE;
+  return self;
+}
+
+@end
+
+
