@@ -20,13 +20,18 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */ 
 
 #include <config.h>
 #include <base/preface.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSCoder.h>
+#include <Foundation/NSArray.h>
+#include <Foundation/NSData.h>
+#include <Foundation/NSDictionary.h>
+#include <Foundation/NSCharacterSet.h>
+#include <Foundation/NSRange.h>
 #include <base/NSGString.h>
 #include <base/NSGCString.h>
 #include <base/IndexedCollection.h>
@@ -37,11 +42,43 @@
 #include <base/Unicode.h>
 #include <base/fast.x>
 
-static	SEL	csInitSel = @selector(initWithCStringNoCopy:length:fromZone:);
+/*
+ *	Include sequence handling code with instructions to generate search
+ *	and compare functions for NSString objects.
+ */
+#define	GSEQ_STRCOMP	strCompCsNs
+#define	GSEQ_STRRANGE	strRangeCsNs
+#define	GSEQ_O	GSEQ_NS
+#define	GSEQ_S	GSEQ_CS
+#include <GSeq.h>
+
+#define	GSEQ_STRCOMP	strCompCsUs
+#define	GSEQ_STRRANGE	strRangeCsUs
+#define	GSEQ_O	GSEQ_US
+#define	GSEQ_S	GSEQ_CS
+#include <GSeq.h>
+
+#define	GSEQ_STRCOMP	strCompCsCs
+#define	GSEQ_STRRANGE	strRangeCsCs
+#define	GSEQ_O	GSEQ_CS
+#define	GSEQ_S	GSEQ_CS
+#include <GSeq.h>
+
+/*
+ *	Include property-list parsing code configured for ascii characters.
+ */
+#define	GSPLUNI	0
+#include "propList.h"
+
+static	SEL	csInitSel = @selector(initWithCStringNoCopy: length: fromZone:);
 static	SEL	msInitSel = @selector(initWithCapacity:);
 static	IMP	csInitImp;	/* designated initialiser for cString	*/
 static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
+@interface NSGMutableCString (GNUDescription)
+- (unsigned char*) _extendBy: (unsigned)len;
+@end
+
 @implementation NSGCString
 
 + (void) initialize
@@ -55,14 +92,14 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     }
 }
 
-+ allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  return NSAllocateObject (self, 0, z);
+  return NSAllocateObject(self, 0, z);
 }
 
-+ alloc
++ (id) alloc
 {
-  return NSAllocateObject (self, 0, NSDefaultMallocZone());
+  return NSAllocateObject(self, 0, NSDefaultMallocZone());
 }
 
 - (void)dealloc
@@ -95,8 +132,12 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 		    fromZone: (NSZone*)zone
 {
   _count = length;
-  _contents_chars = byteString;
+  _contents_chars = (unsigned char*)byteString;
+#if	GS_WITH_GC
+  _zone = byteString ? GSAtomicMallocZone() : 0;
+#else
   _zone = byteString ? zone : 0;
+#endif
   return self;
 }
 
@@ -107,7 +148,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 {
   NSZone	*z;
 
-  if (flag)
+  if (flag && byteString)
     {
       z = NSZoneFromPointer(byteString);
     }
@@ -126,7 +167,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   id a = [[NSGString allocWithZone: z] initWithCharactersNoCopy: chars
 							 length: length
 						       fromZone: z];
-  [self release];
+  RELEASE(self);
   return a;
 }
 
@@ -138,7 +179,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   id a = [[NSGString allocWithZone: z] initWithCharactersNoCopy: chars
 							 length: length
 						   freeWhenDone: flag];
-  [self release];
+  RELEASE(self);
   return a;
 }
 
@@ -161,44 +202,56 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     }
 }
 
-- (void) encodeWithCoder: aCoder
+- (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [aCoder encodeValueOfObjCType:@encode(unsigned) at:&_count];
+  [aCoder encodeValueOfObjCType: @encode(unsigned) at: &_count];
   if (_count > 0)
     {
-      [aCoder encodeArrayOfObjCType:@encode(unsigned char)
-			      count:_count
-				 at:_contents_chars];
+      [aCoder encodeArrayOfObjCType: @encode(unsigned char)
+			      count: _count
+				 at: _contents_chars];
     }
 }
 
-- initWithCoder: aCoder
+- (id) initWithCoder: (NSCoder*)aCoder
 {
-  [aCoder decodeValueOfObjCType:@encode(unsigned)
-			     at:&_count];
+  [aCoder decodeValueOfObjCType: @encode(unsigned)
+			     at: &_count];
   if (_count > 0)
     {
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, _count);
-      [aCoder decodeArrayOfObjCType:@encode(unsigned char)
-			      count:_count
-				 at:_contents_chars];
+      [aCoder decodeArrayOfObjCType: @encode(unsigned char)
+			      count: _count
+				 at: _contents_chars];
     }
   return self;
 }
 
-- copy
+- (id) copy
 {
   NSZone	*z = NSDefaultMallocZone();
 
   if (NSShouldRetainWithZone(self, z) == NO)
     {
       NSGCString	*obj;
-      char		*tmp;
+      unsigned char	*tmp;
 
       obj = (NSGCString*)NSAllocateObject(_fastCls._NSGCString, 0, z);
-      tmp = NSZoneMalloc(z, _count);
-      memcpy(tmp, _contents_chars, _count);
+      if (_count)
+	{
+	  tmp = NSZoneMalloc(z, _count);
+	  memcpy(tmp, _contents_chars, _count);
+	}
+      else
+	{
+	  tmp = 0;
+	  z = 0;
+	}
       obj = (*csInitImp)(obj, csInitSel, tmp, _count, z);
       if (_hash && obj)
         {
@@ -208,20 +261,28 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     }
   else 
     {
-      return [self retain];
+      return RETAIN(self);
     }
 }
 
-- copyWithZone: (NSZone*)z
+- (id) copyWithZone: (NSZone*)z
 {
   if (NSShouldRetainWithZone(self, z) == NO)
     {
       NSGCString	*obj;
-      char		*tmp;
+      unsigned char	*tmp;
 
       obj = (NSGCString*)NSAllocateObject(_fastCls._NSGCString, 0, z);
-      tmp = NSZoneMalloc(z, _count);
-      memcpy(tmp, _contents_chars, _count);
+      if (_count)
+	{
+	  tmp = NSZoneMalloc(z, _count);
+	  memcpy(tmp, _contents_chars, _count);
+	}
+      else
+	{
+	  tmp = 0;
+	  z = 0;
+	}
       obj = (*csInitImp)(obj, csInitSel, tmp, _count, z);
       if (_hash && obj)
         {
@@ -231,11 +292,11 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     }
   else 
     {
-      return [self retain];
+      return RETAIN(self);
     }
 }
 
-- mutableCopy
+- (id) mutableCopy
 {
   NSGMutableCString	*obj;
 
@@ -256,7 +317,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   return obj;
 }
 
-- mutableCopyWithZone: (NSZone*)z
+- (id) mutableCopyWithZone: (NSZone*)z
 {
   NSGMutableCString	*obj;
 
@@ -278,11 +339,20 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
 - (const char *) cString
 {
-  char *r = (char*)_fastMallocBuffer(_count+1);
+  unsigned char	*r = (unsigned char*)_fastMallocBuffer(_count+1);
 
   memcpy(r, _contents_chars, _count);
   r[_count] = '\0';
-  return r;
+  return (const char*)r;
+}
+
+- (const char *) lossyCString
+{
+  unsigned char	*r = (unsigned char*)_fastMallocBuffer(_count+1);
+
+  memcpy(r, _contents_chars, _count);
+  r[_count] = '\0';
+  return (const char*)r;
 }
 
 - (void) getCString: (char*)buffer
@@ -307,10 +377,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 {
   int len;
 
-  if (aRange.location >= _count)
-    [NSException raise: NSRangeException format:@"Invalid location."];
-  if (aRange.length > (_count - aRange.location))
-    [NSException raise: NSRangeException format:@"Invalid location+length."];
+  GS_RANGE_CHECK(aRange, _count);
   if (maxLength < aRange.length)
     {
       len = maxLength;
@@ -357,7 +424,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
 - (void) getCharacters: (unichar*)buffer
 {
-  int i;
+  unsigned	i;
 
   for (i = 0; i < _count; i++)
     buffer[i] = chartouni(((unsigned char *)_contents_chars)[i]);
@@ -365,12 +432,9 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
 - (void) getCharacters: (unichar*)buffer range: (NSRange)aRange
 {
-  int e, i;
+  unsigned	e, i;
 
-  if (aRange.location >= _count)
-    [NSException raise: NSRangeException format:@"Invalid location."];
-  if (aRange.length > (_count - aRange.location))
-    [NSException raise: NSRangeException format:@"Invalid location+length."];
+  GS_RANGE_CHECK(aRange, _count);
   e = aRange.location + aRange.length;
   for (i = aRange.location; i < e; i++)
     *buffer++ = chartouni(((unsigned char *)_contents_chars)[i]);
@@ -378,17 +442,15 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
 - (NSString*) substringFromRange: (NSRange)aRange
 {
-  if (aRange.location > _count)
-    [NSException raise: NSRangeException format:@"Invalid location."];
-  if (aRange.length > (_count - aRange.location))
-    [NSException raise: NSRangeException format:@"Invalid location+length."];
+  GS_RANGE_CHECK(aRange, _count);
   return [[self class] stringWithCString: _contents_chars + aRange.location
 		       length: aRange.length];
 }
 
 - (NSStringEncoding) fastestEncoding
 {
-  if(([NSString defaultCStringEncoding]==NSASCIIStringEncoding)  || ([NSString defaultCStringEncoding]==NSISOLatin1StringEncoding))
+  if (([NSString defaultCStringEncoding] == NSASCIIStringEncoding)
+    || ([NSString defaultCStringEncoding] == NSISOLatin1StringEncoding))
     return [NSString defaultCStringEncoding];
   else
     return NSUnicodeStringEncoding;
@@ -408,7 +470,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     return NO;
   c = fastClassOfInstance(anObject);
 
-  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString || c == _fastCls._NXConstantString)
+  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString)
     {
       NSGCString	*other = (NSGCString*)anObject;
 
@@ -424,6 +486,22 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 	return NO;
       return YES;
     }
+  else if (c == _fastCls._NXConstantString)
+    {
+      NSGCString	*other = (NSGCString*)anObject;
+
+      if (_count != other->_count)
+	return NO;
+      if (memcmp(_contents_chars, other->_contents_chars, _count) != 0)
+	return NO;
+      return YES;
+    }
+  else if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    {
+      if (strCompCsUs(self, anObject, 0, (NSRange){0,_count}) == NSOrderedSame)
+	return YES;
+      return NO;
+    }
   else if (c == nil)
     return NO;
   else if (fastClassIsKindOfClass(c, _fastCls._NSString))
@@ -442,7 +520,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   if (aString == nil)
     return NO;
   c = fastClassOfInstance(aString);
-  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString || c == _fastCls._NXConstantString)
+  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString)
     {
       NSGCString	*other = (NSGCString*)aString;
 
@@ -458,6 +536,22 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 	return NO;
       return YES;
     }
+  else if (c == _fastCls._NXConstantString)
+    {
+      NSGCString	*other = (NSGCString*)aString;
+
+      if (_count != other->_count)
+	return NO;
+      if (memcmp(_contents_chars, other->_contents_chars, _count) != 0)
+	return NO;
+      return YES;
+    }
+  else if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    {
+      if (strCompCsUs(self, aString, 0, (NSRange){0,_count}) == NSOrderedSame)
+	return YES;
+      return NO;
+    }
   else if (c == nil)
     return NO;
   else if (fastClassIsKindOfClass(c, _fastCls._NSString))
@@ -483,16 +577,29 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
 - (id) initWithString: (NSString*)string
 {
-    NSZone	*z = fastZone(self);
-    unsigned	length = [string cStringLength];
-    char	*buf = NSZoneMalloc(z, length+1);  // getCString appends a nul.
+  unsigned	length = [string cStringLength];
+  NSZone	*z;
+  unsigned char	*buf;
 
-    [string getCString: buf];
-    return [self initWithCStringNoCopy: buf length: length fromZone: z];
+  if (length > 0)
+    {
+      z = fastZone(self);
+      buf = NSZoneMalloc(z, length+1);  // getCString appends a nul.
+      [string getCString: buf];
+    }
+  else
+    {
+      z = 0;
+      buf = 0;
+    }
+  return [self initWithCStringNoCopy: buf length: length fromZone: z];
 }
 
 - (void) descriptionTo: (id<GNUDescriptionDestination>)output
 {
+  if (output == nil)
+    return;
+
   if (_count == 0)
     {
       [output appendString: @"\"\""];
@@ -505,7 +612,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
       for (i = 0; i < _count; i++)
 	{
-	  char	val = _contents_chars[i];
+	  unsigned char	val = _contents_chars[i];
 
 	  if (isalnum(val))
 	    {
@@ -513,26 +620,26 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 	    }
 	  switch (val)
 	    {
-	      case '\a':
-	      case '\b':
-	      case '\t':
-	      case '\r':
-	      case '\n':
-	      case '\v':
-	      case '\f':
-	      case '\\':
-	      case '"' :
+	      case '\a': 
+	      case '\b': 
+	      case '\t': 
+	      case '\r': 
+	      case '\n': 
+	      case '\v': 
+	      case '\f': 
+	      case '\\': 
+	      case '"' : 
 		length += 1;
 		break;
 
-	      default:
+	      default: 
 		if (val == ' ' || isprint(val))
 		  {
 		    needQuote = YES;
 		  }
 		else
 		  {
-		    length += 3;
+		    length += 4;
 		  }
 		break;
 	    }
@@ -540,29 +647,39 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 
       if (needQuote || length != _count)
 	{
-	  NSZone	*z = fastZone(self);
-	  char		*buf = NSZoneMalloc(z, length+3);
-	  char		*ptr = buf;
-	  NSString	*result;
+	  Class		c = fastClass(output);
+	  NSZone	*z = NSDefaultMallocZone();
+	  unsigned char	*buf;
+	  unsigned char	*ptr;
 
+	  length += 2;
+	  if (c == _fastCls._NSGMutableCString)
+	    {
+	      buf = [(NSGMutableCString*)output _extendBy: length];
+	    }
+	  else
+	    {
+	      buf = NSZoneMalloc(z, length+1);
+	    }
+	  ptr = buf;
 	  *ptr++ = '"';
 	  for (i = 0; i < _count; i++)
 	    {
-	      char	val = _contents_chars[i];
+	      unsigned char	val = _contents_chars[i];
 
 	      switch (val)
 		{
-		  case '\a':	*ptr++ = '\\'; *ptr++ = 'a';  break;
-		  case '\b':	*ptr++ = '\\'; *ptr++ = 'b';  break;
-		  case '\t':	*ptr++ = '\\'; *ptr++ = 't';  break;
-		  case '\r':	*ptr++ = '\\'; *ptr++ = 'r';  break;
-		  case '\n':	*ptr++ = '\\'; *ptr++ = 'n';  break;
-		  case '\v':	*ptr++ = '\\'; *ptr++ = 'v';  break;
-		  case '\f':	*ptr++ = '\\'; *ptr++ = 'f';  break;
-		  case '\\':	*ptr++ = '\\'; *ptr++ = '\\'; break;
-		  case '"' :	*ptr++ = '\\'; *ptr++ = '"';  break;
+		  case '\a': 	*ptr++ = '\\'; *ptr++ = 'a';  break;
+		  case '\b': 	*ptr++ = '\\'; *ptr++ = 'b';  break;
+		  case '\t': 	*ptr++ = '\\'; *ptr++ = 't';  break;
+		  case '\r': 	*ptr++ = '\\'; *ptr++ = 'r';  break;
+		  case '\n': 	*ptr++ = '\\'; *ptr++ = 'n';  break;
+		  case '\v': 	*ptr++ = '\\'; *ptr++ = 'v';  break;
+		  case '\f': 	*ptr++ = '\\'; *ptr++ = 'f';  break;
+		  case '\\': 	*ptr++ = '\\'; *ptr++ = '\\'; break;
+		  case '"' : 	*ptr++ = '\\'; *ptr++ = '"';  break;
 
-		  default:
+		  default: 
 		    if (isprint(val) || val == ' ')
 		      {
 			*ptr++ = val;
@@ -580,10 +697,15 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 	    }
 	  *ptr++ = '"';
 	  *ptr = '\0';
-	  result = [[_fastCls._NSGCString alloc] initWithCStringNoCopy: buf
-			length: length+2 fromZone: z];
-	  [output appendString: result];
-	  [result release];
+	  if (c != _fastCls._NSGMutableCString)
+	    {
+	      NSString	*result;
+
+	      result = [[_fastCls._NSGCString allocWithZone: z]
+		initWithCStringNoCopy: buf length: length fromZone: z];
+	      [output appendString: result];
+	      RELEASE(result);
+	    }
 	}
       else
 	{
@@ -591,19 +713,110 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 	}
     }
 }
+
+- (id) propertyList
+{
+  id		result;
+  pldata	data;
+
+  data.ptr = _contents_chars;
+  data.pos = 0;
+  data.end = _count;
+  data.lin = 1;
+  data.err = nil;
+
+  if (plInit == 0)
+    setupPl([NSGCString class]);
+
+  result = parsePlItem(&data);
+
+  if (result == nil && data.err != nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"%@ at line %u", data.err, data.lin];
+    }
+  return AUTORELEASE(result);
+}
+
+- (NSDictionary*) propertyListFromStringsFileFormat
+{
+  id		result;
+  pldata	data;
+
+  data.ptr = _contents_chars;
+  data.pos = 0;
+  data.end = _count;
+  data.lin = 1;
+  data.err = nil;
+
+  if (plInit == 0)
+    setupPl([NSGCString class]);
+
+  result = parseSfItem(&data);
+  if (result == nil && data.err != nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"%@ at line %u", data.err, data.lin];
+    }
+  return AUTORELEASE(result);
+}
+
+- (NSRange) rangeOfComposedCharacterSequenceAtIndex: (unsigned)anIndex
+{
+  if (anIndex >= _count)
+    [NSException raise: NSRangeException format: @"Invalid location."];
+  return NSMakeRange(anIndex, 1);
+}
+
+- (NSComparisonResult) compare: (NSString*)aString
+		       options: (unsigned int)mask
+			 range: (NSRange)aRange
+{
+  Class	c;
+
+  if (aString == nil)
+    [NSException raise: NSInvalidArgumentException format: @"compare with nil"];
+  c = fastClass(aString);
+  if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    return strCompCsUs(self, aString, mask, aRange);
+  else if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString
+    || c == _fastCls._NXConstantString)
+    return strCompCsCs(self, aString, mask, aRange);
+  else
+    return strCompCsNs(self, aString, mask, aRange);
+}
+
+- (NSRange) rangeOfString: (NSString *) aString
+		  options: (unsigned int) mask
+		    range: (NSRange) aRange
+{
+  Class	c;
+
+  if (aString == nil)
+    [NSException raise: NSInvalidArgumentException format: @"range of nil"];
+  c = fastClass(aString);
+  if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    return strRangeCsUs(self, aString, mask, aRange);
+  else if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString
+    || c == _fastCls._NXConstantString)
+    return strRangeCsCs(self, aString, mask, aRange);
+  else
+    return strRangeCsNs(self, aString, mask, aRange);
+}
+
 @end
 
 
 @implementation NSGMutableCString
 
-+ allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  return NSAllocateObject (self, 0, z);
+  return NSAllocateObject(self, 0, z);
 }
 
-+ alloc
++ (id) alloc
 {
-  return NSAllocateObject (self, 0, NSDefaultMallocZone());
+  return NSAllocateObject(self, 0, NSDefaultMallocZone());
 }
 
 + (void) initialize
@@ -620,14 +833,27 @@ typedef struct {
   @defs(NSGMutableCString)
 } NSGMutableCStringStruct;
 
+static inline void
+stringGrowBy(NSGMutableCStringStruct *self, unsigned want)
+{
+  want += self->_count + 1;
+  if (want > self->_capacity)
+    self->_capacity += self->_capacity/2;
+  if (want > self->_capacity)
+    self->_capacity = want;
+  self->_contents_chars
+    = NSZoneRealloc(self->_zone, self->_contents_chars, self->_capacity);
+}
+
 static inline void
 stringIncrementCountAndMakeHoleAt(NSGMutableCStringStruct *self, 
 				  int index, int size)
 {
 #ifndef STABLE_MEMCPY
   {
-    int i;
-    for (i = self->_count; i >= index; i--)
+    unsigned i = self->_count;
+
+    while (i-- > index)
       self->_contents_chars[i+size] = self->_contents_chars[i];
   }
 #else
@@ -659,13 +885,17 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 }
 
 /* This is the designated initializer for this class */
-- initWithCapacity: (unsigned)capacity
+- (id) initWithCapacity: (unsigned)capacity
 {
   _count = 0;
   _capacity = capacity;
   if (capacity)
     {
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, _capacity);
     }
   return self;
@@ -680,8 +910,12 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
     {
       _count = length;
       _capacity = length;
-      _contents_chars = byteString;
+      _contents_chars = (unsigned char*)byteString;
+#if	GS_WITH_GC
+      _zone = byteString ? GSAtomicMallocZone() : 0;
+#else
       _zone = byteString ? zone : 0;
+#endif
     }
   return self;
 }
@@ -694,7 +928,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   id a = [[NSGMutableString allocWithZone: z] initWithCharactersNoCopy: chars
 							 length: length
 						       fromZone: z];
-  [self release];
+  RELEASE(self);
   return a;
 }
 
@@ -706,19 +940,27 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   id a = [[NSGMutableString allocWithZone: z] initWithCharactersNoCopy: chars
 							   length: length
 						     freeWhenDone: flag];
-  [self release];
+  RELEASE(self);
   return a;
 }
 
-- copy
+- (id) copy
 {
-  char		*tmp;
+  unsigned char	*tmp;
   NSGCString	*obj;
   NSZone	*z = NSDefaultMallocZone();
 
   obj = (NSGCString*)NSAllocateObject(_fastCls._NSGCString, 0, z);
-  tmp = NSZoneMalloc(z, _count);
-  memcpy(tmp, _contents_chars, _count);
+  if (_count)
+    {
+      tmp = NSZoneMalloc(z, _count);
+      memcpy(tmp, _contents_chars, _count);
+    }
+  else
+    {
+      tmp = 0;
+      z = 0;
+    }
   obj = (*csInitImp)(obj, csInitSel, tmp, _count, z);
   if (_hash && obj)
     {
@@ -729,14 +971,22 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return obj;
 }
 
-- copyWithZone: (NSZone*)z
+- (id) copyWithZone: (NSZone*)z
 {
-  char		*tmp;
+  unsigned char	*tmp;
   NSGCString	*obj;
 
   obj = (NSGCString*)NSAllocateObject(_fastCls._NSGCString, 0, z);
-  tmp = NSZoneMalloc(z, _count);
-  memcpy(tmp, _contents_chars, _count);
+  if (_count)
+    {
+      tmp = NSZoneMalloc(z, _count);
+      memcpy(tmp, _contents_chars, _count);
+    }
+  else
+    {
+      tmp = 0;
+      z = 0;
+    }
   obj = (*csInitImp)(obj, csInitSel, tmp, _count, z);
   if (_hash && obj)
     {
@@ -747,7 +997,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return obj;
 }
 
-- mutableCopy
+- (id) mutableCopy
 {
   NSGMutableCString	*obj;
 
@@ -766,7 +1016,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return obj;
 }
 
-- mutableCopyWithZone: (NSZone*)z
+- (id) mutableCopyWithZone: (NSZone*)z
 {
   NSGMutableCString	*obj;
 
@@ -781,6 +1031,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 	  obj->_hash = _hash;
 	}
     }
+  return obj;
 }
 
 - (void) deleteCharactersInRange: (NSRange)range
@@ -791,21 +1042,19 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 
 // xxx This should be primitive method
 - (void) replaceCharactersInRange: (NSRange)range
-   withString: (NSString*)aString
+		       withString: (NSString*)aString
 {
-  [self deleteCharactersInRange:range];
-  [self insertString:aString atIndex:range.location];
+  [self deleteCharactersInRange: range];
+  [self insertString: aString atIndex: range.location];
 }
 
-- (void) insertString: (NSString*)aString atIndex:(unsigned)index
+- (void) insertString: (NSString*)aString atIndex: (unsigned)index
 {
   unsigned c = [aString cStringLength];
-  char	save;
+  unsigned char	save;
+
   if (_count + c >= _capacity)
-    {
-      _capacity = MAX(_capacity*2, _count+c+1);
-      _contents_chars = NSZoneRealloc(_zone, _contents_chars, _capacity);
-    }
+    stringGrowBy((NSGMutableCStringStruct *)self, c);
   stringIncrementCountAndMakeHoleAt((NSGMutableCStringStruct*)self, index, c);
   save = _contents_chars[index+c];	// getCString will put a nul here.
   [aString getCString: _contents_chars + index];
@@ -824,11 +1073,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
       unsigned		l = other->_count;
 
       if (_count + l > _capacity)
-        {
-          _capacity = MAX(_capacity*2, _count+l);
-          _contents_chars =
-		    NSZoneRealloc(fastZone(self), _contents_chars, _capacity);
-        }
+	stringGrowBy((NSGMutableCStringStruct *)self, l);
       memcpy(_contents_chars + _count, other->_contents_chars, l);
       _count += l;
       _hash = 0;
@@ -837,11 +1082,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
     {
       unsigned l = [aString cStringLength];
       if (_count + l >= _capacity)
-        {
-          _capacity = MAX(_capacity*2, _count+l+1);
-          _contents_chars =
-		    NSZoneRealloc(fastZone(self), _contents_chars, _capacity);
-        }
+	stringGrowBy((NSGMutableCStringStruct *)self, l);
       [aString getCString: _contents_chars + _count];
       _count += l;
       _hash = 0;
@@ -855,7 +1096,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
     {
       _capacity = length+1;
       _contents_chars =
-		NSZoneRealloc(fastZone(self), _contents_chars, _capacity);
+		NSZoneRealloc(_zone, _contents_chars, _capacity);
     }
   [aString getCString: _contents_chars];
   _count = length;
@@ -877,12 +1118,12 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 				    range.location, range.length);
 }
 
-- initWithCoder: aCoder
+- (id) initWithCoder: (NSCoder*)aCoder
 {
   unsigned cap;
   
-  [aCoder decodeValueOfObjCType:@encode(unsigned) at:&cap];
-  [self initWithCapacity:cap];
+  [aCoder decodeValueOfObjCType: @encode(unsigned) at: &cap];
+  [self initWithCapacity: cap];
   _count = cap;
   if (_count > 0)
     {
@@ -912,7 +1153,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
     {
       _capacity *= 2;
       _contents_chars =
-		NSZoneRealloc(fastZone(self), _contents_chars, _capacity);
+		NSZoneRealloc(_zone, _contents_chars, _capacity);
     }
   stringIncrementCountAndMakeHoleAt((NSGMutableCStringStruct*)self, index, 1);
   _contents_chars[index] = [newObject charValue];
@@ -924,41 +1165,68 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   stringDecrementCountAndFillHoleAt((NSGMutableCStringStruct*)self, index, 1);
 }
 
+- (unsigned char*) _extendBy: (unsigned)len
+{
+  unsigned char	*ptr;
+
+  if (len > 0)
+    stringGrowBy((NSGMutableCStringStruct *)self, len);
+  ptr = &_contents_chars[_count];
+  _count += len;
+  _hash = 0;
+  return ptr;
+}
 @end
 
 @implementation NXConstantString
 
++ (id) allocWithZone: (NSZone*)z
+{
+  [NSException raise: NSGenericException
+	      format: @"Attempt to allocate an NXConstantString"];
+  return nil;
+}
+
+- (id) initWithCStringNoCopy: (char*)byteString
+		      length: (unsigned int)length
+		    fromZone: (NSZone*)zone
+{
+  [NSException raise: NSGenericException
+	      format: @"Attempt to init an NXConstantString"];
+  return nil;
+}
+
 /*
  *	NXConstantString overrides [-dealloc] so that it is never deallocated.
  *	If we pass an NXConstantString to another process or record it in an
  *	archive and readi it back, the new copy will never be deallocated -
  *	causing a memory leak.  So we tell the system to use the super class.
  */
-- (Class)classForArchiver
+- (Class) classForArchiver
 {
   return [self superclass];
 }
 
-- (Class)classForCoder
+- (Class) classForCoder
 {
   return [self superclass];
 }
 
-- (Class)classForPortCoder
+- (Class) classForPortCoder
 {
   return [self superclass];
 }
 
-- (void)dealloc
+- (void) dealloc
 {
 }
 
 - (const char*) cString
 {
-  return _contents_chars;
+  return (const char*) _contents_chars;
 }
 
-- retain
+- (id) retain
 {
   return self;
 }
@@ -968,12 +1236,17 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return;
 }
 
-- autorelease
+- (id) autorelease
 {
   return self;
 }
 
-- copyWithZone: (NSZone*)z
+- (id) copy
+{
+  return self;
+}
+
+- (id) copyWithZone: (NSZone*)z
 {
   return self;
 }
@@ -999,4 +1272,139 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return (unichar)_contents_chars[index];
 }
 
+- (unsigned) hash
+{
+  return _fastImp._NSString_hash(self, @selector(hash));
+}
+
+- (BOOL) isEqual: (id)anObject
+{
+  Class	c;
+  if (anObject == self)
+    {
+      return YES;
+    }
+  if (anObject == nil)
+    {
+      return NO;
+    }
+  c = fastClassOfInstance(anObject);
+
+  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString
+    || c == _fastCls._NXConstantString)
+    {
+      NXConstantString	*other = (NXConstantString*)anObject;
+
+      if (_count != other->_count)
+	return NO;
+      if (memcmp(_contents_chars, other->_contents_chars, _count) != 0)
+	return NO;
+      return YES;
+    }
+  else if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    {
+      if (strCompCsUs(self, anObject, 0, (NSRange){0,_count}) == NSOrderedSame)
+	return YES;
+      return NO;
+    }
+  else if (c == nil)
+    {
+      return NO;
+    }
+  else if (fastClassIsKindOfClass(c, _fastCls._NSString))
+    {
+      return _fastImp._NSString_isEqualToString_(self,
+	@selector(isEqualToString:), anObject);
+    }
+  else
+    {
+      return NO;
+    }
+}
+
+- (BOOL) isEqualToString: (NSString*)aString
+{
+  Class	c;
+
+  if (aString == self)
+    {
+      return YES;
+    }
+  if (aString == nil)
+    {
+      return NO;
+    }
+  c = fastClassOfInstance(aString);
+  if (c == _fastCls._NSGCString || c == _fastCls._NSGMutableCString
+    || c == _fastCls._NXConstantString)
+    {
+      NXConstantString	*other = (NXConstantString*)aString;
+
+      if (_count != other->_count)
+	return NO;
+      if (memcmp(_contents_chars, other->_contents_chars, _count) != 0)
+	return NO;
+      return YES;
+    }
+  else if (c == _fastCls._NSGString || c == _fastCls._NSGMutableString)
+    {
+      if (strCompCsUs(self, aString, 0, (NSRange){0,_count}) == NSOrderedSame)
+	return YES;
+      return NO;
+    }
+  else if (c == nil)
+    {
+      return NO;
+    }
+  else if (fastClassIsKindOfClass(c, _fastCls._NSString))
+    {
+      return _fastImp._NSString_isEqualToString_(self,
+	@selector(isEqualToString:), aString);
+    }
+  else
+    {
+      return NO;
+    }
+}
+
+- (id) mutableCopy
+{
+  NSGMutableCString	*obj;
+
+  obj = (NSGMutableCString*)NSAllocateObject(_fastCls._NSGMutableCString,
+		0, NSDefaultMallocZone());
+  if (obj)
+    {
+      obj = (*msInitImp)(obj, msInitSel, _count);
+      if (obj)
+	{
+	  NXConstantString	*tmp = (NXConstantString*)obj;
+
+	  memcpy(tmp->_contents_chars, _contents_chars, _count);
+	  tmp->_count = _count;
+	  tmp->_hash = 0;
+	}
+    }
+  return obj;
+}
+
+- (id) mutableCopyWithZone: (NSZone*)z
+{
+  NSGMutableCString	*obj;
+
+  obj = (NSGMutableCString*)NSAllocateObject(_fastCls._NSGMutableCString, 0, z);
+  if (obj)
+    {
+      obj = (*msInitImp)(obj, msInitSel, _count);
+      if (obj)
+	{
+	  NXConstantString	*tmp = (NXConstantString*)obj;
+
+	  memcpy(tmp->_contents_chars, _contents_chars, _count);
+	  tmp->_count = _count;
+	  tmp->_hash = 0;		// No hash available yet.
+	}
+    }
+  return obj;
+}
 @end
