@@ -135,8 +135,8 @@ static void obsFree(Observation *o);
 #include <base/GSIArray.h>
 
 #define GSI_MAP_RETAIN_KEY(X)  
-#define GSI_MAP_RELEASE_KEY(X) ((((gsaddr)X.obj) & 1) == 0 \
-  ? RELEASE(X.obj) : X.obj)
+#define GSI_MAP_RELEASE_KEY(X) ({if ((((gsaddr)X.obj) & 1) == 0) \
+  RELEASE(X.obj);})
 #define GSI_MAP_HASH(X)        doHash(X.obj)
 #define GSI_MAP_EQUAL(X,Y)     doEqual(X.obj, Y.obj)
 
@@ -192,7 +192,7 @@ typedef struct NCTbl {
   short		cacheIndex;
 } NCTable;
 
-#define	TABLE		((NCTable*)table)
+#define	TABLE		((NCTable*)_table)
 #define	WILDCARD	(TABLE->wildcard)
 #define	NAMELESS	(TABLE->nameless)
 #define	NAMED		(TABLE->named)
@@ -910,7 +910,7 @@ static NSNotificationCenter *default_center = nil;
   /*
    * If this is a recursive posting of a notification, the array will already
    * be in use, so we restrict our operation to array indices beyond the end
-   * of those used by the posting that cuased this one.
+   * of those used by the posting that caused this one.
    */
   arrayBase = GSIArrayCount(a);
 
@@ -1012,21 +1012,18 @@ static NSNotificationCenter *default_center = nil;
 	   * Post the notification to all the observers that specified neither
 	   * NAME nor OBJECT.
 	   */
-	  if (o != ENDOBS)
+	  for (o = WILDCARD; o != ENDOBS; o = o->next)
 	    {
-	      for (o = WILDCARD; o != ENDOBS; o = o->next)
-		{
-		  GSIArrayAddItem(a, (GSIArrayItem)o);
-		}
-	      count = GSIArrayCount(a);
-	      while (count-- > arrayBase)
-		{
-		  o = GSIArrayItemAtIndex(a, count).ext;
-		  if (o->next != 0) 
-		    (*o->method)(o->observer, o->selector, notification);
-		}
-	      GSIArrayRemoveItemsFromIndex(a, arrayBase);
+	      GSIArrayAddItem(a, (GSIArrayItem)o);
 	    }
+	  count = GSIArrayCount(a);
+	  while (count-- > arrayBase)
+	    {
+	      o = GSIArrayItemAtIndex(a, count).ext;
+	      if (o->next != 0) 
+		(*o->method)(o->observer, o->selector, notification);
+	    }
+	  GSIArrayRemoveItemsFromIndex(a, arrayBase);
 
 	  /*
 	   * Post the notification to all the observers that specified OBJECT,
@@ -1056,7 +1053,7 @@ static NSNotificationCenter *default_center = nil;
 
 	  /*
 	   * Post the notification to all the observers of NAME, except those
-	   * observers with a non-nill OBJECT that doesn't match the
+	   * observers with a non-nil OBJECT that doesn't match the
 	   * notification's OBJECT).
 	   */
 	  if (n_name)
