@@ -18,7 +18,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */ 
 
 #include <config.h>
@@ -38,10 +38,15 @@
 #include <Foundation/NSPortCoder.h>
 #include <Foundation/NSDistantObject.h>
 #include <Foundation/NSZone.h>
+#include <Foundation/NSDebug.h>
+#include <Foundation/NSThread.h>
+#include <Foundation/NSNotification.h>
 #include <limits.h>
 
 #include <base/fast.x>
 
+extern BOOL __objc_responds_to(id, SEL);
+
 fastCls	_fastCls;	/* Structure to cache classes.	*/
 fastImp	_fastImp;	/* Structure to cache methods.	*/
 
@@ -100,6 +105,12 @@ void	_fastBuildCache()
  *	correct zone to free memory very fast.
  */
 
+/*
+ * retain_counts_gate is needed when running multi-threaded for retain/release
+ * to work reliably.
+ */
+static objc_mutex_t retain_counts_gate = NULL;
+
 #if	GS_WITH_GC == 0
 #define	REFCNT_LOCAL	1
 #define	CACHE_ZONE	1
@@ -148,6 +159,11 @@ typedef	struct obj_layout *obj;
 
 #if	GS_WITH_GC
 
+unsigned
+NSExtraRefCount(id anObject)
+{
+  return 0;
+}
 void
 NSIncrementExtraRefCount(id anObject)
 {
@@ -167,49 +183,103 @@ NSDecrementExtraRefCountWasZero(id anObject)
  *	depending on whether we are using local or global counting.
  */
 #if	defined(REFCNT_LOCAL)
+
+unsigned
+NSExtraRefCount(id anObject)
+{
+  return ((obj)anObject)[-1].retained;
+}
+
 void
 NSIncrementExtraRefCount(id anObject)
 {
-  ((obj)anObject)[-1].retained++;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      ((obj)anObject)[-1].retained++;
+      objc_mutex_unlock (retain_counts_gate);
+    }
+  else
+    {
+      ((obj)anObject)[-1].retained++;
+    }
 }
 
-#define	NSIncrementExtraRefCount(X) \
-	((obj)(X))[-1].retained++
+#define	NSIncrementExtraRefCount(X) ({ \
+  if (retain_counts_gate != 0) \
+    { \
+      objc_mutex_lock(retain_counts_gate); \
+      ((obj)(X))[-1].retained++;            \
+      objc_mutex_unlock(retain_counts_gate); \
+    } \
+  else \
+    { \
+      ((obj)X)[-1].retained++; \
+    } \
+})
 
 BOOL
 NSDecrementExtraRefCountWasZero(id anObject)
 {
-  if (((obj)anObject)[-1].retained-- == 0)
-    return YES;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      if (((obj)anObject)[-1].retained-- == 0)
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return YES;
+	}
+      else
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return NO;
+	}
+    }
   else
-    return NO;
+    {
+      if (((obj)anObject)[-1].retained-- == 0)
+	{
+	  return YES;
+	}
+      else
+	{
+	  return NO;
+	}
+    }
 }
 
-#define	NSDecrementExtraRefCountWasZero(X) \
-	(((obj)(X))[-1].retained-- == 0 ? YES : NO)
 
-#define	extraRefCount(X)	(((obj)(X))[-1].retained)
+#define	NSExtraRefCount(X)	(((obj)(X))[-1].retained)
 
 #else
 
 /* The maptable of retain counts on objects */
 static o_map_t *retain_counts = NULL;
-/* The mutex lock to protect multi-threaded use of `retain_counts' */
-static objc_mutex_t retain_counts_gate = NULL;
 
 void
 NSIncrementExtraRefCount (id anObject)
 {
   o_map_node_t *node;
-  extern o_map_node_t *o_map_node_for_key (o_map_t *m, const void *k);
+  extern o_map_node_t *o_map_node_for_key(o_map_t *m, const void *k);
 
-  objc_mutex_lock (retain_counts_gate);
-  node = o_map_node_for_key (retain_counts, anObject);
-  if (node)
-    ((int)(node->value))++;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (node)
+	((int)(node->value))++;
+      else
+	o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
+      objc_mutex_unlock(retain_counts_gate);
+    }
   else
-    o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
-  objc_mutex_unlock (retain_counts_gate);
+    {
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (node)
+	((int)(node->value))++;
+      else
+	o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
+    }
 }
 
 BOOL
@@ -219,30 +289,63 @@ NSDecrementExtraRefCountWasZero (id anObject)
   extern o_map_node_t *o_map_node_for_key (o_map_t *m, const void *k);
   extern void o_map_remove_node (o_map_node_t *node);
 
-  objc_mutex_lock (retain_counts_gate);
-  node = o_map_node_for_key (retain_counts, anObject);
-  if (!node)
+  if (retain_counts_gate != 0)
     {
-      objc_mutex_unlock (retain_counts_gate);
-      return YES;
+      objc_mutex_lock(retain_counts_gate);
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (!node)
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return YES;
+	}
+      NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
+      if (!--((int)(node->value)))
+	{
+	  o_map_remove_node (node);
+	}
+      objc_mutex_unlock(retain_counts_gate);
+    }
+  else
+    {
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (!node)
+	{
+	  return YES;
+	}
+      NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
+      if (!--((int)(node->value)))
+	{
+	  o_map_remove_node (node);
+	}
     }
-  NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
-  if (!--((int)(node->value)))
-    o_map_remove_node (node);
-  objc_mutex_unlock (retain_counts_gate);
   return NO;
 }
 
-static unsigned
-extraRefCount (id anObject)
+unsigned
+NSExtraRefCount (id anObject)
 {
   unsigned ret;
 
-  objc_mutex_lock (retain_counts_gate);
-  ret = (unsigned) o_map_value_at_key (retain_counts, anObject);
-  if (ret == (unsigned)o_map_not_a_key_marker(retain_counts) ||
-      ret == (unsigned)o_map_not_a_value_marker(retain_counts)) ret = 0;
-  objc_mutex_unlock (retain_counts_gate);
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      ret = (unsigned) o_map_value_at_key(retain_counts, anObject);
+      if (ret == (unsigned)o_map_not_a_key_marker(retain_counts)
+	|| ret == (unsigned)o_map_not_a_value_marker(retain_counts))
+	{
+	  ret = 0;
+	}
+      objc_mutex_unlock(retain_counts_gate);
+    }
+  else
+    {
+      ret = (unsigned) o_map_value_at_key(retain_counts, anObject);
+      if (ret == (unsigned)o_map_not_a_key_marker(retain_counts)
+	|| ret == (unsigned)o_map_not_a_value_marker(retain_counts))
+	{
+	  ret = 0;
+	}
+    }
   return ret;	/* ExtraRefCount + 1	*/
 }
 
@@ -258,6 +361,9 @@ extraRefCount (id anObject)
  */
 #if	GS_WITH_GC
 
+#include	<gc.h>
+#include	<gc_typed.h>
+
 inline NSZone *
 fastZone(NSObject *object)
 {
@@ -268,42 +374,55 @@ static void
 GSFinalize(void* object, void* data)
 {
   [(id)object gcFinalize];
-
-  /* Set the class of anObject to FREED_OBJECT. The further messages to this
-     object will cause an error to occur. */
-  ((id)object)->class_pointer = __freedObjectClass;
-
 #ifndef	NDEBUG
   GSDebugAllocationRemove(((id)object)->class_pointer);
 #endif
-  ((id)object)->class_pointer = (void*) 0xdeadface;
+  ((id)object)->class_pointer = (void*)0xdeadface;
 }
 
 inline NSObject *
-NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
+NSAllocateObject(Class aClass, unsigned extraBytes, NSZone *zone)
 {
-  id new = nil;
-  int size = aClass->instance_size + extraBytes;
-  if (CLS_ISCLASS (aClass))
-    new = NSZoneMalloc (zone, size);
-  if (new != nil)
+  id	new = nil;
+  int	size = aClass->instance_size + extraBytes;
+
+  NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
+  if (zone == GSAtomicMallocZone())
     {
-      memset (new, 0, size);
-      new->class_pointer = aClass;
+      new = NSZoneMalloc(zone, size);
     }
-  if ([new respondsToSelector: @selector(gcFinalize)])
+  else
     {
-#ifndef	NDEBUG
-      /*
-       *	We only do allocation counting for objects that can be
-       *	finalised - for other objects we have no way of decrementing
-       *	the count when the object is collected.
-       */
-      GSDebugAllocationAdd(aClass);
-#endif
-      GC_REGISTER_FINALIZER (new, GSFinalize, NULL, NULL, NULL);
+      GC_descr	gc_type = (GC_descr)aClass->gc_object_type;
+
+      if (gc_type == 0)
+	{
+	  new = NSZoneMalloc(zone, size);
+	  NSLog(@"No garbage collection information for '%s'", aClass->name);
+	}
+      else
+	{
+	  new = GC_CALLOC_EXPLICTLY_TYPED(1, size, gc_type);
+        }
     }
 
+  if (new != nil)
+    {
+      memset(new, 0, size);
+      new->class_pointer = aClass;
+      if (__objc_responds_to(new, @selector(gcFinalize)))
+	{
+#ifndef	NDEBUG
+	  /*
+	   *	We only do allocation counting for objects that can be
+	   *	finalised - for other objects we have no way of decrementing
+	   *	the count when the object is collected.
+	   */
+	  GSDebugAllocationAdd(aClass);
+#endif
+	  GC_REGISTER_FINALIZER (new, GSFinalize, NULL, NULL, NULL);
+	}
+    }
   return new;
 }
 
@@ -341,6 +460,9 @@ fastZone(NSObject *object)
 inline NSObject *
 NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
 {
+#ifndef	NDEBUG
+  extern void GSDebugAllocationAdd(Class);
+#endif
   id new = nil;
   int size = aClass->instance_size + extraBytes + sizeof(struct obj_layout);
   if (CLS_ISCLASS (aClass))
@@ -367,6 +489,9 @@ NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
 inline void
 NSDeallocateObject(NSObject *anObject)
 {
+#ifndef	NDEBUG
+  extern void GSDebugAllocationRemove(Class);
+#endif
   if ((anObject!=nil) && CLS_ISCLASS(((id)anObject)->class_pointer))
     {
       obj	o = &((obj)anObject)[-1];
@@ -455,25 +580,53 @@ static IMP autorelease_imp = 0;
    This does not need mutex protection. */
 static BOOL double_release_check_enabled = NO;
 
+
 
 @implementation NSObject
 
++ (void) _becomeMultiThreaded: (NSNotification)aNotification
+{
+  if (retain_counts_gate == 0)
+    {
+      retain_counts_gate = objc_mutex_allocate();
+    }
+}
+
 + (void) initialize
 {
   if (self == [NSObject class])
     {
+#ifdef __FreeBSD__
+      // Manipulate the FPU to add the exception mask. (Fixes SIGFPE
+      // problems on *BSD)
+
+      volatile short cw;
+
+      __asm__ volatile ("fstcw (%0)" : : "g" (&cw));
+      cw |= 1; /* Mask 'invalid' exception */
+      __asm__ volatile ("fldcw (%0)" : : "g" (&cw));
+#endif
+
       // Create the global lock
       gnustep_global_lock = [[NSRecursiveLock alloc] init];
-#if !defined(REFCNT_LOCAL)
-      retain_counts = o_map_with_callbacks (o_callbacks_for_non_owned_void_p,
-					    o_callbacks_for_int);
-      retain_counts_gate = objc_mutex_allocate ();
-#endif
       autorelease_class = [NSAutoreleasePool class];
       autorelease_imp = [autorelease_class methodForSelector: autorelease_sel];
       fastMallocClass = [_FastMallocBuffer class];
+#if	GS_WITH_GC == 0
+#if	!defined(REFCNT_LOCAL)
+      retain_counts = o_map_with_callbacks (o_callbacks_for_non_owned_void_p,
+					    o_callbacks_for_int);
+#endif
       fastMallocOffset = fastMallocClass->instance_size % ALIGN;
+#else
+      fastMallocOffset = 0;
+#endif
       _fastBuildCache();
+      [[NSNotificationCenter defaultCenter]
+	addObserver: self
+	   selector: @selector(_becomeMultiThreaded:)
+	       name: NSWillBecomeMultiThreadedNotification
+	     object: nil];
     }
   return;
 }
@@ -508,10 +661,27 @@ static BOOL double_release_check_enabled = NO;
   NSDeallocateObject (self);
 }
 
+static BOOL deallocNotifications = NO;
+
+- (BOOL) deallocNotificationsActive
+{
+  return deallocNotifications;
+}
+
+- (void) setDeallocNotificationsActive: (BOOL)flag
+{
+  deallocNotifications = flag;
+}
+
+- (BOOL) _dealloc
+{
+  return YES;
+}
+
 - free
 {
   [NSException raise: NSGenericException
-	       format: @"Use `dealloc' instead of `free'."];
+	       format: @"Use `dealloc' instead of `free' for %@.", self];
   return nil;
 }
 
@@ -609,13 +779,13 @@ static BOOL double_release_check_enabled = NO;
 
 - (NSString*) description
 {
-  return [NSString stringWithFormat: @"<%s %lx>",
+  return [NSString stringWithFormat: @"<%s: %lx>",
 	object_get_class_name(self), (unsigned long)self];
 }
 
 + (NSString*) description
 {
-  return [NSString stringWithFormat: @"<%s>", object_get_class_name(self)];
+  return [NSString stringWithCString: object_get_class_name(self)];
 }
 
 - (void) descriptionTo: (id<GNUDescriptionDestination>)output
@@ -716,7 +886,7 @@ static BOOL double_release_check_enabled = NO;
 
 /* NSObject protocol */
 
-- autorelease
+- (id) autorelease
 {
 #if	GS_WITH_GC == 0
   if (double_release_check_enabled)
@@ -763,15 +933,8 @@ static BOOL double_release_check_enabled = NO;
 
 + (BOOL) isKindOfClass: (Class)aClass
 {
-  Class class;
-
-  for (class = self; 
-       class != Nil;
-       class = class_get_super_class (class))
-    {
-      if (class == aClass)
-	return YES;
-    }
+  if (aClass == [NSObject class])
+    return YES;
   return NO;
 }
 
@@ -883,7 +1046,12 @@ static BOOL double_release_check_enabled = NO;
     }
 
   if (NSDecrementExtraRefCountWasZero(self))
-    [self dealloc];
+    {
+      if (deallocNotifications == NO || [self _dealloc] == YES)
+	{
+	  [self dealloc];
+	}
+    }
 #endif
 }
 
@@ -922,7 +1090,7 @@ static BOOL double_release_check_enabled = NO;
 #if	GS_WITH_GC
   return UINT_MAX;
 #else
-  return extraRefCount(self) + 1;
+  return NSExtraRefCount(self) + 1;
 #endif
 }
 
@@ -1059,7 +1227,7 @@ static BOOL double_release_check_enabled = NO;
 {
   [NSException
     raise: NSGenericException
-    format: @"method %s not implemented", sel_get_name(aSel)];
+    format: @"method %s not implemented in %s", sel_get_name(aSel), object_get_class_name(self)];
   return nil;
 }
 
@@ -1170,7 +1338,7 @@ static BOOL double_release_check_enabled = NO;
 {
   [NSException
     raise: NSGenericException
-    format: @"subclass should override %s", sel_get_name(aSel)];
+    format: @"subclass %s should override %s", object_get_class_name(self), sel_get_name(aSel)];
   return nil;
 }
 
