@@ -33,9 +33,9 @@
    2038 problem.)
 
    The local time zone can be specified with the user defaults
-   database, the TZ environment variable, the file LOCAL_TIME_FILE, or
-   the fallback time zone (which is UTC), with the ones listed first
-   having precedence.
+   database, the GNUSTEP_TZ environment variable, the file LOCAL_TIME_FILE,
+   the TZ environment variable, or the fallback time zone (which is UTC),
+   with the ones listed first having precedence.
 
    Any time zone must be a file name in ZONES_DIR.
 
@@ -424,7 +424,7 @@ static NSMapTable	*absolutes = 0;
   z = (NSConcreteAbsoluteTimeZone*)NSMapGet(absolutes, (void*)(gsaddr)anOffset);
   if (z)
     {
-      RETAIN(z);
+      IF_NO_GC(RETAIN(z));
       RELEASE(self);
     }
   else
@@ -571,47 +571,67 @@ static NSMapTable	*absolutes = 0;
       zoneDictionary = [[NSMutableDictionary alloc] init];
 
       localZoneString = [[NSUserDefaults standardUserDefaults]
-			  stringForKey: LOCALDBKEY];
+	stringForKey: LOCALDBKEY];
       if (localZoneString == nil)
-        /* Try to get timezone from environment. */
-	localZoneString = [[[NSProcessInfo processInfo]
-			     environment] objectForKey: @"TZ"];
+	{
+	  /*
+	   * Try to get timezone from GNUSTEP_TZ environment variable.
+	   */
+	  localZoneString = [[[NSProcessInfo processInfo]
+	    environment] objectForKey: @"GNUSTEP_TZ"];
+	}
       if (localZoneString == nil)
-       /* Try to get timezone from LOCAL_TIME_FILE. */
-       {
-	 NSString *f = [NSTimeZone getLocalTimeFile];
-         char zone_name[80];
-         FILE *fp;
+	{
+	  /*
+	   * Try to get timezone from LOCAL_TIME_FILE.
+	   */
+	  NSString	*f = [NSTimeZone getLocalTimeFile];
+	  char		zone_name[80];
+	  FILE		*fp;
 
-	 if (f)
-	   {
-#if	defined(__WIN32__)
-	     fp = fopen([f fileSystemRepresentation], "rb");
-#else
-	     fp = fopen([f fileSystemRepresentation], "r");
-#endif
-	     if (fp != NULL)
-	       {
-                 if (fscanf(fp, "%79s", zone_name) == 1)
-                  localZoneString = [NSString stringWithCString: zone_name];
-		 fclose(fp);
-	       }
-	   }
-       }
+	  if (f != 0)
+	    {
+ #if	defined(__WIN32__)
+	      fp = fopen([f fileSystemRepresentation], "rb");
+ #else
+	      fp = fopen([f fileSystemRepresentation], "r");
+ #endif
+	      if (fp != NULL)
+		{
+		  if (fscanf(fp, "%79s", zone_name) == 1)
+		   localZoneString = [NSString stringWithCString: zone_name];
+		  fclose(fp);
+		}
+	    }
+	}
+      if (localZoneString == nil)
+	{
+	  /*
+	   * Try to get timezone from standard unix environment variable.
+	   */
+	  localZoneString = [[[NSProcessInfo processInfo]
+	    environment] objectForKey: @"TZ"];
+	}
       if (localZoneString != nil)
-	localTimeZone = [NSTimeZone timeZoneWithName: localZoneString];
+	{
+	  localTimeZone = [NSTimeZone timeZoneWithName: localZoneString];
+	}
       else
-        NSLog(@"No local time zone specified.");
+	{
+	  NSLog(@"No local time zone specified.");
+	}
 
-      /* If local time zone fails to allocate, then allocate something
-         that is sure to succeed (unless we run out of memory, of
-         course). */
+      /*
+       * If local time zone fails to allocate, then allocate something
+       * that is sure to succeed (unless we run out of memory, of
+       * course).
+       */
       if (localTimeZone == nil)
         {
           NSLog(@"Using time zone with absolute offset 0.");
           localTimeZone = [NSTimeZone timeZoneForSecondsFromGMT: 0];
         }
-      RETAIN(localTimeZone);
+      IF_NO_GC(RETAIN(localTimeZone));
 
       fake_abbrev_dict = [[NSInternalAbbrevDict alloc] init];
     }
@@ -755,11 +775,13 @@ static NSMapTable	*absolutes = 0;
       if (fread(zone_abbrevs, 1, names_size, file) != names_size)
 	[NSException raise: fileException format: errMess];
       abbrevsArray = NSZoneMalloc(NSDefaultMallocZone(), sizeof(id)*names_size);
-      i = 0;
-      while (i < names_size)
+      memset(abbrevsArray, '\0', sizeof(id)*names_size);
+      for (i = 0; i < n_types; i++)
 	{
-	  abbrevsArray[i] = [NSString stringWithCString: zone_abbrevs+i];
-	  i = (strchr(zone_abbrevs+i, '\0')-zone_abbrevs)+1;
+	  int	pos = types[i].abbr_idx;
+
+	  if (abbrevsArray[pos] == nil)
+	    abbrevsArray[pos] = [NSString stringWithCString: zone_abbrevs+pos];
 	}
       NSZoneFree(NSDefaultMallocZone(), zone_abbrevs);
 
@@ -972,12 +994,11 @@ static NSMapTable	*absolutes = 0;
 
 + (NSString*) getTimeZoneFile: (NSString *)name
 {
-  NSString *fileName = [NSString stringWithFormat: @"%@%@",
-				 ZONES_DIR, name];
-
-  return [NSBundle pathForGNUstepResource: fileName
-		   ofType: @""
-		   inDirectory: TIME_ZONE_DIR];
+  NSString *fileName = [NSString stringWithFormat: @"%@%@", ZONES_DIR, name];
+  NSString *path = [NSBundle pathForGNUstepResource: fileName
+					     ofType: @""
+				        inDirectory: TIME_ZONE_DIR];
+  return path;
 }
 
 @end
