@@ -18,7 +18,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 
 #include <config.h>
@@ -30,10 +30,11 @@
 /*
  *	Setup for inline operation of arrays.
  */
-#define	FAST_ARRAY_RETAIN(X)	X
-#define	FAST_ARRAY_RELEASE(X)	
+#define	GSI_ARRAY_RETAIN(X)	
+#define	GSI_ARRAY_RELEASE(X)	
+#define	GSI_ARRAY_TYPES	GSUNION_OBJ|GSUNION_SEL|GSUNION_STR
 
-#include "FastArray.x"
+#include <base/GSIArray.h>
 
 #define	_IN_NSUNARCHIVER_M
 #include <Foundation/NSArchiver.h>
@@ -63,10 +64,8 @@ typeToName1(char type)
       case _C_UINT:	return "unsigned int";
       case _C_LNG:	return "long";
       case _C_ULNG:	return "unsigned long";
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:	return "long long";
       case _C_ULNG_LNG:	return "unsigned long long";
-#endif
       case _C_FLT:	return "float";
       case _C_DBL:	return "double";
       case _C_PTR:	return "pointer";
@@ -109,10 +108,8 @@ typeToName2(char type)
       case _GSC_UINT:	return "unsigned int";
       case _GSC_LNG:	return "long";
       case _GSC_ULNG:	return "unsigned long";
-#ifdef	_GSC_LNG_LNG
       case _GSC_LNG_LNG:	return "long long";
       case _GSC_ULNG_LNG:	return "unsigned long long";
-#endif
       case _GSC_FLT:	return "float";
       case _GSC_DBL:	return "double";
       case _GSC_PTR:	return "pointer";
@@ -226,21 +223,16 @@ static SEL dValSel = @selector(decodeValueOfObjCType:at:);
 }
 - (void) dealloc
 {
-  [original release];
+  RELEASE(original);
   if (name)
     {
-      [name release];
+      RELEASE(name);
     }
   NSDeallocateObject(self);
 }
 - (void) mapToClass: (Class)c withName: (NSString*)n
 {
-  if (n != name)
-    {
-      [n retain];
-      [name release];
-      name = n;
-    }
+  ASSIGN(name, n);
   class = c;
 }
 @end
@@ -320,11 +312,11 @@ mapClassName(NSUnarchiverObjectInfo *info)
     }
   NS_HANDLER
     {
-      [unarchiver release];
+      RELEASE(unarchiver);
       [localException raise];
     }
   NS_ENDHANDLER
-  [unarchiver release];
+  RELEASE(unarchiver);
 
   return obj;
 }
@@ -342,15 +334,15 @@ mapClassName(NSUnarchiverObjectInfo *info)
 
 - (void) dealloc
 {
-  [data release];
-  [objDict release];
+  RELEASE(data);
+  RELEASE(objDict);
   if (clsMap)
     {
       NSZone	*z = clsMap->zone;
 
-      FastArrayClear(clsMap);
-      FastArrayClear(objMap);
-      FastArrayClear(ptrMap);
+      GSIArrayClear(clsMap);
+      GSIArrayClear(objMap);
+      GSIArrayClear(ptrMap);
       NSZoneFree(z, (void*)clsMap);
     }
   [super dealloc];
@@ -389,7 +381,16 @@ mapClassName(NSUnarchiverObjectInfo *info)
       objDict = [[NSMutableDictionary allocWithZone: zone]
 			initWithCapacity: 200];
 
-      [self resetUnarchiverWithData: anObject atIndex: 0];
+      NS_DURING
+	{
+	  [self resetUnarchiverWithData: anObject atIndex: 0];
+	}
+      NS_HANDLER
+	{
+	  RELEASE(self);
+	  [localException raise];
+	}
+      NS_ENDHANDLER
     }
   return self;
 }
@@ -501,26 +502,26 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	    {
 	      if (info & _GSC_XREF)
 		{
-		  if (xref >= FastArrayCount(objMap))
+		  if (xref >= GSIArrayCount(objMap))
 		    {
 		      [NSException raise: NSInternalInconsistencyException
 				  format: @"object crossref missing - %d",
 					xref];
 		    }
-		  obj = FastArrayItemAtIndex(objMap, xref).o;
+		  obj = GSIArrayItemAtIndex(objMap, xref).obj;
 		  /*
 		   *	If it's a cross-reference, we need to retain it in
 		   *	order to give the appearance that it's actually a
 		   *	new object.
 		   */
-		  [obj retain];
+		  IF_NO_GC(RETAIN(obj));
 		}
 	      else
 		{
 		  Class	c;
 		  id	rep;
 
-		  if (xref != FastArrayCount(objMap))
+		  if (xref != GSIArrayCount(objMap))
 		    {
 		      [NSException raise: NSInternalInconsistencyException
 				  format: @"extra object crossref - %d",
@@ -529,20 +530,20 @@ mapClassName(NSUnarchiverObjectInfo *info)
 		  (*dValImp)(self, dValSel, @encode(Class), &c);
 
 		  obj = [c allocWithZone: zone];
-		  FastArrayAddItem(objMap, (FastArrayItem)obj);
+		  GSIArrayAddItem(objMap, (GSIArrayItem)obj);
 
 		  rep = [obj initWithCoder: self];
 		  if (rep != obj)
 		    {
 		      obj = rep;
-		      FastArraySetItemAtIndex(objMap, (FastArrayItem)obj, xref);
+		      GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
 		    }
 
 		  rep = [obj awakeAfterUsingCoder: self];
 		  if (rep != obj)
 		    {
 		      obj = rep;
-		      FastArraySetItemAtIndex(objMap, (FastArrayItem)obj, xref);
+		      GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
 		    }
 		}
 	    }
@@ -567,12 +568,12 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	    }
 	  if (info & _GSC_XREF)
 	    {
-	      if (xref >= FastArrayCount(clsMap))
+	      if (xref >= GSIArrayCount(clsMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"class crossref missing - %d", xref];
 		}
-	      classInfo = (NSUnarchiverObjectInfo*)FastArrayItemAtIndex(clsMap, xref).o;
+	      classInfo = (NSUnarchiverObjectInfo*)GSIArrayItemAtIndex(clsMap, xref).obj;
 	      *(Class*)address = mapClassObject(classInfo);
 	      return;
 	    }
@@ -581,7 +582,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	      unsigned	cver;
 	      NSString	*className;
 
-	      if (xref != FastArrayCount(clsMap))
+	      if (xref != GSIArrayCount(clsMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 				format: @"extra class crossref - %d", xref];
@@ -600,10 +601,10 @@ mapClassName(NSUnarchiverObjectInfo *info)
 		  classInfo = [NSUnarchiverObjectInfo newWithName: className];
 		  [classInfo mapToClass: c withName: className];
 		  [objDict setObject: classInfo forKey: className];
-		  [classInfo release];
+		  RELEASE(classInfo);
 		}
 	      classInfo->version = cver;
-	      FastArrayAddItem(clsMap, (FastArrayItem)classInfo);
+	      GSIArrayAddItem(clsMap, (GSIArrayItem)classInfo);
 	      *(Class*)address = mapClassObject(classInfo);
 	      /*
 	       *	Point the address to a dummy location and read the
@@ -635,22 +636,22 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	    }
 	  if (info & _GSC_XREF)
 	    {
-	      if (xref >= FastArrayCount(ptrMap))
+	      if (xref >= GSIArrayCount(ptrMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"sel crossref missing - %d", xref];
 		}
-	      sel = FastArrayItemAtIndex(ptrMap, xref).C;
+	      sel = GSIArrayItemAtIndex(ptrMap, xref).sel;
 	    }
 	  else
 	    {
-	      if (xref != FastArrayCount(ptrMap))
+	      if (xref != GSIArrayCount(ptrMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"extra sel crossref - %d", xref];
 		}
 	      (*desImp)(src, desSel, &sel, @encode(SEL), &cursor, nil);
-	      FastArrayAddItem(ptrMap, (FastArrayItem)sel);
+	      GSIArrayAddItem(ptrMap, (GSIArrayItem)sel);
 	    }
 	  *(SEL*)address = sel;
 	  return;
@@ -712,19 +713,18 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	    }
 	  if (info & _GSC_XREF)
 	    {
-	      if (xref >= FastArrayCount(ptrMap))
+	      if (xref >= GSIArrayCount(ptrMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"ptr crossref missing - %d", xref];
 		}
-	      *(void**)address = FastArrayItemAtIndex(ptrMap, xref).p;
+	      *(void**)address = GSIArrayItemAtIndex(ptrMap, xref).ptr;
 	    }
 	  else
 	    {
 	      unsigned	size;
-	      NSData	*dat;
 
-	      if (FastArrayCount(ptrMap) != xref)
+	      if (GSIArrayCount(ptrMap) != xref)
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"extra ptr crossref - %d", xref];
@@ -736,7 +736,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	       */
 	      size = objc_sizeof_type(++type);
 	      *(void**)address = _fastMallocBuffer(size);
-	      FastArrayAddItem(ptrMap, (FastArrayItem)*(void**)address);
+	      GSIArrayAddItem(ptrMap, (GSIArrayItem)*(void**)address);
 
 	      /*
 	       *	Decode value and add memory to map for crossrefs.
@@ -748,8 +748,6 @@ mapClassName(NSUnarchiverObjectInfo *info)
 
       case _GSC_CHARPTR:
 	{
-	  char		*str;
-
 	  typeCheck(*type, _GSC_CHARPTR);
 	  /*
 	   *	Special case - a zero crossref value size is a nil pointer.
@@ -761,24 +759,22 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	    }
 	  if (info & _GSC_XREF)
 	    {
-	      if (xref >= FastArrayCount(ptrMap))
+	      if (xref >= GSIArrayCount(ptrMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"string crossref missing - %d", xref];
 		}
-	      *(char**)address = FastArrayItemAtIndex(ptrMap, xref).s;
+	      *(char**)address = GSIArrayItemAtIndex(ptrMap, xref).str;
 	    }
 	  else
 	    {
-	      int	length;
-
-	      if (xref != FastArrayCount(ptrMap))
+	      if (xref != GSIArrayCount(ptrMap))
 		{
 		  [NSException raise: NSInternalInconsistencyException
 			      format: @"extra string crossref - %d", xref];
 		}
 	      (*desImp)(src, desSel, address, @encode(char*), &cursor, nil);
-	      FastArrayAddItem(ptrMap, (FastArrayItem)*(void**)address);
+	      GSIArrayAddItem(ptrMap, (GSIArrayItem)*(void**)address);
 	    }
 	  return;
 	}
@@ -964,7 +960,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	  d = [[NSData allocWithZone: zone] initWithBytesNoCopy: b
 							 length: l
 						       fromZone: zone];
-	  [d autorelease];
+	  IF_NO_GC(AUTORELEASE(d));
 	  [self decodeArrayOfObjCType: @encode(unsigned char)
 				count: l
 				   at: b];
@@ -974,8 +970,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	[NSException raise: NSInternalInconsistencyException
 		    format: @"Decoding data object with unknown type"];
     }
-  else
-    return [NSData data];
+  return [NSData data];
 }
 
 /*
@@ -1007,16 +1002,16 @@ mapClassName(NSUnarchiverObjectInfo *info)
 
   if (info & _GSC_XREF)
     {
-      if (xref >= FastArrayCount(objMap))
+      if (xref >= GSIArrayCount(objMap))
 	{
 	  [NSException raise: NSInternalInconsistencyException
 		      format: @"object crossref missing - %d",
 			    xref];
 	}
-      obj = FastArrayItemAtIndex(objMap, xref).o;
+      obj = GSIArrayItemAtIndex(objMap, xref).obj;
       /*
        *	If it's a cross-reference, we don't need to autorelease it
-       *	since we don't own it.
+       *	since we didn't create it.
        */
       return obj;
     }
@@ -1025,7 +1020,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
       Class	c;
       id	rep;
 
-      if (xref != FastArrayCount(objMap))
+      if (xref != GSIArrayCount(objMap))
 	{
 	  [NSException raise: NSInternalInconsistencyException
 		      format: @"extra object crossref - %d",
@@ -1034,25 +1029,25 @@ mapClassName(NSUnarchiverObjectInfo *info)
       (*dValImp)(self, dValSel, @encode(Class), &c);
 
       obj = [c allocWithZone: zone];
-      FastArrayAddItem(objMap, (FastArrayItem)obj);
+      GSIArrayAddItem(objMap, (GSIArrayItem)obj);
 
       rep = [obj initWithCoder: self];
       if (rep != obj)
 	{
 	  obj = rep;
-	  FastArraySetItemAtIndex(objMap, (FastArrayItem)obj, xref);
+	  GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
 	}
 
       rep = [obj awakeAfterUsingCoder: self];
       if (rep != obj)
 	{
 	  obj = rep;
-	  FastArraySetItemAtIndex(objMap, (FastArrayItem)obj, xref);
+	  GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
 	}
       /*
        *	A newly allocated object needs to be autoreleased.
        */
-      return [obj autorelease];
+      return AUTORELEASE(obj);
     }
 }
 
@@ -1107,7 +1102,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	{
 	  info = [NSUnarchiverClassInfo newWithName: nameInArchive];
 	  [clsDict setObject: info forKey: nameInArchive];
-	  [info release];
+	  RELEASE(info);
 	}
       [info mapToClass: c withName: trueName];
     }
@@ -1145,7 +1140,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	{
 	  info = [NSUnarchiverObjectInfo newWithName: nameInArchive];
 	  [objDict setObject: info forKey: nameInArchive];
-	  [info release];
+	  RELEASE(info);
 	}
       [info mapToClass: c withName: trueName];
     }
@@ -1155,11 +1150,13 @@ mapClassName(NSUnarchiverObjectInfo *info)
 {
   unsigned i;
 
-  for (i = FastArrayCount(objMap) - 1; i > 0; i--)
+  if (replacement == anObject)
+    return;
+  for (i = GSIArrayCount(objMap) - 1; i > 0; i--)
     {
-      if (FastArrayItemAtIndex(objMap, i).o == anObject)
+      if (GSIArrayItemAtIndex(objMap, i).obj == anObject)
 	{
-	  FastArraySetItemAtIndex(objMap, (FastArrayItem)replacement, i);
+	  GSIArraySetItemAtIndex(objMap, (GSIArrayItem)replacement, i);
 	  return;
 	}
     }
@@ -1205,8 +1202,8 @@ mapClassName(NSUnarchiverObjectInfo *info)
     {
       Class	c;
 
-      [data release];
-      data = [anObject retain];
+      TEST_RELEASE(data);
+      data = RETAIN(anObject);
       c = fastClass(data);
       if (src != self)
 	{
@@ -1239,17 +1236,17 @@ mapClassName(NSUnarchiverObjectInfo *info)
       /*
        *	Allocate and initialise arrays to build crossref maps in.
        */
-      clsMap = NSZoneMalloc(zone, sizeof(FastArray_t)*3);
-      FastArrayInitWithZoneAndCapacity(clsMap, zone, sizeC);
-      FastArrayAddItem(clsMap, (FastArrayItem)0);
+      clsMap = NSZoneMalloc(zone, sizeof(GSIArray_t)*3);
+      GSIArrayInitWithZoneAndCapacity(clsMap, zone, sizeC);
+      GSIArrayAddItem(clsMap, (GSIArrayItem)0);
 
       objMap = &clsMap[1];
-      FastArrayInitWithZoneAndCapacity(objMap, zone, sizeO);
-      FastArrayAddItem(objMap, (FastArrayItem)0);
+      GSIArrayInitWithZoneAndCapacity(objMap, zone, sizeO);
+      GSIArrayAddItem(objMap, (GSIArrayItem)0);
 
       ptrMap = &clsMap[2];
-      FastArrayInitWithZoneAndCapacity(ptrMap, zone, sizeP);
-      FastArrayAddItem(ptrMap, (FastArrayItem)0);
+      GSIArrayInitWithZoneAndCapacity(ptrMap, zone, sizeP);
+      GSIArrayAddItem(ptrMap, (GSIArrayItem)0);
     }
   else
     {
@@ -1300,16 +1297,11 @@ mapClassName(NSUnarchiverObjectInfo *info)
 {
   if (name)
     {
-      (*dValImp)(self, dValSel, @encode(id), (void*)name);
+      *name = [self decodeObject];
     }
   else
     {
-      id	obj;
-      (*dValImp)(self, dValSel, @encode(id), (void*)&obj);
-      if (obj)
-	{
-	  [obj release];
-	}
+      (void)[self decodeObject];
     }
   [self decodeArrayOfObjCType: type count: count at: buf];
 }
@@ -1324,16 +1316,11 @@ mapClassName(NSUnarchiverObjectInfo *info)
 {
   if (name)
     {
-      (*dValImp)(self, dValSel, @encode(id), (void*)name);
+      *name = [self decodeObject];
     }
   else
     {
-      id	obj;
-      (*dValImp)(self, dValSel, @encode(id), (void*)&obj);
-      if (obj)
-	{
-	  [obj release];
-	}
+      (void)[self decodeObject];
     }
   (*dValImp)(self, dValSel, type, buf);
 }
@@ -1344,16 +1331,11 @@ mapClassName(NSUnarchiverObjectInfo *info)
 {
   if (name)
     {
-      (*dValImp)(self, dValSel, @encode(id), (void*)name);
+      *name = [self decodeObject];
     }
   else
     {
-      id	obj;
-      (*dValImp)(self, dValSel, @encode(id), (void*)&obj);
-      if (obj)
-	{
-	  [obj release];
-	}
+      (void)[self decodeObject];
     }
   (*dValImp)(self, dValSel, type, buf);
 }
@@ -1363,16 +1345,11 @@ mapClassName(NSUnarchiverObjectInfo *info)
 {
   if (name)
     {
-      (*dValImp)(self, dValSel, @encode(id), (void*)name);
+      *name = [self decodeObject];
     }
   else
     {
-      id	obj;
-      (*dValImp)(self, dValSel, @encode(id), (void*)&obj);
-      if (obj)
-	{
-	  [obj release];
-	}
+      (void)[self decodeObject];
     }
   (*dValImp)(self, dValSel, @encode(id), (void*)anObject);
 }
