@@ -55,18 +55,28 @@
 /* Wait for access */
 #define _MAX_COUNT 5          /* Max 10 sec. */
 
-/* User's Defauls database */
-static NSString* GNU_UserDefaultsDatabase = @"GNUstep/.GNUstepDefaults";
-static NSString* GNU_UserDefaultsDatabaseLock = @"GNUstep/.GNUstepUDLock";
+static SEL	nextObjectSel = @selector(nextObject);
+static SEL	objectForKeySel = @selector(objectForKey:);
+
+/* User's Defaults database */
+static NSString	*GNU_UserDefaultsPrefix = @"GNUstep";
+static NSString	*GNU_UserDefaultsDatabase = @".GNUstepDefaults";
+static NSString	*GNU_UserDefaultsDatabaseLock = @".GNUstepUDLock";
+
+static Class	NSArrayClass;
+static Class	NSDataClass;
+static Class	NSDictionaryClass;
+static Class	NSMutableDictionaryClass;
+static Class	NSStringClass;
 
 /*************************************************************************
  *** Local method definitions
  *************************************************************************/
 @interface NSUserDefaults (__local_NSUserDefaults)
-- (void)__createStandardSearchList;
-- (NSDictionary *)__createArgumentDictionary;
-- (void)__changePersistentDomain: (NSString *)domainName;
-- (void)__timerTicked: (NSTimer*)tim;
+- (void) __createStandardSearchList;
+- (NSDictionary*) __createArgumentDictionary;
+- (void) __changePersistentDomain: (NSString*)domainName;
+- (void) __timerTicked: (NSTimer*)tim;
 @end
 
 @implementation NSUserDefaults: NSObject
@@ -81,16 +91,28 @@ static NSMutableString   *processName = nil;
  *************************************************************************/
 static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 
-+ (void) resetUserDefaults
++ (void) initialize
 {
-  id	defs = sharedDefaults;
-
-  setSharedDefaults = NO;
-  sharedDefaults = nil;
-  RELEASE(defs);
+  if (self == [NSUserDefaults class])
+    {
+      /*
+       * Cache class info for more rapid testing of the types of defaults.
+       */
+      NSArrayClass = [NSArray class];
+      NSDataClass = [NSData class];
+      NSDictionaryClass = [NSDictionary class];
+      NSMutableDictionaryClass = [NSMutableDictionary class];
+      NSStringClass = [NSString class];
+    }
 }
 
-+ (NSUserDefaults *)standardUserDefaults
++ (void) resetUserDefaults
+{
+  setSharedDefaults = NO;
+  DESTROY(sharedDefaults);
+}
+
++ (NSUserDefaults*) standardUserDefaults
   /*
     Returns the shared defaults object. If it doesn't exist yet, it's
     created. The defaults are initialized for the current user.
@@ -216,22 +238,23 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 }
 
 
-+ (NSArray *)userLanguages
++ (NSArray*) userLanguages
 {
-  NSMutableArray *uL = [NSMutableArray arrayWithCapacity: 5];
-  NSArray *currLang = [[self standardUserDefaults] 
-			stringArrayForKey: @"Languages"];
-  NSEnumerator *enumerator;
-  id obj;
+  NSMutableArray	*uL = [NSMutableArray arrayWithCapacity: 5];
+  NSArray		*currLang = [[self standardUserDefaults] 
+			  stringArrayForKey: @"Languages"];
+  NSEnumerator		*enumerator;
+  id			obj;
 	
   if (!currLang)
     {                    // Try to build it from the env 
-      const char *env_list;
-      NSString *env;
+      const char	*env_list;
+      NSString		*env;
+
       env_list = getenv("LANGUAGES");
       if (env_list)
 	{
-	  env = [NSString stringWithCString: env_list];
+	  env = [NSStringClass stringWithCString: env_list];
 	  currLang = RETAIN([env componentsSeparatedByString: @";"]);
 	}
     }
@@ -250,10 +273,10 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
   return uL;
 }
 
-+ (void)setUserLanguages: (NSArray *)languages
++ (void) setUserLanguages: (NSArray*)languages
 {
-  NSMutableDictionary *globDict = [[self standardUserDefaults] 
-				    persistentDomainForName: NSGlobalDomain];
+  NSMutableDictionary	*globDict = [[self standardUserDefaults] 
+			    persistentDomainForName: NSGlobalDomain];
 	
   if (!languages)          // Remove the entry
     [globDict removeObjectForKey: @"Languages"];
@@ -267,72 +290,97 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 /*************************************************************************
  *** Initializing the User Defaults
  *************************************************************************/
-- (id)init
+- (id) init
   /* Initializes defaults for current user calling initWithUser: . */
 {
   return [self initWithUser: NSUserName()];
 }
 
-/* Initializes defaults for the specified user calling initWithFile: . */
-- (id)initWithUser: (NSString *)userName
+static NSString	*pathForUser(NSString *user) 
 {
-  NSString* userHome = NSHomeDirectoryForUser(userName);
-  NSString *filename;
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSString	*home;
+  NSString	*path;
+  BOOL		isDir;
 	
-  // Either userName is empty or it's wrong
-  if (!userHome)
+  home = NSHomeDirectoryForUser(user);
+  if (home == nil)
+    {  
+      return nil;
+    }
+  path = [home stringByAppendingPathComponent: GNU_UserDefaultsPrefix];
+  if ([mgr fileExistsAtPath: path isDirectory: &isDir] == NO)
+    {
+      NSLog(@"Directory '%'@ does not exist - creating it", path);
+      if ([mgr createDirectoryAtPath: path attributes: nil] == NO)
+	{
+	  NSLog(@"Unable to create user GNUstep directory '%@'", path);
+	  return nil;
+	}
+    }
+  if (isDir == NO)
+    {
+      NSLog(@"ERROR - '%@' is not a directory!", path);
+      return nil;
+    }
+  path = [path stringByAppendingPathComponent: GNU_UserDefaultsDatabase];
+  return path;
+}
+
+/* Initializes defaults for the specified user calling initWithFile: . */
+- (id) initWithUser: (NSString*)userName
+{
+  NSString	*path = pathForUser(userName);
+	
+  if (path == nil)
     {  
       RELEASE(self);
       return nil;
     }
-  filename = [NSString stringWithFormat: @"%@/%@",
-	userHome, GNU_UserDefaultsDatabase];
-  return [self initWithContentsOfFile: filename];
+  return [self initWithContentsOfFile: path];
 }
 
-- (id)initWithContentsOfFile: (NSString *)path
+- (id) initWithContentsOfFile: (NSString*)path
   /* Initializes defaults for the specified path. Returns an object with 
      an empty search list. */
 {
   [super init];
 	
   // Find the user's home folder and build the paths (executed only once)
-  if (!defaultsDatabase)
+  if (_defaultsDatabase == nil)
     {
       if (path != nil && [path isEqual: @""] == NO)
-        defaultsDatabase = (NSMutableString*)[path mutableCopy];
+	{
+	  _defaultsDatabase = [path copy];
+	}
       else
 	{
-	  defaultsDatabase =
-	    (NSMutableString*)[NSMutableString stringWithFormat: @"%@/%@",
-			  NSHomeDirectoryForUser(NSUserName()),
-			  GNU_UserDefaultsDatabase];
-	  RETAIN(defaultsDatabase);
+	  _defaultsDatabase = [pathForUser(NSUserName()) copy];
 	}
 
-      if ([[defaultsDatabase lastPathComponent] isEqual: 
-	[GNU_UserDefaultsDatabase lastPathComponent]] == YES)
-	defaultsDatabaseLockName =
-	  (NSMutableString*)[NSMutableString stringWithFormat: @"%@/%@",
-			  [defaultsDatabase stringByDeletingLastPathComponent],
-			  [GNU_UserDefaultsDatabaseLock lastPathComponent]];
+      if ([[_defaultsDatabase lastPathComponent] isEqual: 
+	GNU_UserDefaultsDatabase] == YES)
+	{
+	  path = [_defaultsDatabase stringByDeletingLastPathComponent];
+	}
       else
-        defaultsDatabaseLockName =
-	  (NSMutableString*)[NSMutableString stringWithFormat: @"%@/%@",
-			  NSHomeDirectoryForUser(NSUserName()),
-			  GNU_UserDefaultsDatabaseLock];
-      RETAIN(defaultsDatabaseLockName);
-      defaultsDatabaseLock =
-	RETAIN([NSDistributedLock lockWithPath: defaultsDatabaseLockName]);
-  }
+	{
+	  path = [pathForUser(NSUserName()) stringByDeletingLastPathComponent];
+	}
+      path = [path stringByAppendingPathComponent:
+	GNU_UserDefaultsDatabaseLock];
+      _defaultsDatabaseLockName = [path copy];
+      _defaultsDatabaseLock =
+	RETAIN([NSDistributedLock lockWithPath: _defaultsDatabaseLockName]);
+    }
   if (processName == nil)
     processName = RETAIN([[NSProcessInfo processInfo] processName]);
 
   // Create an empty search list
-  searchList = [[NSMutableArray alloc] initWithCapacity: 10];
+  _searchList = [[NSMutableArray alloc] initWithCapacity: 10];
 	
-  // Initialize persDomains from the archived user defaults (persistent)
-  persDomains = [[NSMutableDictionary alloc] initWithCapacity: 10];
+  // Initialize _persDomains from the archived user defaults (persistent)
+  _persDomains = [[NSMutableDictionaryClass alloc] initWithCapacity: 10];
   if ([self synchronize] == NO)
     {
       NSRunLoop	*runLoop = [NSRunLoop currentRunLoop];
@@ -355,46 +403,46 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
     }
 	
   // Check and if not existent add the Application and the Global domains
-  if (![persDomains objectForKey: processName])
+  if (![_persDomains objectForKey: processName])
     {
-      [persDomains setObject: 
-		     [NSMutableDictionary 
+      [_persDomains setObject: 
+		     [NSMutableDictionaryClass 
 		       dictionaryWithCapacity: 10] forKey: processName];
       [self __changePersistentDomain: processName];
     }
-  if (![persDomains objectForKey: NSGlobalDomain])
+  if (![_persDomains objectForKey: NSGlobalDomain])
     {
-      [persDomains setObject: 
-		   [NSMutableDictionary 
+      [_persDomains setObject: 
+		   [NSMutableDictionaryClass 
 		     dictionaryWithCapacity: 10] forKey: NSGlobalDomain];
       [self __changePersistentDomain: NSGlobalDomain];
     }
 	
   // Create volatile defaults and add the Argument and the Registration domains
-  tempDomains = [[NSMutableDictionary alloc] initWithCapacity: 10];
-  [tempDomains setObject: [self __createArgumentDictionary] 
+  _tempDomains = [[NSMutableDictionaryClass alloc] initWithCapacity: 10];
+  [_tempDomains setObject: [self __createArgumentDictionary] 
 	       forKey: NSArgumentDomain];
-  [tempDomains setObject: 
-		 [NSMutableDictionary
+  [_tempDomains setObject: 
+		 [NSMutableDictionaryClass
 		   dictionaryWithCapacity: 10] forKey: NSRegistrationDomain];
 	
   return self;
 }
 
-- (void)dealloc
+- (void) dealloc
 {
-  if (tickingTimer)
-    [tickingTimer invalidate];
-  RELEASE(lastSync);
-  RELEASE(searchList);
-  RELEASE(persDomains);
-  RELEASE(tempDomains);
-  RELEASE(changedDomains);
-  RELEASE(dictionaryRep);
+  if (_tickingTimer)
+    [_tickingTimer invalidate];
+  RELEASE(_lastSync);
+  RELEASE(_searchList);
+  RELEASE(_persDomains);
+  RELEASE(_tempDomains);
+  RELEASE(_changedDomains);
+  RELEASE(_dictionaryRep);
   [super dealloc];
 }
 
-- (NSString *)description
+- (NSString*) description
 {
   NSMutableString *desc =
     [NSMutableString stringWithFormat: @"%@",[super description]];
@@ -408,98 +456,102 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 /*************************************************************************
  *** Getting and Setting a Default
  *************************************************************************/
-- (NSArray *)arrayForKey: (NSString *)defaultName
+- (NSArray*) arrayForKey: (NSString*)defaultName
 {
-  id obj = [self objectForKey: defaultName];
+  id	obj = [self objectForKey: defaultName];
 	
-  if (obj && [obj isKindOfClass: [NSArray class]])
+  if (obj != nil && [obj isKindOfClass: NSArrayClass])
     return obj;
   return nil;
 }
 
-- (BOOL)boolForKey: (NSString *)defaultName
+- (BOOL) boolForKey: (NSString*)defaultName
 {
-  id obj = [self stringForKey: defaultName];
+  id	obj = [self stringForKey: defaultName];
 	
-  if (obj)
+  if (obj != nil)
     return [obj boolValue];
   return NO;
 }
 
-- (NSData *)dataForKey: (NSString *)defaultName
+- (NSData*) dataForKey: (NSString*)defaultName
 {
-  id obj = [self objectForKey: defaultName];
+  id	obj = [self objectForKey: defaultName];
 	
-  if (obj && [obj isKindOfClass: [NSData class]])
+  if (obj != nil && [obj isKindOfClass: NSDataClass])
     return obj;
   return nil;
 }
 
-- (NSDictionary *)dictionaryForKey: (NSString *)defaultName
+- (NSDictionary*) dictionaryForKey: (NSString*)defaultName
 {
-  id obj = [self objectForKey: defaultName];
+  id	obj = [self objectForKey: defaultName];
 	
-  if (obj && [obj isKindOfClass: [NSDictionary class]])
+  if (obj != nil && [obj isKindOfClass: NSDictionaryClass])
     return obj;
   return nil;
 }
 
-- (float)floatForKey: (NSString *)defaultName
+- (float) floatForKey: (NSString*)defaultName
 {
-  id obj = [self stringForKey: defaultName];
+  id	obj = [self stringForKey: defaultName];
 	
-  if (obj)
+  if (obj != nil)
     return [obj floatValue];
   return 0.0;
 }
 
-- (int)integerForKey: (NSString *)defaultName
+- (int) integerForKey: (NSString*)defaultName
 {
-  id obj = [self stringForKey: defaultName];
+  id	obj = [self stringForKey: defaultName];
 	
-  if (obj)
+  if (obj != nil)
     return [obj intValue];
   return 0;
 }
 
-- (id)objectForKey: (NSString *)defaultName
+- (id) objectForKey: (NSString*)defaultName
 {
-  NSEnumerator *enumerator = [searchList objectEnumerator];
-  id object = nil;
-  id dN;
+  NSEnumerator	*enumerator = [_searchList objectEnumerator];
+  IMP		nImp = [enumerator methodForSelector: nextObjectSel];
+  id		object = nil;
+  id		dN;
+  IMP		pImp = [_persDomains methodForSelector: objectForKeySel];
+  IMP		tImp = [_tempDomains methodForSelector: objectForKeySel];
 	
-  while ((dN = [enumerator nextObject]))
+  while ((dN = (*nImp)(enumerator, nextObjectSel)) != nil)
     {
-      id dict;
+      id	dict;
       
-      dict = [persDomains objectForKey: dN];
-      if (dict && (object = [dict objectForKey: defaultName]))
+      dict = (*pImp)(_persDomains, objectForKeySel, dN);
+      if (dict != nil && (object = [dict objectForKey: defaultName]))
 	break;
-      dict = [tempDomains objectForKey: dN];
-      if (dict && (object = [dict objectForKey: defaultName]))
+      dict = (*tImp)(_tempDomains, objectForKeySel, dN);
+      if (dict != nil && (object = [dict objectForKey: defaultName]))
 	break;
     }
 	
   return object;
 }
 
-- (void)removeObjectForKey: (NSString *)defaultName
+- (void) removeObjectForKey: (NSString*)defaultName
 {
-  id obj = [[persDomains objectForKey: processName] objectForKey: defaultName];
+  id	obj;
 	
-  if (obj)
+  obj = [[_persDomains objectForKey: processName] objectForKey: defaultName];
+  if (obj != nil)
     {
-      id	obj = [persDomains objectForKey: processName];
-      NSMutableDictionary *dict;
+      NSMutableDictionary	*dict;
+      id			obj = [_persDomains objectForKey: processName];
 
-      if ([obj isKindOfClass: [NSMutableDictionary class]] == YES)
+      if ([obj isKindOfClass: NSMutableDictionaryClass] == YES)
 	{
 	  dict = obj;
 	}
       else
 	{
 	  dict = [obj mutableCopy];
-	  [persDomains setObject: dict forKey: processName];
+	  [_persDomains setObject: dict forKey: processName];
 	}
       [dict removeObjectForKey: defaultName];
       [self __changePersistentDomain: processName];
@@ -507,45 +559,48 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
   return;
 }
 
-- (void)setBool: (BOOL)value forKey: (NSString *)defaultName
+- (void) setBool: (BOOL)value forKey: (NSString*)defaultName
 {
-  id obj = (value)?@"YES": @"NO";
+  id	obj = (value)?@"YES": @"NO";
 	
   [self setObject: obj forKey: defaultName];
   return;
 }
 
-- (void)setFloat: (float)value forKey: (NSString *)defaultName
+- (void) setFloat: (float)value forKey: (NSString*)defaultName
 {	
-  char buf[32];
+  char	buf[32];
+
   sprintf(buf,"%g",value);
-  [self setObject: [NSString stringWithCString: buf] forKey: defaultName];
+  [self setObject: [NSStringClass stringWithCString: buf] forKey: defaultName];
   return;
 }
 
-- (void)setInteger: (int)value forKey: (NSString *)defaultName
+- (void) setInteger: (int)value forKey: (NSString*)defaultName
 {
-  char buf[32];
+  char	buf[32];
+
   sprintf(buf,"%d",value);
-  [self setObject: [NSString stringWithCString: buf] forKey: defaultName];
+  [self setObject: [NSStringClass stringWithCString: buf] forKey: defaultName];
   return;
 }
 
-- (void)setObject: (id)value forKey: (NSString *)defaultName
+- (void) setObject: (id)value forKey: (NSString*)defaultName
 {
   if (value && defaultName && ([defaultName length] > 0))
     {
-      id	obj = [persDomains objectForKey: processName];
-      NSMutableDictionary *dict;
+      NSMutableDictionary	*dict;
+      id			obj = [_persDomains objectForKey: processName];
 
-      if ([obj isKindOfClass: [NSMutableDictionary class]] == YES)
+      if ([obj isKindOfClass: NSMutableDictionaryClass] == YES)
 	{
 	  dict = obj;
 	}
       else
 	{
 	  dict = [obj mutableCopy];
-	  [persDomains setObject: dict forKey: processName];
+	  [_persDomains setObject: dict forKey: processName];
+	  RELEASE(dict);
 	}
       [dict setObject: value forKey: defaultName];
       [self __changePersistentDomain: processName];
@@ -553,28 +608,28 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
   return;
 }
 
-- (NSArray *)stringArrayForKey: (NSString *)defaultName
+- (NSArray*) stringArrayForKey: (NSString*)defaultName
 {
-  id arr = [self arrayForKey: defaultName];
+  id	arr = [self arrayForKey: defaultName];
 	
   if (arr)
     {
-      NSEnumerator *enumerator = [arr objectEnumerator];
-      id obj;
+      NSEnumerator	*enumerator = [arr objectEnumerator];
+      id		obj;
 		
       while ((obj = [enumerator nextObject]))
-	if ( ! [obj isKindOfClass: [NSString class]])
+	if ( ! [obj isKindOfClass: NSStringClass])
 	  return nil;
       return arr;
     }
   return nil;
 }
 
-- (NSString *)stringForKey: (NSString *)defaultName
+- (NSString*) stringForKey: (NSString*)defaultName
 {
-  id obj = [self objectForKey: defaultName];
+  id	obj = [self objectForKey: defaultName];
 	
-  if (obj && [obj isKindOfClass: [NSString class]])
+  if (obj != nil && [obj isKindOfClass: NSStringClass])
     return obj;
   return nil;
 }
@@ -582,54 +637,55 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 /*************************************************************************
  *** Returning the Search List
  *************************************************************************/
-- (NSMutableArray *)searchList
+- (NSArray*) searchList
 {
-  return searchList;
+  return AUTORELEASE([_searchList copy]);
 }
 
-- (void)setSearchList: (NSArray*)newList
+- (void) setSearchList: (NSArray*)newList
 {
-  DESTROY(dictionaryRep);
-  RELEASE(searchList);
-  searchList = [newList mutableCopy];
+  DESTROY(_dictionaryRep);
+  RELEASE(_searchList);
+  _searchList = [newList mutableCopy];
 }
 
 /*************************************************************************
  *** Maintaining Persistent Domains
  *************************************************************************/
-- (NSDictionary *)persistentDomainForName: (NSString *)domainName
+- (NSDictionary*) persistentDomainForName: (NSString*)domainName
 {
-  return [[persDomains objectForKey: domainName] copy];
+  return AUTORELEASE([[_persDomains objectForKey: domainName] copy]);
 }
 
-- (NSArray *)persistentDomainNames
+- (NSArray*) persistentDomainNames
 {
-  return [persDomains allKeys];
+  return [_persDomains allKeys];
 }
 
-- (void)removePersistentDomainForName: (NSString *)domainName
+- (void) removePersistentDomainForName: (NSString*)domainName
 {
-  if ([persDomains objectForKey: domainName])
+  if ([_persDomains objectForKey: domainName])
     {
-      [persDomains removeObjectForKey: domainName];
+      [_persDomains removeObjectForKey: domainName];
       [self __changePersistentDomain: domainName];
     }
   return;
 }
 
-- (void)setPersistentDomain: (NSDictionary *)domain 
-		    forName: (NSString *)domainName
+- (void) setPersistentDomain: (NSDictionary*)domain 
+		     forName: (NSString*)domainName
 {
-  id dict = [tempDomains objectForKey: domainName];
+  id	dict = [_tempDomains objectForKey: domainName];
 	
   if (dict)
     {
       [NSException raise: NSInvalidArgumentException 
-		   format: @"Volatile domain with %@ already exists",
-		   domainName];
+		   format: @"Persistant domain %@ already exists", domainName];
       return;
     }
-  [persDomains setObject: domain forKey: domainName];
+  domain = [domain mutableCopy];
+  [_persDomains setObject: domain forKey: domainName];
+  RELEASE(domain);
   [self __changePersistentDomain: domainName];
   return;
 }
@@ -637,13 +693,13 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 - (BOOL) synchronize
 {
   NSFileManager		*mgr = [NSFileManager defaultManager];
-  NSMutableDictionary	*newDict = nil;
+  NSMutableDictionary	*newDict;
   NSDictionary		*attr;
   NSDate		*mod;
 
-  if (tickingTimer == nil)
+  if (_tickingTimer == nil)
     {
-      tickingTimer = [NSTimer scheduledTimerWithTimeInterval: 30
+      _tickingTimer = [NSTimer scheduledTimerWithTimeInterval: 30
 	       target: self
 	       selector: @selector(__timerTicked:)
 	       userInfo: nil
@@ -654,22 +710,22 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
    *	If we haven't changed anything, we only need to synchronise if
    *	the on-disk database has been changed by someone else.
    */
-  if (changedDomains == NO)
+  if (_changedDomains == NO)
     {
       BOOL		wantRead = NO;
 
-      if (lastSync == nil)
+      if (_lastSync == nil)
 	wantRead = YES;
       else
 	{
-	  attr = [mgr fileAttributesAtPath: defaultsDatabase
+	  attr = [mgr fileAttributesAtPath: _defaultsDatabase
 			      traverseLink: YES];
 	  if (attr == nil)
 	    wantRead = YES;
 	  else
 	    {
 	      mod = [attr objectForKey: NSFileModificationDate];
-	      if ([lastSync earlierDate: mod] != lastSync)
+	      if ([_lastSync earlierDate: mod] != _lastSync)
 		wantRead = YES;
 	    }
 	}
@@ -680,12 +736,12 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
   /*
    * Get file lock - break any lock that is more than five minute old.
    */
-  if ([defaultsDatabaseLock tryLock] == NO)
+  if ([_defaultsDatabaseLock tryLock] == NO)
     {
-      if ([[defaultsDatabaseLock lockDate] timeIntervalSinceNow] < -300.0)
+      if ([[_defaultsDatabaseLock lockDate] timeIntervalSinceNow] < -300.0)
 	{
-	  [defaultsDatabaseLock breakLock];
-	  if ([defaultsDatabaseLock tryLock] == NO)
+	  [_defaultsDatabaseLock breakLock];
+	  if ([_defaultsDatabaseLock tryLock] == NO)
 	    {
 	      return NO;
 	    }
@@ -696,40 +752,46 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 	}
     }
 	
-  DESTROY(dictionaryRep);
+  DESTROY(_dictionaryRep);
 
   // Read the persistent data from the stored database
-  if ([mgr fileExistsAtPath: defaultsDatabase])
+  if ([mgr fileExistsAtPath: _defaultsDatabase])
     {
-      newDict = [[NSMutableDictionary allocWithZone: [self zone]]
-        initWithContentsOfFile: defaultsDatabase];
+      newDict = [[NSMutableDictionaryClass allocWithZone: [self zone]]
+        initWithContentsOfFile: _defaultsDatabase];
+      if (newDict == nil)
+	{
+	  [_defaultsDatabaseLock unlock];	// release file lock
+	  NSLog(@"Unable to load defaults from '%@'", _defaultsDatabase);
+	  return NO;
+	}
     }
   else
     {
       attr = [NSDictionary dictionaryWithObjectsAndKeys: 
 		NSUserName(), NSFileOwnerAccountName, nil];
-      NSLog(@"Creating defaults database file %@", defaultsDatabase);
-      [mgr createFileAtPath: defaultsDatabase
+      NSLog(@"Creating defaults database file %@", _defaultsDatabase);
+      [mgr createFileAtPath: _defaultsDatabase
 		   contents: nil
 		 attributes: attr];
-      [[NSDictionary dictionary] writeToFile: defaultsDatabase atomically: YES];
-    }
-
-  if (!newDict)
-    {
-      newDict = [[NSMutableDictionary allocWithZone: [self zone]]
+      newDict = [[NSMutableDictionaryClass allocWithZone: [self zone]]
 		  initWithCapacity: 1];
+      [newDict writeToFile: _defaultsDatabase atomically: YES];
     }
 
-  if (changedDomains)
+  if (_changedDomains)
     {           // Synchronize both dictionaries
-      NSEnumerator *enumerator = [changedDomains objectEnumerator];
-      id obj, dict;
+      NSEnumerator	*enumerator = [_changedDomains objectEnumerator];
+      IMP		nextImp;
+      IMP		pImp;
+      id		obj, dict;
 		
-      while ((obj = [enumerator nextObject]))
+      nextImp = [enumerator methodForSelector: nextObjectSel];
+      pImp = [_persDomains methodForSelector: objectForKeySel];
+      while ((obj = (*nextImp)(enumerator, nextObjectSel)) != nil)
 	{
-	  dict = [persDomains objectForKey: obj];
-	  if (dict)       // Domain was added or changet
+	  dict = (*pImp)(_persDomains, objectForKeySel, obj);
+	  if (dict)       // Domain was added or changed
 	    {
 	      [newDict setObject: dict forKey: obj];
 	    }
@@ -738,31 +800,31 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 	      [newDict removeObjectForKey: obj];
 	    }
 	}
-      RELEASE(persDomains);
-      persDomains = newDict;
+      RELEASE(_persDomains);
+      _persDomains = newDict;
       // Save the changes
-      if (![persDomains writeToFile: defaultsDatabase atomically: YES])
+      if (![_persDomains writeToFile: _defaultsDatabase atomically: YES])
 	{
-	  [defaultsDatabaseLock unlock];
+	  [_defaultsDatabaseLock unlock];
 	  return NO;
 	}
-      attr = [mgr fileAttributesAtPath: defaultsDatabase
+      attr = [mgr fileAttributesAtPath: _defaultsDatabase
 			  traverseLink: YES];
       mod = [attr objectForKey: NSFileModificationDate];
-      ASSIGN(lastSync, mod);
-      [defaultsDatabaseLock unlock];	// release file lock
+      ASSIGN(_lastSync, mod);
+      [_defaultsDatabaseLock unlock];	// release file lock
     }
   else
     {
-      attr = [mgr fileAttributesAtPath: defaultsDatabase
+      attr = [mgr fileAttributesAtPath: _defaultsDatabase
 			  traverseLink: YES];
       mod = [attr objectForKey: NSFileModificationDate];
-      ASSIGN(lastSync, mod);
-      [defaultsDatabaseLock unlock];	// release file lock
-      if ([persDomains isEqual: newDict] == NO)
+      ASSIGN(_lastSync, mod);
+      [_defaultsDatabaseLock unlock];	// release file lock
+      if ([_persDomains isEqual: newDict] == NO)
 	{
-	  RELEASE(persDomains);
-	  persDomains = newDict;
+	  RELEASE(_persDomains);
+	  _persDomains = newDict;
 	  [[NSNotificationCenter defaultCenter] 
 	    postNotificationName: NSUserDefaultsDidChangeNotification
 			  object: nil];
@@ -780,120 +842,134 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
 /*************************************************************************
  *** Maintaining Volatile Domains
  *************************************************************************/
-- (void)removeVolatileDomainForName: (NSString *)domainName
+- (void) removeVolatileDomainForName: (NSString*)domainName
 {
-  DESTROY(dictionaryRep);
-  [tempDomains removeObjectForKey: domainName];
+  DESTROY(_dictionaryRep);
+  [_tempDomains removeObjectForKey: domainName];
 }
 
-- (void)setVolatileDomain: (NSDictionary *)domain 
-		  forName: (NSString *)domainName
+- (void) setVolatileDomain: (NSDictionary*)domain 
+		   forName: (NSString*)domainName
 {
-  id dict = [persDomains objectForKey: domainName];
+  id	dict = [_persDomains objectForKey: domainName];
 	
   if (dict)
     {
       [NSException raise: NSInvalidArgumentException 
-		   format: @"Persistent domain with %@ already exists",
-		   domainName];
+		  format: @"Volatile domain %@ already exists", domainName];
       return;
     }
-  DESTROY(dictionaryRep);
-  [tempDomains setObject: domain forKey: domainName];
+  DESTROY(_dictionaryRep);
+  domain = [domain mutableCopy];
+  [_tempDomains setObject: domain forKey: domainName];
+  RELEASE(domain);
   return;
 }
 
-- (NSDictionary *)volatileDomainForName: (NSString *)domainName
+- (NSDictionary*) volatileDomainForName: (NSString*)domainName
 {
-  return [tempDomains objectForKey: domainName];
+  return AUTORELEASE([[_tempDomains objectForKey: domainName] copy]);
 }
 
-- (NSArray *)volatileDomainNames
+- (NSArray*) volatileDomainNames
 {
-  return [tempDomains allKeys];
+  return [_tempDomains allKeys];
 }
 
 /*************************************************************************
  *** Making Advanced Use of Defaults
  *************************************************************************/
-- (NSDictionary *) dictionaryRepresentation
+- (NSDictionary*) dictionaryRepresentation
 {
-  if (dictionaryRep == nil)
+  if (_dictionaryRep == nil)
     {
       NSEnumerator		*enumerator;
       NSMutableDictionary	*dictRep;
-      id obj;
-      id dict;
+      id			obj;
+      id			dict;
+      static SEL		aSel = @selector(addEntriesFromDictionary:);
+      IMP			nImp;
+      IMP			pImp;
+      IMP			tImp;
+      IMP			aImp;
 	
-      enumerator = [searchList reverseObjectEnumerator];
-      dictRep = [NSMutableDictionary allocWithZone: NSDefaultMallocZone()];
+      pImp = [_persDomains methodForSelector: objectForKeySel];
+      tImp = [_tempDomains methodForSelector: objectForKeySel];
+
+      enumerator = [_searchList reverseObjectEnumerator];
+      nImp = [enumerator methodForSelector: nextObjectSel];
+
+      dictRep = [NSMutableDictionaryClass allocWithZone: NSDefaultMallocZone()];
       dictRep = [dictRep initWithCapacity: 512];
-      while ((obj = [enumerator nextObject]))
+      aImp = [dictRep methodForSelector: aSel];
+
+      while ((obj = (*nImp)(enumerator, nextObjectSel)) != nil)
 	{
-	  if ( (dict = [persDomains objectForKey: obj])
-	       || (dict = [tempDomains objectForKey: obj]) )
-	    [dictRep addEntriesFromDictionary: dict];
+	  if ( (dict = (*pImp)(_persDomains, objectForKeySel, obj)) != nil
+	    || (dict = (*tImp)(_tempDomains, objectForKeySel, obj)) != nil)
+	    (*aImp)(dictRep, aSel, dict);
 	}
-      dictionaryRep = [dictRep copy];
+      _dictionaryRep = [dictRep copy];
       RELEASE(dictRep);
     }
-  return dictionaryRep;
+  return _dictionaryRep;
 }
 
 - (void) registerDefaults: (NSDictionary*)newVals
 {
   NSMutableDictionary	*regDefs;
 
-  regDefs = [tempDomains objectForKey: NSRegistrationDomain];
+  regDefs = [_tempDomains objectForKey: NSRegistrationDomain];
   if (regDefs == nil)
     {
-      regDefs = [NSMutableDictionary dictionaryWithCapacity: [newVals count]];
+      regDefs = [NSMutableDictionaryClass
+	dictionaryWithCapacity: [newVals count]];
     }
-  DESTROY(dictionaryRep);
+  DESTROY(_dictionaryRep);
   [regDefs addEntriesFromDictionary: newVals];
 }
 
 /*************************************************************************
  *** Accessing the User Defaults database
  *************************************************************************/
-- (void)__createStandardSearchList
+- (void) __createStandardSearchList
 {
-  NSArray *uL = [[self class] userLanguages];
-  NSEnumerator *enumerator = [uL objectEnumerator];
-  id object;
+  NSArray	*uL = [[self class] userLanguages];
+  NSEnumerator	*enumerator = [uL objectEnumerator];
+  id		object;
 	
   // Note: The search list should exist!
 	
   // 1. NSArgumentDomain
-  [searchList addObject: NSArgumentDomain];
+  [_searchList addObject: NSArgumentDomain];
 	
   // 2. Application
-  [searchList addObject: processName];
+  [_searchList addObject: processName];
 
   // 3. User's preferred languages
   while ((object = [enumerator nextObject]))
     {
-      [searchList addObject: object];
+      [_searchList addObject: object];
     }
 	
   // 4. NSGlobalDomain
-  [searchList addObject: NSGlobalDomain];
+  [_searchList addObject: NSGlobalDomain];
 	
   // 5. NSRegistrationDomain
-  [searchList addObject: NSRegistrationDomain];
+  [_searchList addObject: NSRegistrationDomain];
 	
   return;
 }
 
-- (NSDictionary *)__createArgumentDictionary
+- (NSDictionary*) __createArgumentDictionary
 {
-  NSArray *args = [[NSProcessInfo processInfo] arguments];
-  //$$$	NSArray *args = searchList;  // $$$
-  NSEnumerator *enumerator = [args objectEnumerator];
+  NSArray	*args = [[NSProcessInfo processInfo] arguments];
+  //$$$	NSArray *args = _searchList;  // $$$
+  NSEnumerator	*enumerator = [args objectEnumerator];
   NSMutableDictionary *argDict =
-    [NSMutableDictionary dictionaryWithCapacity: 2];
-  BOOL done;
-  id key, val;
+    [NSMutableDictionaryClass dictionaryWithCapacity: 2];
+  BOOL		done;
+  id		key, val;
 	
   [enumerator nextObject];	// Skip process name.
   done = ((key = [enumerator nextObject]) == nil);
@@ -941,33 +1017,35 @@ static BOOL setSharedDefaults = NO;	/* Flag to prevent infinite recursion */
   return argDict;
 }
 
-- (void)__changePersistentDomain: (NSString *)domainName
+- (void) __changePersistentDomain: (NSString*)domainName
 {
-  NSEnumerator *enumerator = nil;
-  id obj;
+  NSEnumerator	*enumerator = nil;
+  IMP		nImp;
+  id		obj;
 
-  DESTROY(dictionaryRep);
-  if (!changedDomains)
+  DESTROY(_dictionaryRep);
+  if (!_changedDomains)
     {
-      changedDomains = [[NSMutableArray alloc] initWithCapacity: 5];
+      _changedDomains = [[NSMutableArray alloc] initWithCapacity: 5];
       [[NSNotificationCenter defaultCenter] 
 	postNotificationName: NSUserDefaultsDidChangeNotification object: nil];
     }
 	
-  enumerator = [changedDomains objectEnumerator];
-  while ((obj = [enumerator nextObject]))
+  enumerator = [_changedDomains objectEnumerator];
+  nImp = [enumerator methodForSelector: nextObjectSel];
+  while ((obj = (*nImp)(enumerator, nextObjectSel)) != nil)
     {
       if ([obj isEqualToString: domainName])
 	return;
     }
-  [changedDomains addObject: domainName];
+  [_changedDomains addObject: domainName];
   return;
 }
 
 - (void) __timerTicked: (NSTimer*)tim
 {
-  if (tim == tickingTimer)
-    tickingTimer = nil;
+  if (tim == _tickingTimer)
+    _tickingTimer = nil;
 
   [self synchronize];
 }
