@@ -19,7 +19,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
 /*  Design goals:
 
@@ -86,7 +86,6 @@
 #include <string.h>
 #include <objc/objc-api.h>
 #include <Foundation/NSException.h>
-#include <Foundation/NSPage.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSZone.h>
 #include <Foundation/NSLock.h>
@@ -155,8 +154,8 @@ struct _nfree_block_struct
 };
 
 struct _ffree_block_unpadded {
-    size_t	size;
-    struct _ffree_block_struct *next;
+  size_t	size;
+  struct _ffree_block_struct *next;
 };
 #define	FFCPAD	sizeof(struct _ffree_block_unpadded)
 
@@ -170,20 +169,20 @@ struct _ffree_block_struct
 
 struct _ffree_free_link_unpadded
 {
-    size_t	size;
-    ff_link	*prev;
-    ff_link	*next;
-    size_t	back;	/* Back link at end of 'dead' block.	*/
+  size_t	size;
+  ff_link	*prev;
+  ff_link	*next;
+  size_t	back;	/* Back link at end of 'dead' block.	*/
 };
 #define	FFDPAD	sizeof(struct _ffree_free_link_unpadded)
 
 struct _ffree_free_link
 {
-    size_t	size;
-    ff_link	*prev;
-    ff_link	*next;
-    size_t	back;
-    char	padding[ALIGN - ((FFDPAD % ALIGN) ? (FFDPAD % ALIGN) : ALIGN)];
+  size_t	size;
+  ff_link	*prev;
+  ff_link	*next;
+  size_t	back;
+  char	padding[ALIGN - ((FFDPAD % ALIGN) ? (FFDPAD % ALIGN) : ALIGN)];
 };
 
 /* NSZone structure for freeable zones. */
@@ -221,123 +220,124 @@ roundupto (size_t n, size_t base)
 static inline void*
 chunkToPointer(ff_block *chunk)
 {
-    return (void*)(&chunk[1]);
+  return (void*)(&chunk[1]);
 }
 
 static inline ff_block*
 pointerToChunk(void* ptr)
 {
-    return &(((ff_block*)ptr)[-1]);
+  return &(((ff_block*)ptr)[-1]);
 }
 
 static inline size_t
 chunkIsLive(ff_block* ptr)
 {
-    return ptr->size & LIVE;
+  return ptr->size & LIVE;
 }
 
 static inline size_t
 chunkIsInUse(ff_block* ptr)
 {
-    return ptr->size & INUSE;
+  return ptr->size & INUSE;
 }
 
 static inline size_t
 chunkIsPrevInUse(ff_block* ptr)
 {
-    return ptr->size & PREVUSE;
+  return ptr->size & PREVUSE;
 }
 
 static inline size_t
 chunkSize(ff_block* ptr)
 {
-    return ptr->size & ~SIZE_BITS;
+  return ptr->size & ~SIZE_BITS;
 }
 
 static inline void
 chunkClrInUse(ff_block* ptr)
 {
-    ptr->size &= ~INUSE;
+  ptr->size &= ~INUSE;
 }
 
 static inline size_t
 chunkClrLive(ff_block* ptr)
 {
-    return ptr->size &= ~LIVE;
+  return ptr->size &= ~LIVE;
 }
 
 static inline void
 chunkClrPrevInUse(ff_block* ptr)
 {
-    ptr->size &= ~PREVUSE;
+  ptr->size &= ~PREVUSE;
 }
 
 static inline void
 chunkSetInUse(ff_block* ptr)
 {
-    ptr->size |= INUSE;
+  ptr->size |= INUSE;
 }
 
 static inline size_t
 chunkSetLive(ff_block* ptr)
 {
-    return ptr->size |= LIVE;
+  return ptr->size |= LIVE;
 }
 
 static inline void
 chunkSetPrevInUse(ff_block* ptr)
 {
-    ptr->size |= PREVUSE;
+  ptr->size |= PREVUSE;
 }
 
 static inline void
 chunkSetSize(ff_block* ptr, size_t size)
 {
-    ptr->size = size;
+  ptr->size = size;
 }
 
 static inline ff_block*
 chunkNext(ff_block *ptr)
 {
-    return (ff_block*) ((void*)ptr+chunkSize(ptr));
+  return (ff_block*) ((void*)ptr+chunkSize(ptr));
 }
 
 static inline void
 chunkMakeLink(ff_block *ptr)
 {
-    NSAssert(!chunkIsInUse(ptr), NSInternalInconsistencyException);
-    NSAssert(!chunkIsLive(ptr), NSInternalInconsistencyException);
-    (&(chunkNext(ptr)->size))[-1] = chunkSize(ptr);
+  NSAssert(!chunkIsInUse(ptr), NSInternalInconsistencyException);
+  NSAssert(!chunkIsLive(ptr), NSInternalInconsistencyException);
+  (&(chunkNext(ptr)->size))[-1] = chunkSize(ptr);
 }
 
 static inline ff_block*
 chunkChop(ff_block *ptr, size_t size)
 {
-    ff_block	*remainder;
-    size_t	left = chunkSize(ptr)-size;
+  ff_block	*remainder;
+  size_t	left = chunkSize(ptr)-size;
 
-    NSAssert((chunkSize(ptr) % MINCHUNK) == 0, NSInternalInconsistencyException);
-    NSAssert(chunkSize(ptr) > size, NSInternalInconsistencyException);
-    remainder = (ff_block*)((void*)ptr+size);
-    chunkSetSize(remainder, left | PREVUSE);
-    chunkMakeLink(remainder);
-    chunkSetSize(ptr, size | chunkIsPrevInUse(ptr) | INUSE);
-    return remainder;
+  NSAssert((chunkSize(ptr) % MINCHUNK) == 0, NSInternalInconsistencyException);
+  NSAssert(chunkSize(ptr) > size, NSInternalInconsistencyException);
+  remainder = (ff_block*)((void*)ptr+size);
+  chunkSetSize(remainder, left | PREVUSE);
+  chunkMakeLink(remainder);
+  chunkSetSize(ptr, size | chunkIsPrevInUse(ptr) | INUSE);
+  return remainder;
 }
 
 static inline ff_block*
 chunkPrev(ff_block *ptr)
 {
-    size_t	offset;
-    ff_block	*prev;
+  size_t	offset;
+  ff_block	*prev;
 
-    NSAssert(!chunkIsPrevInUse(ptr), NSInternalInconsistencyException);
-    offset = (&(ptr->size))[-1];
-    NSAssert(offset > 0 && (offset % MINCHUNK) == 0, NSInternalInconsistencyException);
-    prev = (ff_block*)((void*)ptr-offset);
-    NSAssert(chunkSize(prev) == offset, NSInternalInconsistencyException);
-    NSAssert(!chunkIsInUse(prev), NSInternalInconsistencyException);
-    return prev;
+  NSAssert(!chunkIsPrevInUse(ptr), NSInternalInconsistencyException);
+  offset = (&(ptr->size))[-1];
+  NSAssert(offset > 0 && (offset % MINCHUNK) == 0,
+    NSInternalInconsistencyException);
+  prev = (ff_block*)((void*)ptr-offset);
+  NSAssert(chunkSize(prev) == offset, NSInternalInconsistencyException);
+  NSAssert(!chunkIsInUse(prev), NSInternalInconsistencyException);
+  return prev;
 }
 
 /* NSZone structure for nonfreeable zones. */
@@ -413,21 +413,23 @@ static NSZone	*zone_list = 0;
 inline NSZone*
 NSZoneFromPointer(void *ptr)
 {
-    NSZone	*zone;
+  NSZone	*zone;
 
-    if (ptr == 0) return 0;
+  if (ptr == 0) return 0;
 
-    /*
-     *	See if we can find the zone in our list of all zones.
-     */
-    [gnustep_global_lock lock];
-    for (zone = zone_list; zone != 0; zone = zone->next) {
-	if ((zone->lookup)(zone, ptr) == YES) {
-	    break;
+  /*
+   *	See if we can find the zone in our list of all zones.
+   */
+  [gnustep_global_lock lock];
+  for (zone = zone_list; zone != 0; zone = zone->next)
+    {
+      if ((zone->lookup)(zone, ptr) == YES)
+	{
+	  break;
 	}
     }
-    [gnustep_global_lock unlock];
-    return (zone == 0) ? __nszone_private_hidden_default_zone : zone;
+  [gnustep_global_lock unlock];
+  return (zone == 0) ? __nszone_private_hidden_default_zone : zone;
 }
 
 static inline void
@@ -464,17 +466,19 @@ default_realloc (NSZone *zone, void *ptr, size_t size)
 {
   void *mem;
 
-  if (size == 0) {
-    objc_free(ptr);
-    return NULL;
-  }
-  if (ptr == 0) {
-    mem = objc_malloc(size);
-    if (mem == NULL)
-      [NSException raise: NSMallocException
-                   format: @"Default zone has run out of memory"];
-    return mem;
-  }
+  if (size == 0)
+    {
+      objc_free(ptr);
+      return NULL;
+    }
+  if (ptr == 0)
+    {
+      mem = objc_malloc(size);
+      if (mem == NULL)
+	[NSException raise: NSMallocException
+		     format: @"Default zone has run out of memory"];
+      return mem;
+    }
   mem = objc_realloc(ptr, size);
   if (mem == NULL)
     [NSException raise: NSMallocException
@@ -493,7 +497,7 @@ default_recycle (NSZone *zone)
 {
   /* Recycle the default zone?  Thou hast got to be kiddin'. */
   [NSException raise: NSGenericException
-               format: @"Trying to recycle default zone"];
+              format: @"Trying to recycle default zone"];
 }
 
 static BOOL
@@ -501,7 +505,7 @@ default_check (NSZone *zone)
 {
   /* We can't check memory managed by objc_malloc(). */
   [NSException raise: NSGenericException
-		format: @"No checking for default zone"];
+	      format: @"No checking for default zone"];
   return NO;
 }
 
@@ -519,7 +523,7 @@ default_stats (NSZone *zone)
   
   /* We can't obtain statistics from the memory managed by objc_malloc(). */
   [NSException raise: NSGenericException
-		format: @"No statistics for default zone"];
+	      format: @"No statistics for default zone"];
   return dummy;
 }
  
@@ -560,7 +564,8 @@ fmalloc (NSZone *zone, size_t size)
           ptr_buf[i] = ptr_buf[bufsize];
 
           NSAssert(chunkIsInUse(chunkhead), NSInternalInconsistencyException);
-          NSAssert((chunkSize(chunkhead) % MINCHUNK) == 0, NSInternalInconsistencyException);
+          NSAssert((chunkSize(chunkhead) % MINCHUNK) == 0,
+	    NSInternalInconsistencyException);
         }
       else
         {
@@ -584,16 +589,18 @@ fmalloc (NSZone *zone, size_t size)
           objc_mutex_unlock(zptr->lock);
           if (zone->name != nil)
             [NSException raise: NSMallocException
-                         format: @"Zone %s has run out of memory",
+                        format: @"Zone %s has run out of memory",
                          [zone->name cString]];
           else
             [NSException raise: NSMallocException
-                         format: @"Out of memory"];
+                        format: @"Out of memory"];
         }
 
       NSAssert(chunkIsInUse(chunkhead), NSInternalInconsistencyException);
-      NSAssert(chunkIsPrevInUse(chunkNext(chunkhead)), NSInternalInconsistencyException);
-      NSAssert((chunkSize(chunkhead) % MINCHUNK) == 0, NSInternalInconsistencyException);
+      NSAssert(chunkIsPrevInUse(chunkNext(chunkhead)),
+	NSInternalInconsistencyException);
+      NSAssert((chunkSize(chunkhead) % MINCHUNK) == 0,
+	NSInternalInconsistencyException);
     }
   chunkhead->next = (ff_block*)(chunkToPointer(chunkhead)+size);
   *((char*)chunkhead->next) = (char)42;
@@ -618,7 +625,8 @@ frealloc (NSZone *zone, void *ptr, size_t size)
   ff_block *chunkhead, *slack;
   void *result;
 
-  NSAssert(ptr == NULL || NSZoneFromPointer(ptr) == zone, NSInternalInconsistencyException);
+  NSAssert(ptr == NULL || NSZoneFromPointer(ptr) == zone,
+    NSInternalInconsistencyException);
   if (ptr == NULL)
     return fmalloc(zone, size);
   chunkhead = pointerToChunk(ptr);
@@ -681,11 +689,11 @@ frealloc (NSZone *zone, void *ptr, size_t size)
               objc_mutex_unlock(zptr->lock);
               if (zone->name != nil)
                 [NSException raise: NSMallocException
-                             format: @"Zone %s has run out of memory",
+                            format: @"Zone %s has run out of memory",
                              [zone->name cString]];
               else
                 [NSException raise: NSMallocException
-                             format: @"Out of memory"];
+                            format: @"Out of memory"];
             }
           memcpy((void*)(&newchunk[1]), (void*)(&chunkhead[1]), realsize-FBSZ);
           add_buf(zptr, chunkhead);
@@ -901,19 +909,21 @@ inconsistent: // Jump here if an inconsistency was found.
 static BOOL
 flookup (NSZone *zone, void *ptr)
 {
-    ffree_zone *zptr = (ffree_zone*)zone;
-    ff_block *block;
-    BOOL found = NO;
-  
-    objc_mutex_lock(zptr->lock);
-    for (block = zptr->blocks; block != NULL; block = block->next) {
-	if (ptr >= (void*)block && ptr < (void*)chunkNext(block)) {
-	    found = YES;
-	    break;
+  ffree_zone	*zptr = (ffree_zone*)zone;
+  ff_block	*block;
+  BOOL		found = NO;
+
+  objc_mutex_lock(zptr->lock);
+  for (block = zptr->blocks; block != NULL; block = block->next)
+    {
+      if (ptr >= (void*)block && ptr < (void*)chunkNext(block))
+	{
+	  found = YES;
+	  break;
 	}
     }
-    objc_mutex_unlock(zptr->lock);
-    return found;
+  objc_mutex_unlock(zptr->lock);
+  return found;
 }
 
 /* Obtain statistics about the zone.  Doesn't have to be particularly
@@ -1072,7 +1082,8 @@ get_chunk (ffree_zone *zone, size_t size)
 
           NSAssert(!chunkIsInUse(chunk), NSInternalInconsistencyException);
           NSAssert(size < chunkSize(chunk), NSInternalInconsistencyException);
-          NSAssert((chunkSize(chunk) % MINCHUNK) == 0, NSInternalInconsistencyException);
+          NSAssert((chunkSize(chunk) % MINCHUNK) == 0,
+	    NSInternalInconsistencyException);
           
           take_chunk(zone, chunk);
 	  slack = chunkChop(chunk, size);
@@ -1089,7 +1100,8 @@ get_chunk (ffree_zone *zone, size_t size)
       NSAssert((chunksize % MINCHUNK) == 0, NSInternalInconsistencyException);
       NSAssert(!chunkIsInUse(chunk), NSInternalInconsistencyException);
       NSAssert(chunkIsPrevInUse(chunk), NSInternalInconsistencyException);
-      NSAssert(chunkIsInUse(chunkNext(chunk)), NSInternalInconsistencyException);
+      NSAssert(chunkIsInUse(chunkNext(chunk)),
+	NSInternalInconsistencyException);
       
       take_chunk(zone, chunk);
       if (chunksize > size)
@@ -1104,14 +1116,16 @@ get_chunk (ffree_zone *zone, size_t size)
           ff_block *nextchunk = chunkNext(chunk);
 
           NSAssert(!chunkIsInUse(chunk), NSInternalInconsistencyException);
-          NSAssert(!chunkIsPrevInUse(nextchunk), NSInternalInconsistencyException);
+          NSAssert(!chunkIsPrevInUse(nextchunk),
+	    NSInternalInconsistencyException);
           NSAssert(chunksize == size, NSInternalInconsistencyException);
 	  chunkSetInUse(chunk);
 	  chunkSetPrevInUse(nextchunk);
         }
     }
   NSAssert(chunkIsInUse(chunk), NSInternalInconsistencyException);
-  NSAssert(chunkIsPrevInUse(chunkNext(chunk)), NSInternalInconsistencyException);
+  NSAssert(chunkIsPrevInUse(chunkNext(chunk)),
+    NSInternalInconsistencyException);
   return chunk;
 }
 
@@ -1157,7 +1171,8 @@ put_chunk (ffree_zone *zone, ff_block *chunk)
   size_t class = segindex(size);
   ff_link *links = (ff_link*)chunk;
 
-  NSAssert((chunkSize(chunk) % MINCHUNK) == 0, NSInternalInconsistencyException);
+  NSAssert((chunkSize(chunk) % MINCHUNK) == 0,
+    NSInternalInconsistencyException);
   NSAssert(!chunkIsInUse(chunk), NSInternalInconsistencyException);
   NSAssert(chunkIsPrevInUse(chunk), NSInternalInconsistencyException);
   NSAssert(chunkIsInUse(chunkNext(chunk)), NSInternalInconsistencyException);
@@ -1165,7 +1180,8 @@ put_chunk (ffree_zone *zone, ff_block *chunk)
   chunkMakeLink(chunk);
   if (zone->segtaillist[class] == NULL)
     {
-      NSAssert(zone->segheadlist[class] == NULL, NSInternalInconsistencyException);
+      NSAssert(zone->segheadlist[class] == NULL,
+	NSInternalInconsistencyException);
       
       zone->segheadlist[class] = zone->segtaillist[class] = links;
       links->prev = links->next = NULL;
@@ -1174,7 +1190,8 @@ put_chunk (ffree_zone *zone, ff_block *chunk)
     {
       ff_link *prevlink = zone->segtaillist[class];
       
-      NSAssert(zone->segheadlist[class] != NULL, NSInternalInconsistencyException);
+      NSAssert(zone->segheadlist[class] != NULL,
+	NSInternalInconsistencyException);
 
       links->next = NULL;
       links->prev = prevlink;
@@ -1193,7 +1210,8 @@ add_buf (ffree_zone *zone, ff_block *chunk)
 
   NSAssert(bufsize < BUFFER, NSInternalInconsistencyException);
   NSAssert(chunkIsInUse(chunk), NSInternalInconsistencyException);
-  NSAssert((chunkSize(chunk) % MINCHUNK) == 0, NSInternalInconsistencyException);
+  NSAssert((chunkSize(chunk) % MINCHUNK) == 0,
+    NSInternalInconsistencyException);
   NSAssert(chunkSize(chunk) >= MINCHUNK, NSInternalInconsistencyException);
   
   zone->bufsize++;
@@ -1239,8 +1257,10 @@ flush_buf (ffree_zone *zone)
         {
           size_t nextsize = chunkSize(nextchunk);
 
-	  NSAssert(chunkIsPrevInUse(nextchunk), NSInternalInconsistencyException);
-          NSAssert((nextsize % MINCHUNK) == 0, NSInternalInconsistencyException);
+	  NSAssert(chunkIsPrevInUse(nextchunk),
+	    NSInternalInconsistencyException);
+          NSAssert((nextsize % MINCHUNK) == 0,
+	    NSInternalInconsistencyException);
           size += nextsize;
           take_chunk(zone, nextchunk);
 	  nextchunk = chunkNext(nextchunk);
@@ -1250,7 +1270,8 @@ flush_buf (ffree_zone *zone)
       chunkClrPrevInUse(nextchunk);
       NSAssert(chunkNext(chunk) == nextchunk, NSInternalInconsistencyException);
       NSAssert(chunkPrev(nextchunk) == chunk, NSInternalInconsistencyException);
-      NSAssert((chunkSize(chunk) % MINCHUNK) == 0, NSInternalInconsistencyException);
+      NSAssert((chunkSize(chunk) % MINCHUNK) == 0,
+	NSInternalInconsistencyException);
       NSAssert(!chunkIsInUse(chunk), NSInternalInconsistencyException);
       NSAssert(chunkIsPrevInUse(chunk), NSInternalInconsistencyException);
       NSAssert(chunkIsInUse(nextchunk), NSInternalInconsistencyException);
@@ -1292,7 +1313,7 @@ nmalloc (NSZone *zone, size_t size)
       /* First, get the block list in decreasing free size order. */
       preblock = NULL;
       while ((block->next != NULL)
-             && (freesize < block->next->size-block->next->top))
+        && (freesize < block->next->size-block->next->top))
         {
           preblock = block;
           block = block->next;
@@ -1314,11 +1335,11 @@ nmalloc (NSZone *zone, size_t size)
               objc_mutex_unlock(zptr->lock);
               if (zone->name != nil)
                 [NSException raise: NSMallocException
-                             format: @"Zone %s has run out of memory",
+                            format: @"Zone %s has run out of memory",
                              [zone->name cString]];
               else
                 [NSException raise: NSMallocException
-                             format: @"Out of memory"];
+                            format: @"Out of memory"];
             }
           block->next = zptr->blocks;
           block->size = blocksize;
@@ -1477,19 +1498,21 @@ ncheck (NSZone *zone)
 static BOOL
 nlookup (NSZone *zone, void *ptr)
 {
-    nfree_zone *zptr = (nfree_zone*)zone;
-    nf_block *block;
-    BOOL found = NO;
-  
-    objc_mutex_lock(zptr->lock);
-    for (block = zptr->blocks; block != NULL; block = block->next) {
-	if (ptr >= (void*)block &&  ptr < ((void*)block)+block->size) {
-	    found = YES;
-	    break;
+  nfree_zone *zptr = (nfree_zone*)zone;
+  nf_block *block;
+  BOOL found = NO;
+
+  objc_mutex_lock(zptr->lock);
+  for (block = zptr->blocks; block != NULL; block = block->next)
+    {
+      if (ptr >= (void*)block &&  ptr < ((void*)block)+block->size)
+	{
+	  found = YES;
+	  break;
 	}
     }
-    objc_mutex_unlock(zptr->lock);
-    return found;
+  objc_mutex_unlock(zptr->lock);
+  return found;
 }
 
 /* Return statistics for a nonfreeable zone.  Doesn't have to
@@ -1576,7 +1599,6 @@ NSCreateZone (size_t start, size_t gran, BOOL canFree)
       ffree_zone *zone;
       ff_block *block;
       ff_block *chunk;
-      ff_block *header;
       ff_block *tailer;
 
       zone = objc_malloc(sizeof(ffree_zone));
@@ -1679,6 +1701,11 @@ NSDefaultMallocZone (void)
   return __nszone_private_hidden_default_zone;
 }
 
+NSZone* GSAtomicMallocZone (void)
+{
+  return NSDefaultMallocZone();
+}
+
 inline void*
 NSZoneMalloc (NSZone *zone, size_t size)
 {
@@ -1704,10 +1731,10 @@ NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
 inline void
 NSRecycleZone (NSZone *zone)
 {
-    if (zone == 0)
-	zone == NSDefaultMallocZone();
+  if (zone == 0)
+    zone = NSDefaultMallocZone();
 
-    (zone->recycle)(zone);
+  (zone->recycle)(zone);
 }
 
 inline void
@@ -1739,6 +1766,14 @@ NSZoneName (NSZone *zone)
   return zone->name;
 }
 
+/* Not in OpenStep. */
+
+void*
+NSZoneMallocAtomic (NSZone *zone, size_t size)
+{
+  return NSZoneMalloc(GSAtomicMallocZone(), size);
+}
+
 /* Not in OpenStep. */
 inline BOOL
 NSZoneCheck (NSZone *zone)
@@ -1757,5 +1792,110 @@ NSZoneStats (NSZone *zone)
   return (zone->stats)(zone);
 }
 
+#else
+
+#include <gc.h>
+
+/*
+ * Dummy zones used with garbage collection.
+ * The 'atomic' zone is for memory that will be assumed not to contain
+ * pointers for garbage collection purposes.
+ */
+static NSZone default_zone =
+{
+  0, 0, 0, 0, 0, 0, 0, 0, @"default", 0
+};
+NSZone* __nszone_private_hidden_default_zone = &default_zone;
+
+static NSZone atomic_zone =
+{
+  0, 0, 0, 0, 0, 0, 0, 0, @"default", 0
+};
+NSZone* __nszone_private_hidden_atomic_zone = &atomic_zone;
+
+NSZone* NSCreateZone (size_t start, size_t gran, BOOL canFree)
+{ return __nszone_private_hidden_default_zone; }
+
+NSZone* NSDefaultMallocZone (void)
+{ return __nszone_private_hidden_default_zone; }
+
+NSZone* GSAtomicMallocZone (void)
+{ return __nszone_private_hidden_atomic_zone; }
+
+NSZone* NSZoneFromPointer (void *ptr)
+{ return __nszone_private_hidden_default_zone; }
+
+void* NSZoneMalloc (NSZone *zone, size_t size)
+{
+  void	*ptr;
+
+  if (zone == GSAtomicMallocZone())
+    ptr = (void*)GC_MALLOC_ATOMIC(size);
+  else
+    ptr = (void*)GC_MALLOC(size);
+
+  if (ptr == 0)
+    ptr = GSOutOfMemory(size, YES);
+  return ptr;
+}
+
+void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes)
+{
+  size_t	size = elems * bytes;
+  void		*ptr;
+
+  if (zone == __nszone_private_hidden_atomic_zone)
+    ptr = (void*)GC_MALLOC_ATOMIC(size);
+  else
+    ptr = (void*)GC_MALLOC(size);
+
+  if (ptr == 0)
+    ptr = GSOutOfMemory(size, NO);
+  memset(ptr, '\0', size);
+  return ptr;
+}
+
+void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
+{
+  ptr = GC_REALLOC(ptr, size);
+  if (ptr == 0)
+    GSOutOfMemory(size, NO);
+  return ptr;
+}
+
+void NSRecycleZone (NSZone *zone)
+{
+}
+
+void NSZoneFree (NSZone *zone, void *ptr)
+{
+  GC_FREE(ptr);
+}
+
+void NSSetZoneName (NSZone *zone, NSString *name)
+{
+}
+
+NSString* NSZoneName (NSZone *zone)
+{
+  return nil;
+}
+
+void* NSZoneMallocAtomic (NSZone *zone, size_t size)
+{
+  return NSZoneMalloc(GSAtomicMallocZone(), size);
+}
+
+BOOL NSZoneCheck (NSZone *zone)
+{
+  return YES;
+}
+
+struct NSZoneStats NSZoneStats (NSZone *zone)
+{
+  struct NSZoneStats stats = { 0 };
+  return stats;
+}
+
 #endif	/* GS_WITH_GC */
 
