@@ -49,12 +49,12 @@
 #include <Foundation/NSPortMessage.h>
 #include <Foundation/NSPortNameServer.h>
 #include <Foundation/NSLock.h>
+#include <Foundation/NSHost.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 #if	!defined(__WIN32__) || defined(__CYGWIN__)
 #include <unistd.h>		/* for gethostname() */
-#include <sys/param.h>		/* for MAXHOSTNAMELEN */
 #include <netinet/in.h>		/* for inet_ntoa() */
 #include <fcntl.h>
 #include <arpa/inet.h>
@@ -219,7 +219,7 @@ static NSHashTable	*tcpHandleTable = 0;
 	  handle->ports = 
                 NSCreateHashTable(NSNonRetainedObjectHashCallBacks, 0);
 	  NSHashInsert(tcpHandleTable, (void*)handle);
-	  AUTORELEASE(handle);
+	  IF_NO_GC(AUTORELEASE(handle));
 	}
     }
   [tcpHandleLock unlock];
@@ -455,7 +455,7 @@ static GSTcpPort	*dummyPort = nil;
       port->handles = NSCreateHashTable(NSObjectHashCallBacks, 0); 
       NSMapInsert(thePorts, (void*)(gsaddr)sockaddr->sin_addr.s_addr,
 	(void*)port);
-      AUTORELEASE(port);
+      IF_NO_GC(AUTORELEASE(port));
     }
 
   if (port != nil && handle != nil)
@@ -672,7 +672,7 @@ static GSTcpPort	*dummyPort = nil;
 		}
 	    }
 	}
-      AUTORELEASE(port);
+      IF_NO_GC(AUTORELEASE(port));
     }
 
   [tcpPortLock unlock];
@@ -1080,7 +1080,7 @@ static NSMapTable* port_number_2_port;
      don't create a new one, just return the old one. */
   if ((p = (id) NSMapGet (port_number_2_port, (void*)((int)n))))
     {
-      NSAssert(p->is_valid, NSInternalInconsistencyException);
+      NSAssert(p->_is_valid, NSInternalInconsistencyException);
       return [p retain];
     }
 
@@ -1110,8 +1110,7 @@ static NSMapTable* port_number_2_port;
      that when we encode the address, another machine can find us. */
   {
     struct hostent *hp;
-    char hostname[MAXHOSTNAMELEN];
-    int len = MAXHOSTNAMELEN;
+    NSString	*hostname;
     int	r;
 
     /* Set the re-use socket option so that we don't get this socket
@@ -1186,25 +1185,13 @@ static NSMapTable* port_number_2_port;
        machine so that, when we encoded our _LISTENING_ADDRESS for a
        Distributed Objects connection to another machine, they get our
        unique host address that can identify us across the network. */
-    if (gethostname (hostname, len) < 0)
-      {
-	[p release];
-	[NSException raise: NSInternalInconsistencyException
-	  format: @"[TcpInPort +newForReceivingFromPortNumber:] gethostname(): %s",
-	  strerror(errno)];
-      }
-    /* Terminate the name at the first dot. */
-    {
-      char *first_dot = strchr (hostname, '.');
-      if (first_dot)
-	*first_dot = '\0';
-    }
-    hp = gethostbyname (hostname);
+    hostname = [[NSHost currentHost] name];
+    hp = gethostbyname ([hostname cString]);
     if (!hp)
       hp = gethostbyname ("localhost");
     if (hp == 0)
       {
-	NSLog(@"Unable to get IP address of '%s' or of 'localhost'", hostname);
+	NSLog(@"Unable to get IP address of '%@' or of 'localhost'", hostname);
 #ifndef HAVE_INET_ATON
 	p->_listening_address.sin_addr.s_addr = inet_addr("127.0.0.1");
 #else
@@ -1283,7 +1270,7 @@ static NSMapTable* port_number_2_port;
 
 - (struct sockaddr_in*) _listeningSockaddr
 {
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
   return &_listening_address;
 }
 
@@ -1520,7 +1507,7 @@ static NSMapTable* port_number_2_port;
 {
   int s = [p _port_socket];
 
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
   /* Make sure it hasn't already been added. */
   NSAssert(!NSMapGet (_client_sock_2_out_port, (void*)s), NSInternalInconsistencyException);
 
@@ -1534,7 +1521,7 @@ static NSMapTable* port_number_2_port;
   id packet;
   int s = [p _port_socket];
 
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
   if (debug_tcp_port)
     NSLog(@"%s: Closed connection from\n %@\n",
 	     object_get_class_name (self), [p description]);
@@ -1572,7 +1559,7 @@ static NSMapTable* port_number_2_port;
 
 - (void) invalidate
 {
-  if (is_valid)
+  if (_is_valid)
     {
       NSMapEnumerator me = NSEnumerateMapTable (_client_sock_2_out_port);
       int count = NSCountMapTable (_client_sock_2_out_port);
@@ -1644,7 +1631,7 @@ static NSMapTable* port_number_2_port;
   return [NSString
 	   stringWithFormat: @"%s%c0x%x port %hd socket %d",
 	   object_get_class_name (self),
-	   is_valid ? ' ' : '-',
+	   _is_valid ? ' ' : '-',
 	   (unsigned)self,
 	   GSSwapBigI16ToHost(_listening_address.sin_port),
 	   _port_socket];
@@ -1668,7 +1655,7 @@ static NSMapTable* port_number_2_port;
 
 - (void) encodeWithCoder: aCoder
 {
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
   /* We are actually encoding a "send right" (ala Mach), 
      not a receive right.  
      These values must match those expected by [TcpOutPort +newWithCoder] */
@@ -1770,7 +1757,7 @@ static NSMapTable *out_port_bag = NULL;
 	    /* Assume that sin_family is equal.  Using memcmp() doesn't
 	       work because sin_zero's may differ. */
 	    {
-	      NSAssert(p->is_valid, NSInternalInconsistencyException);
+	      NSAssert(p->_is_valid, NSInternalInconsistencyException);
 	      return [p retain];
 	    }
 	}
@@ -1835,7 +1822,7 @@ static NSMapTable *out_port_bag = NULL;
 	}
       if (p)
 	{
-	  NSAssert(p->is_valid, NSInternalInconsistencyException);
+	  NSAssert(p->_is_valid, NSInternalInconsistencyException);
 	  return [p retain];
 	}
     }
@@ -1934,27 +1921,13 @@ static NSMapTable *out_port_bag = NULL;
   struct hostent *hp;
   const char *host_cstring;
   struct sockaddr_in addr;
-  /* Only used if no hostname is passed in. */
-  char local_hostname[MAXHOSTNAMELEN];
 
   /* Look up the hostname. */
   if (!hostname || ![hostname length])
     {
-      int len = sizeof (local_hostname);
-      char *first_dot;
-      if (gethostname (local_hostname, len) < 0)
-	{
-	  [NSException raise: NSInternalInconsistencyException
-	      format: @"[TcpInPort newForSendingToPortNumber:onHost:] gethostname(): %s",
-	      strerror(errno)];
-	}
-      host_cstring = local_hostname;
-      first_dot = strchr (host_cstring, '.');
-      if (first_dot)
-	*first_dot = '\0';
+      hostname = [[NSHost currentHost] name];
     }
-  else
-    host_cstring = [hostname cString];
+  host_cstring = [hostname cString];
   hp = gethostbyname ((char*)host_cstring);
   if (!hp)
     [self error: "unknown host: \"%s\"", host_cstring];
@@ -2016,7 +1989,7 @@ static NSMapTable *out_port_bag = NULL;
 {
   id reply_port = [packet replyInPort];
 
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
 
   /* If the socket of this TcpOutPort isn't already being polled
      for incoming data by a TcpInPort, and if the packet's REPLY_PORT
@@ -2070,7 +2043,7 @@ static NSMapTable *out_port_bag = NULL;
 
 - (void) invalidate
 {
-  if (is_valid)
+  if (_is_valid)
     {
       id	port = _polling_in_port;
 
@@ -2145,7 +2118,7 @@ static NSMapTable *out_port_bag = NULL;
   return [NSString 
 	   stringWithFormat: @"%s%c0x%x host %s port %hd socket %d",
 	   object_get_class_name (self),
-	   is_valid ? ' ' : '-',
+	   _is_valid ? ' ' : '-',
 	   (unsigned)self,
 	   inet_ntoa (_remote_in_port_address.sin_addr),
 	   GSSwapBigI16ToHost(_remote_in_port_address.sin_port),
@@ -2154,7 +2127,7 @@ static NSMapTable *out_port_bag = NULL;
 
 - (void) encodeWithCoder: aCoder
 {
-  NSAssert(is_valid, NSInternalInconsistencyException);
+  NSAssert(_is_valid, NSInternalInconsistencyException);
   NSAssert(!_polling_in_port 
 	  || (GSSwapBigI16ToHost(_remote_in_port_address.sin_port)
       != [_polling_in_port portNumber]), NSInternalInconsistencyException);
