@@ -1,5 +1,5 @@
 /* Implementation for UnixFileHandle for GNUStep
-   Copyright (C) 1997 Free Software Foundation, Inc.
+   Copyright (C) 1997-1999 Free Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Date: 1997
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 
 #include <config.h>
@@ -36,7 +36,7 @@
 #include <Foundation/NSHost.h>
 #include <Foundation/NSByteOrder.h>
 
-#ifdef WIN32
+#if	defined(__WIN32__) && !defined(__CYGWIN__)
 #include <Windows32/Sockets.h>
 #else
 #include <time.h>
@@ -46,7 +46,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <signal.h>
-#endif /* WIN32 */
+#endif /* __WIN32__ */
 
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -69,6 +69,18 @@
 #define NBLK_OPT     FNDELAY
 #endif
 
+#ifndef	O_BINARY
+#ifdef	_O_BINARY
+#define	O_BINARY	_O_BINARY
+#else
+#define	O_BINARY	0
+#endif
+#endif
+
+#ifndef	INADDR_NONE
+#define	INADDR_NONE	-1
+#endif
+
 // Maximum data in single I/O operation
 #define	NETBUF_SIZE	4096
 
@@ -88,8 +100,8 @@ static NSString*	NotificationKey = @"NSFileHandleNotificationKey";
 static BOOL
 getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 {
-  const char*		proto = "tcp";
-  struct servent*	sp;
+  const char		*proto = "tcp";
+  struct servent	*sp;
 
   if (pcl)
     proto = [pcl cString];
@@ -111,6 +123,8 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 #ifndef	HAVE_INET_ATON
       sin->sin_addr.s_addr = inet_addr([name cString]);
+      if (sin->sin_addr.s_addr == INADDR_NONE)
+	return NO;
 #else
       if (inet_aton([name cString], &sin->sin_addr) == 0)
 	return NO;
@@ -184,13 +198,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
   if (descriptor != -1)
     {
+      [self setNonBlocking: wasNonBlocking];
       if (closeOnDealloc == YES)
 	{
 	  close(descriptor);
 	  descriptor = -1;
 	}
-      else if (isNonBlocking != wasNonBlocking)
-	[self setNonBlocking: wasNonBlocking];
     }
   [readInfo release];
   [writeInfo release];
@@ -199,14 +212,14 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 // Initializing a UnixFileHandle Object
 
-- (id)init
+- (id) init
 {
   return [self initWithNullDevice];
 }
 
-- (id)initAsClientAtAddress: a
-		    service: s
-		   protocol: p
+- (id) initAsClientAtAddress: (NSString*)a
+		     service: (NSString*)s
+		    protocol: (NSString*)p
 {
   int	net;
   struct sockaddr_in	sin;
@@ -251,10 +264,10 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initAsClientInBackgroundAtAddress: a
-				service: s
-			       protocol: p
-			       forModes: modes
+- (id) initAsClientInBackgroundAtAddress: (NSString*)a
+				 service: (NSString*)s
+			        protocol: (NSString*)p
+			        forModes: (NSArray*)modes
 {
   int	net;
   struct sockaddr_in	sin;
@@ -314,9 +327,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initAsServerAtAddress: a
-		    service: s
-		   protocol: p
+- (id) initAsServerAtAddress: (NSString*)a
+		     service: (NSString*)s
+		    protocol: (NSString*)p
 {
   int	status = 1;
   int	net;
@@ -376,9 +389,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initForReadingAtPath: (NSString*)path
+- (id) initForReadingAtPath: (NSString*)path
 {
-  int	d = open([path fileSystemRepresentation], O_RDONLY);
+  int	d = open([path fileSystemRepresentation], O_RDONLY|O_BINARY);
 
   if (d < 0)
     {
@@ -394,9 +407,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (id)initForWritingAtPath: (NSString*)path
+- (id) initForWritingAtPath: (NSString*)path
 {
-  int	d = open([path fileSystemRepresentation], O_WRONLY);
+  int	d = open([path fileSystemRepresentation], O_WRONLY|O_BINARY);
 
   if (d < 0)
     {
@@ -412,9 +425,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (id)initForUpdatingAtPath: (NSString*)path
+- (id) initForUpdatingAtPath: (NSString*)path
 {
-  int	d = open([path fileSystemRepresentation], O_RDWR);
+  int	d = open([path fileSystemRepresentation], O_RDWR|O_BINARY);
 
   if (d < 0)
     {
@@ -427,7 +440,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (id)initWithStandardError
+- (id) initWithStandardError
 {
   if (fh_stderr)
     {
@@ -445,7 +458,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initWithStandardInput
+- (id) initWithStandardInput
 {
   if (fh_stdin)
     {
@@ -463,7 +476,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initWithStandardOutput
+- (id) initWithStandardOutput
 {
   if (fh_stdout)
     {
@@ -481,17 +494,18 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initWithNullDevice
+- (id) initWithNullDevice
 {
-  self = [self initWithFileDescriptor: open("/dev/null", O_RDWR)
+  self = [self initWithFileDescriptor: open("/dev/null", O_RDWR|O_BINARY)
 		       closeOnDealloc: YES];
-  if (self) {
-    isNullDevice = YES;
-  }
+  if (self)
+    {
+      isNullDevice = YES;
+    }
   return self;
 }
 
-- (id)initWithFileDescriptor: (int)desc closeOnDealloc: (BOOL)flag
+- (id) initWithFileDescriptor: (int)desc closeOnDealloc: (BOOL)flag
 {
   self = [super init];
   if (self)
@@ -511,10 +525,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
         isStandardFile = NO;
 
       if ((e = fcntl(desc, F_GETFL, 0)) >= 0)
-        if (e & NBLK_OPT)
-	  wasNonBlocking = YES;
-	else
-	  wasNonBlocking = NO;
+	{
+	  if (e & NBLK_OPT)
+	    wasNonBlocking = YES;
+	  else
+	    wasNonBlocking = NO;
+	}
 
       isNonBlocking = wasNonBlocking;
       descriptor = desc;
@@ -529,17 +545,17 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return self;
 }
 
-- (id)initWithNativeHandle: (void*)hdl
+- (id) initWithNativeHandle: (void*)hdl
 {
   return [self initWithFileDescriptor: (gsaddr)hdl closeOnDealloc: NO];
 }
 
-- (id)initWithNativeHandle: (void*)hdl closeOnDealloc: (BOOL)flag
+- (id) initWithNativeHandle: (void*)hdl closeOnDealloc: (BOOL)flag
 {
   return [self initWithFileDescriptor: (gsaddr)hdl closeOnDealloc: flag];
 }
 
-- (void)checkAccept
+- (void) checkAccept
 {
   if (acceptOK == NO)
     {
@@ -563,7 +579,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)checkConnect
+- (void) checkConnect
 {
   if (connectOK == NO)
     {
@@ -588,7 +604,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)checkRead
+- (void) checkRead
 {
   if (readOK == NO)
     {
@@ -612,7 +628,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)checkWrite
+- (void) checkWrite
 {
   if (writeOK == NO)
     {
@@ -634,21 +650,19 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 // Returning file handles
 
-- (int)fileDescriptor
+- (int) fileDescriptor
 {
-  if (isNullDevice)
-    return -1;
   return descriptor;
 }
 
-- (void*)nativeHandle
+- (void*) nativeHandle
 {
   return (void*)0;
 }
 
 // Synchronous I/O operations
 
-- (NSData*)availableData
+- (NSData*) availableData
 {
   char			buf[NETBUF_SIZE];
   NSMutableData*	d;
@@ -667,32 +681,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
   else
     {
-      int	count;
-
-      /*
-       *	Determine number of bytes readable on descriptor.
-       */
-      if (ioctl(descriptor, FIONREAD, (char*)&count) < 0)
+      if ((len = read(descriptor, buf, sizeof(buf))) > 0)
 	{
-	  [NSException raise: NSFileHandleOperationException
-		      format: @"unable to use FIONREAD on descriptor - %s",
-		      strerror(errno)];
-	}
-
-      if (count == 0)
-	{
-	  len = 0;	/* End-of-file	*/
-	}
-      else
-	{
-	  if (count > sizeof(buf))
-	    {
-	      count = sizeof(buf);
-	    }
-	  if ((len = read(descriptor, buf, count)) > 0)
-	    {
-	      [d appendBytes: buf length: len];
-	    }
+	  [d appendBytes: buf length: len];
 	}
     }
   if (len < 0)
@@ -776,7 +767,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return d;
 }
 
-- (void)writeData: (NSData*)item
+- (void) writeData: (NSData*)item
 {
   int		rval = 0;
   const void*	ptr = [item bytes];
@@ -808,7 +799,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 // Asynchronous I/O operations
 
-- (void)acceptConnectionInBackgroundAndNotifyForModes: (NSArray*)modes
+- (void) acceptConnectionInBackgroundAndNotifyForModes: (NSArray*)modes
 {
   [self checkAccept];
   readPos = 0;
@@ -819,12 +810,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self watchReadDescriptorForModes: modes];
 }
 
-- (void)acceptConnectionInBackgroundAndNotify
+- (void) acceptConnectionInBackgroundAndNotify
 {
   [self acceptConnectionInBackgroundAndNotifyForModes: nil];
 }
 
-- (void)readInBackgroundAndNotifyForModes: (NSArray*)modes
+- (void) readInBackgroundAndNotifyForModes: (NSArray*)modes
 {
   [self checkRead];
   readPos = 0;
@@ -837,12 +828,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self watchReadDescriptorForModes: modes];
 }
 
-- (void)readInBackgroundAndNotify
+- (void) readInBackgroundAndNotify
 {
   return [self readInBackgroundAndNotifyForModes: nil];
 }
 
-- (void)readToEndOfFileInBackgroundAndNotifyForModes: (NSArray*)modes
+- (void) readToEndOfFileInBackgroundAndNotifyForModes: (NSArray*)modes
 {
   [self checkRead];
   readPos = 0;
@@ -855,12 +846,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self watchReadDescriptorForModes: modes];
 }
 
-- (void)readToEndOfFileInBackgroundAndNotify
+- (void) readToEndOfFileInBackgroundAndNotify
 {
   return [self readToEndOfFileInBackgroundAndNotifyForModes: nil];
 }
 
-- (void)waitForDataInBackgroundAndNotifyForModes: (NSArray*)modes
+- (void) waitForDataInBackgroundAndNotifyForModes: (NSArray*)modes
 {
   [self checkRead];
   readPos = 0;
@@ -873,14 +864,14 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self watchReadDescriptorForModes: modes];
 }
 
-- (void)waitForDataInBackgroundAndNotify
+- (void) waitForDataInBackgroundAndNotify
 {
   return [self waitForDataInBackgroundAndNotifyForModes: nil];
 }
 
 // Seeking within a file
 
-- (unsigned long long)offsetInFile
+- (unsigned long long) offsetInFile
 {
   off_t	result = -1;
 
@@ -895,7 +886,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return (unsigned long long)result;
 }
 
-- (unsigned long long)seekToEndOfFile
+- (unsigned long long) seekToEndOfFile
 {
   off_t	result = -1;
 
@@ -910,7 +901,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   return (unsigned long long)result;
 }
 
-- (void)seekToFileOffset: (unsigned long long)pos
+- (void) seekToFileOffset: (unsigned long long)pos
 {
   off_t	result = -1;
 
@@ -927,7 +918,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 // Operations on file
 
-- (void)closeFile
+- (void) closeFile
 {
   if (descriptor < 0)
     [NSException raise: NSFileHandleOperationException
@@ -936,6 +927,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self ignoreReadDescriptor];
   [self ignoreWriteDescriptor];
 
+  [self setNonBlocking: wasNonBlocking];
   (void)close(descriptor);
   descriptor = -1;
   acceptOK = NO;
@@ -965,20 +957,20 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)synchronizeFile
+- (void) synchronizeFile
 {
   if (isStandardFile)
     (void)sync();
 }
 
-- (void)truncateFileAtOffset: (unsigned long long)pos
+- (void) truncateFileAtOffset: (unsigned long long)pos
 {
   if (isStandardFile && descriptor >= 0)
     (void)ftruncate(descriptor, pos);
-   [self seekToFileOffset: pos];
+  [self seekToFileOffset: pos];
 }
 
-- (void)writeInBackgroundAndNotify: (NSData*)item forModes: (NSArray*)modes
+- (void) writeInBackgroundAndNotify: (NSData*)item forModes: (NSArray*)modes
 {
   NSMutableDictionary*	info;
 
@@ -996,17 +988,17 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   [self watchWriteDescriptor];
 }
 
-- (void)writeInBackgroundAndNotify: (NSData*)item;
+- (void) writeInBackgroundAndNotify: (NSData*)item;
 {
   [self writeInBackgroundAndNotify: item forModes: nil];
 }
 
-- (void)postReadNotification
+- (void) postReadNotification
 {
-  NSMutableDictionary*	info = readInfo;
-  NSNotification*	n;
-  NSArray*		modes;
-  NSString*		name;
+  NSMutableDictionary	*info = readInfo;
+  NSNotification	*n;
+  NSArray		*modes;
+  NSString		*name;
 
   [self ignoreReadDescriptor];
   readInfo = nil;
@@ -1023,12 +1015,12 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 		forModes: modes];
 }
 
-- (void)postWriteNotification
+- (void) postWriteNotification
 {
-  NSMutableDictionary*	info = [writeInfo objectAtIndex: 0];
-  NSNotification*	n;
-  NSArray*		modes;
-  NSString*		name;
+  NSMutableDictionary	*info = [writeInfo objectAtIndex: 0];
+  NSNotification	*n;
+  NSArray		*modes;
+  NSString		*name;
 
   [self ignoreWriteDescriptor];
   modes = (NSArray*)[info objectForKey: NSFileHandleNotificationMonitorModes];
@@ -1047,21 +1039,21 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     [self watchWriteDescriptor];	/* In case of queued writes.	*/
 }
 
-- (BOOL)readInProgress
+- (BOOL) readInProgress
 {
   if (readInfo)
     return YES;
   return NO;
 }
 
-- (BOOL)writeInProgress
+- (BOOL) writeInProgress
 {
   if ([writeInfo count] > 0)
     return YES;
   return NO;
 }
 
-- (void)ignoreReadDescriptor
+- (void) ignoreReadDescriptor
 {
   NSRunLoop	*l;
   NSArray	*modes;
@@ -1094,7 +1086,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	       all: YES];
 }
 
-- (void)ignoreWriteDescriptor
+- (void) ignoreWriteDescriptor
 {
   NSRunLoop	*l;
   NSArray	*modes;
@@ -1131,7 +1123,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	       all: YES];
 }
 
-- (void)watchReadDescriptorForModes: (NSArray*)modes;
+- (void) watchReadDescriptorForModes: (NSArray*)modes;
 {
   NSRunLoop	*l;
 
@@ -1162,7 +1154,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)watchWriteDescriptor
+- (void) watchWriteDescriptor
 {
   if (descriptor < 0)
     return;
@@ -1199,10 +1191,10 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (void)receivedEvent: (void*)data
-                 type: (RunLoopEventType)type
-		extra: (void*)extra
-	      forMode: (NSString*)mode
+- (void) receivedEvent: (void*)data
+                  type: (RunLoopEventType)type
+		 extra: (void*)extra
+	       forMode: (NSString*)mode
 {
   NSString	*operation;
 
@@ -1216,7 +1208,6 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	  struct sockaddr_in	buf;
 	  int			desc;
 	  int			blen = sizeof(buf);
-	  NSFileHandle*		hdl;
 
 	  desc = accept(descriptor, (struct sockaddr*)&buf, &blen);
 	  if (desc < 0)
@@ -1233,7 +1224,8 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	      struct sockaddr_in	sin;
 	      int			size = sizeof(sin);
 
-	      h = [[UnixFileHandle alloc] initWithFileDescriptor: desc];
+	      h = [[UnixFileHandle alloc] initWithFileDescriptor: desc
+						  closeOnDealloc: YES];
 	      getpeername(desc, (struct sockaddr*)&sin, &size);
 	      [h setAddr: &sin];
 	      [readInfo setObject: h
@@ -1351,9 +1343,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
 }
 
-- (NSDate*)timedOutEvent: (void*)data
-		    type: (RunLoopEventType)type
-		 forMode: (NSString*)mode
+- (NSDate*) timedOutEvent: (void*)data
+		     type: (RunLoopEventType)type
+		  forMode: (NSString*)mode
 {
   return nil;		/* Don't restart timed out events	*/
 }
@@ -1368,7 +1360,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   protocol = @"tcp";
 }
 
-- (void)setNonBlocking: (BOOL)flag
+- (void) setNonBlocking: (BOOL)flag
 {
   int	e;
 
