@@ -165,7 +165,7 @@ mframe_build_signature(const char *typePtr, int *size, int *narg, char *buf)
    */
   if (doMalloc)
     {
-      char	*tmp = objc_malloc(dest - buf + 1);
+      char	*tmp = NSZoneMalloc(NSDefaultMallocZone(), dest - buf + 1);
 
       strcpy(tmp, buf);
       buf = tmp;
@@ -218,6 +218,9 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	  case _C_BYREF:  info->qual |= _F_BYREF; break;
 #endif
 	  case _C_ONEWAY: info->qual |= _F_ONEWAY; break;
+#ifdef	_C_GCINVISIBLE
+	  case _C_GCINVISIBLE:  info->qual |= _F_GCINVISIBLE; break;
+#endif
 	  default: flag = NO;
 	}
       if (flag)
@@ -288,7 +291,6 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	info->align = __alignof__(unsigned long);
 	break;
 
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:
 	info->size = sizeof(long long);
 	info->align = __alignof__(long long);
@@ -299,7 +301,6 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	info->align = __alignof__(unsigned long long);
 	break;
 
-#endif
       case _C_FLT:
 	info->size = sizeof(float);
 	info->align = __alignof__(float);
@@ -1355,7 +1356,7 @@ mframe_build_return_opts (arglist_t argframe,
 		  retLength = objc_sizeof_type(tmptype);
 		  /* Allocate memory to hold the value we're pointing to. */
 		  *(void**)retframe = 
-		    objc_malloc (retLength);
+		    NSZoneMalloc(NSDefaultMallocZone(), retLength);
 		  /* We are responsible for making sure this memory gets free'd
 		     eventually.  Ask NSData class to autorelease it. */
 		  [NSData dataWithBytesNoCopy: *(void**)retframe
@@ -1559,9 +1560,9 @@ mframe_destroy_argframe(const char *types, arglist_t argframe)
 
   if (stack_argsize)
     {
-      objc_free(argframe->arg_ptr);
+      NSZoneFree(NSDefaultMallocZone(), argframe->arg_ptr);
     }
-  objc_free(argframe);
+  NSZoneFree(NSDefaultMallocZone(), argframe);
 }
 
 
@@ -1650,7 +1651,6 @@ mframe_decode_return (const char *type, void* buffer, void* retframe)
 	break;
       }
 
-#ifdef	_C_LNG_LNG
     case _C_LNG_LNG:
     case _C_ULNG_LNG:
       {
@@ -1661,7 +1661,6 @@ mframe_decode_return (const char *type, void* buffer, void* retframe)
 	*(unsigned long long*)buffer = retframe_longlong(retframe);
 	break;
       }
-#endif
 
     case _C_FLT:
       {
