@@ -4,21 +4,18 @@
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Created: October 1997
 
-   This file is part of the GNUstep Base Library.
+   This file is part of the GNUstep Project
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+    
+   You should have received a copy of the GNU General Public  
+   License along with this library; see the file COPYING.LIB.
+   If not, write to the Free Software Foundation,
+   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
 
 #include	<Foundation/NSArray.h>
@@ -55,7 +52,7 @@ main(int argc, char** argv)
 
     args = [proc arguments];
 
-    for (i = 0; i < [args count]; i++) {
+    for (i = 1; i < [args count]; i++) {
         if ([[args objectAtIndex: i] isEqual: @"--help"]) {
 	    printf(
 "The 'dwrite' command lets you modify a user's defaults database.\n"
@@ -83,7 +80,7 @@ main(int argc, char** argv)
 	}
     }
 
-    i = 0;
+    i = 1;
     if ([args count] > i && [[args objectAtIndex: i] isEqual: @"-u"]) {
 	if ([args count] > ++i) {
 	    user = [args objectAtIndex: i++];
