@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <string.h>		/* for strchr() */
+#include <ctype.h>		/* for strchr() */
 #ifndef __WIN32__
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -96,7 +97,7 @@ typedef	unsigned char	*uptr;
 int	debug = 0;		/* Extra debug logging.			*/
 int	nofork = 0;		/* turn off fork() for debugging.	*/
 int	noprobe = 0;		/* turn off probe for unknown servers.	*/
-int	interval = 300;		/* Minimum time (sec) between probes.	*/
+int	interval = 600;		/* Minimum time (sec) between probes.	*/
 
 int	udp_sent = 0;
 int	tcp_sent = 0;
@@ -451,7 +452,7 @@ map_by_port(unsigned p, unsigned char t)
     {
       if (debug > 2)
 	{
-	  fprintf(stderr, "Found port %d with name %.*s\n",
+	  fprintf(stderr, "Found port %d with name %s\n",
 		map[index]->port, map[index]->name);
 	}
       return map[index];
@@ -508,17 +509,17 @@ typedef struct	{
 } prb_type;
 prb_type	**prb = 0;
 
-static prb_type	*prb_get(struct in_addr *old);
-
 /*
  *	Name -		prb_add()
- *	Purpose -	Create a new probe entry in the list in the
- *			appropriate position.
+ *	Purpose -	Create a new probe entry in the list.
+ *			The new entry is always placed at the end of the list
+ *			so that the list remains in the order in which hosts
+ *			have been contancted.
  */
 static void
 prb_add(struct in_addr *p)
 {
-  prb_type	*n;
+  prb_type	*n = 0;
   int		i;
 
   if (is_local_host(*p) != 0)
@@ -530,16 +531,37 @@ prb_add(struct in_addr *p)
       return;
     }
     
-  n = prb_get(p);
-  if (n)
+  /*
+   * If we already have an entry for this address, remove it from the list
+   * ready for re-insertion in the correct place.
+   */
+  for (i = 0; i < prb_used; i++)
     {
-      n->when = time(0);
-      return;
+      if (memcmp(&prb[i]->sin, p, IASIZE) == 0)
+	{
+	  n = prb[i];
+	  for (i++; i < prb_used; i++)
+	    {
+	      prb[i-1] = prb[i];
+	    }
+	  prb_used--;
+	}
+    }
+
+  /*
+   * Create a new entry structure if necessary.
+   * Set the current time in the structure, so we know when we last had contact.
+   */
+  if (n == 0)
+    {
+      n = (prb_type*)malloc(sizeof(prb_type));
+      n->sin = *p;
     }
-  n = (prb_type*)malloc(sizeof(prb_type));
-  n->sin = *p;
   n->when = time(0);
 
+  /*
+   * Grow the list if we need more space.
+   */
   if (prb_used >= prb_size)
     {
       int	size = (prb_size + 16) * sizeof(prb_type*);
@@ -555,57 +577,14 @@ prb_add(struct in_addr *p)
 	  prb_size = 16;
 	}
     }
-  for (i = 0; i < prb_used; i++)
-    {
-      if (memcmp((char*)&prb[i]->sin, (char*)&n->sin, IASIZE) > 0)
-	{
-	  int	j;
 
-	  for (j = prb_used+1; j > i; j--)
-	    {
-	      prb[j] = prb[j-1];
-	    }
-	  break;
-	}
-    }
-  prb[i] = n;
+  /*
+   * Append the new item at the end of the list.
+   */
+  prb[prb_used] = n;
   prb_used++;
 }
 
-/*
- *	Name -		prb_get()
- *	Purpose -	Search the list for an entry for a particular addr
- */
-static prb_type*
-prb_get(struct in_addr *p)
-{
-  int		lower = 0;
-  int		upper = prb_used;
-  int		index;
-
-  for (index = upper/2; upper != lower; index = lower + (upper - lower)/2)
-    {
-      int	i = memcmp(&prb[index]->sin, p, IASIZE);
-
-      if (i < 0)
-	{
-	  lower = index + 1;
-        }
-      else if (i > 0)
-	{
-	  upper = index;
-        }
-      else
-	{
-	  break;
-        }
-    }
-  if (index<prb_used && memcmp(&prb[index]->sin,p,IASIZE)==0)
-    {
-      return prb[index];
-    }
-  return 0;
-}
 
 /*
  *	Name -		prb_del()
@@ -700,7 +679,7 @@ dump_stats()
   fprintf(stderr, "tcp messages waiting for send - %d\n", tcp_pending);
   fprintf(stderr, "udp messages waiting for send - %d\n", udp_pending);
   fprintf(stderr, "size of name-to-port map - %d\n", map_used);
-  fprintf(stderr, "number of known name servers - %d\n", prb_used);
+  fprintf(stderr, "number of known name servers - %ld\n", prb_used);
   fprintf(stderr, "TCP %d read, %d sent\n", tcp_read, tcp_sent);
   fprintf(stderr, "UDP %d read, %d sent\n", udp_read, udp_sent);
 }
@@ -777,57 +756,54 @@ init_iface()
   final = (struct ifreq*)&ifc.ifc_buf[ifc.ifc_len];
   for (ifr = ifc.ifc_req; ifr < final; ifr++)
     {
-      if (ifr->ifr_addr.sa_family == AF_INET)
-	{	/* IP interface */
-	  ifreq = *ifr;
-	  if (ioctl(desc, SIOCGIFFLAGS, (char *)&ifreq) < 0)
-	    {
-	      perror("SIOCGIFFLAGS");
-	    }
-	  else if (ifreq.ifr_flags & IFF_UP)
-	    {	/* active interface */
-	      if (ioctl(desc, SIOCGIFADDR, (char *)&ifreq) < 0)
-		{
-		  perror("SIOCGIFADDR");
-		}
-	      else
-		{
-		  if (interfaces >= MAX_IFACE)
-		    {
-		      fprintf(stderr,
+      ifreq = *ifr;
+      if (ioctl(desc, SIOCGIFFLAGS, (char *)&ifreq) < 0)
+        {
+          perror("SIOCGIFFLAGS");
+        }
+      else if (ifreq.ifr_flags & IFF_UP)
+        {  /* interface is up */
+          if (ioctl(desc, SIOCGIFADDR, (char *)&ifreq) < 0)
+            {
+              perror("SIOCGIFADDR");
+            }
+          else if (ifreq.ifr_addr.sa_family == AF_INET)
+            {	/* IP interface */
+	      if (interfaces >= MAX_IFACE)
+	        {
+	          fprintf(stderr,
 "You have too many network interfaces on your machine (in which case you need\n"
 "to change the 'MAX_IFACE' constant in gdomap.c and rebuild it), or your\n"
 "system is buggy, and you need to use the '-a' command line flag for\n"
 "gdomap to manually set the interface addresses and masks to be used.\n");
-		      close(desc);
-		      exit(1);
-		    }
-		  addr[interfaces] =
-			((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
-		  if (ioctl(desc, SIOCGIFNETMASK, (char *)&ifreq) < 0)
-		    {
-		      perror("SIOCGIFNETMASK");
-		      /*
-		       *	If we can't get a netmask - assume a class-c
-		       *	network.
-		       */
-		      mask[interfaces] = class_c_mask;
-		    }
-		  else
-		    {
+	          close(desc);
+	          exit(1);
+	        }
+	      addr[interfaces] =
+		((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
+	      if (ioctl(desc, SIOCGIFNETMASK, (char *)&ifreq) < 0)
+	        {
+	          perror("SIOCGIFNETMASK");
+		  /*
+		   *	If we can't get a netmask - assume a class-c
+		   *	network.
+		   */
+		   mask[interfaces] = class_c_mask;
+		}
+              else
+                {
 /*
  *	Some systems don't have ifr_netmask
  */
 #ifdef	ifr_netmask
-		      mask[interfaces] =
-		        ((struct sockaddr_in *)&ifreq.ifr_netmask)->sin_addr;
+                  mask[interfaces] =
+                    ((struct sockaddr_in *)&ifreq.ifr_netmask)->sin_addr;
 #else
-		      mask[interfaces] =
-		        ((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
+                  mask[interfaces] =
+                    ((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
 #endif
-		    }
-		  interfaces++;
-		}
+                }
+              interfaces++;
 	    }
 	}
     }
@@ -1083,6 +1059,12 @@ init_ports()
   if (bind(udp_desc, (void*)&sa, sizeof(sa)) < 0)
     {
       fprintf(stderr, "Unable to bind address to UDP socket\n");
+      if (errno == EACCES)
+	{
+	  fprintf(stderr, "You probably need to run gdomap as root,\n");
+	  fprintf(stderr, "or run the nameserver on a non-standard\n");
+	  fprintf(stderr, "port that does not require root privilege.\n");
+	}
       exit(1);
     }
 
@@ -1120,6 +1102,12 @@ init_ports()
   if (bind(tcp_desc, (void*)&sa, sizeof(sa)) < 0)
     {
       fprintf(stderr, "Unable to bind address to TCP socket\n");
+      if (errno == EACCES)
+	{
+	  fprintf(stderr, "You probably need to run gdomap as root,\n");
+	  fprintf(stderr, "or run the nameserver on a non-standard\n");
+	  fprintf(stderr, "port that does not require root privilege.\n");
+	}
       exit(1);
     }
   if (listen(tcp_desc, QUEBACKLOG) < 0)
@@ -1544,7 +1532,6 @@ handle_io()
       else if (rval == 0)
 	{
 	  long		now = time(0);
-	  int		i;
 
 	  /*
 	   *	Let's handle a timeout.
@@ -1706,7 +1693,7 @@ handle_request(int desc)
 	{
 	  fprintf(stderr, "request type '%c' from chan %d", type, desc);
 	}
-      fprintf(stderr, " - name: '%.*s' port: %d\n", size, buf, port);
+      fprintf(stderr, " - name: '%.*s' port: %ld\n", size, buf, port);
     }
 
   if (ptype != GDO_TCP_GDO && ptype != GDO_TCP_FOREIGN &&
@@ -1819,7 +1806,7 @@ handle_request(int desc)
 		    {
 		      if (debug > 1)
 			{
-			  fprintf(stderr, "re-register from %d to %d\n",
+			  fprintf(stderr, "re-register from %d to %ld\n",
 				m->port, port);
 			}
 		      m->port = port;
@@ -1972,15 +1959,24 @@ handle_request(int desc)
   else if (type == GDO_SERVERS)
     {
       int	i;
+      int	j;
 
       free(w_info[desc].buf);
       w_info[desc].buf = (char*)malloc(sizeof(unsigned long) +
 		(prb_used+1)*IASIZE);
       *(unsigned long*)w_info[desc].buf = htonl(prb_used+1);
       mcopy(&w_info[desc].buf[4], &r_info[desc].addr.sin_addr, IASIZE);
-      for (i = 0; i < prb_used; i++)
+
+      /*
+       * Copy the addresses of the hosts we have probed into the buffer.
+       * During the copy, reverse the order of the addresses so that the
+       * address we have contacted most recently is first.  This should
+       * ensure that the client process will attempt to contact live
+       * hosts before dead ones.
+       */
+      for (i = 0, j = prb_used; i < prb_used; i++)
 	{
-	  mcopy(&w_info[desc].buf[4+(i+1)*IASIZE], &prb[i]->sin, IASIZE);
+	  mcopy(&w_info[desc].buf[4+(i+1)*IASIZE], &prb[--j]->sin, IASIZE);
 	}
       w_info[desc].len = 4 + (prb_used+1)*IASIZE;
     }
@@ -2197,9 +2193,7 @@ handle_send()
 		  fprintf(stderr, "failed sendto for %s\n",
 			    inet_ntoa(entry->addr.sin_addr));
 		}
-	      u_queue = entry->next;
-	      free(entry->dat);
-	      free(entry);
+	      queue_pop();
 	    }
 	}
       else
@@ -2903,7 +2897,7 @@ main(int argc, char** argv)
 	    printf("-c file		use config file for probe.\n");
 	    printf("-d		extra debug logging.\n");
 	    printf("-f		avoid fork() to make debugging easy\n");
-	    printf("-i seconds	re-probe at this interval (roughly)\n");
+	    printf("-i seconds	re-probe at this interval (roughly), min 60\n");
 	    printf("-p		obsolete no-op\n");
 	    printf("\n");
 	    exit(0);
@@ -2970,7 +2964,7 @@ printf(
 		fprintf(stderr, "Registration will take place locally.\n");
 	      }
 	    doregister(optarg, port, ptype);
-	    return;
+	    return 0;
 
 	  case 'T':
 	    if (strcmp(optarg, "tcp_gdo") == 0)
@@ -3171,16 +3165,15 @@ printf(
    *	Ensure we don't have any open file descriptors which may refer
    *	to sockets bound to ports we may try to use.
    *
-   *	Use '/dev/tty' to produce logging output and use '/dev/null'
-   *	for stdin and stdout.
+   *	Use '/dev/null' for stdin and stdout.  Assume stderr is ok.
    */
   for (c = 0; c < FD_SETSIZE; c++)
     {
-      (void)close(c);
+      if (c != 2)
+	(void)close(c);
     }
   (void)open("/dev/null", O_RDONLY);	/* Stdin.	*/
   (void)open("/dev/null", O_WRONLY);	/* Stdout.	*/
-  (void)open("/dev/tty", O_WRONLY);	/* Stderr.	*/
 
   init_my_port();	/* Determine port to listen on.		*/
   if (interfaces == 0)
