@@ -55,10 +55,12 @@
 @class NSText,NSCStringText;
 // Controls
 @class NSControl, NSButton, NSTextField, NSScroller, NSBox, NSForm, NSMatrix;
-@class NSPopUpButton, NSSlider, NSBrowser, NSForm;
+@class NSPopUpButton, NSProgressIndicator, NSSlider, NSBrowser, NSForm;
+@class NSComboBox;
 // Cells
 @class NSCell, NSActionCell, NSButtonCell, NSTextFieldCell, NSFormCell;
 @class NSSliderCell, NSMenuCell, NSBrowserCell, NSFormCell;
+@class NSComboBoxCell;
 @class NSEvent, NSCursor;
 @class NSColor, NSColorList, NSColorPanel, NSColorPicker, NSColorWell;
 @class NSImage, NSImageCell, NSImageRep, NSBitmapImageRep, NSCachedImageRep;
@@ -73,6 +75,7 @@
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSAttributedString.h>
+#include <AppKit/NSBezierPath.h>
 #include <AppKit/NSBitmapImageRep.h>
 #include <AppKit/NSBox.h>
 #include <AppKit/NSBrowser.h>
@@ -89,6 +92,8 @@
 #include <AppKit/NSColorPicker.h>
 #include <AppKit/NSColorPicking.h>
 #include <AppKit/NSColorWell.h>
+#include <AppKit/NSComboBox.h>
+#include <AppKit/NSComboBoxCell.h>
 #include <AppKit/NSControl.h>
 #include <AppKit/NSCursor.h>
 #include <AppKit/NSCustomImageRep.h>
@@ -110,6 +115,7 @@
 #include <AppKit/NSImageRep.h>
 #include <AppKit/NSImageView.h>
 #include <AppKit/NSInterfaceStyle.h>
+#include <AppKit/NSLayoutManager.h>
 #include <AppKit/NSMatrix.h>
 #include <AppKit/NSMenu.h>
 #include <AppKit/NSMenuItem.h>
@@ -127,6 +133,7 @@
 #include <AppKit/NSPrintInfo.h>
 #include <AppKit/NSPrintOperation.h>
 #include <AppKit/NSPrintPanel.h>
+#include <AppKit/NSProgressIndicator.h>
 #include <AppKit/NSResponder.h>
 #include <AppKit/NSSavePanel.h>
 #include <AppKit/NSScreen.h>
@@ -140,11 +147,14 @@
 #include <AppKit/NSSpellServer.h>
 #include <AppKit/NSSplitView.h>
 #include <AppKit/NSStringDrawing.h>
+#include <AppKit/NSTableColumn.h>
 #include <AppKit/NSTabView.h>
 #include <AppKit/NSTabViewItem.h>
 #include <AppKit/NSText.h>
+#include <AppKit/NSTextContainer.h>
 #include <AppKit/NSTextField.h>
 #include <AppKit/NSTextFieldCell.h>
+#include <AppKit/NSTextStorage.h>
 #include <AppKit/NSTextView.h>
 #include <AppKit/NSView.h>
 #include <AppKit/NSWindow.h>
