@@ -20,14 +20,14 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
 #ifndef _GNUstep_H_GMAppKit
 #define _GNUstep_H_GMAppKit
 
 #include <AppKit/AppKit.h>
-#import <extensions/GMArchiver.h>
+#import <AppKit/GMArchiver.h>
 
 @interface NSApplication (GMArchiverMethods) <ModelCoding>
 @end
@@ -50,8 +50,10 @@
 @interface NSControl (GMArchiverMethods) <ModelCoding>
 @end
 
+#ifndef __APPLE__ /* On MacOSX NSCStringText is depricated. */
 @interface NSCStringText (GMArchiverMethods) <ModelCoding>
 @end
+#endif
 
 @interface NSFont (GMArchiverMethods) <ModelCoding>
 @end
