@@ -32,20 +32,14 @@
 #include <AppKit/NSCell.h>
 
 @class NSImage;
-@class NSTextFieldCell;
 
 @interface NSBrowserCell : NSCell <NSCoding>
 {
   // Attributes
   NSImage *_alternateImage;
-  NSImage *_branchImage;
-  NSImage *_highlightBranchImage;
-  NSCell *_browserText;
-  BOOL _isLeaf;
-  BOOL _isLoaded;
-
-  // reserved for the backend
-  void *_be_bfc_reserved;
+  // Think of the following ones as of two BOOL ivars
+#define _browsercell_is_leaf    _cell.subclass_bool_one
+#define _browsercell_is_loaded  _cell.subclass_bool_two
 }
 
 //
