@@ -36,11 +36,14 @@
 #include <Foundation/NSCoder.h>
 #include <Foundation/NSSet.h>
 
-#include <AppKit/NSPanel.h>
 #include <AppKit/NSBrowser.h>
+#include <AppKit/NSButton.h>
+#include <AppKit/NSForm.h>
+#include <AppKit/NSFormCell.h>
+#include <AppKit/NSPanel.h>
+#include <AppKit/NSTextField.h>
 
 @class NSString;
-@class NSView;
 
 enum {
   NSFileHandlingPanelImageButton,
@@ -56,29 +59,33 @@ enum {
 
 @interface NSSavePanel : NSPanel <NSCoding>
 {
-  id _accessoryView;
-  id _bottomView;
-  id _delegate;
-  id _browser;
-  id _form;
-  id _prompt;
-  id _titleField;
-  id _topView;
+  NSView *_accessoryView;
+  NSView *_bottomView;
+  NSBrowser *_browser;
+  NSForm *_form;
+  NSButton *_okButton;
+  NSTextField *_titleField;
+  NSView *_topView;
+
+  NSSize _originalMinSize;
+  NSSize _originalSize;
 
   NSString *_requiredFileType;
-  NSString *_lastValidPath;
+  NSString *_directory;
+  NSString *_fullFileName;
 
   BOOL _treatsFilePackagesAsDirectories;
   BOOL _delegateHasCompareFilter;
-  BOOL _delegateHasFilenameFilter;
+  BOOL _delegateHasShowFilenameFilter;
   BOOL _delegateHasValidNameFilter;
+  BOOL _selfHasShowExtensionFilter;
 
-  NSRect _oldContentFrame;
-  NSRect _oldTopViewFrame;
+  // YES when we stopped because the user pressed 'OK'
+  BOOL _OKButtonPressed;
 }
 
 /*
- * Creating an NSSavePanel
+ * Getting the NSSavePanel shared instance
  */
 + (NSSavePanel *) savePanel;
 
@@ -190,20 +197,12 @@ enum {
  * Responding to User Input
  */
 - (void) selectText: (id)sender;
-
-/*
- * NSCoding protocol
- */
-- (void) encodeWithCoder: (id)aCoder;
-- (id) initWithCoder: (id)aDecoder;
-
 @end
 
 /*
  * Methods Implemented by the Delegate 
  */
 @interface NSObject (NSSavePanelDelegate)
-
 /*
  * The NSSavePanel sends this message just before the end of a 
  * modal session for each file name displayed or selected 
@@ -221,7 +220,6 @@ enum {
 			with: (NSString *)filename2
 	       caseSensitive: (BOOL)caseSensitive;	 
 - (BOOL) panel: (id)sender shouldShowFilename: (NSString *)filename;
-
 @end
 
 #endif /* _GNUstep_H_NSSavePanel */
