@@ -27,13 +27,19 @@
 #import <Foundation/NSException.h>
 #include "AppKit/GMAppKit.h"
 
+#ifndef AUTORELEASE
+#define AUTORELEASE(object)	[object autorelease]
+#define RELEASE(object)		[object release]
+#define RETAIN(object)		[object retain]
+#endif
+
 void __dummy_GMAppKit_functionForLinking() {}
 
 @implementation NSApplication (GMArchiverMethods)
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-#if NeXT_GUI_LIBRARY
+#ifndef GNU_GUI_LIBRARY
   NSArray* windows1 = [self windows];
   NSMutableArray* windows2 = [NSMutableArray array];
   int i, count = [windows1 count];
@@ -77,7 +83,7 @@ void __dummy_GMAppKit_functionForLinking() {}
   if (anObject)
     [self setDelegate:anObject];
 
-#if NeXT_GUI_LIBRARY
+#ifndef GNU_GUI_LIBRARY
   mainMenu = [unarchiver decodeObjectWithName:@"mainMenu"];
   if (mainMenu)
     [self setMainMenu:mainMenu];
@@ -372,6 +378,7 @@ void __dummy_GMAppKit_functionForLinking() {}
     int nr, nc;
     NSArray *cell_array;
     int i;
+    id decodedDelegate;
     
     self = [super initWithModelUnarchiver:unarchiver];
 
@@ -399,9 +406,11 @@ void __dummy_GMAppKit_functionForLinking() {}
     for (i = 0; (i < [cell_array count]) && (i < nr*nc); i++) {
         [self putCell:[cell_array objectAtIndex:i] atRow:i/nc column:i%nc];
     }
-    
-    [self setDelegate:[unarchiver decodeObjectWithName:@"delegate"]];
-    
+
+    decodedDelegate = [unarchiver decodeObjectWithName:@"delegate"];
+    if (decodedDelegate)
+      [self setDelegate:decodedDelegate];
+
 
     [self setTarget:[unarchiver decodeObjectWithName:@"target"]];
     [self setAction:[unarchiver decodeSelectorWithName:@"action"]];
@@ -425,28 +434,40 @@ void __dummy_GMAppKit_functionForLinking() {}
     return [NSArray array];
 }
 
-- (void)encodeWithModelArchiver:(GMArchiver*)archiver
+- (void) encodeWithModelArchiver: (GMArchiver*)archiver
 {
-    [super encodeWithModelArchiver:archiver];
+  [super encodeWithModelArchiver: archiver];
 
-    [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
-    [archiver encodeInt:[self borderType] withName:@"borderType"];
-    [archiver encodeBOOL:[self hasHorizontalScroller] withName:@"hasHorizontalScroller"];
-    [archiver encodeBOOL:[self hasVerticalScroller] withName:@"hasVerticalScroller"];
-    [archiver encodeObject:[self documentView] withName:@"documentView"];
+  [archiver encodeObject: [self backgroundColor]
+		withName: @"backgroundColor"];
+  [archiver encodeInt: [self borderType]
+	     withName: @"borderType"];
+  [archiver encodeBOOL: [self hasHorizontalScroller]
+	      withName: @"hasHorizontalScroller"];
+  [archiver encodeBOOL: [self hasVerticalScroller]
+	      withName: @"hasVerticalScroller"];
+  [archiver encodeObject: [self documentView]
+		withName: @"documentView"];
 }
 
-- (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
+- (id) initWithModelUnarchiver: (GMUnarchiver*)unarchiver
 {
-    self = [super initWithModelUnarchiver:unarchiver];
+  self = [super initWithModelUnarchiver: unarchiver];
 
-    [self setBackgroundColor:[unarchiver decodeObjectWithName:@"backgroundColor"]];
-    [self setBorderType:[unarchiver decodeIntWithName:@"borderType"]];
-    [self setHasHorizontalScroller:[unarchiver decodeBOOLWithName:@"hasHorizontalScroller"]];
-    [self setHasVerticalScroller:[unarchiver decodeBOOLWithName:@"hasVerticalScroller"]];
-    [self setDocumentView:[unarchiver decodeObjectWithName:@"documentView"]];
+  [self setContentView: AUTORELEASE([NSClipView new])];
 
-    return self;
+  [self setBackgroundColor:
+    [unarchiver decodeObjectWithName: @"backgroundColor"]];
+  [self setBorderType:
+    [unarchiver decodeIntWithName: @"borderType"]];
+  [self setHasHorizontalScroller:
+    [unarchiver decodeBOOLWithName: @"hasHorizontalScroller"]];
+  [self setHasVerticalScroller:
+    [unarchiver decodeBOOLWithName: @"hasVerticalScroller"]];
+  [self setDocumentView:
+    [unarchiver decodeObjectWithName: @"documentView"]];
+
+  return self;
 }
 
 @end /* NSScrollView (GMArchiverMethods) */
@@ -599,6 +620,8 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
+    id decodedCell;
+  
     self = [super initWithModelUnarchiver:unarchiver];
 
     // if (model_version == 1) {
@@ -612,7 +635,15 @@ void __dummy_GMAppKit_functionForLinking() {}
     //[self setIgnoresMultiClick:
     //            [unarchiver decodeBOOLWithName:@"ignoresMultiClick"]];
     // } else {
-    [self setCell:[unarchiver decodeObjectWithName:@"cell"]];
+    {
+      // So that custom NSControls, which do not encode the cell, 
+      // can still work.
+      decodedCell = [unarchiver decodeObjectWithName:@"cell"];
+      if (decodedCell)
+	[self setCell: decodedCell];
+      else
+	[self setCell: AUTORELEASE([[[self class] cellClass] new])];
+    }
     [self setEnabled:[unarchiver decodeBOOLWithName:@"isEnabled"]];
     [self setTag:[unarchiver decodeIntWithName:@"tag"]];
     [self setIgnoresMultiClick:
@@ -691,31 +722,62 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-#if XDPS_BACKEND_LIBRARY || XRAW_BACKEND_LIBRARY || XGPS_BACKEND_LIBRARY
-  [super encodeWithModelArchiver:archiver];
-#endif
-
+  [archiver encodeString:[self title] withName:@"title"];
+  if ([self respondsToSelector: @selector(image)])
+    [archiver encodeObject:[self image] withName:@"image"];
+  if ([self respondsToSelector: @selector(onStateImage)])
+    [archiver encodeObject:[self onStateImage] withName:@"onStateImage"];
+  if ([self respondsToSelector: @selector(offStateImage)])
+    [archiver encodeObject:[self offStateImage] withName:@"offStateImage"];
+  if ([self respondsToSelector: @selector(mixedStateImage)])
+    [archiver encodeObject:[self mixedStateImage] withName:@"mixedStateImage"];
+  [archiver encodeString:[self keyEquivalent] withName:@"keyEquivalent"];
+  if ([self respondsToSelector: @selector(state)])
+    [archiver encodeInt:[self state] withName:@"state"];
   [archiver encodeObject:[self target] withName:@"target"];
   [archiver encodeSelector:[self action] withName:@"action"];
-  [archiver encodeString:[self title] withName:@"title"];
   [archiver encodeInt:[self tag] withName:@"tag"];
   [archiver encodeBOOL:[self isEnabled] withName:@"isEnabled"];
-  [archiver encodeString:[self keyEquivalent] withName:@"keyEquivalent"];
+  if ([self respondsToSelector: @selector(changesState)])
+    [archiver encodeBOOL:[self changesState] withName:@"changesState"];
+  if ([self respondsToSelector: @selector(submenu)])
+    [archiver encodeObject:[self submenu] withName:@"submenu"];
+  [archiver encodeConditionalObject:[self representedObject]
+	                   withName:@"representedObject"];
 }
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
-#if XDPS_BACKEND_LIBRARY || XRAW_BACKEND_LIBRARY || XGPS_BACKEND_LIBRARY
-  self = [super initWithModelUnarchiver:unarchiver];
-#endif
-
+  [self setTitle:[unarchiver decodeStringWithName:@"title"]];
+  [self setImage:[unarchiver decodeObjectWithName:@"image"]];
+  [self setOnStateImage:[unarchiver decodeObjectWithName:@"onStateImage"]];
+  [self setOffStateImage:[unarchiver decodeObjectWithName:@"offStateImage"]];
+  [self setMixedStateImage:[unarchiver
+			     decodeObjectWithName:@"mixedStateImage"]];
+  [self setKeyEquivalent:[unarchiver decodeStringWithName:@"keyEquivalent"]];
+  [self setState:[unarchiver decodeIntWithName:@"state"]];
   [self setTarget:[unarchiver decodeObjectWithName:@"target"]];
   [self setAction:[unarchiver decodeSelectorWithName:@"action"]];
-  [self setTitle:[unarchiver decodeStringWithName:@"title"]];
   [self setTag:[unarchiver decodeIntWithName:@"tag"]];
   [self setEnabled:[unarchiver decodeBOOLWithName:@"isEnabled"]];
-  [self setKeyEquivalent:[unarchiver decodeStringWithName:@"keyEquivalent"]];
+  [self setChangesState:[unarchiver decodeBOOLWithName:@"changesState"]];
+  [self setMenu:nil];
+  [self setSubmenu:[unarchiver decodeObjectWithName:@"submenu"]];
+  [self setRepresentedObject:[unarchiver
+			       decodeObjectWithName:@"representedObject"]];
 
+#ifdef GNU_GUI_LIBRARY
+  /*
+   * Set submenu from target if not set
+   */
+  if ([NSStringFromSelector ([self action]) 
+      isEqualToString: @"submenuAction:"])
+    {
+      if ([self submenu] == nil)
+	[self setSubmenu: [self target]];
+    }
+#endif
+  
 #if 0
   NSLog (@"menu item %@: target = %@, isEnabled = %d",
 	[self title], [self target], [self isEnabled]);
@@ -731,9 +793,9 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
+  [archiver encodeString:[self title] withName:@"title"];
   [archiver encodeObject:[self itemArray] withName:@"itemArray"];
   [archiver encodeBOOL:[self autoenablesItems] withName:@"autoenablesItems"];
-  [archiver encodeString:[self title] withName:@"title"];
 }
 
 /* Define this method here because on OPENSTEP 4.x the NSMenu is inherited from
@@ -754,18 +816,14 @@ void __dummy_GMAppKit_functionForLinking() {}
 
   for (i = 0, count = [decodedItems count]; i < count; i++)
     [self addItem:[decodedItems objectAtIndex:i]];
-//    [self addItemWithTitle:@"dummy" action:NULL keyEquivalent:@""];
 
-//  [itemArray replaceObjectsInRange:NSMakeRange(0, count)
-//	     withObjectsFromArray:decodedItems];
+  for (i = 0; i < count; i++)
+    {
+      id item = [itemArray objectAtIndex:i];
 
-  for (i = 0; i < count; i++) {
-    id item = [itemArray objectAtIndex:i];
-    id target = [item target];
-
-    if ([target isKindOfClass:[NSMenu class]])
-      [self setSubmenu:target forItem:item];
-  }
+      if ([item hasSubmenu])
+	[self setSubmenu:[item submenu] forItem:item];
+    }
 
   [self setAutoenablesItems:
 	  [unarchiver decodeBOOLWithName:@"autoenablesItems"]];
@@ -777,23 +835,22 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 @end /* NSMenu (GMArchiverMethods) */
 
-#if 0
 @implementation NSPopUpButton (GMArchiverMethods)
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
   [archiver encodeBOOL:[self pullsDown] withName:@"pullsDown"];
   
-#if 0
+#if 1
   /* OUCH! This code crashes the translator; probably we interfere somehow with
      the way NSPopUpButton is handled by the NeXT's NIB code. Sorry, the
      popup buttons cannot be handled by the convertor! */
   [archiver encodeArray:[self itemArray] withName:@"itemArray"];
-  [archiver encodeString:[self titleOfSelectedItem] withName:@"selectedItem"];
-  [super encodeWithModelArchiver:archiver];
-#else // need frame for workarounds to know where to place the popup
-  [archiver encodeRect:[self frame] withName:@"frame"];
+  //[archiver encodeString:[self titleOfSelectedItem] withName:@"selectedItem"];
+  //[archiver encodeString:[self title] withName:@"selectedItem"];
+  //[super encodeWithModelArchiver:archiver];
 #endif
+  [archiver encodeRect:[self frame] withName:@"frame"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -817,7 +874,7 @@ void __dummy_GMAppKit_functionForLinking() {}
 
   int i, count;
   NSMutableArray* decodedItems
-      = [unarchiver decodeObjectWithName:@"itemArray"];
+      = [unarchiver decodeArrayWithName:@"itemArray"];
 
   self = [super initWithModelUnarchiver:unarchiver];
 
@@ -844,7 +901,6 @@ void __dummy_GMAppKit_functionForLinking() {}
 }
 
 @end /* NSPopUpButton (GMArchiverMethods) */
-#endif
 
 @implementation NSResponder (GMArchiverMethods)
 
@@ -887,8 +943,6 @@ void __dummy_GMAppKit_functionForLinking() {}
   [archiver encodeBOOL:[self drawsBackground] withName:@"drawsBackground"];
   [archiver encodeBOOL:[self isBordered] withName:@"isBordered"];
   [archiver encodeBOOL:[self isBezeled] withName:@"isBezeled"];
-  [archiver encodeObject:[self nextText] withName:@"nextText"];
-  [archiver encodeObject:[self previousText] withName:@"previousText"];
   [archiver encodeObject:[self delegate] withName:@"delegate"];
   [archiver encodeString:[self stringValue] withName:@"stringValue"];
   [archiver encodeBOOL:[self isEditable] withName:@"isEditable"];
@@ -911,8 +965,6 @@ void __dummy_GMAppKit_functionForLinking() {}
   [self setBordered:[unarchiver decodeBOOLWithName:@"isBordered"]];
   [self setBezeled:[unarchiver decodeBOOLWithName:@"isBezeled"]];
 
-  [self setNextText:[unarchiver decodeObjectWithName:@"nextText"]];
-  [self setPreviousText:[unarchiver decodeObjectWithName:@"previousText"]];
   [self setDelegate:[unarchiver decodeObjectWithName:@"delegate"]];
 
   theCell = [self cell];
@@ -951,6 +1003,9 @@ void __dummy_GMAppKit_functionForLinking() {}
 	    withName:@"autoresizesSubviews"];
   [archiver encodeUnsignedInt:[self autoresizingMask]
 	    withName:@"autoresizingMask"];
+  [archiver encodeConditionalObject:[self nextKeyView] withName:@"nextKeyView"];
+  [archiver encodeConditionalObject:[self previousKeyView] 
+	    withName:@"previousKeyView"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -988,11 +1043,14 @@ void __dummy_GMAppKit_functionForLinking() {}
 	[unarchiver decodeBOOLWithName:@"autoresizesSubviews"]];
   [self setAutoresizingMask:
 	[unarchiver decodeUnsignedIntWithName:@"autoresizingMask"]];
+  [self setNextKeyView: [unarchiver decodeObjectWithName:@"nextKeyView"]];
+  [self setPreviousKeyView: 
+	  [unarchiver decodeObjectWithName:@"previousKeyView"]];
 
-#ifndef NeXT_GUI_LIBRARY
+
+#ifdef GNU_GUI_LIBRARY
   _rFlags.flipped_view = [self isFlipped];
-  _rFlags.opaque_view = [self isOpaque];
-  if ([sub_views count])
+  if ([_sub_views count])
     _rFlags.has_subviews = 1;
 #endif
 
@@ -1030,6 +1088,8 @@ void __dummy_GMAppKit_functionForLinking() {}
   [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
   [archiver encodeUnsignedInt:[self styleMask] withName:@"styleMask"];
   [archiver encodeUnsignedInt:[self backingType] withName:@"backingType"];
+  [archiver encodeConditionalObject:[self initialFirstResponder] 
+	    withName:@"initialFirstResponder"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -1059,10 +1119,12 @@ void __dummy_GMAppKit_functionForLinking() {}
   if (frameAutosaveName)
     [self setFrameAutosaveName:frameAutosaveName];
 
-#ifndef NeXT_GUI_LIBRARY
+#ifdef GNU_GUI_LIBRARY
   window_level = [unarchiver decodeIntWithName:@"level"];
 #endif
 
+  [self setInitialFirstResponder: 
+	  [unarchiver decodeObjectWithName:@"initialFirstResponder"]];
   [self setAutodisplay:[unarchiver decodeBOOLWithName:@"isAutodisplay"]];
   [self setTitle:[unarchiver decodeStringWithName:@"title"]];
   [self setRepresentedFilename:
@@ -1113,6 +1175,8 @@ void __dummy_GMAppKit_functionForLinking() {}
     [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
     [archiver encodeUnsignedInt:[self styleMask] withName:@"styleMask"];
     [archiver encodeUnsignedInt:[self backingType] withName:@"backingType"];
+    [archiver encodeConditionalObject:[self initialFirstResponder] 
+	      withName:@"initialFirstResponder"];
 
     [archiver encodeBOOL:[self isFloatingPanel] withName:@"isFloatingPanel"];
     [archiver encodeBOOL:[self becomesKeyOnlyIfNeeded]
@@ -1146,10 +1210,12 @@ void __dummy_GMAppKit_functionForLinking() {}
     if (frameAutosaveName)
 	[self setFrameAutosaveName:frameAutosaveName];
     
-#ifndef NeXT_GUI_LIBRARY
+#ifdef GNU_GUI_LIBRARY
     window_level = [unarchiver decodeIntWithName:@"level"];
 #endif
-    
+
+    [self setInitialFirstResponder: 
+	    [unarchiver decodeObjectWithName:@"initialFirstResponder"]];    
     [self setAutodisplay:[unarchiver decodeBOOLWithName:@"isAutodisplay"]];
     [self setTitle:[unarchiver decodeStringWithName:@"title"]];
     [self setRepresentedFilename:
@@ -1203,6 +1269,8 @@ void __dummy_GMAppKit_functionForLinking() {}
     [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
     [archiver encodeUnsignedInt:[self styleMask] withName:@"styleMask"];
     [archiver encodeUnsignedInt:[self backingType] withName:@"backingType"];
+    [archiver encodeConditionalObject:[self initialFirstResponder] 
+	      withName:@"initialFirstResponder"];
 
     // NSPanel specific
     [archiver encodeBOOL:[self isFloatingPanel] withName:@"isFloatingPanel"];
@@ -1253,10 +1321,12 @@ void __dummy_GMAppKit_functionForLinking() {}
     if (frameAutosaveName)
 	[self setFrameAutosaveName:frameAutosaveName];
     
-#ifndef NeXT_GUI_LIBRARY
+#ifdef GNU_GUI_LIBRARY
     window_level = [unarchiver decodeIntWithName:@"level"];
 #endif
     
+    [self setInitialFirstResponder: 
+	    [unarchiver decodeObjectWithName:@"initialFirstResponder"]];
     [self setAutodisplay:[unarchiver decodeBOOLWithName:@"isAutodisplay"]];
     [self setTitle:[unarchiver decodeStringWithName:@"title"]];
     [self setRepresentedFilename:
@@ -1296,6 +1366,26 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 @end  /* NSSavePanel (GMArchiverMethods) */
 
+@implementation NSOpenPanel (GMArchiverMethods)
+
+- (void)encodeWithModelArchiver:(GMArchiver*)archiver
+{
+  [super encodeWithModelArchiver:(GMArchiver*)archiver];
+  [archiver encodeBOOL:[self canChooseFiles] 
+	    withName:@"canChooseFiles"];
+  [archiver encodeBOOL:[self canChooseDirectories] 
+	    withName:@"canChooseDirectories"];
+}
+
+- (id)initWithModelUnarchiver:(GMUnarchiver *)unarchiver
+{
+  [super initWithModelUnarchiver:(GMUnarchiver *)unarchiver];
+  [self setCanChooseFiles:[unarchiver decodeBOOLWithName:@"canChooseFiles"]];
+  [self setCanChooseDirectories:[unarchiver decodeBOOLWithName:
+					      @"canChooseDirectories"]];
+  return self;
+}
+@end  /* NSOpenPanel (GMArchiverMethods) */
 
 @implementation NSBrowser (GMArchiverMethods)
 
