@@ -27,6 +27,7 @@
 */ 
 
 #include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSBundle.h>
 
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSEvent.h>
@@ -40,8 +41,10 @@ char **NSArgv = NULL;
 int
 NSApplicationMain(int argc, const char **argv)
 {
-  NSAutoreleasePool	*pool;
-
+  NSDictionary		*infoDict;
+  NSString		*className;
+  Class			appClass;
+  CREATE_AUTORELEASE_POOL(pool);
 #if LIB_FOUNDATION_LIBRARY
   extern char		**environ;
 
@@ -50,15 +53,25 @@ NSApplicationMain(int argc, const char **argv)
 			     environment: environ];
 #endif
 
-  pool = [NSAutoreleasePool new];
-
 #ifndef NX_CURRENT_COMPILER_RELEASE
   initialize_gnustep_backend();
 #endif
 
-  [[NSApplication sharedApplication] run];
+  infoDict = [[NSBundle mainBundle] infoDictionary];
+  className = [infoDict objectForKey: @"NSPrincipalClass"];
+  appClass = NSClassFromString(className);
 
-  [pool release];
+  if (appClass == 0)
+    {
+      NSLog(@"Bad application class '%@' specified", className);
+      appClass = [NSApplication class];
+    }
+
+  [[appClass sharedApplication] run];
+
+  DESTROY(NSApp);
+
+  RELEASE(pool);
 
   return 0;
 }
