@@ -40,6 +40,7 @@
 #include <AppKit/NSPasteboard.h>
 #include <AppKit/NSView.h>
 #include <AppKit/NSGraphics.h>
+#include <AppKit/PSOperators.h>
 
 NSString	*NSSystemColorsDidChangeNotification =
 			@"NSSystemColorsDidChangeNotification";
@@ -48,7 +49,7 @@ NSString	*NSSystemColorsDidChangeNotification =
 
 // Class variables
 static BOOL gnustep_gui_ignores_alpha = YES;
-static NSColorList	*systemColors = nil;
+static NSColorList		*systemColors = nil;
 static NSMutableDictionary	*colorStrings = nil;
 
 //
@@ -658,7 +659,7 @@ static NSMutableDictionary	*colorStrings = nil;
    *	consisting of the three componets values in a quoted string.
    */
   if ([colorspace_name isEqualToString: NSCalibratedRGBColorSpace] &&
-	alpha_component == 0)
+	alpha_component == 1.0)
     return [NSString stringWithFormat: @"\"%f %f %f\"",
 	RGB_component.red, RGB_component.green, RGB_component.blue];
  
@@ -808,6 +809,53 @@ static NSMutableDictionary	*colorStrings = nil;
     *alpha = alpha_component;
 }
 
+- (BOOL) isEqual: (id)other
+{
+  if (other == self)
+    return YES;
+  if ([other isKindOfClass: [NSColor class]] == NO)
+    return NO;
+  else
+    {
+      NSColor	*col = (NSColor*)other;
+
+      if (col->active_component != active_component)
+	return NO;
+      if (col->alpha_component != alpha_component)
+	return NO;
+      switch (active_component)
+	{
+	  case GNUSTEP_GUI_RGB_ACTIVE:
+	    if (col->RGB_component.red != RGB_component.red
+	      || col->RGB_component.green != RGB_component.green
+	      || col->RGB_component.blue != RGB_component.blue)
+	      return NO;
+	    return YES;
+
+	  case GNUSTEP_GUI_CMYK_ACTIVE:
+	    if (col->CMYK_component.cyan != CMYK_component.cyan
+	      || col->CMYK_component.magenta != CMYK_component.magenta
+	      || col->CMYK_component.yellow != CMYK_component.yellow
+	      || col->CMYK_component.black != CMYK_component.black)
+	      return NO;
+	    return YES;
+
+	  case GNUSTEP_GUI_HSB_ACTIVE:
+	    if (col->HSB_component.hue != HSB_component.hue
+	      || col->HSB_component.saturation != HSB_component.saturation
+	      || col->HSB_component.brightness != HSB_component.brightness)
+	      return NO;
+	    return YES;
+
+	  case GNUSTEP_GUI_WHITE_ACTIVE:
+	    if (col->white_component != white_component)
+	      return NO;
+	    return YES;
+	}
+      return NO;
+    }
+}
+
 //
 // Retrieving Individual Components
 //
@@ -1004,7 +1052,7 @@ static NSMutableDictionary	*colorStrings = nil;
   return [NSColor colorWithCalibratedRed: red
 				   green: green
 				    blue: blue
-				   alpha: 0];
+				   alpha: 1.0];
 }
 
 - (NSColor*) colorWithAlphaComponent: (float)alpha
@@ -1049,6 +1097,35 @@ static NSMutableDictionary	*colorStrings = nil;
 
 - (void) set
 {
+  switch (active_component)
+    {
+      case GNUSTEP_GUI_RGB_ACTIVE:
+	NSDebugLLog(@"NSColor", @"RGB %f %f %f\n", RGB_component.red,
+		   RGB_component.green, RGB_component.blue);
+	PSsetrgbcolor(RGB_component.red, RGB_component.green, 
+		      RGB_component.blue);
+	break;
+
+      case GNUSTEP_GUI_CMYK_ACTIVE:
+	NSDebugLLog(@"NSColor", @"CMYK %f %f %f %f\n", CMYK_component.cyan, 
+		   CMYK_component.magenta,
+		   CMYK_component.yellow, CMYK_component.black);
+	PSsetcmykcolor(CMYK_component.cyan, CMYK_component.magenta,
+		       CMYK_component.yellow, CMYK_component.black);
+	break;
+
+      case GNUSTEP_GUI_HSB_ACTIVE:
+	NSDebugLLog(@"NSColor", @"HSB %f %f %f\n", HSB_component.hue,
+		   HSB_component.saturation, HSB_component.brightness);
+	PSsethsbcolor(HSB_component.hue, HSB_component.saturation,
+		      HSB_component.brightness);
+	break;
+
+      case GNUSTEP_GUI_WHITE_ACTIVE:
+	NSDebugLLog(@"NSColor", @"Gray %f\n", white_component);
+	PSsetgray(white_component);
+    }
+  PSsetalpha(alpha_component);
 }
 
 //
@@ -1147,7 +1224,15 @@ static NSMutableDictionary	*colorStrings = nil;
       if ((space = [dict objectForKey: @"ColorSpace"]) == nil)
 	return nil;
 
-      alpha = [[dict objectForKey: @"Alpha"] floatValue];
+      str = [dict objectForKey: @"Alpha"];
+      if (str == nil || [str isEqualToString: @""])
+	{
+	  alpha = 1.0;
+	}
+      else
+	{
+	  alpha = [str floatValue];
+	}
 
       if ([space isEqual: NSCalibratedWhiteColorSpace])
 	{
@@ -1218,7 +1303,7 @@ static NSMutableDictionary	*colorStrings = nil;
       return [self colorWithCalibratedRed: r
 				    green: g
 				     blue: b
-				    alpha: 0];
+				    alpha: 1.0];
     }
   return nil;
 }
@@ -1244,7 +1329,7 @@ static NSMutableDictionary	*colorStrings = nil;
 }
 
 /*
- *	Go through all the names of ssystem colors - for each color where
+ *	Go through all the names of system colors - for each color where
  *	there is a value in the defaults database, see if the current
  *	istring value of the color differs from the old one.
  *	Where there is a difference, update the color strings dictionary 
