@@ -51,7 +51,8 @@
 #include <AppKit/NSScreen.h>
 #include <AppKit/NSColor.h>
 
-BOOL	doesCaching = NO;
+BOOL	NSImageDoesCaching = YES;	/* enable caching	*/
+BOOL	NSImageForceCaching = NO;	/* use on missmatch	*/
 
 // Resource directories
 static NSString* gnustep_libdir = @GNUSTEP_INSTALL_LIBDIR;
@@ -94,12 +95,9 @@ static NSString* NSImage_PATH = @"Images";
 
 - (void) dealloc
 {
-  if (fileName)
-    [fileName release];
-  if (rep)
-    [rep release];
-  if (bg)
-    [bg release];
+  TEST_RELEASE(fileName);
+  TEST_RELEASE(rep);
+  TEST_RELEASE(bg);
   NSDeallocateObject(self);
 }
 @end
@@ -110,18 +108,19 @@ NSArray *iterate_reps_for_types(NSArray *imageReps, SEL method);
 GSRepData*
 repd_for_rep(NSArray *_reps, NSImageRep *rep)
 {
-  unsigned	i, count;
+  NSEnumerator	*enumerator = [_reps objectEnumerator];
+  IMP		nextImp = [enumerator methodForSelector: @selector(nextObject)];
   GSRepData	*repd;
 
-  count = [_reps count];
-  for (i = 0; i < count; i++) 
+  while ((repd = (*nextImp)(enumerator, @selector(nextObject))) != nil)
     {
-      repd = [_reps objectAtIndex: i];
       if (repd->rep == rep)
-        return repd;
+	{
+	  return repd;
+	}
     }
   [NSException raise: NSInternalInconsistencyException
-  	format: @"Cannot find stored representation"];
+	      format: @"Cannot find stored representation"];
   /* NOT REACHED */
   return nil;
 }
@@ -133,16 +132,20 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 - (BOOL) useFromFile: (NSString *)fileName;
 - (BOOL) loadFromData: (NSData *)data;
 - (BOOL) loadFromFile: (NSString *)fileName;
-- (NSImageRep *) lastRepresentation;
+- (NSImageRep*) cacheForRep: (NSImageRep*)rep
+		   onDevice: (NSDictionary*)deviceDescription;
+- (NSImageRep*) _doImageCache: (NSDictionary*)deviceDescription;
 @end
 
 @implementation NSImage
 
 /* Class variables and functions for class methods */
-static NSMutableDictionary* nameDict = nil;
-static NSDictionary* nsmapping = nil;
+static NSMutableDictionary	*nameDict = nil;
+static NSDictionary		*nsmapping = nil;
+static NSColor			*clearColor = nil;
+static Class			cacheClass = 0;
 
-+ (void)initialize
++ (void) initialize
 {
   if (self == [NSImage class])
     {
@@ -156,13 +159,14 @@ static NSDictionary* nsmapping = nil;
       // initialize the class variables
       nameDict = [[NSMutableDictionary alloc] initWithCapacity: 10];
       if (path)
-	nsmapping = [[[NSString stringWithContentsOfFile: path]
-				propertyListFromStringsFileFormat]
-				retain];
+	nsmapping = RETAIN([[NSString stringWithContentsOfFile: path]
+				propertyListFromStringsFileFormat]);
+      clearColor = RETAIN([NSColor clearColor]);
+      cacheClass = [NSCachedImageRep class];
     }
 }
 
-+ imageNamed: (NSString *)aName
++ (id) imageNamed: (NSString *)aName
 {
   NSString	*realName = [nsmapping objectForKey: aName];
 
@@ -174,10 +178,10 @@ static NSDictionary* nsmapping = nil;
     {
       NSString* ext;
       NSString* path = nil;
-      NSBundle* main;
+      NSBundle* main_bundle;
       NSArray *array;
       NSString *the_name = aName;
-      main = [NSBundle mainBundle];
+      main_bundle = [NSBundle mainBundle];
       ext  = [aName pathExtension];
 
       /* Check if extension is one of the image types */
@@ -198,7 +202,7 @@ static NSDictionary* nsmapping = nil;
 
       /* First search locally */
       if (ext)
-	path = [main pathForResource: the_name ofType: ext];
+	path = [main_bundle pathForResource: the_name ofType: ext];
       else 
 	{
 	  id o, e;
@@ -207,7 +211,7 @@ static NSDictionary* nsmapping = nil;
 	  while ((o = [e nextObject]))
 	    {
 	      NSDebugLog(@"extension %s\n", [o cString]);
-	      path = [main pathForResource: the_name 
+	      path = [main_bundle pathForResource: the_name 
 		        ofType: o];
 	      if (path != nil && [path length] != 0)
 		break;
@@ -249,10 +253,11 @@ static NSDictionary* nsmapping = nil;
 
 	  image = [[self allocWithZone: NSDefaultMallocZone()]
 		initByReferencingFile: path];
-	  if (image)
+	  if (image != nil)
 	    {
 	      [image setName: aName];
-	      [image release];		// Retained in dictionary.
+	      RELEASE(image);		// Retained in dictionary.
+	      image->_flags.archiveByName = YES;
 	    }
 	  return image;
 	}
@@ -261,11 +266,21 @@ static NSDictionary* nsmapping = nil;
   return [nameDict objectForKey: aName];
 }
 
+- (BOOL) isFlipped
+{
+  return _flags.flipDraw;
+}
+
+- (void) setFlipped: (BOOL)flag
+{
+  _flags.flipDraw = flag;
+}
+
 // Designated initializer for nearly everything.
-- initWithSize: (NSSize)aSize
+- (id) initWithSize: (NSSize)aSize
 {
   [super init];
-  _reps = [[NSMutableArray arrayWithCapacity: 2] retain];
+  _reps = [[NSMutableArray alloc] initWithCapacity: 2];
   if (aSize.width && aSize.height) 
     {
       _size = aSize;
@@ -273,6 +288,7 @@ static NSDictionary* nsmapping = nil;
     }
   _flags.colorMatchPreferred = YES;
   _flags.multipleResolutionMatching = YES;
+  _color = RETAIN(clearColor);
   
   return self;
 }
@@ -292,7 +308,7 @@ static NSDictionary* nsmapping = nil;
       _flags.dataRetained = NO;
       if (![self useFromFile: fileName])
 	{
-	  [self release];
+	  RELEASE(self);
 	  return nil;
 	}
     }
@@ -308,7 +324,7 @@ static NSDictionary* nsmapping = nil;
       _flags.dataRetained = YES;
       if (![self useFromFile: fileName])
 	{
-	  [self release];
+	  RELEASE(self);
 	  return nil;
 	}
     }
@@ -323,14 +339,14 @@ static NSDictionary* nsmapping = nil;
     {
       if (![self loadFromData: data])
 	{
-	  [self release];
+	  RELEASE(self);
 	  return nil;
 	}
     }
   return self;
 }
 
-- initWithPasteboard: (NSPasteboard *)pasteboard
+- (id) initWithPasteboard: (NSPasteboard *)pasteboard
 {
   [self notImplemented: _cmd];
   return nil;
@@ -346,7 +362,8 @@ static NSDictionary* nsmapping = nil;
 {
   if (_size.width == 0) 
     {
-      NSImageRep* rep = [self bestRepresentationForDevice: nil];
+      NSImageRep	*rep = [self bestRepresentationForDevice: nil];
+
       _size = [rep size];
     }
   return _size;
@@ -355,52 +372,46 @@ static NSDictionary* nsmapping = nil;
 - (void) dealloc
 {
   [self representations];
-  [_repList release];
-  [_reps release];
+  RELEASE(_reps);
   /* Make sure we don't remove name from the nameDict if we are just a copy
      of the named image, not the original image */
   if (name && self == [nameDict objectForKey: name]) 
     [nameDict removeObjectForKey: name];
-  [name release];
+  RELEASE(name);
   [super dealloc];
 }
 
-- copyWithZone: (NSZone *)zone
+- (id) copyWithZone: (NSZone *)zone
 {
-  NSImage* copy;
+  NSImage	*copy;
 
   // FIXME: maybe we should retain if _flags.dataRetained = NO
   copy = (NSImage*)NSCopyObject (self, 0, zone);
 
-  [name retain];
+  RETAIN(name);
   copy->_reps = [NSMutableArray new];
-  copy->_repList = [NSMutableArray new];
-  [_color retain];
+  RETAIN(_color);
   _lockedView = nil;
-  [copy addRepresentations: [[[self representations] copyWithZone: zone]
-				autorelease]];
+  [copy addRepresentations: [self representations]];
   
   return copy;
 }
 
-- (BOOL)setName: (NSString *)string
+- (BOOL) setName: (NSString *)string
 {
   if (!nameDict)
-    nameDict = [[NSMutableDictionary dictionaryWithCapacity: 2] retain];
+    nameDict = [[NSMutableDictionary alloc] initWithCapacity: 2];
 
   if (!string || [nameDict objectForKey: string])
     return NO;
 
-  [string retain];
-  if (name)
-    [name release];
-  name = string;
+  ASSIGN(name, string);
 
   [nameDict setObject: self forKey: name];
   return YES;
 }
 
-- (NSString *)name
+- (NSString *) name
 {
   return name;
 }
@@ -411,7 +422,7 @@ static NSDictionary* nsmapping = nil;
   _flags.useEPSOnResolutionMismatch = flag;
 }
 
-- (BOOL)usesEPSOnResolutionMismatch
+- (BOOL) usesEPSOnResolutionMismatch
 {
   return _flags.useEPSOnResolutionMismatch;
 }
@@ -421,7 +432,7 @@ static NSDictionary* nsmapping = nil;
   _flags.colorMatchPreferred = flag;
 }
 
-- (BOOL)prefersColorMatch
+- (BOOL) prefersColorMatch
 {
   return _flags.colorMatchPreferred;
 }
@@ -431,7 +442,7 @@ static NSDictionary* nsmapping = nil;
   _flags.multipleResolutionMatching = flag;
 }
 
-- (BOOL)matchesOnMultipleResolution
+- (BOOL) matchesOnMultipleResolution
 {
   return _flags.multipleResolutionMatching;
 }
@@ -481,16 +492,18 @@ static NSDictionary* nsmapping = nil;
     {
       GSRepData	 *repd = (GSRepData*)[_reps objectAtIndex: i];
 
-      if (repd->bg != nil
-	|| [repd->rep isKindOfClass: [NSCachedImageRep class]] == NO)
-	valid = YES;
+      if (repd->bg != nil || [repd->rep isKindOfClass: cacheClass] == NO)
+	{
+	  valid = YES;
+	  break;
+	}
     }
   return valid;
 }
 
 - (void) recache
 {
-  int i, count;
+  unsigned i, count;
 
   count = [_reps count];
   for (i = 0; i < count; i++) 
@@ -500,8 +513,8 @@ static NSDictionary* nsmapping = nil;
       repd = (GSRepData*)[_reps objectAtIndex: i];
       if (repd->bg != nil)
 	{
-	  [repd->bg release];
-	  repd->bg = nil;
+	  DESTROY(repd->bg);
+	  [repd->rep setOpaque: YES];
 	}
     }
 }
@@ -518,18 +531,15 @@ static NSDictionary* nsmapping = nil;
 
 - (void) setBackgroundColor: (NSColor *)aColor
 {
-  if (_color != aColor)
+  if (aColor == nil)
     {
-      if (_color)
-	[_color release];
-      _color = [aColor retain];
+      aColor = clearColor;
     }
+  ASSIGN(_color, aColor);
 }
 
 - (NSColor *) backgroundColor
 {
-  if (_color == nil)
-    _color = [[NSColor clearColor] retain];
   return _color;
 }
 
@@ -540,7 +550,7 @@ static NSDictionary* nsmapping = nil;
   unsigned	i, count;
   GSRepData	*repd;
 
-  _syncLoad = NO;
+  _flags.syncLoad = NO;
   count = [_reps count];
   for (i = 0; i < count; i++) 
     {
@@ -565,40 +575,24 @@ static NSDictionary* nsmapping = nil;
 // a cache and no cache exists, create one and draw the representation in it
 // If a cache exists, but is not valid, redraw the cache from the original
 // image (if there is one).
-- (NSImageRep *)_doImageCache
+- (NSImageRep *)_doImageCache: (NSDictionary*)deviceDesc
 {
   NSImageRep	*rep = nil;
   GSRepData	*repd;
 
-  repd = repd_for_rep(_reps, [self bestRepresentationForDevice: nil]);
+  repd = repd_for_rep(_reps, [self bestRepresentationForDevice: deviceDesc]);
   rep = repd->rep;
 
-  if (doesCaching)
+  NSDebugLLog(@"NSImage", @"Getting image rep %d", (int)rep);
+  if (NSImageDoesCaching == YES)
     {
       /*
        * If this is not a cached image rep - create a cache to be used to
        * render the image rep into, and switch to the cached rep.
        */
-      if ([rep isKindOfClass: [NSCachedImageRep class]] == NO) 
-	{
-	  NSScreen		*cur = [NSScreen mainScreen];
-	  NSCachedImageRep	*cachedRep;
-	  NSSize		imageSize;
-  
-	  imageSize = [self size];
-	  if (imageSize.width == 0 || imageSize.height == 0)
-	    return nil;
-
-	  cachedRep = [[NSCachedImageRep alloc] initWithSize: _size
-						       depth: [cur depth]
-						    separate: NO
-						       alpha: NO];
-	  [self addRepresentation: cachedRep];
-	  [cachedRep release];		/* Retained in _reps array.	*/
-	  repd = repd_for_rep(_reps, cachedRep);
-	  repd->original = rep;
-	  rep = repd->rep;
-	} 
+      rep = [self cacheForRep: rep onDevice: deviceDesc];
+      repd = repd_for_rep(_reps, rep);
+      NSDebugLLog(@"NSImage", @"Cached image rep is %d", (int)rep);
 
       /*
        * if the cache is not valid, it's background color will not exist
@@ -607,24 +601,36 @@ static NSDictionary* nsmapping = nil;
        */
       if (repd->bg == nil) 
 	{
-	  NSRect	bounds;
+	  NSRect	drawRect = NSMakeRect(0, 0, _size.width, _size.height);
 
 	  [self lockFocusOnRepresentation: rep];
-	  /*
-	   * If this is not a cache - the lockFocus will have created a
-	   * cache that we can use instead.
-	   */
-	  if (repd->original == nil)
+	  if (_color != nil && [_color alphaComponent] != 0.0)
 	    {
-	      repd = repd_for_rep(_reps, [self lastRepresentation]);
+	      NSRect	bounds = [_lockedView bounds];
+
+	      [_color set];
+	      NSRectFill(bounds);
 	    }
-	  bounds = [_lockedView bounds];
-	  [_color set];
-	  NSEraseRect(bounds);
-	  [self drawRepresentation: repd->original 
-	    inRect: NSMakeRect(0, 0, _size.width, _size.height)];
+	  [self drawRepresentation: repd->original inRect: drawRect];
 	  [self unlockFocus];
-	  repd->bg = [_color copy];
+	  if (_color == nil)
+	    {
+	      repd->bg = [clearColor copy];
+	    }
+	  else
+	    {
+	      repd->bg = [_color copy];
+	    }
+	  if ([repd->bg alphaComponent] == 1.0)
+	    {
+	      [rep setOpaque: YES];
+	    }
+	  else
+	    {
+	      [rep setOpaque: [repd->original isOpaque]];
+	    }
+	  NSDebugLLog(@"NSImage", @"Rendered rep %d on background %@",
+	    (int)rep, repd->bg);
 	}
     }
   
@@ -652,7 +658,7 @@ static NSDictionary* nsmapping = nil;
   // xxx If fromRect specifies something other than full image
   // then we need to construct a subimage to draw
 
-  rep = [self _doImageCache];
+  rep = [self _doImageCache: nil];
   [self drawRepresentation: rep inRect: rect];
 }
 
@@ -674,7 +680,7 @@ static NSDictionary* nsmapping = nil;
   // xxx If fromRect specifies something other than full image
   // then we need to construct a subimage to draw
 
-  rep = [self _doImageCache];
+  rep = [self _doImageCache: nil];
   [self drawRepresentation: rep inRect: rect];
 }
 
@@ -741,10 +747,10 @@ static NSDictionary* nsmapping = nil;
   if ([array indexOfObject: ext] == NSNotFound)
     return NO;
   repd = [GSRepData new];
-  repd->fileName = [fileName retain];
+  repd->fileName = RETAIN(fileName);
   [_reps addObject: repd];
-  [repd release];
-  _syncLoad = YES;
+  RELEASE(repd);
+  _flags.syncLoad = YES;
   return YES;
 }
 
@@ -758,40 +764,20 @@ static NSDictionary* nsmapping = nil;
   unsigned	i, count;
   GSRepData	*repd;
 
-  if (!imageRepArray)
-    return;
-
-  if (_syncLoad)
-    [self _loadImageFilenames];
+  if (_flags.syncLoad == YES)
+    {
+      [self _loadImageFilenames];
+    }
   count = [imageRepArray count];
   for (i = 0; i < count; i++)
     {
       repd = [GSRepData new];
-      repd->rep = [[imageRepArray objectAtIndex: i] retain];
+      repd->rep = RETAIN([imageRepArray objectAtIndex: i]);
       [_reps addObject: repd]; 
-      [repd release];
+      RELEASE(repd);
     }
 }
 
-- (BOOL) useCacheWithDepth: (int)depth
-{
-  NSSize imageSize;
-  NSCachedImageRep* rep;
-  
-  imageSize = [self size];
-  if (!imageSize.width || !imageSize.height)
-    return NO;
-
-
-  // FIXME: determine alpha? separate?
-  rep = [[NSCachedImageRep alloc] initWithSize: _size
-       depth: depth
-       separate: NO
-       alpha: NO];
-  [self addRepresentation: rep];
-  return YES;
-}
-
 - (void) removeRepresentation: (NSImageRep *)imageRep
 {
   unsigned	i;
@@ -814,64 +800,39 @@ static NSDictionary* nsmapping = nil;
 
 - (void) lockFocus
 {
-  NSScreen	*cur = [NSScreen mainScreen];
   NSImageRep	*rep;
 
   if (!(rep = [self bestRepresentationForDevice: nil])) 
     {
-      [self useCacheWithDepth: [cur depth]];
-      rep = [self lastRepresentation];
+      rep = [self cacheForRep: rep onDevice: nil];
     }
   [self lockFocusOnRepresentation: rep];
 }
 
 - (void) lockFocusOnRepresentation: (NSImageRep *)imageRep
 {
-  NSScreen	*cur = [NSScreen mainScreen];
-  NSWindow	*window;
-
-  if (!imageRep)
+  if (imageRep == nil)
     [NSException raise: NSInvalidArgumentException
       format: @"Cannot lock focus on nil rep"];
 
-  if (doesCaching)
+  if (NSImageDoesCaching == YES)
     {
-      if (![imageRep isKindOfClass: [NSCachedImageRep class]]) 
-	{
-	  GSRepData	*repd, *cached;
-	  int		depth;
+      NSWindow	*window;
 
-	  if (_flags.unboundedCacheDepth)
-	    depth = [cur depth];      // FIXME: get depth correctly
-	  else
-	    depth = [cur depth];
-	  if (![self useCacheWithDepth: depth]) 
-	    {
-	      [NSException raise: NSImageCacheException
-		format: @"Unable to create cache"];
-	    }
-	  cached = repd_for_rep(_reps, [self lastRepresentation]);
-	  cached->original = imageRep;
-	  imageRep = cached->rep;
-	}
-	window = [(NSCachedImageRep *)imageRep window];
-	_lockedView = [window contentView];
-	[_lockedView lockFocus];
+      imageRep = [self cacheForRep: imageRep onDevice: nil];
+      window = [(NSCachedImageRep *)imageRep window];
+      _lockedView = [window contentView];
+      [_lockedView lockFocus];
     }
 }
 
 - (void) unlockFocus
 {
-  if (_lockedView)
-    [_lockedView unlockFocus];
-  _lockedView = nil;
-}
-
-- (NSImageRep *) lastRepresentation
-{
-  // Reconstruct the rep list if it has changed
-  [self representations];
-  return [_repList lastObject];
+  if (_lockedView != nil)
+    {
+      [_lockedView unlockFocus];
+      _lockedView = nil;
+    }
 }
 
 - (NSImageRep*) bestRepresentationForDevice: (NSDictionary*)deviceDescription
@@ -880,7 +841,7 @@ static NSDictionary* nsmapping = nil;
   unsigned	count;
 
   /* Make sure we have the images loaded in. */
-  if (_syncLoad)
+  if (_flags.syncLoad)
     [self _loadImageFilenames];
 
   count = [_reps count];
@@ -892,6 +853,8 @@ static NSDictionary* nsmapping = nil;
 
       /*
        *	What's the best representation? FIXME
+       *	At the moment we take the last bitmap we find.
+       *	If we can't find a bitmap, we take whatever we can!
        */
       [_reps getObjects: reps];
       for (i = 0; i < count; i++)
@@ -902,86 +865,169 @@ static NSDictionary* nsmapping = nil;
 	    {
 	      rep = repd->rep;
 	    }
-	}
-
-      /*
-       * If we got a representation - see if we already have it cached.
-       */
-      if (doesCaching)
-	{
-	  if (rep != nil)
+	  else if (rep == nil)
 	    {
-	      GSRepData	*invalidCache = nil;
-	      GSRepData	*validCache = nil;
-
-	      /*
-	       * Search the cached image reps for any whose original is our
-	       * 'best' image rep.  See if we can notice any invalidated
-	       * cache as we go - if we don't find a valid cache, we want to
-	       * re-use an invalidated one rather than createing a new one.
-	       */
-	      for (i = 0; i < count; i++)
-		{
-		  GSRepData	*repd = reps[i];
-    
-		  if (repd->original == rep)
-		    {
-		      if (repd->bg == nil)
-			{
-			  invalidCache = repd;
-			}
-		      else if ([repd->bg isEqual: _color] == YES)
-			{
-			  validCache = repd;
-			  break;
-			}
-		    }
-		}
-
-	      if (validCache)
-		{
-		  /*
-		   * If the image rep has transparencey and we are drawing
-		   * without a background (background is clear) then the
-		   * cache can't really be valid 'cos we might be drawing
-		   * transparency on top of anything.  So we invalidate
-		   * the cache by removing the background color information.
-		   */
-		  if ([rep hasAlpha]
-		    && [validCache->bg isEqual: [NSColor clearColor]])
-		    {
-		      [validCache->bg release];
-		      validCache->bg = nil;
-		    }
-		  rep = validCache->rep;
-		}
-	      else if (invalidCache)
-		{
-		  rep = invalidCache->rep;
-		}
+	      rep = repd->rep;
 	    }
 	}
     }
   return rep;
 }
 
+- (NSImageRep*) cacheForRep: (NSImageRep*)rep
+		   onDevice: (NSDictionary*)deviceDescription
+{
+  if (NSImageDoesCaching == YES && [rep isKindOfClass: cacheClass] == NO)
+    {
+      NSImageRep	*cacheRep = nil;
+      unsigned		count = [_reps count];
+
+      if (count > 0)
+	{
+	  GSRepData	*invalidCache = nil;
+	  GSRepData	*partialCache = nil;
+	  GSRepData	*validCache = nil;
+	  GSRepData	*reps[count];
+          unsigned	partialCount = 0;
+	  unsigned	i;
+	  BOOL		opaque = [rep isOpaque];
+
+	  [_reps getObjects: reps];
+
+	  /*
+	   * Search the cached image reps for any whose original is our
+	   * 'best' image rep.  See if we can notice any invalidated
+	   * cache as we go - if we don't find a valid cache, we want to
+	   * re-use an invalidated one rather than creating a new one.
+	   * NB. If the image rep is opaque, then any cached rep is valid
+	   * irrespective of the background color it was drawn with.
+	   */
+	  for (i = 0; i < count; i++)
+	    {
+	      GSRepData	*repd = reps[i];
+
+	      if (repd->original == rep && repd->rep != rep)
+		{
+		  if (repd->bg == nil)
+		    {
+NSDebugLLog(@"NSImage", @"Invalid %@ ... %@ %d", repd->bg, _color, repd->rep);
+		      invalidCache = repd;
+		    }
+		  else if (opaque == YES || [repd->bg isEqual: _color] == YES)
+		    {
+NSDebugLLog(@"NSImage", @"Exact %@ ... %@ %d", repd->bg, _color, repd->rep);
+		      validCache = repd;
+		      break;
+		    }
+		  else
+		    {
+NSDebugLLog(@"NSImage", @"Partial %@ ... %@ %d", repd->bg, _color, repd->rep);
+		      partialCache = repd;
+		      partialCount++;
+		    }
+		}
+	    }
+
+	  if (validCache != nil)
+	    {
+	      if (NSImageForceCaching == NO && [rep isOpaque] == NO)
+		{
+		  /*
+		   * If the image rep is not opaque and we are drawing
+		   * without an opaque background then the cache can't
+		   * really be valid 'cos we might be drawing transparency
+		   * on top of anything.  So we invalidate the cache by
+		   * removing the background color information.
+		   */
+		  if ([validCache->bg alphaComponent] != 1.0)
+		    {
+		      DESTROY(validCache->bg);
+		    }
+		}
+	      cacheRep = validCache->rep;
+	    }
+	  else if (partialCache != nil && partialCount > 2)
+	    {
+	      /*
+	       * Only re-use partially correct caches if there are already
+	       * a few partial matches - otherwise we fall default to
+	       * creating a new cache.
+	       */
+	      if (NSImageForceCaching == NO && [rep isOpaque] == NO)
+		{
+		  if (invalidCache != nil)
+		    {
+		      /*
+		       * If there is an unused cache - use it rather than
+		       * re-using this one, since we might get a request
+		       * to draw with this color again.
+		       */
+		      partialCache = invalidCache;
+		    }
+		  else
+		    {
+		      DESTROY(partialCache->bg);
+		    }
+		}
+	      cacheRep = partialCache->rep;
+	    }
+	  else if (invalidCache != nil)
+	    {
+	      cacheRep = invalidCache->rep;
+	    }
+	}
+      if (cacheRep == nil)
+	{
+	  NSScreen	*cur = [NSScreen mainScreen];
+	  NSSize	imageSize;
+	  GSRepData	*repd;
+
+	  imageSize = [self size];
+	  if (imageSize.width == 0 || imageSize.height == 0)
+	    return nil;
+
+	  cacheRep = [[cacheClass alloc] initWithSize: _size
+						depth: [cur depth]
+					     separate: NO
+						alpha: NO];
+	  [self addRepresentation: cacheRep];
+	  RELEASE(cacheRep);		/* Retained in _reps array.	*/
+          repd = repd_for_rep(_reps, cacheRep);
+          repd->original = rep;
+	}
+      return cacheRep;
+    }
+  else
+    {
+      return rep;
+    }
+}
+
 - (NSArray *) representations
 {
-  unsigned	i, count;
+  unsigned	count;
 
-  if (!_repList)
-    _repList = [[NSMutableArray alloc] init];
-  if (_syncLoad)
-    [self _loadImageFilenames];
-  [_repList removeAllObjects];
-  count = [_reps count];
-  for (i = 0; i < count; i++) 
+  if (_flags.syncLoad)
     {
-      GSRepData	*repd = [_reps objectAtIndex: i];
+      [self _loadImageFilenames];
+    }
+  count = [_reps count];
+  if (count == 0)
+    {
+      return [NSArray array];
+    }
+  else
+    {
+      id	repList[count];
+      unsigned	i;
 
-      [_repList addObject: repd->rep];
+      [_reps getObjects: repList];
+      for (i = 0; i < count; i++) 
+	{
+	  repList[i] = ((GSRepData*)repList[i])->rep;
+	}
+      return [NSArray arrayWithObjects: repList count: count];
     }
-  return _repList;
 }
 
 - (void) setDelegate: anObject
@@ -1021,10 +1067,114 @@ static NSDictionary* nsmapping = nil;
 // NSCoding
 - (void) encodeWithCoder: (NSCoder*)coder
 {
+  BOOL	flag;
+
+  flag = _flags.archiveByName;
+  [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+  if (flag == YES)
+    {
+      /*
+       * System image - just encode the name.
+       */
+      [coder encodeValueOfObjCType: @encode(id) at: &name];
+    }
+  else
+    {
+      NSMutableArray	*a;
+      NSEnumerator	*e;
+      NSImageRep	*r;
+
+      /*
+       * If the image is not yet loaded - load it now.
+       */
+      if (_flags.syncLoad)
+	{
+	  [self _loadImageFilenames];
+	}
+
+      /*
+       * Normal image - encode the ivars
+       */
+      [coder encodeValueOfObjCType: @encode(NSSize) at: &_size];
+      [coder encodeValueOfObjCType: @encode(id) at: &_color];
+      flag = _flags.scalable;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.dataRetained;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.flipDraw;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.sizeWasExplicitlySet;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.useEPSOnResolutionMismatch;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.colorMatchPreferred;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.multipleResolutionMatching;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.cacheSeparately;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _flags.unboundedCacheDepth;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+
+      /*
+       * Now encode an array of all the image reps (excluding cache)
+       */
+      a = [NSMutableArray arrayWithCapacity: 2];
+      e = [[self representations] objectEnumerator];
+      while ((r = [e nextObject]) != nil)
+	{
+	  if ([r isKindOfClass: [NSCachedImageRep class]] == NO)
+	    {
+	      [a addObject: r];
+	    }
+	}
+      [coder encodeValueOfObjCType: @encode(id) at: &a];
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)coder
 {
+  BOOL	flag;
+
+  [coder decodeValueOfObjCType: @encode(BOOL) at: &flag];
+  if (flag == YES)
+    {
+      NSString	*theName = [coder decodeObject];
+
+      RELEASE(self);
+      self = RETAIN([NSImage imageNamed: theName]);
+    }
+  else
+    {
+      NSArray	*a;
+
+      [coder decodeValueOfObjCType: @encode(NSSize) at: &_size];
+      [coder decodeValueOfObjCType: @encode(id) at: &_color];
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.scalable = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.dataRetained = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.flipDraw = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.sizeWasExplicitlySet = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.useEPSOnResolutionMismatch = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.colorMatchPreferred = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.multipleResolutionMatching = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.cacheSeparately = flag;
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      _flags.unboundedCacheDepth = flag;
+
+      /*
+       * get the image reps and add them.
+       */
+      a = [coder decodeObject];
+      [self addRepresentations: a];
+    }
   return self;
 }
 
