@@ -50,18 +50,11 @@
 #include <AppKit/GMArchiver.h>
 
 
-
-//*****************************************************************************
-//
-// 		NSPanel
-//
-//*****************************************************************************
-
 @implementation	NSPanel
 
-//
-// Class methods
-//
+/*
+ * Class methods
+ */
 + (void)initialize
 {
   if (self == [NSPanel class])
@@ -70,9 +63,9 @@
     }
 }
 
-//
-// Instance methods
-//
+/*
+ * Instance methods
+ */
 - (id) init
 {
   int style = NSTitledWindowMask | NSClosableWindowMask;
@@ -83,35 +76,29 @@
 			     defer: NO];
 }
 
-- (id) initWithContentRect: (NSRect)contentRect
-		 styleMask: (unsigned int)aStyle
-		   backing: (NSBackingStoreType)bufferingType
-		     defer: (BOOL)flag
-		    screen: (NSScreen*)aScreen
-{
-  self = [super initWithContentRect: contentRect
-			  styleMask: aStyle
-			    backing: bufferingType
-			      defer: flag
-			     screen: aScreen];
-  [self setReleasedWhenClosed: NO];
-  return self;
-}
-
 - (void) initDefaults
 {
   [super initDefaults];
-  menu_exclude = YES;		// Don't show panels in windows menu.
+  [self setReleasedWhenClosed: NO];
+  [self setHidesOnDeactivate: YES];
+  [self setExcludedFromWindowsMenu: YES];
 }
 
 - (BOOL) canBecomeKeyWindow
+{
+  if (_becomesKeyOnlyIfNeeded)
+    return NO;
+  return YES;
+}
+
+- (BOOL) canBecomeMainWindow
 {
   return NO;
 }
 
-//
-// If we receive an escape, close.
-//
+/*
+ * If we receive an escape, close.
+ */
 - (void) keyDown: (NSEvent*)theEvent
 {
   if ([@"\e" isEqual: [theEvent charactersIgnoringModifiers]] &&
@@ -121,9 +108,9 @@
     [super keyDown: theEvent];
 }
 
-//
-// Determining the Panel's Behavior
-//
+/*
+ * Determining the Panel's Behavior
+ */
 - (BOOL) isFloatingPanel
 {
   return _isFloatingPanel;
@@ -131,7 +118,18 @@
 
 - (void) setFloatingPanel: (BOOL)flag
 {
-  _isFloatingPanel = flag;
+  if (_isFloatingPanel != flag)
+    {
+      _isFloatingPanel = flag;
+      if (flag == YES)
+	{
+	  [self setLevel: NSFloatingWindowLevel];
+	}
+      else
+	{
+	  [self setLevel: NSNormalWindowLevel];
+	}
+    }
 }
 
 - (BOOL) worksWhenModal
@@ -149,14 +147,14 @@
   return _becomesKeyOnlyIfNeeded;
 }
 
-- (void)setBecomesKeyOnlyIfNeeded: (BOOL)flag
+- (void) setBecomesKeyOnlyIfNeeded: (BOOL)flag
 {
   _becomesKeyOnlyIfNeeded = flag;
 }
 
-//
-// NSCoding protocol
-//
+/*
+ * NSCoding protocol
+ */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
   [super encodeWithCoder: aCoder];
@@ -171,12 +169,6 @@
 
 @end /* NSPanel */
 
-//*****************************************************************************
-//
-// 		GSAlertPanel
-//
-//*****************************************************************************
-
 #define	PANX	362.0
 #define	PANY	161.0
 
@@ -189,6 +181,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 
 @interface	GSAlertPanel : NSPanel
 {
+  NSSize        originalSize;
   NSButton	*defButton;
   NSButton	*altButton;
   NSButton	*othButton;
@@ -210,9 +203,9 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 
 @implementation	GSAlertPanel
 
-//
-// Class methods
-//
+/*
+ * Class methods
+ */
 + (void)initialize
 {
   if (self == [GSAlertPanel class])
@@ -260,7 +253,6 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 		(unsigned)sender);
     }
   active = NO;
-  [self orderOut: self];
   [[NSApplication sharedApplication] stopModal];
 }
 
@@ -284,6 +276,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 - (void) encodeWithModelArchiver: (GMArchiver *)archiver
 {
   [super encodeWithModelArchiver: archiver];
+  [archiver encodeSize: originalSize withName: @"OriginalSize"];
   [archiver encodeObject: defButton withName: @"DefaultButton"];
   [archiver encodeObject: altButton withName: @"AlternateButton"];
   [archiver encodeObject: othButton withName: @"OtherButton"];
@@ -313,6 +306,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
       NSRect	rect;
       NSBox	*box;
 
+      originalSize = r.size;
       [self setMaxSize: r.size];
       [self setMinSize: r.size];
       [self setTitle: @" "];
@@ -328,7 +322,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
       [box setTitlePosition: NSNoTitle];
       [box setBorderType: NSGrooveBorder];
       [content addSubview: box];
-      [box release];
+      RELEASE(box);
 
       rect.size.height = bh;
       rect.size.width = bw;
@@ -377,14 +371,14 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 
       rect.size.height = 36.0;
       rect.size.width = 344.0;
-      rect.origin.y = 46.0;
+      rect.origin.y = 55.0; 
       rect.origin.x = 8.0;
       messageField = [[NSTextField alloc] initWithFrame: rect];
       [messageField setAutoresizingMask:
-		NSViewWidthSizable | NSViewHeightSizable | NSViewMaxYMargin];
+		      NSViewMinYMargin | NSViewMaxYMargin];
       [messageField setEditable: NO];
       [messageField setSelectable: NO];
-      [messageField setBordered: NO];
+      [messageField setBezeled: NO];
       [messageField setDrawsBackground: NO];
       [messageField setAlignment: NSCenterTextAlignment];
       [messageField setStringValue: @""];
@@ -395,10 +389,10 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
       rect.origin.y = 121.0;
       rect.origin.x = 64.0;
       titleField = [[NSTextField alloc] initWithFrame: rect];
-      [titleField setAutoresizingMask: NSViewWidthSizable | NSViewMinYMargin];
+      [titleField setAutoresizingMask: NSViewMinYMargin];
       [titleField setEditable: NO];
       [titleField setSelectable: NO];
-      [titleField setBordered: NO];
+      [titleField setBezeled: NO];
       [titleField setDrawsBackground: NO];
       [titleField setStringValue: @""];
       [titleField setFont: [NSFont systemFontOfSize: 18.0]];
@@ -410,6 +404,7 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 - (id) initWithModelUnarchiver: (GMUnarchiver*)unarchiver
 {
   self = [super initWithModelUnarchiver: unarchiver];
+  originalSize = [unarchiver decodeSizeWithName: @"OriginalSize"];
   defButton = [[unarchiver decodeObjectWithName: @"DefaultButton"] retain];
   altButton = [[unarchiver decodeObjectWithName: @"AlternateButton"] retain];
   othButton = [[unarchiver decodeObjectWithName: @"OtherButton"] retain];
@@ -427,11 +422,9 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 
 - (int) runModal
 {
-  NSApplication	*app;
-
-  app = [NSApplication sharedApplication];
   active = YES;
-  [app runModalForWindow: self];
+  [NSApp runModalForWindow: self];
+  [self orderOut: self];
   return result;
 }
 
@@ -442,13 +435,33 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	    other: (NSString*)otherButton
 {
   NSView	*content = [self contentView];
+  float		button_origin;
+  float		excessWidth = 0;
+  float         width;
+  NSRect	bRect;
+  NSSize        buttonSize = NSMakeSize (72, 24);
+  int           numberOfButtons = 0;
 
-  if (defaultButton)
+  /* First step, get minimum size needed for objects */
+  /* Put all the objects in the panel now, otherwise resizing the panel 
+     won't do the vertical position computations for us */
+  if ([icoButton superview] == nil) 
+    [content addSubview: icoButton];
+
+  /* NB: Need to make all the buttons of the same size */
+  if (defaultButton != nil)
     {
+      numberOfButtons++;
       [defButton setTitle: defaultButton];
+      [defButton sizeToFit];
+      bRect = [defButton frame];
+      if (bRect.size.height > buttonSize.height)
+	buttonSize.height = bRect.size.height;
+      if (bRect.size.width > buttonSize.width)
+	buttonSize.width = bRect.size.width;
+      
       if ([defButton superview] == nil)
 	[content addSubview: defButton];
-      [self makeFirstResponder: defButton];
     }
   else
     {
@@ -456,9 +469,17 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	[defButton removeFromSuperview];
     }
 
-  if (alternateButton)
+  if (alternateButton != nil)
     {
+      numberOfButtons++;
       [altButton setTitle: alternateButton];
+      [altButton sizeToFit];
+      bRect = [altButton frame];
+      if (bRect.size.height > buttonSize.height)
+	buttonSize.height = bRect.size.height;
+      if (bRect.size.width > buttonSize.width)
+	buttonSize.width = bRect.size.width;
+
       if ([altButton superview] == nil)
 	[content addSubview: altButton];
     }
@@ -468,9 +489,17 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	[altButton removeFromSuperview];
     }
 
-  if (otherButton)
+  if (otherButton != nil)
     {
+      numberOfButtons++;
       [othButton setTitle: otherButton];
+      [othButton sizeToFit];
+      bRect = [othButton frame];
+      if (bRect.size.height > buttonSize.height)
+	buttonSize.height = bRect.size.height;
+      if (bRect.size.width > buttonSize.width)
+	buttonSize.width = bRect.size.width;
+      
       if ([othButton superview] == nil)
 	[content addSubview: othButton];
     }
@@ -480,9 +509,31 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	[othButton removeFromSuperview];
     }
 
-  if (message)
+  if (numberOfButtons > 0)
+    {
+      // Compute total width taken up by buttons
+      // buttonSize: size of a button
+      // 10: spacing between buttons
+      // 5: spacing between last button and window border
+      width = (10 + buttonSize.width) * numberOfButtons;
+
+      // If needed, take note that we have to resize the panel
+      if (width - originalSize.width > excessWidth)
+	excessWidth = width - originalSize.width;
+
+      // Secondary TODO: Check also height.
+    }	
+
+  if (message != nil)
     {
       [messageField setStringValue: message];
+      [messageField sizeToFit];
+      // 5: spacing between text and window borders
+      width = [messageField frame].size.width + 10;
+
+      if (width - originalSize.width > excessWidth)
+	excessWidth = width - originalSize.width;
+
       if ([messageField superview] == nil)
 	[content addSubview: messageField];
     }
@@ -492,9 +543,12 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	[messageField removeFromSuperview];
     }
 
-  if (title)
+  if (title != nil)
     {
       [titleField setStringValue: title];
+      [titleField sizeToFit];
+      // TODO: Check title dimension
+
       if ([titleField superview] == nil)
 	[content addSubview: titleField];
     }
@@ -504,18 +558,69 @@ static GSAlertPanel	*gmodelAlertPanel = nil;
 	[titleField removeFromSuperview];
     }
 
+  /* Second step, resize the panel to comfortable size */
+  if (excessWidth > 0)
+    {
+      NSSize s = NSMakeSize (originalSize.width + excessWidth, 
+			     originalSize.height + (excessWidth/2));
+      [self setMaxSize: s];
+      [self setMinSize: s];
+      [self setContentSize: s];
+    }
+  else
+    {
+      // Resize the panel to standard dimension
+      [self setMaxSize: originalSize];
+      [self setMinSize: originalSize];
+      [self setContentSize: originalSize];
+    }
+
+  /* Third step, now that the panel size is OK, move objects 
+     in the right position */
+  button_origin = [content bounds].size.width - 5 - buttonSize.width;
+
+  if (defaultButton != nil)
+    {
+      bRect.size = buttonSize;
+      bRect.origin.x = button_origin;
+      bRect.origin.y = [defButton frame].origin.y;
+      [defButton setFrame: bRect];
+      button_origin -= 10 + buttonSize.width;
+      
+      [self makeFirstResponder: defButton];
+    }
+  
+  if (alternateButton != nil)
+    {
+      bRect.size = buttonSize;
+      bRect.origin.x = button_origin;
+      bRect.origin.y = [altButton frame].origin.y;
+      [altButton setFrame: bRect];
+      button_origin -= 10 + buttonSize.width;
+    }
+  
+  if (otherButton != nil)
+    {
+      bRect.size = buttonSize;
+      bRect.origin.x = button_origin;
+      bRect.origin.y = [othButton frame].origin.y;
+      [othButton setFrame: bRect];
+    }
+  
+  if (message != nil)
+    {
+      // Center the message
+      bRect = [messageField frame];
+      bRect.origin.x = ([content frame].size.width - bRect.size.width) / 2;
+      [messageField setFrameOrigin: NSMakePoint (bRect.origin.x, bRect.origin.y)];
+    }
+
   result = NSAlertErrorReturn;	/* If no button was pressed	*/
   [content display];
 }
 
 @end /* GSAlertPanel */
 
-//*****************************************************************************
-//
-// 		Alert panel functions
-//
-//*****************************************************************************
-
 id
 NSGetAlertPanel(NSString *title,
 		NSString *msg,
@@ -545,12 +650,11 @@ NSGetAlertPanel(NSString *title,
 #else
 
       panel = [GSAlertPanel alloc];
-      panel = [panel initWithContentRect:NSMakeRect(0,0,PANX,PANY)
-					 styleMask: NSTitledWindowMask
-					 backing: NSBackingStoreRetained
-					 defer: YES
-					 screen: nil];
-	  [panel center];
+      panel = [panel initWithContentRect: NSMakeRect(0, 0, PANX, PANY)
+			       styleMask: NSTitledWindowMask
+				 backing: NSBackingStoreRetained
+				   defer: YES
+				  screen: nil];
 
 #endif
     }
@@ -641,9 +745,9 @@ NSGetInformationalAlertPanel(NSString *title,
 void
 NSReleaseAlertPanel(id alertPanel)
 {
-  if (alertPanel != standardAlertPanel &&
-      alertPanel != informationalAlertPanel &&
-      alertPanel != criticalAlertPanel)
+  if (alertPanel != standardAlertPanel
+    && alertPanel != informationalAlertPanel
+    && alertPanel != criticalAlertPanel)
     [alertPanel release];
 }
 
@@ -702,7 +806,7 @@ NSRunCriticalAlertPanel(NSString *title,
 
   va_start (ap, otherButton);
   panel = NSGetCriticalAlertPanel(title, msg,
-		defaultButton, alternateButton, otherButton, ap);
+    defaultButton, alternateButton, otherButton, ap);
   va_end (ap);
 
   result = [panel runModal];
@@ -723,7 +827,7 @@ NSRunInformationalAlertPanel(NSString *title,
 
   va_start (ap, otherButton);
   panel = NSGetInformationalAlertPanel(title, msg,
-		defaultButton, alternateButton, otherButton, ap);
+    defaultButton, alternateButton, otherButton, ap);
   va_end (ap);
 
   result = [panel runModal];
@@ -784,7 +888,7 @@ NSRunLocalizedAlertPanel(NSString *table,
   else
     {
       panel = NSGetAlertPanel(title, message, defaultButton,
-			alternateButton, otherButton, ap);
+	alternateButton, otherButton, ap);
       standardAlertPanel = panel;
     }
 
