@@ -195,6 +195,12 @@ static Class rulerViewClass = nil;
 
 - (void) setContentView: (NSClipView*)aView
 {
+  if (aView == nil)
+    [NSException raise: NSInvalidArgumentException
+		format: @"Attempt to set nil content view"];
+  if ([aView isKindOfClass: [NSView class]] == NO)
+    [NSException raise: NSInvalidArgumentException
+		format: @"Attempt to set non-view object as content view"];
   ASSIGN((id)_contentView, (id)aView);
   [self addSubview: _contentView];
   [_contentView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
@@ -467,8 +473,7 @@ static Class rulerViewClass = nil;
 
 - (void) tile
 {
-  NSRect boundsRect = [self bounds];
-  NSSize contentSize = [isa contentSizeForFrameSize: boundsRect.size
+  NSSize contentSize = [isa contentSizeForFrameSize: _bounds.size
 			      hasHorizontalScroller: _hasHorizScroller
 				hasVerticalScroller: _hasVertScroller
 					 borderType: _borderType];
@@ -478,7 +483,7 @@ static Class rulerViewClass = nil;
   NSRect horizScrollerRect = NSZeroRect;
   float borderThickness = 0;
 
-  switch ([self borderType])
+  switch (_borderType)
     {
       case NSNoBorder:
 	break;
@@ -498,10 +503,10 @@ static Class rulerViewClass = nil;
 
   if (_hasVertScroller)
     {
-      vertScrollerRect.origin.x = boundsRect.origin.x + borderThickness;
-      vertScrollerRect.origin.y = boundsRect.origin.y + borderThickness;
+      vertScrollerRect.origin.x = _bounds.origin.x + borderThickness;
+      vertScrollerRect.origin.y = _bounds.origin.y + borderThickness;
       vertScrollerRect.size.width = scrollerWidth;
-      vertScrollerRect.size.height =bounds.size.height - 2 * borderThickness;
+      vertScrollerRect.size.height = _bounds.size.height - 2 * borderThickness;
 
       contentRect.origin.x += scrollerWidth + 1;
     }
@@ -509,7 +514,7 @@ static Class rulerViewClass = nil;
   if (_hasHorizScroller)
     {
       horizScrollerRect.origin.x = contentRect.origin.x;
-      horizScrollerRect.origin.y = boundsRect.origin.y + borderThickness;
+      horizScrollerRect.origin.y = _bounds.origin.y + borderThickness;
       horizScrollerRect.size.width = contentRect.size.width;
       horizScrollerRect.size.height = scrollerWidth;
 
@@ -533,11 +538,11 @@ static Class rulerViewClass = nil;
 {
   NSGraphicsContext	*ctxt = GSCurrentContext();
   float scrollerWidth = [NSScroller scrollerWidth];
-  float horizLinePosition, horizLineLength = [self bounds].size.width;
+  float horizLinePosition, horizLineLength = _bounds.size.width;
   float borderThickness = 0;
 
   DPSgsave(ctxt);
-  switch ([self borderType])
+  switch (_borderType)
     {
       case NSNoBorder:
 	break;
@@ -567,8 +572,8 @@ static Class rulerViewClass = nil;
     {
       horizLinePosition = scrollerWidth + borderThickness;
       horizLineLength -= scrollerWidth + 2 * borderThickness;
-      DPSmoveto(ctxt, horizLinePosition, borderThickness + 1);
-      DPSrlineto(ctxt, 0, [self bounds].size.height - 2 * borderThickness - 1);
+      DPSmoveto(ctxt, horizLinePosition, borderThickness);
+      DPSrlineto(ctxt, 0, _bounds.size.height - 2 * borderThickness - 1);
       DPSstroke(ctxt);
     }
 
@@ -577,7 +582,7 @@ static Class rulerViewClass = nil;
       float	ypos = scrollerWidth + borderThickness + 1;
 
       if (_rFlags.flipped_view)
-	ypos = [self bounds].size.height - ypos;
+	ypos = _bounds.size.height - ypos;
       DPSmoveto(ctxt, horizLinePosition, ypos);
       DPSrlineto(ctxt, horizLineLength - 1, 0);
       DPSstroke(ctxt);
@@ -778,4 +783,65 @@ static Class rulerViewClass = nil;
   return _vertScroller;
 }
 
+/*
+ * NSCoding protocol
+ */
+- (void) encodeWithCoder: (NSCoder*)aCoder
+{
+  [super encodeWithCoder: aCoder];
+
+  NSDebugLLog(@"NSScrollView", @"NSScrollView: start encoding\n");
+  [aCoder encodeObject: _contentView];
+  [aCoder encodeValueOfObjCType: @encode(NSBorderType) at: &_borderType];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_scrollsDynamically];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_rulersVisible];
+  [aCoder encodeValueOfObjCType: @encode(float) at: &_hLineScroll];
+  [aCoder encodeValueOfObjCType: @encode(float) at: &_hPageScroll];
+  [aCoder encodeValueOfObjCType: @encode(float) at: &_vLineScroll];
+  [aCoder encodeValueOfObjCType: @encode(float) at: &_vPageScroll];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_hasHorizScroller];
+  if (_hasHorizScroller)
+    [aCoder encodeObject: _horizScroller];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_hasVertScroller];
+  if (_hasVertScroller)
+    [aCoder encodeObject: _vertScroller];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_hasHorizRuler];
+  if (_hasHorizRuler)
+    [aCoder encodeObject: _horizRuler];
+  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_hasVertRuler];
+  if (_hasVertRuler)
+    [aCoder encodeObject: _vertRuler];
+  NSDebugLLog(@"NSScrollView", @"NSScrollView: finish encoding\n");
+}
+
+- (id) initWithCoder: (NSCoder*)aDecoder
+{
+  [super initWithCoder: aDecoder];
+
+  NSDebugLLog(@"NSScrollView", @"NSScrollView: start decoding\n");
+  [aDecoder decodeValueOfObjCType: @encode(id) at: &_contentView];
+  [aDecoder decodeValueOfObjCType: @encode(NSBorderType) at: &_borderType];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_scrollsDynamically];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_rulersVisible];
+  [aDecoder decodeValueOfObjCType: @encode(float) at: &_hLineScroll];
+  [aDecoder decodeValueOfObjCType: @encode(float) at: &_hPageScroll];
+  [aDecoder decodeValueOfObjCType: @encode(float) at: &_vLineScroll];
+  [aDecoder decodeValueOfObjCType: @encode(float) at: &_vPageScroll];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_horizScroller];
+  if (_horizScroller)
+    [aDecoder decodeValueOfObjCType: @encode(id) at: &_horizScroller];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_vertScroller];
+  if (_vertScroller)
+    [aDecoder decodeValueOfObjCType: @encode(id) at: &_vertScroller];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_horizRuler];
+  if (_horizRuler)
+    [aDecoder decodeValueOfObjCType: @encode(id) at: &_horizRuler];
+  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_vertRuler];
+  if (_vertRuler)
+    [aDecoder decodeValueOfObjCType: @encode(id) at: &_vertRuler];
+  NSDebugLLog(@"NSScrollView", @"NSScrollView: finish decoding\n");
+
+  return self;
+}
+
 @end
