@@ -252,7 +252,7 @@ static const float buttonsWidth = 16;
 
 - (void) checkSpaceForParts
 {
-  NSSize	frameSize = [self frame].size;
+  NSSize	frameSize = _frame.size;
   float		size = (_isHorizontal ? frameSize.width : frameSize.height);
   float		scrollerWidth = [isa scrollerWidth];
 
@@ -357,8 +357,8 @@ static const float buttonsWidth = 16;
    */
   NSRect rect;
 
-  if (thePoint.x <= 0 || thePoint.x >= frame.size.width
-    || thePoint.y <= 0 || thePoint.y >= frame.size.height)
+  if (thePoint.x <= 0 || thePoint.x >= _frame.size.width
+    || thePoint.y <= 0 || thePoint.y >= _frame.size.height)
     return NSScrollerNoPart;
 
   rect = [self rectForPart: NSScrollerDecrementLine];
@@ -613,10 +613,8 @@ static const float buttonsWidth = 16;
 
       if (theCell)
 	{
-	  [self lockFocus];
 	  [theCell highlight: YES withFrame: rect inView: self];
-	  [self unlockFocus];
-	  [window flushWindow];
+	  [_window flushWindow];
 
 	  NSDebugLog (@"tracking cell %x", theCell);
 
@@ -625,10 +623,8 @@ static const float buttonsWidth = 16;
 				      ofView: self
 				untilMouseUp: YES];
 
-	  [self lockFocus];
 	  [theCell highlight: NO withFrame: rect inView: self];
-	  [self unlockFocus];
-	  [window flushWindow];
+	  [_window flushWindow];
 	}
 
       if (shouldReturn)
@@ -652,6 +648,9 @@ static const float buttonsWidth = 16;
   NSDebugLog (@"NSScroller drawRect: ((%f, %f), (%f, %f))",
 	    rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
 
+  [[_window backgroundColor] set];
+  NSRectFill (_bounds);
+
   [self drawKnobSlot];
   [self drawKnob];
 
@@ -718,7 +717,7 @@ static const float buttonsWidth = 16;
 
 - (NSRect) rectForPart: (NSScrollerPart)partCode
 {
-  NSRect scrollerFrame = frame;
+  NSRect scrollerFrame = _frame;
   float x = 1, y = 1;
   float width, height;
   float buttonsSize = 2 * buttonsWidth + 2;
