@@ -1,4 +1,4 @@
-/* 
+/*
    NSWorkspace.m
 
    Description...
@@ -44,11 +44,87 @@
 #include <Foundation/NSProcessInfo.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSNotificationQueue.h>
+#include <Foundation/NSDistributedNotificationCenter.h>
 #include <Foundation/NSConnection.h>
 
 #define stringify_it(X) #X
 #define	mkpath(X) stringify_it(X) "/Tools"
 
+static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
+
+@interface	_GSWorkspaceCenter: NSNotificationCenter
+{
+  NSDistributedNotificationCenter	*remote;
+}
+- (void) _handleRemoteNotification: (NSNotification*)aNotification;
+@end
+
+@implementation	_GSWorkspaceCenter
+
+- (void) dealloc
+{
+  [remote removeObserver: self name: nil object: GSWorkspaceNotification];
+  RELEASE(remote);
+  [super dealloc];
+}
+
+- (id) init
+{
+  self = [super init];
+  if (self != nil)
+    {
+      remote = RETAIN([NSDistributedNotificationCenter defaultCenter]);
+      [remote addObserver: self
+		 selector: @selector(_handleRemoteNotification:)
+		     name: nil
+		   object: GSWorkspaceNotification];
+    }
+  return self;
+}
+
+/*
+ * Post notification remotely - since we are listening for distributed
+ * notifications, we will observe the notification arriving from the
+ * distributed notification center, and it will get sent out locally too.
+ */
+- (void) postNotification: (NSNotification*)aNotification
+{
+  NSNotification	*rem;
+
+  rem = [NSNotification notificationWithName: [aNotification name]
+				      object: GSWorkspaceNotification
+				    userInfo: [aNotification userInfo]];
+  [remote postNotification: rem];
+}
+
+- (void) postNotificationName: (NSString*)name 
+		       object: (id)object
+{
+  [self postNotification: [NSNotification notificationWithName: name
+							object: object]];
+}
+
+- (void) postNotificationName: (NSString*)name 
+		       object: (id)object
+		     userInfo: (NSDictionary*)info
+{
+  [self postNotification: [NSNotification notificationWithName: name
+							object: object
+						      userInfo: info]];
+}
+
+/*
+ * Forward a notification from a remote application to observers in this
+ * application.
+ */
+- (void) _handleRemoteNotification: (NSNotification*)aNotification
+{
+  [super postNotification: aNotification];
+}
+
+@end
+
+
 
 @implementation	NSWorkspace
 
@@ -92,7 +168,7 @@ static NSString			*_rootPath = @"/";
 
       beenHere = YES;
 
-      workspaceCenter = [NSNotificationCenter new];
+      workspaceCenter = [_GSWorkspaceCenter new];
       iconMap = [NSMutableDictionary new];
 
       /*
@@ -182,9 +258,21 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
       if ([file isAbsolutePath] == NO)
 	{
 	  NSString	*path;
+	  NSString	*iconPath;
+	  NSBundle	*bundle;
 
 	  path = [ws fullPathForApplication: appName];
-	  file = [path stringByAppendingPathComponent: file];
+	  bundle = [NSBundle bundleWithPath: path];
+	  iconPath = [bundle pathForImageResource: file];
+	  /*
+	   * If the icon is not in the Resources of the app, try looking
+	   * directly in the app wrapper.
+	   */
+	  if (iconPath == nil)
+	    {
+	      iconPath = [path stringByAppendingPathComponent: file];
+	    }
+	  file = iconPath;
 	}
       if ([[NSFileManager defaultManager] isReadableFileAtPath: file] == YES)
 	{
@@ -509,7 +597,7 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
     andDeactivate: (BOOL)flag
 {
   NSString      *port = [appName stringByDeletingPathExtension];
-  id            app;
+  id            app = nil;
 
   /*
    *	Try to contact a running application.
@@ -529,6 +617,7 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
     {
       NSString	*path;
       NSArray	*args;
+      NSDictionary  *userinfo = nil;
 
       path = [self locateApplicationBinary: appName];
       if (path == nil)
@@ -539,6 +628,16 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
 	    @"Continue", nil, nil);
 	  return NO;
 	}
+
+     // App being launched, raise NSWorkspaceWillLaunchApplicationNotification
+     userinfo = [NSDictionary dictionaryWithObject: port
+					    forKey: @"NSApplicationName"];
+
+     [workspaceCenter
+          postNotificationName: NSWorkspaceWillLaunchApplicationNotification
+          object: self
+          userInfo: userinfo];
+
       args = [NSArray arrayWithObjects: @"-GSFilePath", fullPath, nil];
       if ([NSTask launchedTaskWithLaunchPath: path arguments: args] == nil)
 	{
@@ -548,6 +647,7 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
 	    @"Continue", nil, nil);
 	  return NO;
 	}
+
       return YES;
     }
   else
@@ -555,9 +655,9 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
       NS_DURING
 	{
 	  if (flag == NO)
-	    [app application: nil openFileWithoutUI: fullPath];
+	    [app application: NSApp openFileWithoutUI: fullPath];
 	  else
-	    [app application: nil openFile: fullPath];
+	    [app application: NSApp openFile: fullPath];
 	}
       NS_HANDLER
 	{
@@ -642,12 +742,15 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
 - (NSImage *) iconForFile: (NSString *)aPath
 {
   NSImage	*image = nil;
-  BOOL		isDir = NO;
   NSString	*iconPath = nil;
   NSString	*pathExtension = [[aPath pathExtension] lowercaseString];
   NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSDictionary	*attributes;
+  NSString	*fileType;
 
-  if ([mgr fileExistsAtPath: aPath isDirectory: &isDir] && isDir)
+  attributes = [mgr fileAttributesAtPath: aPath traverseLink: YES];
+  fileType = [attributes objectForKey: NSFileType];
+  if ([fileType isEqual: NSFileTypeDirectory] == YES)
     {
       if ([pathExtension isEqualToString: @"app"]
 	|| [pathExtension isEqualToString: @"debug"]
@@ -659,12 +762,26 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
 	  iconPath = [[bundle infoDictionary] objectForKey: @"NSIcon"];
 	  if (iconPath && [iconPath isAbsolutePath] == NO)
 	    {
-	      iconPath = [aPath stringByAppendingPathComponent: iconPath];
+	      NSString	*file = iconPath;
+
+	      iconPath = [bundle pathForImageResource: file];
+
+	      /*
+	       * If there is no icon in the Resources of the app, try
+	       * looking directly in the app wrapper.
+	       */
+	      if (iconPath == nil)
+		{
+		  iconPath = [aPath stringByAppendingPathComponent: file];
+		  if ([mgr isReadableFileAtPath: iconPath] == NO)
+		    {
+		      iconPath = nil;
+		    }
+		}
 	    }
 	  /*
 	   *	If there is no icon specified in the Info.plist for app
-	   *	try 'wrapper/app.tiff' and 'wrapper/.dir.tiff' as
-	   *	possible locations for the application icon.
+	   *	try 'wrapper/app.tiff'
 	   */
 	  if (iconPath == nil)
 	    {
@@ -675,39 +792,48 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
 	      iconPath = [iconPath stringByAppendingPathExtension: @"tiff"];
 	      if ([mgr isReadableFileAtPath: iconPath] == NO)
 		{
-		  str = @".dir.tiff";
-		  iconPath = [aPath stringByAppendingPathComponent: str];
-		  if ([mgr isReadableFileAtPath: iconPath] == NO)
-		    iconPath = nil;
+		  iconPath = nil;
 		}
 	    }
+	}
 
-	  if (iconPath)
+      /*
+       *	If we have no iconPath, try 'dir/.dir.tiff' as a
+       *	possible locations for the directory icon.
+       */
+      if (iconPath == nil)
+	{
+	  iconPath = [aPath stringByAppendingPathComponent: @".dir.tiff"];
+	  if ([mgr isReadableFileAtPath: iconPath] == NO)
 	    {
-	      NS_DURING
-		{
-		  image = [[NSImage alloc] initWithContentsOfFile: iconPath];
-		  AUTORELEASE(image);
-		}
-	      NS_HANDLER
-		{
-		  NSLog(@"BAD TIFF FILE '%@'", iconPath);
-		}
-	      NS_ENDHANDLER
+	      iconPath = nil;
 	    }
 	}
 
+      if (iconPath != nil)
+	{
+	  NS_DURING
+	    {
+	      image = [[NSImage alloc] initWithContentsOfFile: iconPath];
+	      AUTORELEASE(image);
+	    }
+	  NS_HANDLER
+	    {
+	      NSLog(@"BAD TIFF FILE '%@'", iconPath);
+	    }
+	  NS_ENDHANDLER
+	}
+
       if (image == nil)
 	{
 	  image = [self _iconForExtension: pathExtension];
-	}
-
-      if (image == nil)
-	{
-	  if ([aPath isEqual: _rootPath])
-	    image = [self rootImage];
-	  else
-	    image= [self folderImage];
+	  if (image == nil || image == [self unknownFiletypeImage])
+	    {
+	      if ([aPath isEqual: _rootPath])
+		image = [self rootImage];
+	      else
+		image = [self folderImage];
+	    }
 	}
     }
   else
@@ -822,12 +948,25 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
 	        autolaunch: (BOOL)autolaunch
 {
   NSString	*path;
+  NSDictionary  *userinfo;
 
   path = [self locateApplicationBinary: appName];
+
   if (path == nil)
     return NO;
+
+  // App being launched, raise NSWorkspaceWillLaunchApplicationNotification
+  userinfo = [NSDictionary dictionaryWithObject:
+    [[appName lastPathComponent] stringByDeletingPathExtension]
+    forKey: @"NSApplicationName"];
+  [workspaceCenter
+    postNotificationName: NSWorkspaceWillLaunchApplicationNotification
+    object: self
+    userInfo: userinfo];
+
   if ([NSTask launchedTaskWithLaunchPath: path arguments: nil] == nil)
     return NO;
+
   return YES;
 }
 
