@@ -3,11 +3,13 @@
 
    Functions for dealing with tiff images.
 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996,1999 Free Software Foundation, Inc.
    
    Author:  Adam Fedor <fedor@colorado.edu>
    Date: Feb 1996
-   
+
+   Support for writing tiffs: Richard Frith-Macdonald
+
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
@@ -73,7 +75,8 @@ typedef struct {
   long  size;
   long  position;
   const char* mode;
-  realloc_data_callback* realloc_data;
+  char **outdata;
+  long *outposition;
 } chandle_t;
 
 /* Client functions that provide reading/writing of data for libtiff */
@@ -81,7 +84,7 @@ static tsize_t
 TiffHandleRead(thandle_t handle, tdata_t buf, toff_t count)
 {
   chandle_t* chand = (chandle_t *)handle;
-  NSDebugLog (@"TiffHandleRead\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleRead\n");
   if (chand->position >= chand->size)
     return 0;
   if (chand->position + count > chand->size)
@@ -94,19 +97,22 @@ static tsize_t
 TiffHandleWrite(thandle_t handle, tdata_t buf, toff_t count)
 {
   chandle_t* chand = (chandle_t *)handle;
-  NSDebugLog (@"TiffHandleWrite\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleWrite\n");
   if (chand->mode == "r")
     return 0;
   if (chand->position + count > chand->size)
     {
-      if (chand->realloc_data == NULL)
-	return 0;
       chand->size = chand->position + count + 1;
-      chand->data = chand->realloc_data(chand->data, chand->size);
+      chand->data = objc_realloc(chand->data, chand->size);
+      *(chand->outdata) = chand->data;
       if (chand->data == NULL)
 	return 0;
     }
   memcpy(chand->data + chand->position, buf, count);
+  chand->position += count;
+  if (chand->position > *(chand->outposition))
+    *(chand->outposition) = chand->position;
+  
   return count;
 }
 
@@ -114,7 +120,7 @@ static toff_t
 TiffHandleSeek(thandle_t handle, toff_t offset, int mode)
 {
   chandle_t* chand = (chandle_t *)handle;
-  NSDebugLog (@"TiffHandleSeek\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleSeek\n");
   switch(mode) 
     {
     case SEEK_SET: chand->position = offset; break;
@@ -133,7 +139,7 @@ TiffHandleClose(thandle_t handle)
 {
   chandle_t* chand = (chandle_t *)handle;
 
-  NSDebugLog (@"TiffHandleClose\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleClose\n");
   /* Presumably, we don't need the handle anymore */
   OBJC_FREE(chand);
   return 0;
@@ -143,7 +149,7 @@ static toff_t
 TiffHandleSize(thandle_t handle)
 {
   chandle_t* chand = (chandle_t *)handle;
-  NSDebugLog (@"TiffHandleSize\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleSize\n");
   return chand->size;
 }
 
@@ -152,7 +158,7 @@ TiffHandleMap(thandle_t handle, tdata_t* data, toff_t* size)
 {
   chandle_t* chand = (chandle_t *)handle;
   
-  NSDebugLog (@"TiffHandleMap\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleMap\n");
   *data = chand->data;
   *size = chand->size;
     
@@ -162,24 +168,44 @@ TiffHandleMap(thandle_t handle, tdata_t* data, toff_t* size)
 static void
 TiffHandleUnmap(thandle_t handle, tdata_t data, toff_t size)
 {
-  NSDebugLog (@"TiffHandleUnmap\n");
+  NSDebugLLog(@"NSImage", @"TiffHandleUnmap\n");
   /* Nothing to unmap. */
 }
 
 /* Open a tiff from a stream. Returns NULL if can't read tiff information.  */
 TIFF* 
-NSTiffOpenData(char* data, long size, const char* mode,
-			    realloc_data_callback* realloc_data)
+NSTiffOpenDataRead(const char* data, long size)
 {
   chandle_t* handle;
-  NSDebugLog (@"NSTiffOpenData\n");
+  NSDebugLLog(@"NSImage", @"NSTiffOpenData\n");
   OBJC_MALLOC(handle, chandle_t, 1);
-  handle->data = data;
+  handle->data = (char*)data;
+  handle->outdata = 0;
   handle->position = 0;
+  handle->outposition = 0;
   handle->size = size;
-  handle->mode = mode;
-  handle->realloc_data = realloc_data;
-  return TIFFClientOpen("NSData", mode,
+  handle->mode = "r";
+  return TIFFClientOpen("NSData", "r",
+			(thandle_t)handle,
+			TiffHandleRead, TiffHandleWrite,
+			TiffHandleSeek, TiffHandleClose,
+			TiffHandleSize,
+			TiffHandleMap, TiffHandleUnmap);
+}
+
+TIFF* 
+NSTiffOpenDataWrite(char **data, long *size)
+{
+  chandle_t* handle;
+  NSDebugLLog(@"NSImage", @"NSTiffOpenData\n");
+  OBJC_MALLOC(handle, chandle_t, 1);
+  handle->data = *data;
+  handle->outdata = data;
+  handle->position = 0;
+  handle->outposition = size;
+  handle->size = *size;
+  handle->mode = "w";
+  return TIFFClientOpen("NSData", "w",
 			(thandle_t)handle,
 			TiffHandleRead, TiffHandleWrite,
 			TiffHandleSeek, TiffHandleClose,
@@ -214,6 +240,7 @@ NSTiffGetInfo(int imageNumber, TIFF* image)
   TIFFGetField(image, TIFFTAG_IMAGEWIDTH, &info->width);
   TIFFGetField(image, TIFFTAG_IMAGELENGTH, &info->height);
   TIFFGetField(image, TIFFTAG_COMPRESSION, &info->compression);
+  TIFFGetField(image, TIFFTAG_JPEGQUALITY, &info->quality);
   TIFFGetField(image, TIFFTAG_SUBFILETYPE, &info->subfileType);
 
   /* If the following tags aren't present then use the TIFF defaults. */
@@ -249,8 +276,8 @@ NSTiffGetInfo(int imageNumber, TIFF* image)
 }
 
 #define READ_SCANLINE(sample) \
-	if ( TIFFReadScanline( image, buf, row, sample ) < 0 ) { \
-	    NSLog(@"tiff: bad data read on line %d\n", row ); \
+	if (TIFFReadScanline(image, buf, row, sample) < 0) { \
+	    NSLog(@"tiff: bad data read on line %d\n", row); \
 	    error = 1; \
 	    break; \
 	} \
@@ -290,51 +317,51 @@ NSTiffRead(int imageNumber, TIFF* image, NSTiffInfo* info, char* data)
     memcpy(info, newinfo, sizeof(NSTiffInfo));
 
   map = NULL;
-  if ( newinfo->photoInterp == PHOTOMETRIC_PALETTE) 
+  if (newinfo->photoInterp == PHOTOMETRIC_PALETTE) 
     {
       map = NSTiffGetColormap(image);
       if (!map)
 	return -1;
     }
 
-  maxval = ( 1 << newinfo->bitsPerSample ) - 1;
+  maxval = (1 << newinfo->bitsPerSample) - 1;
   line   = ceil((float)newinfo->width * newinfo->bitsPerSample / 8.0);
-  size   = ceil((float)line * newinfo->height * newinfo->samplesPerPixel );
+  size   = ceil((float)line * newinfo->height * newinfo->samplesPerPixel);
   scan_line_size = TIFFScanlineSize(image);
   OBJC_MALLOC(buf, u_char, TIFFScanlineSize(image));
   
   raster = (u_char *)data;
   outP = raster;
-  switch ( newinfo->photoInterp ) 
+  switch (newinfo->photoInterp) 
     {
     case PHOTOMETRIC_MINISBLACK:
     case PHOTOMETRIC_MINISWHITE:
       if (newinfo->planarConfig == PLANARCONFIG_CONTIG) 
 	{
-	  for ( row = 0; row < newinfo->height; ++row ) 
+	  for (row = 0; row < newinfo->height; ++row) 
 	    {
 	      READ_SCANLINE(0)
-		for ( col = 0; col < line*newinfo->samplesPerPixel; col++) 
+		for (col = 0; col < line*newinfo->samplesPerPixel; col++) 
 		  *outP++ = *inP++;
 	    }
 	} 
       else 
 	{
 	  for (i = 0; i < newinfo->samplesPerPixel; i++)
-	    for ( row = 0; row < newinfo->height; ++row ) 
+	    for (row = 0; row < newinfo->height; ++row) 
 	      {
 		READ_SCANLINE(i)
-		  for ( col = 0; col < line; col++) 
+		  for (col = 0; col < line; col++) 
 		    *outP++ = *inP++;
 	      }
 	}
       break;
     case PHOTOMETRIC_PALETTE:
       {
-	for ( row = 0; row < newinfo->height; ++row ) 
+	for (row = 0; row < newinfo->height; ++row) 
 	  {
 	    READ_SCANLINE(0)
-	      for ( col = 0; col < newinfo->width; col++) 
+	      for (col = 0; col < newinfo->width; col++) 
 		{
 		  *outP++ = map->red[*inP] / 256;
 		  *outP++ = map->green[*inP] / 256;
@@ -351,11 +378,11 @@ NSTiffRead(int imageNumber, TIFF* image, NSTiffInfo* info, char* data)
     case PHOTOMETRIC_RGB:
       if (newinfo->planarConfig == PLANARCONFIG_CONTIG) 
 	{
-	  NSDebugLog(@"PHOTOMETRIC_RGB: CONTIG\n");
-	  for ( row = 0; row < newinfo->height; ++row ) 
+	  NSDebugLLog(@"NSImage", @"PHOTOMETRIC_RGB: CONTIG\n");
+	  for (row = 0; row < newinfo->height; ++row) 
 	    {
 	      READ_SCANLINE(0)
-		for ( col = 0; col < newinfo->width; col++) 
+		for (col = 0; col < newinfo->width; col++) 
 		  for (i = 0; i < newinfo->samplesPerPixel; i++)
 		    {
 		      *outP++ = *inP++;
@@ -364,12 +391,12 @@ NSTiffRead(int imageNumber, TIFF* image, NSTiffInfo* info, char* data)
 	} 
       else 
 	{
-	  NSDebugLog(@"PHOTOMETRIC_RGB: NOT CONTIG\n");
+	  NSDebugLLog(@"NSImage", @"PHOTOMETRIC_RGB: NOT CONTIG\n");
 	  for (i = 0; i < newinfo->samplesPerPixel; i++)
-	    for ( row = 0; row < newinfo->height; ++row ) 
+	    for (row = 0; row < newinfo->height; ++row) 
 	      {
 		READ_SCANLINE(i)
-		  for ( col = 0; col < newinfo->width; col++) 
+		  for (col = 0; col < newinfo->width; col++) 
 		    {
 		      *outP++ = *inP++;
 		    }
@@ -386,10 +413,91 @@ NSTiffRead(int imageNumber, TIFF* image, NSTiffInfo* info, char* data)
   return error;
 }
 
+#define WRITE_SCANLINE(sample) \
+	if (TIFFWriteScanline(image, buf, row, sample) != 1) { \
+	    NSLog(@"tiff: bad data write on line %d\n", row); \
+	    error = 1; \
+	    break; \
+	}
+
 int  
-NSWriteTiff(TIFF* image, NSTiffInfo* info, char* data)
+NSTiffWrite(TIFF* image, NSTiffInfo* info, char* data)
 {
-  return 0;
+  tdata_t	buf = (tdata_t)data;
+  int		i;
+  int		row;
+  int		error = 0;
+
+  TIFFSetField(image, TIFFTAG_IMAGEWIDTH, info->width);
+  TIFFSetField(image, TIFFTAG_IMAGELENGTH, info->height);
+  TIFFSetField(image, TIFFTAG_COMPRESSION, info->compression);
+  TIFFSetField(image, TIFFTAG_JPEGQUALITY, info->quality);
+  TIFFSetField(image, TIFFTAG_SUBFILETYPE, info->subfileType);
+  TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, info->bitsPerSample);
+  TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL, info->samplesPerPixel);
+  TIFFSetField(image, TIFFTAG_PLANARCONFIG, info->planarConfig);
+  TIFFSetField(image, TIFFTAG_PHOTOMETRIC, info->photoInterp);
+
+  switch (info->photoInterp) 
+    {
+      case PHOTOMETRIC_MINISBLACK:
+      case PHOTOMETRIC_MINISWHITE:
+	if (info->planarConfig == PLANARCONFIG_CONTIG) 
+	  {
+	    int	line = ceil((float)info->width * info->bitsPerSample / 8.0);
+
+	    for (row = 0; row < info->height; ++row) 
+	      {
+		WRITE_SCANLINE(0)
+		buf += line;
+	      }
+	  } 
+	else 
+	  {
+	    int	line = ceil((float)info->width / 8.0);
+
+	    for (i = 0; i < info->samplesPerPixel; i++)
+	      {
+		for (row = 0; row < info->height; ++row) 
+		  {
+		    WRITE_SCANLINE(i)
+		    buf += line;
+		  }
+	      }
+	  }
+	break;
+
+      case PHOTOMETRIC_RGB:
+	if (info->planarConfig == PLANARCONFIG_CONTIG) 
+	  {
+	    NSDebugLLog(@"NSImage", @"PHOTOMETRIC_RGB: CONTIG\n");
+	    for (row = 0; row < info->height; ++row) 
+	      {
+		WRITE_SCANLINE(0)
+		buf += info->width * info->samplesPerPixel;
+	      }
+	  } 
+	else 
+	  {
+	    NSDebugLLog(@"NSImage", @"PHOTOMETRIC_RGB: NOT CONTIG\n");
+	    for (i = 0; i < info->samplesPerPixel; i++)
+	      {
+		for (row = 0; row < info->height; ++row) 
+		  {
+		    WRITE_SCANLINE(i)
+		    buf += info->width;
+		  }
+	      }
+	  }
+	break;
+
+      default:
+	TIFFError(TIFFFileName(image),
+		  "Can't write photometric %d", info->photoInterp);
+	break;
+    }
+    
+  return error;
 }
 
 /*------------------------------------------------------------------------*/
