@@ -7,24 +7,22 @@
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Created: November 1998
 
-   This file is part of the GNUstep Base Library.
+   This file is part of the GNUstep Project
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+    
+   You should have received a copy of the GNU General Public  
+   License along with this library; see the file COPYING.LIB.
+   If not, write to the Free Software Foundation,
+   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
 
 #include	<Foundation/NSArray.h>
+#include	<Foundation/NSBundle.h>
 #include	<Foundation/NSDictionary.h>
 #include	<Foundation/NSFileManager.h>
 #include	<Foundation/NSString.h>
@@ -33,6 +31,7 @@
 #include	<Foundation/NSDebug.h>
 #include	<Foundation/NSDistributedLock.h>
 #include	<Foundation/NSAutoreleasePool.h>
+#include	<Foundation/NSPathUtilities.h>
 #include	<Foundation/NSSerialization.h>
 
 static void scanDirectory(NSMutableDictionary *services, NSString *path);
@@ -42,7 +41,6 @@ static NSMutableDictionary *validateService(NSDictionary *service, NSString* pat
 
 static NSString		*appsName = @".GNUstepAppList";
 static NSString		*cacheName = @".GNUstepServices";
-static NSString		*infoLoc = @"Resources/Info-gnustep.plist";
 
 static	BOOL verbose = NO;
 static	NSMutableDictionary	*serviceMap;
@@ -50,6 +48,11 @@ static	NSMutableDictionary	*filterMap;
 static	NSMutableDictionary	*printMap;
 static	NSMutableDictionary	*spellMap;
 static	NSMutableDictionary	*applicationMap;
+static	NSMutableDictionary	*extensionsMap;
+
+static Class aClass;
+static Class dClass;
+static Class sClass;
 
 int
 main(int argc, char** argv)
@@ -67,11 +70,16 @@ main(int argc, char** argv)
   NSString		*str;
   unsigned		index;
   BOOL			isDir;
+  BOOL			usedPrefixes = NO;
   NSMutableDictionary	*fullMap;
   NSDictionary		*oldMap;
 
   pool = [NSAutoreleasePool new];
 
+  aClass = [NSArray class];
+  dClass = [NSDictionary class];
+  sClass = [NSString class];
+
   proc = [NSProcessInfo processInfo];
   if (proc == nil)
     {
@@ -87,11 +95,12 @@ main(int argc, char** argv)
   printMap = [NSMutableDictionary dictionaryWithCapacity: 8];
   spellMap = [NSMutableDictionary dictionaryWithCapacity: 8];
   applicationMap = [NSMutableDictionary dictionaryWithCapacity: 64];
+  extensionsMap = [NSMutableDictionary dictionaryWithCapacity: 64];
 
   env = [proc environment];
   args = [proc arguments];
 
-  for (index = 0; index < [args count]; index++)
+  for (index = 1; index < [args count]; index++)
     {
       if ([[args objectAtIndex: index] isEqual: @"--verbose"])
 	verbose = YES;
@@ -138,6 +147,32 @@ main(int argc, char** argv)
     }
 
   roots = [NSMutableArray arrayWithCapacity: 3];
+
+  /*
+   *	Set up an array of root paths from the prefix list if possible.
+   *	If we managed to get any paths, we set a flag so we know not to
+   *	get and add default values later.
+   */
+  str = [env objectForKey: @"GNUSTEP_PATHPREFIX_LIST"];
+  if (str != nil && [str isEqualToString: @""] == NO)
+    {
+      NSArray	*a = [str componentsSeparatedByString: @":"];
+      unsigned	index;
+
+      for (index = 0; index < [a count]; index++)
+	{
+	  str = [a objectAtIndex: index];
+	  if ([str isEqualToString: @""] == NO)
+	    {
+	      if ([roots containsObject: str] == NO)
+		{
+		  [roots addObject: str];
+		  usedPrefixes = YES;
+		}
+	    }
+	}
+    }
+
   services = [NSMutableDictionary dictionaryWithCapacity: 200];
 
   /*
@@ -150,7 +185,7 @@ main(int argc, char** argv)
   if (str != nil)
     usrRoot = str;
   else
-    usrRoot = [NSString stringWithFormat: @"%@/GNUstep", NSHomeDirectory()];
+    usrRoot = [sClass stringWithFormat: @"%@/GNUstep", NSHomeDirectory()];
 
   mgr = [NSFileManager defaultManager];
   if (([mgr fileExistsAtPath: usrRoot isDirectory: &isDir] && isDir) == 0)
@@ -162,9 +197,10 @@ main(int argc, char** argv)
 	  exit(1);
 	}
     }
-  [roots addObject: usrRoot];
 
-  usrRoot = [usrRoot stringByAppendingPathComponent: @"Services"];
+  str = usrRoot;	/* Record for adding into roots array */
+
+  usrRoot = [str stringByAppendingPathComponent: @"Services"];
   if (([mgr fileExistsAtPath: usrRoot isDirectory: &isDir] && isDir) == 0)
     {
       if ([mgr createDirectoryAtPath: usrRoot attributes: nil] == NO)
@@ -175,17 +211,32 @@ main(int argc, char** argv)
 	}
     }
 
-  str = [env objectForKey: @"GNUSTEP_LOCAL_ROOT"];
-  if (str != nil)
-    [roots addObject: str];
-  else
-    [roots addObject: @"/usr/GNUstep/Local"];
+  if (usedPrefixes == NO)
+    {
+      /*
+       * Ensure that the user root (or default user root) is in the path.
+       */
+      if ([roots containsObject: str] == NO)
+	[roots addObject: str];
 
-  str = [env objectForKey: @"GNUSTEP_SYSTEM_ROOT"];
-  if (str != nil)
-    [roots addObject: str];
-  else
-    [roots addObject: @"/usr/GNUstep"];
+      /*
+       * Ensure that the local root (or default local root) is in the path.
+       */
+      str = [env objectForKey: @"GNUSTEP_LOCAL_ROOT"];
+      if (str == nil)
+	str = @"/usr/GNUstep/Local";
+      if ([roots containsObject: str] == NO)
+	[roots addObject: str];
+
+      /*
+       * Ensure that the system root (or default system root) is in the path.
+       */
+      str = [env objectForKey: @"GNUSTEP_SYSTEM_ROOT"];
+      if (str == nil)
+	str = @"/usr/GNUstep";
+      if ([roots containsObject: str] == NO)
+	[roots addObject: str];
+    }
 
   /*
    *	Before doing the main scan, we examine the 'Services' directory to
@@ -254,6 +305,7 @@ main(int argc, char** argv)
     {
       oldMap = nil;
     }
+  [applicationMap setObject: extensionsMap forKey: @"GSExtensionsMap"];
   if ([applicationMap isEqual: oldMap] == NO)
     {
       data = [NSSerializer serializePropertyList: applicationMap];
@@ -269,6 +321,122 @@ main(int argc, char** argv)
   exit(0);
 }
 
+/*
+ * Load information about the types of files that an application supports.
+ * For each extension found, produce a dictionary, keyed by app name, that
+ * contains dictionaries giving type info for that extension.
+ * NB. in order to make extensions case-insensiteve - we always convert
+ * to lowercase.
+ */
+static void addExtensionsForApplication(NSDictionary *info, NSString *app)
+{
+  unsigned int  i;
+  id            o0;
+  NSArray       *a0;
+
+
+  o0 = [info objectForKey: @"NSTypes"];
+  if (o0)
+    {
+      if ([o0 isKindOfClass: aClass] == NO)
+        {
+          NSLog(@"bad app NSTypes (not an array) - %@\n", app);
+          return;
+        }
+      a0 = (NSArray*)o0;
+      i = [a0 count];
+      while (i-- > 0)
+        {
+          NSDictionary          *t;
+          NSArray               *a1;
+          id                    o1 = [a0 objectAtIndex: i];
+          unsigned int          j;
+
+          if ([o1 isKindOfClass: dClass] == NO)
+            {
+              NSLog(@"bad app NSTypes (type not a dictionary) - %@\n", app);
+              return;
+            }
+	  /*
+	   * Set 't' to the dictionary defining a particular file type.
+	   */
+          t = (NSDictionary*)o1;
+          o1 = [t objectForKey: @"NSUnixExtensions"];
+          if (o1 == nil)
+            {
+              continue;
+            }
+          if ([o1 isKindOfClass: aClass] == NO)
+            {
+              NSLog(@"bad app NSType (extensions not an array) - %@\n", app);
+              return;
+            }
+          a1 = (NSArray*)o1;
+          j = [a1 count];
+          while (j-- > 0)
+            {
+              NSString			*e;
+              NSMutableDictionary	*d;
+
+              e = [[a1 objectAtIndex: j] lowercaseString];
+              d = [extensionsMap objectForKey: e];
+              if (d == nil)
+                {
+                  d = [NSMutableDictionary dictionaryWithCapacity: 1];
+                  [extensionsMap setObject: d forKey: e];
+                }
+              if ([d objectForKey: app] == NO)
+                {
+                  [d setObject: t forKey: app];
+                } 
+            }
+        }
+    }
+  else
+    {
+      /*
+       *    If we have an old format list of extensions
+       *    handled by this application - ensure that
+       *    the name of the application is listed in
+       *    the dictionary of applications handling each of
+       *    the extensions.
+       */
+      o0 = [info objectForKey: @"NSExtensions"];
+      if (o0 == nil)
+        {
+          return;
+        }
+      if ([o0 isKindOfClass: aClass] == NO)
+        {
+          NSLog(@"bad app NSExtensions (not an array) - %@\n", app);
+          return;
+        }
+      a0 = (NSArray*)o0;
+      i = [a0 count];
+      while (i-- > 0)
+        {
+          NSString		*e;
+          NSMutableDictionary	*d;
+
+          e = [[a0 objectAtIndex: i] lowercaseString];
+          d = [extensionsMap objectForKey: e];
+          if (d == nil)
+            {
+	      d = [NSMutableDictionary dictionaryWithCapacity: 1];
+	      [extensionsMap setObject: d forKey: e];
+            }
+          if ([d objectForKey: app] == nil)
+            {
+	      NSDictionary	*info;
+
+	      info = [NSDictionary dictionaryWithObjectsAndKeys:
+			nil];
+              [d setObject: info forKey: app];
+            } 
+        }
+    }
+}
+
 static void
 scanDirectory(NSMutableDictionary *services, NSString *path)
 {
@@ -284,51 +452,63 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
       NSString	*newPath;
       BOOL	isDir;
 
-
       if (ext != nil &&
-	  ([ext isEqualToString: @"app"] || [ext isEqualToString: @"debug"]))
+	  ([ext isEqualToString: @"app"] || [ext isEqualToString: @"debug"]
+	  || [ext isEqualToString: @"profile"]))
 	{
 	  newPath = [path stringByAppendingPathComponent: name];
 	  if ([mgr fileExistsAtPath: newPath isDirectory: &isDir] && isDir)
 	    {
-	      NSString	*infPath;
+	      NSString		*oldPath;
+	      NSBundle		*bundle;
+	      NSDictionary	*info;
 
 	      /*
-	       *	All (non-debug) application paths are noted by name
+	       *	All application paths are noted by name
 	       *	in the 'applicationMap' dictionary.
 	       */
-	      if ([ext isEqualToString: @"app"])
-		{
-		  if ([applicationMap objectForKey: name] == nil)
-		    {
-		      [applicationMap setObject: newPath forKey: name];
-		    }
-		}
-	      infPath = [newPath stringByAppendingPathComponent: infoLoc];
-	      if ([mgr fileExistsAtPath: infPath isDirectory: &isDir] && !isDir)
-		{
-		  NSDictionary	*info;
+              if ((oldPath = [applicationMap objectForKey: name]) == nil)
+                {
+                  [applicationMap setObject: newPath forKey: name];
+                }
+              else
+                {
+                  /*
+                   * If we already have an entry for an application with
+                   * this name, we skip this one - the first one takes
+                   * precedence.
+                   */
+                  NSLog(@"duplicate app (%@) at '%@' and '%@'\n",
+                        name, oldPath, newPath);
+                  continue;
+                }
 
-		  info = [NSDictionary dictionaryWithContentsOfFile: infPath];
-		  if (info)
-		    {
-		      id	svcs = [info objectForKey: @"NSServices"];
+	      bundle = [NSBundle bundleWithPath: newPath];
+	      info = [bundle infoDictionary];
+	      if (info)
+		{
+		  id	obj;
 
-		      if (svcs)
+		  /*
+		   * Load and validate any services definitions.
+		   */
+		  obj = [info objectForKey: @"NSServices"];
+		  if (obj)
+		    {
+		      NSMutableArray	*entry;
+
+		      entry = validateEntry(obj, newPath);
+		      if (entry)
 			{
-			  NSMutableArray	*entry;
-
-			  entry = validateEntry(svcs, newPath);
-			  if (entry)
-			    {
-			      [services setObject: entry forKey: newPath];
-			    }
+			  [services setObject: entry forKey: newPath];
 			}
 		    }
-		  else
-		    {
-		      NSLog(@"bad app info - %@\n", infPath);
-		    }
+
+		  addExtensionsForApplication(info, name);
+		}
+	      else
+		{
+		  NSLog(@"bad app info - %@\n", newPath);
 		}
 	    }
 	  else
@@ -341,38 +521,34 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
 	  newPath = [path stringByAppendingPathComponent: name];
 	  if ([mgr fileExistsAtPath: newPath isDirectory: &isDir] && isDir)
 	    {
-	      NSString	*infPath;
+	      NSBundle		*bundle;
+	      NSDictionary	*info;
 
-	      infPath = [newPath stringByAppendingPathComponent: infoLoc];
-	      if ([mgr fileExistsAtPath: infPath isDirectory: &isDir] && !isDir)
+	      bundle = [NSBundle bundleWithPath: newPath];
+	      info = [bundle infoDictionary];
+	      if (info)
 		{
-		  NSDictionary	*info;
+		  id	svcs = [info objectForKey: @"NSServices"];
 
-		  info = [NSDictionary dictionaryWithContentsOfFile: infPath];
-		  if (info)
+		  if (svcs)
 		    {
-		      id	svcs = [info objectForKey: @"NSServices"];
+		      NSMutableArray	*entry;
 
-		      if (svcs)
+		      entry = validateEntry(svcs, newPath);
+		      if (entry)
 			{
-			  NSMutableArray	*entry;
-
-			  entry = validateEntry(svcs, newPath);
-			  if (entry)
-			    {
-			      [services setObject: entry forKey: newPath];
-			    }
-			}
-		      else
-			{
-			  NSLog(@"missing info - %@\n", infPath);
+			  [services setObject: entry forKey: newPath];
 			}
 		    }
 		  else
 		    {
-		      NSLog(@"bad service info - %@\n", infPath);
+		      NSLog(@"missing info - %@\n", newPath);
 		    }
 		}
+	      else
+		{
+		  NSLog(@"bad service info - %@\n", newPath);
+		}
 	    }
 	  else
 	    {
@@ -446,7 +622,7 @@ validateEntry(id svcs, NSString *path)
   NSArray		*services;
   unsigned		pos;
 
-  if ([svcs isKindOfClass: [NSArray class]] == NO)
+  if ([svcs isKindOfClass: aClass] == NO)
     {
       NSLog(@"NSServices entry not an array - %@\n", path);
       return nil;
@@ -459,7 +635,7 @@ validateEntry(id svcs, NSString *path)
       id			svc;
 
       svc = [services objectAtIndex: pos];
-      if ([svc isKindOfClass: [NSDictionary class]])
+      if ([svc isKindOfClass: dClass])
 	{
 	  NSDictionary		*service = (NSDictionary*)svc;
 	  NSMutableDictionary	*newService;
@@ -483,9 +659,6 @@ static NSMutableDictionary*
 validateService(NSDictionary *service, NSString *path, unsigned pos)
 {
   static NSDictionary	*fields = nil;
-  static Class		aClass;
-  static Class		dClass;
-  static Class		sClass;
   NSEnumerator		*e;
   NSMutableDictionary	*result;
   NSString		*k;
@@ -493,9 +666,6 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
   if (fields == nil)
     {
-      aClass = [NSArray class];
-      dClass = [NSDictionary class];
-      sClass = [NSString class];
       fields = [NSDictionary dictionaryWithObjectsAndKeys:
 	@"string", @"NSMessage",
 	@"string", @"NSPortName",
