@@ -5,10 +5,11 @@
 #   Return the name of the environment variable for the operating
 #   system that is used by the dynamic loader.
 #
-#   Copyright (C) 1997 Free Software Foundation, Inc.
+#   Copyright (C) 1997,1999 Free Software Foundation, Inc.
 #
 #   Author:  Scott Christley <scottc@net-community.com>
 #   Author:  Ovidiu Predescu <ovidiu@net-community.com>
+#   Rewrite: Richard Frith-Macdoanld <richard@brainstorm.co.uk>
 #   This file is part of the GNUstep Makefile Package.
 #
 #   This library is free software; you can redistribute it and/or
@@ -38,35 +39,36 @@ old_IFS="$IFS"
 IFS=" 
 "
 
+lib_paths="$GNUSTEP_USER_ROOT/$last_path_part:$GNUSTEP_USER_ROOT/$tool_path_part:$GNUSTEP_LOCAL_ROOT/$last_path_part:$GNUSTEP_LOCAL_ROOT/$tool_path_part:$GNUSTEP_SYSTEM_ROOT/$last_path_part:$GNUSTEP_SYSTEM_ROOT/$tool_path_part"
+
+if [ -n "$additional_library_paths" ]; then
+  for dir in $additional_library_paths; do
+    additional="${additional}${dir}:"
+  done
+fi
+lib_paths="${additional}${lib_paths}"
+
 case "$host_os" in
 
   *nextstep4*)
     if [ -z "$DYLD_LIBRARY_PATH" ]; then
-	DYLD_LIBRARY_PATH="$GNUSTEP_USER_ROOT/$last_path_part:$GNUSTEP_USER_ROOT/$tool_path_part:$GNUSTEP_LOCAL_ROOT/$last_path_part:$GNUSTEP_LOCAL_ROOT/$tool_path_part:$GNUSTEP_SYSTEM_ROOT/$last_path_part:$GNUSTEP_SYSTEM_ROOT/$tool_path_part"
+      DYLD_LIBRARY_PATH="$lib_paths"
     else
-	DYLD_LIBRARY_PATH="$GNUSTEP_USER_ROOT/$last_path_part:$GNUSTEP_USER_ROOT/$tool_path_part:$GNUSTEP_LOCAL_ROOT/$last_path_part:$GNUSTEP_LOCAL_ROOT/$tool_path_part:$GNUSTEP_SYSTEM_ROOT/$last_path_part:$GNUSTEP_SYSTEM_ROOT/$tool_path_part:$DYLD_LIBRARY_PATH"
+      if ( echo ${DYLD_LIBRARY_PATH}|fgrep -v "${lib_paths}" >/dev/null ); then
+	DYLD_LIBRARY_PATH="$lib_paths:$DYLD_LIBRARY_PATH"
+      fi
     fi
-    if [ -n "$additional_library_paths" ]; then
-      for dir in $additional_library_paths; do
-	additional="${additional}${dir}:"
-      done
-    fi
-    DYLD_LIBRARY_PATH="${additional}${DYLD_LIBRARY_PATH}"
     export DYLD_LIBRARY_PATH
     ;;
 
   *)
     if [ -z "$LD_LIBRARY_PATH" ]; then
-	LD_LIBRARY_PATH="$GNUSTEP_USER_ROOT/$last_path_part:$GNUSTEP_USER_ROOT/$tool_path_part:$GNUSTEP_LOCAL_ROOT/$last_path_part:$GNUSTEP_LOCAL_ROOT/$tool_path_part:$GNUSTEP_SYSTEM_ROOT/$last_path_part:$GNUSTEP_SYSTEM_ROOT/$tool_path_part"
+      LD_LIBRARY_PATH="$lib_paths"
     else
-	LD_LIBRARY_PATH="$GNUSTEP_USER_ROOT/$last_path_part:$GNUSTEP_USER_ROOT/$tool_path_part:$GNUSTEP_LOCAL_ROOT/$last_path_part:$GNUSTEP_LOCAL_ROOT/$tool_path_part:$GNUSTEP_SYSTEM_ROOT/$last_path_part:$GNUSTEP_SYSTEM_ROOT/$tool_path_part:$LD_LIBRARY_PATH"
+      if ( echo ${LD_LIBRARY_PATH}|fgrep -v "${lib_paths}" >/dev/null ); then
+	LD_LIBRARY_PATH="$lib_paths:$LD_LIBRARY_PATH"
+      fi
     fi
-    if [ -n "$additional_library_paths" ]; then
-      for dir in $additional_library_paths; do
-	additional="${additional}${dir}:"
-      done
-    fi
-    LD_LIBRARY_PATH="${additional}${LD_LIBRARY_PATH}"
     export LD_LIBRARY_PATH;;
 
 esac
