@@ -31,7 +31,7 @@
 #include <AppKit/NSGraphics.h>
 #include <AppKit/NSCStringText.h>
 #include <AppKit/NSEvent.h>
-#include <gnustep/xdps/NSDPSContext.h>
+#include <gnustep/xdps/NSDPSContextWindow.h>
 #include "drawingfuncs.h"
 
 #define	CUR_CONTEXT	((NSDPSContext*)[NSDPSContext currentContext])
@@ -510,10 +510,12 @@ void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
   Drawable source, draw;
   NSWindow *window;
   NSDPSContext *ctxt;
+  gswindow_device_t *windev;
 
   ctxt = (NSDPSContext *)GSCurrentContext();
   window = [[GSCurrentContext() focusView] window];
-  draw = [(PXKWindow *)window xDrawable];
+  windev = [NSDPSContext _windowWithTag: [window windowNumber]];
+  draw = (windev->buffer) ? windev->buffer : windev->ident;
   if (draw == 0)
     return;
 
@@ -521,37 +523,44 @@ void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
   if (srcGstate != 0)
     {
       int gc, x, y;
+      NSRect tRect = [ctxt XRectFromUserRect: srcRect];
       PSgsave();
       PSsetgstate(srcGstate);
       PScurrentXgcdrawable(&gc, (int *)(&source), &x, &y);
       if (source == 0)
         return;
-      srcRect.origin = [ctxt XPointFromUserPoint: srcRect.origin];
-      srcRect.size = [ctxt XSizeFromUserSize: srcRect.size];
+      NSDebugLLog (@"Copy", @"Orig  %@\n", NSStringFromRect(srcRect));
+      srcRect = [ctxt XRectFromUserRect: srcRect];
+      NSDebugLLog (@"Copy", @"XCoor %@\n", NSStringFromRect(srcRect));
       PSgrestore();
     }
   else
     {
       source = draw;
-      srcRect.origin = [ctxt XPointFromUserPoint: srcRect.origin];
-      srcRect.size = [ctxt XSizeFromUserSize: srcRect.size];
+      srcRect = [ctxt XRectFromUserRect: srcRect];
     }
+  NSDebugLLog (@"Copy", @"ODest %@\n", NSStringFromPoint(destPoint));
   destPoint = [ctxt XPointFromUserPoint: destPoint];
-  srcRect.origin.y -= NSHeight(srcRect);
-  destPoint.y -= NSHeight(srcRect);
+  /* FIXME: Why is this needed? */
+  if (![[GSCurrentContext() focusView] isFlipped])
+    destPoint.y -= NSHeight(srcRect);
+  NSDebugLLog (@"Copy", @"XDest %@\n", NSStringFromPoint(destPoint));
 
   src.x = NSMinX(srcRect); src.y = NSMinY(srcRect);
   src.width = NSWidth(srcRect); src.height = NSHeight(srcRect);
   dst.x = destPoint.x; dst.y = destPoint.y;
   NSDebugLLog (@"NSWindow", @"Copying bitmap from (%d %d %d %d) to (%d %d)\n",
   	src.x, src.y, src.width, src.height, dst.x, dst.y);
-  XCopyArea([ctxt xDisplay], source, draw, [(PXKWindow *)window xGC],
+  XCopyArea([ctxt xDisplay], source, draw, windev->gc,
                 src.x, src.y, src.width, src.height, dst.x, dst.y);
 }
 
 //
 // Render Bitmap Images
 //
+static	char *hex = "0123456789abcdef";
+#define	PUTHEX(cont, c)	DPSPrintf(cont, "%c%c", hex[((c)>>4)&0xf], hex[(c)&0xf]);
+
 void NSDrawBitmap(NSRect rect,
                   int pixelsWide,
                   int pixelsHigh,
@@ -563,7 +572,84 @@ void NSDrawBitmap(NSRect rect,
                   BOOL hasAlpha, 
                   NSString *colorSpaceName, 
                   const unsigned char *const data[5])
-{}
+{
+  int pixels;
+  DPSContext ctxt;
+  ctxt = DPSGetCurrentContext();
+
+  /* Save scaling */
+  PSmatrix(); PScurrentmatrix();
+  PSmoveto(NSMinX(rect), NSMinY(rect));
+  PSscale(NSWidth(rect), NSHeight(rect));
+
+  /* Prepare the image operands */
+  if (bitsPerSample == 0)
+    bitsPerSample = 8;
+  DPSPrintf(ctxt, "%d %d %d\n", pixelsWide, pixelsHigh, bitsPerSample);
+  DPSPrintf(ctxt, "[%lu 0 0 -%lu 0 %lu]\n", pixelsWide, pixelsHigh, 
+	    pixelsHigh);
+
+  /* Send the data source */
+  pixels = bytesPerRow * pixelsHigh;
+  if (isPlanar)
+    {
+      int i;
+      for (i = 0; i < samplesPerPixel; i++)
+	PSsendchararray(data[i], pixels);
+    }
+  else
+    {
+      int i, j, pos, row, col;
+      float adjust;
+      unsigned char dot;
+      DPSPrintf(ctxt, "<");
+      j = 0;
+      pos = 0;
+      for (row = 0; row < pixelsHigh; row++)
+	for (col = 0; col < pixelsWide; col++)
+	  for (i = 0; i < samplesPerPixel; i++)
+	    {
+	      if (hasAlpha)
+		{
+		  adjust = data[0][pos + (3 - (pos % 4))];
+		  adjust = adjust/255;
+		}
+	      else
+		adjust = 1;
+	      if ((hasAlpha) && ((pos % 4) != 3))
+		{
+		  dot = 255 + (data[0][pos] - 255)*adjust;
+		  PUTHEX(ctxt, dot);
+		  ++j;
+		}
+	      ++pos;
+	      if (j == 33)
+		{
+		  j = 0;
+		}
+	    }
+      DPSPrintf(ctxt, "> \n");
+    }
+
+  /* Now image it */
+  if (samplesPerPixel > 1)
+    {
+      if (hasAlpha)
+	samplesPerPixel -= 1;
+      PSsendboolean(isPlanar);
+      PSsendint(samplesPerPixel);
+      /* alphaimage doesn't work on DGS yet. Maybe not at all on other impl */
+      if (hasAlpha)
+	DPSPrintf(ctxt, "colorimage\n");
+      else
+	PScolorimage();
+    }
+  else
+    PSimage();
+  
+  /* Restore original scaling */
+  PSsetmatrix();
+}
 
 //
 // Other Application Kit Functions
