@@ -25,8 +25,21 @@
 */ 
 
 #include "config.h"
-#include <gnustep/xdps/PXKEPSImageRep.h>
+#include <gnustep/xdps/NSDPSContext.h>
+
+@interface PXKEPSImageRep : NSEPSImageRep
+{
+}
+
+@end
 
 @implementation PXKEPSImageRep
 
+- (Class) classForCoder: aCoder
+{
+  if ([self class] == [PXKEPSImageRep class])
+    return [super class];
+  return [self class];
+}
+
 @end
