@@ -32,6 +32,7 @@
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/PSOperators.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSUserDefaults.h>
 
 #define	CUR_CONTEXT	((XGContext*)[XGContext currentContext])
 
@@ -41,20 +42,17 @@ void XGInitializeDrawingEngine(void);
 BOOL 
 initialize_gnustep_backend(void)
 {                           
+  id obj;
   NSDebugLog(@"Using the GNUstep GUI X/GPS backend.\n");
 
   [NSGraphicsContext setDefaultContextClass: [XGContext class]];
-  [XGScreen poseAsClass:[NSScreen class]];
   [XGFontManager poseAsClass:[NSFontManager class]];
   [XGFont poseAsClass:[NSFont class]];  
-  [XGApplication poseAsClass:[NSApplication class]];
-  [XGMenuWindow poseAsClass:[NSMenuWindow class]];
-  [XGColor class];	// poseAs: done in initialize
-  [XGImage poseAsClass: [NSImage class]];
-  [XGBitmapImageRep poseAsClass: [NSBitmapImageRep class]];
-  [XGCachedImageRep poseAsClass: [NSCachedImageRep class]];
-  [XGCursor poseAsClass: [NSCursor class]];
-  [XGWindow poseAsClass:[NSWindow class]];
+  obj = [[NSUserDefaults standardUserDefaults] stringForKey: @"ImageCompositing"];
+  if (obj && [obj boolValue] == NO)
+    [XGBitmapImageRep poseAsClass: [NSBitmapImageRep class]];
+  else
+    NSDebugLog(@"Using ImageCompositing functions");
 
   NSDebugLog(@"Initializing XGPS drawing engine\n");
   XGInitializeDrawingEngine();
@@ -112,7 +110,7 @@ void NSRectFillListWithGrays(const NSRect *rects,const float *grays,int count)
 {
 }
 
-void NSDrawButton(NSRect aRect, NSRect clipRect)
+void NSDrawButton(const NSRect aRect, const NSRect clipRect)
 {
   float x, y, w, h, v;
   XGContext	*ctxt;
@@ -143,12 +141,9 @@ void NSDrawButton(NSRect aRect, NSRect clipRect)
   else
     y++;
 
-  // Top and left inner line in light grey
+  // Fill the inner rectangle in light gray
   DPSsetgray(ctxt, 0.667);
-  DPSmoveto(ctxt, x+1, y+v);
-  DPSrlineto(ctxt, 0, h-2*v);
-  DPSrlineto(ctxt, w-2, 0);
-  DPSstroke(ctxt);
+  DPSrectfill(ctxt, x, y, w, h);
 
   // Top and left outer line in white
   DPSsetgray(ctxt, 1.0);
@@ -172,7 +167,7 @@ void NSDrawButton(NSRect aRect, NSRect clipRect)
   DPSstroke(ctxt);
 }
 
-void NSDrawGrayBezel(NSRect aRect, NSRect clipRect)
+void NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
 {
   float x, y, w, h, v;
   XGContext	*ctxt;
@@ -203,6 +198,10 @@ void NSDrawGrayBezel(NSRect aRect, NSRect clipRect)
 
   DPSsetlinewidth(ctxt, 1.0);
 
+  // Fill the inner rectangle in light gray
+  DPSsetgray(ctxt, 0.667);
+  DPSrectfill(ctxt, x, y, w, h);
+
   // Top and left outer line in dark grey
   DPSsetgray(ctxt, 0.333);
   DPSmoveto(ctxt, x, y);
@@ -219,20 +218,20 @@ void NSDrawGrayBezel(NSRect aRect, NSRect clipRect)
 
   // Bottom and right outer line in white
   DPSsetgray(ctxt, 1.0);
-  DPSmoveto(ctxt, x+1, y);
-  DPSrlineto(ctxt, w-1, 0);
-  DPSrlineto(ctxt, 0, h);
+  DPSmoveto(ctxt, x, y);
+  DPSrlineto(ctxt, w, 0);
+  DPSrlineto(ctxt, 0, h-v);
   DPSstroke(ctxt);
 
   // Bottom and right inner line in light grey
   DPSsetgray(ctxt, 0.667);
-  DPSmoveto(ctxt, x+2, y+v);
-  DPSrlineto(ctxt, w-3, 0);
-  DPSrlineto(ctxt, 0, 2*v-h);
+  DPSmoveto(ctxt, x+1, y+v);
+  DPSrlineto(ctxt, w-2, 0);
+  DPSrlineto(ctxt, 0, h-2*v);
   DPSstroke(ctxt);
 }
 
-void NSDrawGroove(NSRect aRect, NSRect clipRect)
+void NSDrawGroove(const NSRect aRect, const NSRect clipRect)
 {
   float x, y, w, h, v;
   XGContext	*ctxt;
@@ -263,28 +262,29 @@ void NSDrawGroove(NSRect aRect, NSRect clipRect)
 
   DPSsetlinewidth(ctxt, 1.0);
 
+  // Fill the inner rectangle in light gray
+  DPSsetgray(ctxt, 0.667);
+  DPSrectfill(ctxt, x, y, w, h);
+
+  // Top and left outer line in dark gray
   DPSsetgray(ctxt, 0.333);
   DPSmoveto(ctxt, x, y);
   DPSrlineto(ctxt, 0, h);
   DPSrlineto(ctxt, w, 0);
   DPSstroke(ctxt);
 
-  DPSsetgray(ctxt, 1.0);
-  DPSmoveto(ctxt, x+1, y);
-  DPSrlineto(ctxt, 0, h-v);
-  DPSrlineto(ctxt, w-1, 0);
-  DPSrlineto(ctxt, 0, -h+v);
-  DPSrlineto(ctxt, 1-w, 0);
-  DPSstroke(ctxt);
-    
-  DPSsetgray(ctxt, 0.333);
+  // Bottom and right inner line in dark gray (too)
   DPSmoveto(ctxt, x+2, y+v);
   DPSrlineto(ctxt, w-3, 0);
   DPSrlineto(ctxt, 0, h-3*v);
   DPSstroke(ctxt);
+
+  // Draw remaining white lines using a single white rect
+  DPSsetgray(ctxt, 1.0);
+  DPSrectstroke(ctxt, x+1, y-v, w, h);
 }
 
-void NSFrameRect(NSRect aRect)  
+void NSFrameRect(const NSRect aRect)  
 {
   float x, y, w, h;
   XGContext	*ctxt;
@@ -302,7 +302,7 @@ void NSFrameRect(NSRect aRect)
   DPSrectstroke(ctxt, x, y, w, h);
 }
 
-void NSFrameRectWithWidth(NSRect aRect, float frameWidth)
+void NSFrameRectWithWidth(const NSRect aRect, float frameWidth)
 {
   float x, y, w, h;
   XGContext	*ctxt;
@@ -396,13 +396,75 @@ void XRRemoveClipPath(void)
 //
 void NSEraseRect(NSRect aRect)
 {
-  [[NSColor whiteColor] set];
+  XGContext	*ctxt;
+  ctxt = CUR_CONTEXT;
+  DPSgsave(ctxt);
+  DPSsetgray(ctxt, 1.0);
   NSRectFill(aRect);
+  DPSgrestore(ctxt);
 }
 
-void NSDrawWhiteBezel(NSRect aRect, NSRect clipRect)  	// like a TextField ?
+void NSDrawWhiteBezel(const NSRect aRect, const NSRect clipRect)
 {
-  NSDrawGrayBezel(aRect, clipRect);
+  float x, y, w, h, v;
+  XGContext	*ctxt;
+  XGGState	*current;
+
+  if (!NSIsEmptyRect(clipRect) && NSIntersectsRect(aRect, clipRect) == NO)
+    return;
+
+  ctxt = CUR_CONTEXT;
+  current = [ctxt xrCurrentGState];
+
+/* The lines drawn should be offset by half a linewidth inwards so
+   that the entire line gets drawn inside the rectangle. */
+  x = NSMinX(aRect);
+  y = NSMinY(aRect);
+  w = NSWidth(aRect) - 1.0;
+  h = NSHeight(aRect) - 1.0;
+  v = 1.0;
+  
+  if (current->viewIsFlipped == YES)
+    {
+      y += h;
+      h = -h;
+      v = -v;
+    }
+  else
+    y++;
+
+  DPSsetlinewidth(ctxt, 1.0);
+
+  // Fill the inner rectangle in white
+  DPSsetgray(ctxt, 1.0);
+  DPSrectfill(ctxt, x, y, w, h);
+
+  // Top and left outer line in dark grey
+  DPSsetgray(ctxt, 0.333);
+  DPSmoveto(ctxt, x, y);
+  DPSrlineto(ctxt, 0, h);
+  DPSrlineto(ctxt, w, 0);
+  DPSstroke(ctxt);
+
+  // Top and left inner line in dark gray (too)
+  DPSmoveto(ctxt, x+1, y+v);
+  DPSrlineto(ctxt, 0, h-2*v);
+  DPSrlineto(ctxt, w-2, 0);
+  DPSstroke(ctxt);
+
+  // Bottom and right outer line in white
+  DPSsetgray(ctxt, 1.0);
+  DPSmoveto(ctxt, x, y);
+  DPSrlineto(ctxt, w, 0);
+  DPSrlineto(ctxt, 0, h-v);
+  DPSstroke(ctxt);
+
+  // Bottom and right inner line in light grey
+  DPSsetgray(ctxt, 0.667);
+  DPSmoveto(ctxt, x+1, y+v);
+  DPSrlineto(ctxt, w-2, 0);
+  DPSrlineto(ctxt, 0, h-3*v);
+  DPSstroke(ctxt);
 }
 
 void NSDrawBezel(NSRect aRect, NSRect clipRect)
@@ -613,23 +675,6 @@ void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
 	       NSCompositeCopy);
 }
 
-//
-// Render Bitmap Images
-//
-void NSDrawBitmap(NSRect rect,
-                  int pixelsWide,
-                  int pixelsHigh,
-                  int bitsPerSample,
-                  int samplesPerPixel,
-                  int bitsPerPixel,
-                  int bytesPerRow,
-                  BOOL isPlanar,
-                  BOOL hasAlpha,
-                  NSString *colorSpaceName,
-                  const unsigned char *const data[5])
-{
-}
-
 //
 // Play the System Beep
 //
@@ -675,7 +720,16 @@ GSWViewIsFlipped(GSCTXT *ctxt, BOOL flipped)
 {
   ([(XGContext *)ctxt xrCurrentGState])->viewIsFlipped = flipped;
 }
-  
+
+void
+GSWinitcontext(GSCTXT *ctxt, int window_number, GC xgc, Drawable drawable, 
+	int xoff, int yoff)
+{
+  DPSsetgcdrawable(ctxt, xgc, (void *)drawable, xoff, yoff);
+  DPSinitmatrix(ctxt);
+  DPSinitclip(ctxt);
+  [[(XGContext *)ctxt xrCurrentGState] setWindow: window_number];
+}
 
 //*****************************************************************************
 //
