@@ -91,9 +91,9 @@ NSMutableDictionary	*pasteboards = nil;
 
   if (d)
     {
-      d->type = [aType retain];
-      d->owner = [anObject retain];
-      d->pboard = [anotherObject retain];
+      d->type = RETAIN(aType);
+      d->owner = RETAIN(anObject);
+      d->pboard = RETAIN(anotherObject);
       d->wantsChangedOwner = wants;
       d->hasStdDataForType = StdData;
       d->hasGNUDataForType = GNUData;
@@ -110,20 +110,20 @@ NSMutableDictionary	*pasteboards = nil;
     {
       o = owner;
       owner = nil;
-      [o release];
+      RELEASE(o);
       o = pboard;
       pboard = nil;
-      [o release];
+      RELEASE(o);
       ourConnection = YES;
     }
   if (pboard && [pboard isProxy] && [pboard connectionForProxy] == c)
     {
       o = owner;
       owner = nil;
-      [o release];
+      RELEASE(o);
       o = pboard;
       pboard = nil;
-      [o release];
+      RELEASE(o);
       ourConnection = YES;
     }
   return ourConnection;
@@ -131,10 +131,10 @@ NSMutableDictionary	*pasteboards = nil;
 
 - (void) dealloc
 {
-  [type release];
-  [data release];
-  [owner release];
-  [pboard release];
+  RELEASE(type);
+  RELEASE(data);
+  RELEASE(owner);
+  RELEASE(pboard);
   [super dealloc];
 }
 
@@ -162,8 +162,7 @@ NSMutableDictionary	*pasteboards = nil;
   if (data != nil && owner != nil
     && [owner isProxy] == NO && [owner isKindOfClass: xPbClass] == YES)
     {
-      [data release];
-      data = nil;
+      DESTROY(data);
     }
 
   if (data == nil && (owner && pboard))
@@ -199,9 +198,7 @@ NSMutableDictionary	*pasteboards = nil;
     {
       NSLog(@"set data for %x\n", (unsigned)self);
     }
-  [d retain];
-  [data release];
-  data = d;
+  ASSIGN(data, d);
 }
 
 - (id) type
@@ -256,8 +253,8 @@ NSMutableDictionary	*pasteboards = nil;
     {
       int     i;
 
-      e->owner = [anOwner retain];
-      e->pboard = [aPboard retain];
+      e->owner = RETAIN(anOwner);
+      e->pboard = RETAIN(aPboard);
 
       if (anOwner && [anOwner respondsToSelector:
 		@selector(pasteboardChangedOwner:)])
@@ -288,7 +285,7 @@ NSMutableDictionary	*pasteboards = nil;
 			hasStdDataForType: e->hasStdDataForType
 			hasGNUDataForType: e->hasGNUDataForType];
 	  [e->items addObject: d];
-	  [d release];
+	  RELEASE(d);
 	}
       e->refNum = count;
       if (verbose > 1)
@@ -337,7 +334,7 @@ NSMutableDictionary	*pasteboards = nil;
 			hasStdDataForType: StdData
 			hasGNUDataForType: GNUData];
 	  [items addObject: d];
-	  [d release];
+	  RELEASE(d);
 	}
     }
   if (verbose > 1)
@@ -356,10 +353,10 @@ NSMutableDictionary	*pasteboards = nil;
     {
       o = owner;
       owner = nil;
-      [o release];
+      RELEASE(o);
       o = pboard;
       pboard = nil;
-      [o release];
+      RELEASE(o);
       ourConnection = YES;
     }
 
@@ -367,10 +364,10 @@ NSMutableDictionary	*pasteboards = nil;
     {
       o = owner;
       owner = nil;
-      [o release];
+      RELEASE(o);
       o = pboard;
       pboard = nil;
-      [o release];
+      RELEASE(o);
       ourConnection = YES;
     }
 
@@ -392,9 +389,9 @@ NSMutableDictionary	*pasteboards = nil;
 
 - (void) dealloc
 {
-  [owner release];
-  [pboard release];
-  [items release];
+  RELEASE(owner);
+  RELEASE(pboard);
+  RELEASE(items);
   [super dealloc];
 }
 
@@ -436,7 +433,8 @@ NSMutableDictionary	*pasteboards = nil;
 	{
 	  PasteboardData*   d = [items objectAtIndex: i];
 
-	  if ([d wantsChangedOwner] && [a containsObject: [d owner]] == NO)
+	  if ([d wantsChangedOwner]
+	    && [a indexOfObjectIdenticalTo: [d owner]] == NSNotFound)
 	    {
 	      [a addObject: [d owner]];
 	    }
@@ -444,8 +442,8 @@ NSMutableDictionary	*pasteboards = nil;
 
       if (wantsChangedOwner)
 	{
-	  [a removeObject: owner];
 	  [owner pasteboardChangedOwner: pboard];
+	  [a removeObjectIdenticalTo: owner];
 	}
 
       for (i = 0; i < [items count] && [a count] > 0; i++)
@@ -456,7 +454,7 @@ NSMutableDictionary	*pasteboards = nil;
 	  if ([a containsObject: o])
 	    {
 	      [o pasteboardChangedOwner: [d pboard]];
-	      [a removeObject: o];
+	      [a removeObjectIdenticalTo: o];
 	    }
 	}
     }
@@ -563,13 +561,13 @@ NSMutableDictionary	*pasteboards = nil;
   if (pb == nil)
     {
       pb = [PasteboardObject alloc];
-      pb->name = [aName retain];
+      pb->name = RETAIN(aName);
       pb->nextCount = 1;
       pb->histLength = 1;
       pb->history = [[NSMutableArray alloc] initWithCapacity: 2];
       pb->current = nil;
       [pasteboards setObject: pb forKey: aName];
-      [pb autorelease];
+      AUTORELEASE(pb);
     }
   [dictionary_lock unlock];
   return pb;
@@ -672,7 +670,8 @@ NSMutableDictionary	*pasteboards = nil;
     }
   if (verbose)
     {
-      NSLog(@"get data for type '%@' version %d\n", type, e ? [e refNum] : -1);
+      NSLog(@"%@ get data for type '%@' version %d\n",
+	self, type, e ? [e refNum] : -1);
     }
   if (e)
     {
@@ -688,8 +687,8 @@ NSMutableDictionary	*pasteboards = nil;
 
 - (void) dealloc
 {
-  [name release];
-  [history release];
+  RELEASE(name);
+  RELEASE(history);
   [super dealloc];
 }
 
@@ -697,7 +696,7 @@ NSMutableDictionary	*pasteboards = nil;
 	       owner: (id)owner
 	  pasteboard: (NSPasteboard*)pb
 {
-  PasteboardEntry	*old = [current retain];
+  PasteboardEntry	*old = RETAIN(current);
   id			x = [XPbOwner ownerByOsPb: name];
 
   /*
@@ -715,19 +714,24 @@ NSMutableDictionary	*pasteboards = nil;
 				   pboard: pb
 				      ref: nextCount++];
   [history addObject: current];
-  [current release];
+  RELEASE(current);
   if ([history count] > histLength)
     {
       [history removeObjectAtIndex: 0];
     }
   [old lostOwnership];
-  [old release];
+  RELEASE(old);
   /*
    * If there is an interested X pasteboard - inform it of the ownership
    * change.
    */
   if (x != nil)
     [x pasteboardChangedOwner: pb];
+  if (verbose)
+    {
+      NSLog(@"%@ declare types '%@' version %d\n",
+	self, types, [current refNum]);
+    }
   return [current refNum];
 }
 
@@ -780,7 +784,7 @@ NSMutableDictionary	*pasteboards = nil;
 
   if (verbose)
     {
-      NSLog(@"set data for type '%@' version %d\n", type, count);
+      NSLog(@"%@ set data for type '%@' version %d\n", self, type, count);
     }
   e = [self entryByCount: count];
   if (e)
@@ -931,7 +935,7 @@ NSMutableDictionary	*pasteboards = nil;
 
 - (void) dealloc
 {
-  [permenant release];
+  RELEASE(permenant);
   [super dealloc];
 }
 
@@ -1084,7 +1088,7 @@ init(int argc, char** argv)
 int
 main(int argc, char** argv)
 {
-  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+  CREATE_AUTORELEASE_POOL(pool);
 
   init(argc, argv);
 
@@ -1120,7 +1124,7 @@ main(int argc, char** argv)
       NSLog(@"GNU pasteboard server startup.\n");
     }
   [[NSRunLoop currentRunLoop] run];
-  [pool release];
+  RELEASE(pool);
   exit(0);
 }
 
